.wave_on_click
    overflow: hidden
    position: relative
    min-height: 20px
    .wave
        opacity: 0.8
        width: 2px
        height: 2px
        background-color: $waves
        position: absolute
        border-radius: 100%
