#phanterpwa-snippet-phanterpwa-logo-wrapper
    background: #020024
    background: radial-gradient(circle, #060075  0%, #020024 87%, #000000 100%)
    h3 

        color: white
        width: 80%
        margin: auto
    h1
        color: orange
    .phanterpwa-background-logo
        height: 200px

        svg
            height: 100%
            width: auto
svg
    .phanterpwa-fil0
        fill: #336699
    .phanterpwa-fil1
        fill: #330066
    .phanterpwa-full_logo-fil2
        fill: #336699

    .phanterpwa-full_logo-fil3
        fill: #330066

    .phanterpwa-full_logo-fil0
        fill: #FFCC00
        fill-rule: nonzero

    .phanterpwa-full_logo-fil1
        fill: #990000
        fill-rule: nonzero

    .phanterpwa-full_logo-fil4
        fill: #330066

    .phanterpwa-full_logo-fil5
        fill: #330066
