// This is the list of predefined variables
// that are added when compiling your Sass file.
// You can overwrite or use them.
// The app-version variable is programmatically
// changed automatically using the app version.


$black: #000000
$white: #ffffff
$red_lighten_5: #ffebee
$red_lighten_4: #ffcdd2
$red_lighten_3: #ef9a9a
$red_lighten_2: #e57373
$red_lighten_1: #ef5350
$red: #f44336
$red_darken_1: #e53935
$red_darken_2: #d32f2f
$red_darken_3: #c62828
$red_darken_4: #b71c1c
$red_accent_1: #ff8a80
$red_accent_2: #ff5252
$red_accent_3: #ff1744
$red_accent_4: #d50000
$pink_lighten_5: #fce4ec
$pink_lighten_4: #f8bbd0
$pink_lighten_3: #f48fb1
$pink_lighten_2: #f06292
$pink_lighten_1: #ec407a
$pink: #e91e63
$pink_darken_1: #d81b60
$pink_darken_2: #c2185b
$pink_darken_3: #ad1457
$pink_darken_4: #880e4f
$pink_accent_1: #ff80ab
$pink_accent_2: #ff4081
$pink_accent_3: #f50057
$pink_accent_4: #c51162
$purple_lighten_5: #f3e5f5
$purple_lighten_4: #e1bee7
$purple_lighten_3: #ce93d8
$purple_lighten_2: #ba68c8
$purple_lighten_1: #ab47bc
$purple: #9c27b0
$purple_darken_1: #8e24aa
$purple_darken_2: #7b1fa2
$purple_darken_3: #6a1b9a
$purple_darken_4: #4a148c
$purple_accent_1: #ea80fc
$purple_accent_2: #e040fb
$purple_accent_3: #d500f9
$purple_accent_4: #aa00ff
$deep_purple_lighten_5: #ede7f6
$deep_purple_lighten_4: #d1c4e9
$deep_purple_lighten_3: #b39ddb
$deep_purple_lighten_2: #9575cd
$deep_purple_lighten_1: #7e57c2
$deep_purple: #673ab7
$deep_purple_darken_1: #5e35b1
$deep_purple_darken_2: #512da8
$deep_purple_darken_3: #4527a0
$deep_purple_darken_4: #311b92
$deep_purple_accent_1: #b388ff
$deep_purple_accent_2: #7c4dff
$deep_purple_accent_3: #651fff
$deep_purple_accent_4: #6200ea
$indigo_lighten_5: #e8eaf6
$indigo_lighten_4: #c5cae9
$indigo_lighten_3: #9fa8da
$indigo_lighten_2: #7986cb
$indigo_lighten_1: #5c6bc0
$indigo: #3f51b5
$indigo_darken_1: #3949ab
$indigo_darken_2: #303f9f
$indigo_darken_3: #283593
$indigo_darken_4: #1a237e
$indigo_accent_1: #8c9eff
$indigo_accent_2: #536dfe
$indigo_accent_3: #3d5afe
$indigo_accent_4: #304ffe
$blue_lighten_5: #e3f2fd
$blue_lighten_4: #bbdefb
$blue_lighten_3: #90caf9
$blue_lighten_2: #64b5f6
$blue_lighten_1: #42a5f5
$blue: #2196f3
$blue_darken_1: #1e88e5
$blue_darken_2: #1976d2
$blue_darken_3: #1565c0
$blue_darken_4: #0d47a1
$blue_accent_1: #82b1ff
$blue_accent_2: #448aff
$blue_accent_3: #2979ff
$blue_accent_4: #2962ff
$light_blue_lighten_5: #e1f5fe
$light_blue_lighten_4: #b3e5fc
$light_blue_lighten_3: #81d4fa
$light_blue_lighten_2: #4fc3f7
$light_blue_lighten_1: #29b6f6
$light_blue: #03a9f4
$light_blue_darken_1: #039be5
$light_blue_darken_2: #0288d1
$light_blue_darken_3: #0277bd
$light_blue_darken_4: #01579b
$light_blue_accent_1: #80d8ff
$light_blue_accent_2: #40c4ff
$light_blue_accent_3: #00b0ff
$light_blue_accent_4: #0091ea
$cyan_lighten_5: #e0f7fa
$cyan_lighten_4: #b2ebf2
$cyan_lighten_3: #80deea
$cyan_lighten_2: #4dd0e1
$cyan_lighten_1: #26c6da
$cyan: #00bcd4
$cyan_darken_1: #00acc1
$cyan_darken_2: #0097a7
$cyan_darken_3: #00838f
$cyan_darken_4: #006064
$cyan_accent_1: #84ffff
$cyan_accent_2: #18ffff
$cyan_accent_3: #00e5ff
$cyan_accent_4: #00b8d4
$teal_lighten_5: #e0f2f1
$teal_lighten_4: #b2dfdb
$teal_lighten_3: #80cbc4
$teal_lighten_2: #4db6ac
$teal_lighten_1: #26a69a
$teal: #009688
$teal_darken_1: #00897b
$teal_darken_2: #00796b
$teal_darken_3: #00695c
$teal_darken_4: #004d40
$teal_accent_1: #a7ffeb
$teal_accent_2: #64ffda
$teal_accent_3: #1de9b6
$teal_accent_4: #00bfa5
$green_lighten_5: #e8f5e9
$green_lighten_4: #c8e6c9
$green_lighten_3: #a5d6a7
$green_lighten_2: #81c784
$green_lighten_1: #66bb6a
$green: #4caf50
$green_darken_1: #43a047
$green_darken_2: #388e3c
$green_darken_3: #2e7d32
$green_darken_4: #1b5e20
$green_accent_1: #b9f6ca
$green_accent_2: #69f0ae
$green_accent_3: #00e676
$green_accent_4: #00c853
$light_green_lighten_5: #f1f8e9
$light_green_lighten_4: #dcedc8
$light_green_lighten_3: #c5e1a5
$light_green_lighten_2: #aed581
$light_green_lighten_1: #9ccc65
$light_green: #8bc34a
$light_green_darken_1: #7cb342
$light_green_darken_2: #689f38
$light_green_darken_3: #558b2f
$light_green_darken_4: #33691e
$light_green_accent_1: #ccff90
$light_green_accent_2: #b2ff59
$light_green_accent_3: #76ff03
$light_green_accent_4: #64dd17
$lime_lighten_5: #f9fbe7
$lime_lighten_4: #f0f4c3
$lime_lighten_3: #e6ee9c
$lime_lighten_2: #dce775
$lime_lighten_1: #d4e157
$lime: #cddc39
$lime_darken_1: #c0ca33
$lime_darken_2: #afb42b
$lime_darken_3: #9e9d24
$lime_darken_4: #827717
$lime_accent_1: #f4ff81
$lime_accent_2: #eeff41
$lime_accent_3: #c6ff00
$lime_accent_4: #aeea00
$yellow_lighten_5: #fffde7
$yellow_lighten_4: #fff9c4
$yellow_lighten_3: #fff59d
$yellow_lighten_2: #fff176
$yellow_lighten_1: #ffee58
$yellow: #ffeb3b
$yellow_darken_1: #fdd835
$yellow_darken_2: #fbc02d
$yellow_darken_3: #f9a825
$yellow_darken_4: #f57f17
$yellow_accent_1: #ffff8d
$yellow_accent_2: #ffff00
$yellow_accent_3: #ffea00
$yellow_accent_4: #ffd600
$amber_lighten_5: #fff8e1
$amber_lighten_4: #ffecb3
$amber_lighten_3: #ffe082
$amber_lighten_2: #ffd54f
$amber_lighten_1: #ffca28
$amber: #ffc107
$amber_darken_1: #ffb300
$amber_darken_2: #ffa000
$amber_darken_3: #ff8f00
$amber_darken_4: #ff6f00
$amber_accent_1: #ffe57f
$amber_accent_2: #ffd740
$amber_accent_3: #ffc400
$amber_accent_4: #ffab00
$orange_lighten_5: #fff3e0
$orange_lighten_4: #ffe0b2
$orange_lighten_3: #ffcc80
$orange_lighten_2: #ffb74d
$orange_lighten_1: #ffa726
$orange: #ff9800
$orange_darken_1: #fb8c00
$orange_darken_2: #f57c00
$orange_darken_3: #ef6c00
$orange_darken_4: #e65100
$orange_accent_1: #ffd180
$orange_accent_2: #ffab40
$orange_accent_3: #ff9100
$orange_accent_4: #ff6d00
$deep_orange_lighten_5: #fbe9e7
$deep_orange_lighten_4: #ffccbc
$deep_orange_lighten_3: #ffab91
$deep_orange_lighten_2: #ff8a65
$deep_orange_lighten_1: #ff7043
$deep_orange: #ff5722
$deep_orange_darken_1: #f4511e
$deep_orange_darken_2: #e64a19
$deep_orange_darken_3: #d84315
$deep_orange_darken_4: #bf360c
$deep_orange_accent_1: #ff9e80
$deep_orange_accent_2: #ff6e40
$deep_orange_accent_3: #ff3d00
$deep_orange_accent_4: #dd2c00
$brown_lighten_5: #efebe9
$brown_lighten_4: #d7ccc8
$brown_lighten_3: #bcaaa4
$brown_lighten_2: #a1887f
$brown_lighten_1: #8d6e63
$brown: #795548
$brown_darken_1: #6d4c41
$brown_darken_2: #5d4037
$brown_darken_3: #4e342e
$brown_darken_4: #3e2723
$grey_lighten_5: #fafafa
$grey_lighten_4: #f5f5f5
$grey_lighten_3: #eeeeee
$grey_lighten_2: #e0e0e0
$grey_lighten_1: #bdbdbd
$grey: #9e9e9e
$grey_darken_1: #757575
$grey_darken_2: #616161
$grey_darken_3: #424242
$grey_darken_4: #212121
$blue_grey_lighten_5: #eceff1
$blue_grey_lighten_4: #cfd8dc
$blue_grey_lighten_3: #b0bec5
$blue_grey_lighten_2: #90a4ae
$blue_grey_lighten_1: #78909c
$blue_grey: #607d8b
$blue_grey_darken_1: #546e7a
$blue_grey_darken_2: #455a64
$blue_grey_darken_3: #37474f
$blue_grey_darken_4: #263238

$default_font_color: rgba(0, 0, 0, 0.8)

$white: white
$material_level00: $white
$material_level01: #F8F8F8
$material_level02: #F5F5F5
$material_level03: #F0F0F0
$material_level04: #E8E8E8
$material_level05: #E0E0E0
$material_level06: #C1C1C1
$material_level07: #979797
$grey01: #F5F5F5
$grey02: #E0E0E0
$grey03: #D3D3D3
$grey04: #C0C0C0
$grey05: #B0B0B0
$grey06: #A0A0A0
$grey07: #888888
$grey08: #707070
$grey09: #606060
$grey10: #484848
$grey11: #303030
$grey12: #181818
$black: black

$alert: red
$warning: #c8c364
$attention: orange
$components: #4e6eaa
$fg_components: white
$ok: #62d062

$waves: rgba(200, 200, 200, 0.5)

$width1: "screen and (min-width: 0) and (max-width: 301px)"
$width2: "screen and (min-width: 300px) and (max-width: 601px)"
$width3: "screen and (min-width: 600px) and (max-width: 901px)"
$width4: "screen and (min-width: 900px) and (max-width: 1201px)"
$width5: "screen and (min-width: 1200px) and (max-width: 1501px)"
$width6: "screen and (min-width: 1500px) and (max-width: 1801px)"
$width7: "screen and (min-width: 1800px) and (max-width: 2100px)"
$width8: "screen and (min-width: 2100px)"

$width2_col: "screen and (min-width: 300px)"
$width3_col: "screen and (min-width: 600px)"
$width4_col: "screen and (min-width: 900px)"
$width5_col: "screen and (min-width: 1200px)"
$width6_col: "screen and (min-width: 1500px)"
$width7_col: "screen and (min-width: 1800px)"
$width8_col: "screen and (min-width: 2100px)"

$base-url: "/static/#{$app-version}/fonts/"
