.icon_awesome-wrapper
    font-size: 3rem
    padding: 6px
    text-align: center
    .icon_awesome-button
        &:hover
            color: orange
            cursor: pointer
    .icon_awesome-panel-container
        position: relative
    .icon_awesome_panel_wrapper 
        position: absolute
        padding: 10px
        font-size: 1rem
        display: block
        z-index: 900
        .icon_awesome_panel 
            background-color: rgb(255, 255, 255)
            padding: 10px
            font-size: 1rem
            display: block
            padding: 9px
            box-shadow: 1px 1px 4px rgb(128, 128, 128)
            padding-left: 33px
            .copy_buttom
                position: absolute
                left: 22px
                top: 21px
                &:hover
                    color: orange
                    cursor: pointer
