.phanterpwa-component-pseudomodal-container
    position: absolute
    z-index: 1006
    top: 0
    .phanterpwa-component-pseudomodal-wrapper
        position: relative
        .phanterpwa-component-pseudomodal-content-wrapper
            position: absolute
            width: auto
            box-shadow: 0px 1px 6px gray
            max-height: 50%
            display: table
            overflow: auto
            .phanterpwa-component-pseudomodal-content
                width: 100%
                overflow: auto
                max-height: 350px
                box-shadow: 1px 1px 4px gray
                background-color: white


