.phanterpwa-gallery-upload-image-button
     color: $components
     text-align: center
     i
          font-size: 3rem

.phanterpwa-gallery-upload-input-file
     display: none

.phanterpwa-gallery-upload-image-simple-view-buttons, .phanterpwa-gallery-upload-file-simple-view-buttons
     padding: 2px 4px
     background-color: rgba(0,0,0,0.7)
     position: absolute
     right: 5px
     top: 5px
     color: white
     border-radius: 1px
     i
          cursor: pointer
          &:hover
               color: $attention
.phanterpwa-gallery-upload-image-button
     i
          cursor: pointer
          &:hover
               color: $attention
.phanterpwa-gallery-wrapper
     position: relative

.phanterpwa-gallery-upload-file-button
     color: $components
     text-align: center
     i
          font-size: 3rem
          cursor: pointer
          &:hover
               color: $attention

.phanterpwa-gallery-upload-file-simple-view
     .phanterpwa-gallery-upload-file-icon_and_name
          .file-icon
               font-size: 100px

