@media #{$width1}
    .hidden_on_width1, .show_on_width2, .show_on_width3, .show_on_width4, .show_on_width5, .show_on_width6, .show_on_width7, .show_on_width8
        display: none !important
    .p-hidden_on_width1, .p-show_on_width2, .p-show_on_width3, .p-show_on_width4, .p-show_on_width5, .p-show_on_width6, .p-show_on_width7, .p-show_on_width8
        display: none !important
    .phanterpwa-hidden_on_width1, .phanterpwa-show_on_width2, .phanterpwa-show_on_width3, .phanterpwa-show_on_width4, .phanterpwa-show_on_width5, .phanterpwa-show_on_width6, .phanterpwa-show_on_width7, .phanterpwa-show_on_width8
        display: none !important
@media #{$width2}
    .show_on_width1, .hidden_on_width2, .show_on_width3, .show_on_width4, .show_on_width5, .show_on_width6, .show_on_width7, .show_on_width8
        display: none !important
    .p-show_on_width1, .p-hidden_on_width2, .p-show_on_width3, .p-show_on_width4, .p-show_on_width5, .p-show_on_width6, .p-show_on_width7, .p-show_on_width8
        display: none !important
    .phanterpwa-show_on_width1, .phanterpwa-hidden_on_width2, .phanterpwa-show_on_width3, .phanterpwa-show_on_width4, .phanterpwa-show_on_width5, .phanterpwa-show_on_width6, .phanterpwa-show_on_width7, .phanterpwa-show_on_width8
        display: none !important
@media #{$width3}
    .show_on_width1, .show_on_width2, .hidden_on_width3, .show_on_width4, .show_on_width5, .show_on_width6, .show_on_width7, .show_on_width8
        display: none !important
    .p-show_on_width1, .p-show_on_width2, .p-hidden_on_width3, .p-show_on_width4, .p-show_on_width5, .p-show_on_width6, .p-show_on_width7, .p-show_on_width8
        display: none !important
    .phanterpwa-show_on_width1, .phanterpwa-show_on_width2, .phanterpwa-hidden_on_width3, .phanterpwa-show_on_width4, .phanterpwa-show_on_width5, .phanterpwa-show_on_width6, .phanterpwa-show_on_width7, .phanterpwa-show_on_width8
        display: none !important
@media #{$width4}
    .show_on_width1, .show_on_width2, .show_on_width3, .hidden_on_width4, .show_on_width5, .show_on_width6, .show_on_width7, .show_on_width8
        display: none !important
    .p-show_on_width1, .p-show_on_width2, .p-show_on_width3, .p-hidden_on_width4, .p-show_on_width5, .p-show_on_width6, .p-show_on_width7, .p-show_on_width8
        display: none !important
    .phanterpwa-show_on_width1, .phanterpwa-show_on_width2, .phanterpwa-show_on_width3, .phanterpwa-hidden_on_width4, .phanterpwa-show_on_width5, .phanterpwa-show_on_width6, .phanterpwa-show_on_width7, .phanterpwa-show_on_width8
        display: none !important
@media #{$width5}
    .show_on_width1, .show_on_width2, .show_on_width3, .show_on_width4, .hidden_on_width5, .show_on_width6, .show_on_width7, .show_on_width8
        display: none !important
    .p-show_on_width1, .p-show_on_width2, .p-show_on_width3, .p-show_on_width4, .p-hidden_on_width5, .p-show_on_width6, .p-show_on_width7, .p-show_on_width8
        display: none !important
    .phanterpwa-show_on_width1, .phanterpwa-show_on_width2, .phanterpwa-show_on_width3, .phanterpwa-show_on_width4, .phanterpwa-hidden_on_width5, .phanterpwa-show_on_width6, .phanterpwa-show_on_width7, .phanterpwa-show_on_width8
        display: none !important
@media #{$width6}
    .show_on_width1, .show_on_width2, .show_on_width3, .show_on_width4, .show_on_width5, .hidden_on_width6, .show_on_width7, .show_on_width8
        display: none !important
    .p-show_on_width1, .p-show_on_width2, .p-show_on_width3, .p-show_on_width4, .p-show_on_width5, .p-hidden_on_width6, .p-show_on_width7, .p-show_on_width8
        display: none !important
    .phanterpwa-show_on_width1, .phanterpwa-show_on_width2, .phanterpwa-show_on_width3, .phanterpwa-show_on_width4, .phanterpwa-show_on_width5, .phanterpwa-hidden_on_width6, .phanterpwa-show_on_width7, .phanterpwa-show_on_width8
        display: none !important
@media #{$width7}
    .show_on_width1, .show_on_width2, .show_on_width3, .show_on_width4, .show_on_width5, .show_on_width6, .hidden_on_width7, .show_on_width8
        display: none !important
    .p-show_on_width1, .p-show_on_width2, .p-show_on_width3, .p-show_on_width4, .p-show_on_width5, .p-show_on_width6, .p-hidden_on_width7, .p-show_on_width8
        display: none !important
    .phanterpwa-show_on_width1, .phanterpwa-show_on_width2, .phanterpwa-show_on_width3, .phanterpwa-show_on_width4, .phanterpwa-show_on_width5, .phanterpwa-show_on_width6, .phanterpwa-hidden_on_width7, .phanterpwa-show_on_width8
        display: none !important
@media #{$width8}
    .show_on_width1, .show_on_width2, .show_on_width3, .show_on_width4, .show_on_width5, .show_on_width6, .show_on_width7, .hidden_on_width8
        display: none !important
    .p-show_on_width1, .p-show_on_width2, .p-show_on_width3, .p-show_on_width4, .p-show_on_width5, .p-show_on_width6, .p-show_on_width7, .p-hidden_on_width8
        display: none !important
    .phanterpwa-show_on_width1, .phanterpwa-show_on_width2, .phanterpwa-show_on_width3, .phanterpwa-show_on_width4, .phanterpwa-show_on_width5, .phanterpwa-show_on_width6, .phanterpwa-show_on_width7, .phanterpwa-hidden_on_width8
        display: none !important


.row, .p-row, .phanterpwa-row
    margin: 0px
    margin-left: auto
    margin-right: auto
    width: 100%
    &::after
        content: ""
        display: block
        clear: both
    .col, .p-col, .phanterpwa-col
        float: left
        margin-left: auto
        left: auto
        right: auto
        margin-right: auto
        min-height: 1px
        &.w1p1
            width: 1%
        &.w1p2
            width: 2%
        &.w1p3
            width: 3%
        &.w1p4
            width: 4%
        &.w1p5
            width: 5%
        &.w1p6
            width: 6%
        &.w1p7
            width: 7%
        &.w1p8
            width: 8%
        &.w1p9
            width: 9%
        &.w1p10
            width: 10%
        &.w1p11
            width: 11%
        &.w1p12
            width: 12%
        &.w1p13
            width: 13%
        &.w1p14
            width: 14%
        &.w1p15
            width: 15%
        &.w1p16
            width: 16%
        &.w1p17
            width: 17%
        &.w1p18
            width: 18%
        &.w1p19
            width: 19%
        &.w1p20
            width: 20%
        &.w1p21
            width: 21%
        &.w1p22
            width: 22%
        &.w1p23
            width: 23%
        &.w1p24
            width: 24%
        &.w1p25
            width: 25%
        &.w1p26
            width: 26%
        &.w1p27
            width: 27%
        &.w1p28
            width: 28%
        &.w1p29
            width: 29%
        &.w1p30
            width: 30%
        &.w1p31
            width: 31%
        &.w1p32
            width: 32%
        &.w1p33
            width: 33%
        &.w1p34
            width: 34%
        &.w1p35
            width: 35%
        &.w1p36
            width: 36%
        &.w1p37
            width: 37%
        &.w1p38
            width: 38%
        &.w1p39
            width: 39%
        &.w1p40
            width: 40%
        &.w1p41
            width: 41%
        &.w1p42
            width: 42%
        &.w1p43
            width: 43%
        &.w1p44
            width: 44%
        &.w1p45
            width: 45%
        &.w1p46
            width: 46%
        &.w1p47
            width: 47%
        &.w1p48
            width: 48%
        &.w1p49
            width: 49%
        &.w1p50
            width: 50%
        &.w1p51
            width: 51%
        &.w1p52
            width: 52%
        &.w1p53
            width: 53%
        &.w1p54
            width: 54%
        &.w1p55
            width: 55%
        &.w1p56
            width: 56%
        &.w1p57
            width: 57%
        &.w1p58
            width: 58%
        &.w1p59
            width: 59%
        &.w1p60
            width: 60%
        &.w1p61
            width: 61%
        &.w1p62
            width: 62%
        &.w1p63
            width: 63%
        &.w1p64
            width: 64%
        &.w1p65
            width: 65%
        &.w1p66
            width: 66%
        &.w1p67
            width: 67%
        &.w1p68
            width: 68%
        &.w1p69
            width: 69%
        &.w1p70
            width: 70%
        &.w1p71
            width: 71%
        &.w1p72
            width: 72%
        &.w1p73
            width: 73%
        &.w1p74
            width: 74%
        &.w1p75
            width: 75%
        &.w1p76
            width: 76%
        &.w1p77
            width: 77%
        &.w1p78
            width: 78%
        &.w1p79
            width: 79%
        &.w1p80
            width: 80%
        &.w1p81
            width: 81%
        &.w1p82
            width: 82%
        &.w1p83
            width: 83%
        &.w1p84
            width: 84%
        &.w1p85
            width: 85%
        &.w1p86
            width: 86%
        &.w1p87
            width: 87%
        &.w1p88
            width: 88%
        &.w1p89
            width: 89%
        &.w1p90
            width: 90%
        &.w1p91
            width: 91%
        &.w1p92
            width: 92%
        &.w1p93
            width: 93%
        &.w1p94
            width: 94%
        &.w1p95
            width: 95%
        &.w1p96
            width: 96%
        &.w1p97
            width: 97%
        &.w1p98
            width: 98%
        &.w1p99
            width: 99%
        &.w1p100
            width: 100%
        @media #{$width2_col}
            &.w2p1
                width: 1%
            &.w2p2
                width: 2%
            &.w2p3
                width: 3%
            &.w2p4
                width: 4%
            &.w2p5
                width: 5%
            &.w2p6
                width: 6%
            &.w2p7
                width: 7%
            &.w2p8
                width: 8%
            &.w2p9
                width: 9%
            &.w2p10
                width: 10%
            &.w2p11
                width: 11%
            &.w2p12
                width: 12%
            &.w2p13
                width: 13%
            &.w2p14
                width: 14%
            &.w2p15
                width: 15%
            &.w2p16
                width: 16%
            &.w2p17
                width: 17%
            &.w2p18
                width: 18%
            &.w2p19
                width: 19%
            &.w2p20
                width: 20%
            &.w2p21
                width: 21%
            &.w2p22
                width: 22%
            &.w2p23
                width: 23%
            &.w2p24
                width: 24%
            &.w2p25
                width: 25%
            &.w2p26
                width: 26%
            &.w2p27
                width: 27%
            &.w2p28
                width: 28%
            &.w2p29
                width: 29%
            &.w2p30
                width: 30%
            &.w2p31
                width: 31%
            &.w2p32
                width: 32%
            &.w2p33
                width: 33%
            &.w2p34
                width: 34%
            &.w2p35
                width: 35%
            &.w2p36
                width: 36%
            &.w2p37
                width: 37%
            &.w2p38
                width: 38%
            &.w2p39
                width: 39%
            &.w2p40
                width: 40%
            &.w2p41
                width: 41%
            &.w2p42
                width: 42%
            &.w2p43
                width: 43%
            &.w2p44
                width: 44%
            &.w2p45
                width: 45%
            &.w2p46
                width: 46%
            &.w2p47
                width: 47%
            &.w2p48
                width: 48%
            &.w2p49
                width: 49%
            &.w2p50
                width: 50%
            &.w2p51
                width: 51%
            &.w2p52
                width: 52%
            &.w2p53
                width: 53%
            &.w2p54
                width: 54%
            &.w2p55
                width: 55%
            &.w2p56
                width: 56%
            &.w2p57
                width: 57%
            &.w2p58
                width: 58%
            &.w2p59
                width: 59%
            &.w2p60
                width: 60%
            &.w2p61
                width: 61%
            &.w2p62
                width: 62%
            &.w2p63
                width: 63%
            &.w2p64
                width: 64%
            &.w2p65
                width: 65%
            &.w2p66
                width: 66%
            &.w2p67
                width: 67%
            &.w2p68
                width: 68%
            &.w2p69
                width: 69%
            &.w2p70
                width: 70%
            &.w2p71
                width: 71%
            &.w2p72
                width: 72%
            &.w2p73
                width: 73%
            &.w2p74
                width: 74%
            &.w2p75
                width: 75%
            &.w2p76
                width: 76%
            &.w2p77
                width: 77%
            &.w2p78
                width: 78%
            &.w2p79
                width: 79%
            &.w2p80
                width: 80%
            &.w2p81
                width: 81%
            &.w2p82
                width: 82%
            &.w2p83
                width: 83%
            &.w2p84
                width: 84%
            &.w2p85
                width: 85%
            &.w2p86
                width: 86%
            &.w2p87
                width: 87%
            &.w2p88
                width: 88%
            &.w2p89
                width: 89%
            &.w2p90
                width: 90%
            &.w2p91
                width: 91%
            &.w2p92
                width: 92%
            &.w2p93
                width: 93%
            &.w2p94
                width: 94%
            &.w2p95
                width: 95%
            &.w2p96
                width: 96%
            &.w2p97
                width: 97%
            &.w2p98
                width: 98%
            &.w2p99
                width: 99%
            &.w2p100
                width: 100%
        @media #{$width3_col}
            &.w3p1
                width: 1%
            &.w3p2
                width: 2%
            &.w3p3
                width: 3%
            &.w3p4
                width: 4%
            &.w3p5
                width: 5%
            &.w3p6
                width: 6%
            &.w3p7
                width: 7%
            &.w3p8
                width: 8%
            &.w3p9
                width: 9%
            &.w3p10
                width: 10%
            &.w3p11
                width: 11%
            &.w3p12
                width: 12%
            &.w3p13
                width: 13%
            &.w3p14
                width: 14%
            &.w3p15
                width: 15%
            &.w3p16
                width: 16%
            &.w3p17
                width: 17%
            &.w3p18
                width: 18%
            &.w3p19
                width: 19%
            &.w3p20
                width: 20%
            &.w3p21
                width: 21%
            &.w3p22
                width: 22%
            &.w3p23
                width: 23%
            &.w3p24
                width: 24%
            &.w3p25
                width: 25%
            &.w3p26
                width: 26%
            &.w3p27
                width: 27%
            &.w3p28
                width: 28%
            &.w3p29
                width: 29%
            &.w3p30
                width: 30%
            &.w3p31
                width: 31%
            &.w3p32
                width: 32%
            &.w3p33
                width: 33%
            &.w3p34
                width: 34%
            &.w3p35
                width: 35%
            &.w3p36
                width: 36%
            &.w3p37
                width: 37%
            &.w3p38
                width: 38%
            &.w3p39
                width: 39%
            &.w3p40
                width: 40%
            &.w3p41
                width: 41%
            &.w3p42
                width: 42%
            &.w3p43
                width: 43%
            &.w3p44
                width: 44%
            &.w3p45
                width: 45%
            &.w3p46
                width: 46%
            &.w3p47
                width: 47%
            &.w3p48
                width: 48%
            &.w3p49
                width: 49%
            &.w3p50
                width: 50%
            &.w3p51
                width: 51%
            &.w3p52
                width: 52%
            &.w3p53
                width: 53%
            &.w3p54
                width: 54%
            &.w3p55
                width: 55%
            &.w3p56
                width: 56%
            &.w3p57
                width: 57%
            &.w3p58
                width: 58%
            &.w3p59
                width: 59%
            &.w3p60
                width: 60%
            &.w3p61
                width: 61%
            &.w3p62
                width: 62%
            &.w3p63
                width: 63%
            &.w3p64
                width: 64%
            &.w3p65
                width: 65%
            &.w3p66
                width: 66%
            &.w3p67
                width: 67%
            &.w3p68
                width: 68%
            &.w3p69
                width: 69%
            &.w3p70
                width: 70%
            &.w3p71
                width: 71%
            &.w3p72
                width: 72%
            &.w3p73
                width: 73%
            &.w3p74
                width: 74%
            &.w3p75
                width: 75%
            &.w3p76
                width: 76%
            &.w3p77
                width: 77%
            &.w3p78
                width: 78%
            &.w3p79
                width: 79%
            &.w3p80
                width: 80%
            &.w3p81
                width: 81%
            &.w3p82
                width: 82%
            &.w3p83
                width: 83%
            &.w3p84
                width: 84%
            &.w3p85
                width: 85%
            &.w3p86
                width: 86%
            &.w3p87
                width: 87%
            &.w3p88
                width: 88%
            &.w3p89
                width: 89%
            &.w3p90
                width: 90%
            &.w3p91
                width: 91%
            &.w3p92
                width: 92%
            &.w3p93
                width: 93%
            &.w3p94
                width: 94%
            &.w3p95
                width: 95%
            &.w3p96
                width: 96%
            &.w3p97
                width: 97%
            &.w3p98
                width: 98%
            &.w3p99
                width: 99%
            &.w3p100
                width: 100%
        @media #{$width4_col}
            &.w4p1
                width: 1%
            &.w4p2
                width: 2%
            &.w4p3
                width: 3%
            &.w4p4
                width: 4%
            &.w4p5
                width: 5%
            &.w4p6
                width: 6%
            &.w4p7
                width: 7%
            &.w4p8
                width: 8%
            &.w4p9
                width: 9%
            &.w4p10
                width: 10%
            &.w4p11
                width: 11%
            &.w4p12
                width: 12%
            &.w4p13
                width: 13%
            &.w4p14
                width: 14%
            &.w4p15
                width: 15%
            &.w4p16
                width: 16%
            &.w4p17
                width: 17%
            &.w4p18
                width: 18%
            &.w4p19
                width: 19%
            &.w4p20
                width: 20%
            &.w4p21
                width: 21%
            &.w4p22
                width: 22%
            &.w4p23
                width: 23%
            &.w4p24
                width: 24%
            &.w4p25
                width: 25%
            &.w4p26
                width: 26%
            &.w4p27
                width: 27%
            &.w4p28
                width: 28%
            &.w4p29
                width: 29%
            &.w4p30
                width: 30%
            &.w4p31
                width: 31%
            &.w4p32
                width: 32%
            &.w4p33
                width: 33%
            &.w4p34
                width: 34%
            &.w4p35
                width: 35%
            &.w4p36
                width: 36%
            &.w4p37
                width: 37%
            &.w4p38
                width: 38%
            &.w4p39
                width: 39%
            &.w4p40
                width: 40%
            &.w4p41
                width: 41%
            &.w4p42
                width: 42%
            &.w4p43
                width: 43%
            &.w4p44
                width: 44%
            &.w4p45
                width: 45%
            &.w4p46
                width: 46%
            &.w4p47
                width: 47%
            &.w4p48
                width: 48%
            &.w4p49
                width: 49%
            &.w4p50
                width: 50%
            &.w4p51
                width: 51%
            &.w4p52
                width: 52%
            &.w4p53
                width: 53%
            &.w4p54
                width: 54%
            &.w4p55
                width: 55%
            &.w4p56
                width: 56%
            &.w4p57
                width: 57%
            &.w4p58
                width: 58%
            &.w4p59
                width: 59%
            &.w4p60
                width: 60%
            &.w4p61
                width: 61%
            &.w4p62
                width: 62%
            &.w4p63
                width: 63%
            &.w4p64
                width: 64%
            &.w4p65
                width: 65%
            &.w4p66
                width: 66%
            &.w4p67
                width: 67%
            &.w4p68
                width: 68%
            &.w4p69
                width: 69%
            &.w4p70
                width: 70%
            &.w4p71
                width: 71%
            &.w4p72
                width: 72%
            &.w4p73
                width: 73%
            &.w4p74
                width: 74%
            &.w4p75
                width: 75%
            &.w4p76
                width: 76%
            &.w4p77
                width: 77%
            &.w4p78
                width: 78%
            &.w4p79
                width: 79%
            &.w4p80
                width: 80%
            &.w4p81
                width: 81%
            &.w4p82
                width: 82%
            &.w4p83
                width: 83%
            &.w4p84
                width: 84%
            &.w4p85
                width: 85%
            &.w4p86
                width: 86%
            &.w4p87
                width: 87%
            &.w4p88
                width: 88%
            &.w4p89
                width: 89%
            &.w4p90
                width: 90%
            &.w4p91
                width: 91%
            &.w4p92
                width: 92%
            &.w4p93
                width: 93%
            &.w4p94
                width: 94%
            &.w4p95
                width: 95%
            &.w4p96
                width: 96%
            &.w4p97
                width: 97%
            &.w4p98
                width: 98%
            &.w4p99
                width: 99%
            &.w4p100
                width: 100%
        @media #{$width5_col}
            &.w5p1
                width: 1%
            &.w5p2
                width: 2%
            &.w5p3
                width: 3%
            &.w5p4
                width: 4%
            &.w5p5
                width: 5%
            &.w5p6
                width: 6%
            &.w5p7
                width: 7%
            &.w5p8
                width: 8%
            &.w5p9
                width: 9%
            &.w5p10
                width: 10%
            &.w5p11
                width: 11%
            &.w5p12
                width: 12%
            &.w5p13
                width: 13%
            &.w5p14
                width: 14%
            &.w5p15
                width: 15%
            &.w5p16
                width: 16%
            &.w5p17
                width: 17%
            &.w5p18
                width: 18%
            &.w5p19
                width: 19%
            &.w5p20
                width: 20%
            &.w5p21
                width: 21%
            &.w5p22
                width: 22%
            &.w5p23
                width: 23%
            &.w5p24
                width: 24%
            &.w5p25
                width: 25%
            &.w5p26
                width: 26%
            &.w5p27
                width: 27%
            &.w5p28
                width: 28%
            &.w5p29
                width: 29%
            &.w5p30
                width: 30%
            &.w5p31
                width: 31%
            &.w5p32
                width: 32%
            &.w5p33
                width: 33%
            &.w5p34
                width: 34%
            &.w5p35
                width: 35%
            &.w5p36
                width: 36%
            &.w5p37
                width: 37%
            &.w5p38
                width: 38%
            &.w5p39
                width: 39%
            &.w5p40
                width: 40%
            &.w5p41
                width: 41%
            &.w5p42
                width: 42%
            &.w5p43
                width: 43%
            &.w5p44
                width: 44%
            &.w5p45
                width: 45%
            &.w5p46
                width: 46%
            &.w5p47
                width: 47%
            &.w5p48
                width: 48%
            &.w5p49
                width: 49%
            &.w5p50
                width: 50%
            &.w5p51
                width: 51%
            &.w5p52
                width: 52%
            &.w5p53
                width: 53%
            &.w5p54
                width: 54%
            &.w5p55
                width: 55%
            &.w5p56
                width: 56%
            &.w5p57
                width: 57%
            &.w5p58
                width: 58%
            &.w5p59
                width: 59%
            &.w5p60
                width: 60%
            &.w5p61
                width: 61%
            &.w5p62
                width: 62%
            &.w5p63
                width: 63%
            &.w5p64
                width: 64%
            &.w5p65
                width: 65%
            &.w5p66
                width: 66%
            &.w5p67
                width: 67%
            &.w5p68
                width: 68%
            &.w5p69
                width: 69%
            &.w5p70
                width: 70%
            &.w5p71
                width: 71%
            &.w5p72
                width: 72%
            &.w5p73
                width: 73%
            &.w5p74
                width: 74%
            &.w5p75
                width: 75%
            &.w5p76
                width: 76%
            &.w5p77
                width: 77%
            &.w5p78
                width: 78%
            &.w5p79
                width: 79%
            &.w5p80
                width: 80%
            &.w5p81
                width: 81%
            &.w5p82
                width: 82%
            &.w5p83
                width: 83%
            &.w5p84
                width: 84%
            &.w5p85
                width: 85%
            &.w5p86
                width: 86%
            &.w5p87
                width: 87%
            &.w5p88
                width: 88%
            &.w5p89
                width: 89%
            &.w5p90
                width: 90%
            &.w5p91
                width: 91%
            &.w5p92
                width: 92%
            &.w5p93
                width: 93%
            &.w5p94
                width: 94%
            &.w5p95
                width: 95%
            &.w5p96
                width: 96%
            &.w5p97
                width: 97%
            &.w5p98
                width: 98%
            &.w5p99
                width: 99%
            &.w5p100
                width: 100%
        @media #{$width6_col}
            &.w6p1
                width: 1%
            &.w6p2
                width: 2%
            &.w6p3
                width: 3%
            &.w6p4
                width: 4%
            &.w6p5
                width: 5%
            &.w6p6
                width: 6%
            &.w6p7
                width: 7%
            &.w6p8
                width: 8%
            &.w6p9
                width: 9%
            &.w6p10
                width: 10%
            &.w6p11
                width: 11%
            &.w6p12
                width: 12%
            &.w6p13
                width: 13%
            &.w6p14
                width: 14%
            &.w6p15
                width: 15%
            &.w6p16
                width: 16%
            &.w6p17
                width: 17%
            &.w6p18
                width: 18%
            &.w6p19
                width: 19%
            &.w6p20
                width: 20%
            &.w6p21
                width: 21%
            &.w6p22
                width: 22%
            &.w6p23
                width: 23%
            &.w6p24
                width: 24%
            &.w6p25
                width: 25%
            &.w6p26
                width: 26%
            &.w6p27
                width: 27%
            &.w6p28
                width: 28%
            &.w6p29
                width: 29%
            &.w6p30
                width: 30%
            &.w6p31
                width: 31%
            &.w6p32
                width: 32%
            &.w6p33
                width: 33%
            &.w6p34
                width: 34%
            &.w6p35
                width: 35%
            &.w6p36
                width: 36%
            &.w6p37
                width: 37%
            &.w6p38
                width: 38%
            &.w6p39
                width: 39%
            &.w6p40
                width: 40%
            &.w6p41
                width: 41%
            &.w6p42
                width: 42%
            &.w6p43
                width: 43%
            &.w6p44
                width: 44%
            &.w6p45
                width: 45%
            &.w6p46
                width: 46%
            &.w6p47
                width: 47%
            &.w6p48
                width: 48%
            &.w6p49
                width: 49%
            &.w6p50
                width: 50%
            &.w6p51
                width: 51%
            &.w6p52
                width: 52%
            &.w6p53
                width: 53%
            &.w6p54
                width: 54%
            &.w6p55
                width: 55%
            &.w6p56
                width: 56%
            &.w6p57
                width: 57%
            &.w6p58
                width: 58%
            &.w6p59
                width: 59%
            &.w6p60
                width: 60%
            &.w6p61
                width: 61%
            &.w6p62
                width: 62%
            &.w6p63
                width: 63%
            &.w6p64
                width: 64%
            &.w6p65
                width: 65%
            &.w6p66
                width: 66%
            &.w6p67
                width: 67%
            &.w6p68
                width: 68%
            &.w6p69
                width: 69%
            &.w6p70
                width: 70%
            &.w6p71
                width: 71%
            &.w6p72
                width: 72%
            &.w6p73
                width: 73%
            &.w6p74
                width: 74%
            &.w6p75
                width: 75%
            &.w6p76
                width: 76%
            &.w6p77
                width: 77%
            &.w6p78
                width: 78%
            &.w6p79
                width: 79%
            &.w6p80
                width: 80%
            &.w6p81
                width: 81%
            &.w6p82
                width: 82%
            &.w6p83
                width: 83%
            &.w6p84
                width: 84%
            &.w6p85
                width: 85%
            &.w6p86
                width: 86%
            &.w6p87
                width: 87%
            &.w6p88
                width: 88%
            &.w6p89
                width: 89%
            &.w6p90
                width: 90%
            &.w6p91
                width: 91%
            &.w6p92
                width: 92%
            &.w6p93
                width: 93%
            &.w6p94
                width: 94%
            &.w6p95
                width: 95%
            &.w6p96
                width: 96%
            &.w6p97
                width: 97%
            &.w6p98
                width: 98%
            &.w6p99
                width: 99%
            &.w6p100
                width: 100%
        @media #{$width7_col}
            &.w7p1
                width: 1%
            &.w7p2
                width: 2%
            &.w7p3
                width: 3%
            &.w7p4
                width: 4%
            &.w7p5
                width: 5%
            &.w7p6
                width: 6%
            &.w7p7
                width: 7%
            &.w7p8
                width: 8%
            &.w7p9
                width: 9%
            &.w7p10
                width: 10%
            &.w7p11
                width: 11%
            &.w7p12
                width: 12%
            &.w7p13
                width: 13%
            &.w7p14
                width: 14%
            &.w7p15
                width: 15%
            &.w7p16
                width: 16%
            &.w7p17
                width: 17%
            &.w7p18
                width: 18%
            &.w7p19
                width: 19%
            &.w7p20
                width: 20%
            &.w7p21
                width: 21%
            &.w7p22
                width: 22%
            &.w7p23
                width: 23%
            &.w7p24
                width: 24%
            &.w7p25
                width: 25%
            &.w7p26
                width: 26%
            &.w7p27
                width: 27%
            &.w7p28
                width: 28%
            &.w7p29
                width: 29%
            &.w7p30
                width: 30%
            &.w7p31
                width: 31%
            &.w7p32
                width: 32%
            &.w7p33
                width: 33%
            &.w7p34
                width: 34%
            &.w7p35
                width: 35%
            &.w7p36
                width: 36%
            &.w7p37
                width: 37%
            &.w7p38
                width: 38%
            &.w7p39
                width: 39%
            &.w7p40
                width: 40%
            &.w7p41
                width: 41%
            &.w7p42
                width: 42%
            &.w7p43
                width: 43%
            &.w7p44
                width: 44%
            &.w7p45
                width: 45%
            &.w7p46
                width: 46%
            &.w7p47
                width: 47%
            &.w7p48
                width: 48%
            &.w7p49
                width: 49%
            &.w7p50
                width: 50%
            &.w7p51
                width: 51%
            &.w7p52
                width: 52%
            &.w7p53
                width: 53%
            &.w7p54
                width: 54%
            &.w7p55
                width: 55%
            &.w7p56
                width: 56%
            &.w7p57
                width: 57%
            &.w7p58
                width: 58%
            &.w7p59
                width: 59%
            &.w7p60
                width: 60%
            &.w7p61
                width: 61%
            &.w7p62
                width: 62%
            &.w7p63
                width: 63%
            &.w7p64
                width: 64%
            &.w7p65
                width: 65%
            &.w7p66
                width: 66%
            &.w7p67
                width: 67%
            &.w7p68
                width: 68%
            &.w7p69
                width: 69%
            &.w7p70
                width: 70%
            &.w7p71
                width: 71%
            &.w7p72
                width: 72%
            &.w7p73
                width: 73%
            &.w7p74
                width: 74%
            &.w7p75
                width: 75%
            &.w7p76
                width: 76%
            &.w7p77
                width: 77%
            &.w7p78
                width: 78%
            &.w7p79
                width: 79%
            &.w7p80
                width: 80%
            &.w7p81
                width: 81%
            &.w7p82
                width: 82%
            &.w7p83
                width: 83%
            &.w7p84
                width: 84%
            &.w7p85
                width: 85%
            &.w7p86
                width: 86%
            &.w7p87
                width: 87%
            &.w7p88
                width: 88%
            &.w7p89
                width: 89%
            &.w7p90
                width: 90%
            &.w7p91
                width: 91%
            &.w7p92
                width: 92%
            &.w7p93
                width: 93%
            &.w7p94
                width: 94%
            &.w7p95
                width: 95%
            &.w7p96
                width: 96%
            &.w7p97
                width: 97%
            &.w7p98
                width: 98%
            &.w7p99
                width: 99%
            &.w7p100
                width: 100%
        @media #{$width8_col}
            &.w8p1
                width: 1%
            &.w8p2
                width: 2%
            &.w8p3
                width: 3%
            &.w8p4
                width: 4%
            &.w8p5
                width: 5%
            &.w8p6
                width: 6%
            &.w8p7
                width: 7%
            &.w8p8
                width: 8%
            &.w8p9
                width: 9%
            &.w8p10
                width: 10%
            &.w8p11
                width: 11%
            &.w8p12
                width: 12%
            &.w8p13
                width: 13%
            &.w8p14
                width: 14%
            &.w8p15
                width: 15%
            &.w8p16
                width: 16%
            &.w8p17
                width: 17%
            &.w8p18
                width: 18%
            &.w8p19
                width: 19%
            &.w8p20
                width: 20%
            &.w8p21
                width: 21%
            &.w8p22
                width: 22%
            &.w8p23
                width: 23%
            &.w8p24
                width: 24%
            &.w8p25
                width: 25%
            &.w8p26
                width: 26%
            &.w8p27
                width: 27%
            &.w8p28
                width: 28%
            &.w8p29
                width: 29%
            &.w8p30
                width: 30%
            &.w8p31
                width: 31%
            &.w8p32
                width: 32%
            &.w8p33
                width: 33%
            &.w8p34
                width: 34%
            &.w8p35
                width: 35%
            &.w8p36
                width: 36%
            &.w8p37
                width: 37%
            &.w8p38
                width: 38%
            &.w8p39
                width: 39%
            &.w8p40
                width: 40%
            &.w8p41
                width: 41%
            &.w8p42
                width: 42%
            &.w8p43
                width: 43%
            &.w8p44
                width: 44%
            &.w8p45
                width: 45%
            &.w8p46
                width: 46%
            &.w8p47
                width: 47%
            &.w8p48
                width: 48%
            &.w8p49
                width: 49%
            &.w8p50
                width: 50%
            &.w8p51
                width: 51%
            &.w8p52
                width: 52%
            &.w8p53
                width: 53%
            &.w8p54
                width: 54%
            &.w8p55
                width: 55%
            &.w8p56
                width: 56%
            &.w8p57
                width: 57%
            &.w8p58
                width: 58%
            &.w8p59
                width: 59%
            &.w8p60
                width: 60%
            &.w8p61
                width: 61%
            &.w8p62
                width: 62%
            &.w8p63
                width: 63%
            &.w8p64
                width: 64%
            &.w8p65
                width: 65%
            &.w8p66
                width: 66%
            &.w8p67
                width: 67%
            &.w8p68
                width: 68%
            &.w8p69
                width: 69%
            &.w8p70
                width: 70%
            &.w8p71
                width: 71%
            &.w8p72
                width: 72%
            &.w8p73
                width: 73%
            &.w8p74
                width: 74%
            &.w8p75
                width: 75%
            &.w8p76
                width: 76%
            &.w8p77
                width: 77%
            &.w8p78
                width: 78%
            &.w8p79
                width: 79%
            &.w8p80
                width: 80%
            &.w8p81
                width: 81%
            &.w8p82
                width: 82%
            &.w8p83
                width: 83%
            &.w8p84
                width: 84%
            &.w8p85
                width: 85%
            &.w8p86
                width: 86%
            &.w8p87
                width: 87%
            &.w8p88
                width: 88%
            &.w8p89
                width: 89%
            &.w8p90
                width: 90%
            &.w8p91
                width: 91%
            &.w8p92
                width: 92%
            &.w8p93
                width: 93%
            &.w8p94
                width: 94%
            &.w8p95
                width: 95%
            &.w8p96
                width: 96%
            &.w8p97
                width: 97%
            &.w8p98
                width: 98%
            &.w8p99
                width: 99%
            &.w8p100
                width: 100%