.e-padding_10
    padding: 10px !important
    hr
        width: calc(100% + 20px)
        margin-left: -10px
        color: rgb(255, 255, 255)
        outline: none
        background-color: gray
        margin-bottom: 20px
    &.phanterpwa-container-section
        .phanterpwa-section
            &>label
                top: 5px
                left: 20px

.e-padding_bottom_10
    padding-bottom: 10px !important

.e-padding_top_10
    padding-top: 10px !important

.e-padding_left_10
    padding-left: 10px !important

.e-padding_right_10
    margin-right: 10px !important

.e-padding_20
    padding: 20px !important
    hr
        width: calc(100% + 40px)
        margin-left: -20px
    &.phanterpwa-container-section
        .phanterpwa-section
            &>label
                top: 10px
                left: 30px
.e-padding_auto
    padding: 20px !important
    @media #{$width1}
        padding: 5px !important
    @media #{$width2}
        padding: 10px !important
    @media #{$width3}
        padding: 15px !important

.e-padding_bottom_20
    padding-bottom: 20px !important

.e-padding_top_20
    padding-top: 20px !important

.e-padding_left_20
    padding-left: 20px !important

.e-padding_right_20
    padding-right: 20px !important


.e-display_hidden
    display: none !important

.e-overflow_hidden
    overflow: hidden !important

.e-text_align_center
    text-align: center !important

.e-float_left
    float: left !important

.e-float_right
    float: right !important

.e-title-alert
    text-align: center !important
    color: red !important

.e-margin_20
    margin: 20px

.e-margin_bottom_20
    margin-bottom: 20px !important

.e-margin_top_20
    margin-top: 20px !important

.e-margin_left_20
    margin-left: 20px !important

.e-margin_right_20
    margin-right: 20px !important

.e-margin_10
    margin: 10px !important

.e-margin_bottom_10
    margin-bottom: 10px !important

.e-margin_top_10
    margin-top: 10px !important

.e-margin_left_10
    margin-left: 10px !important

.e-margin_right_10
    margin-right: 10px !important

.e-tagger-wrapper
    overflow: hidden
    text-overflow: ellipsis
    border: 1px solid $components
    margin: 2px
    border-radius: 5px
    line-height: 2rem
    strong
        background-color: $components
        padding: 0px 10px
        color: white
        height: 2rem
        display: block
        float: left
    span
        padding: 6px
        white-space: nowrap

    @media #{$width2}
        strong
            float: none
    @media #{$width1}
        strong
            float: none

    .e-tagger-button
        float: right
        font-size: 0.7rem
        width: 2rem
        height: 2rem
        text-align: center
        color: $components

.e-hidden
    display: none

.e-link
    cursor: pointer
    &:hover
        color: $attention

.e-text_align-center
    text-align: center !important
.e-text_align-right
    text-align: right !important
.e-text_align-justify
    text-align: justify !important