.phanterpwa-paper-a4
    width: 210mm
    height: 297mm
    overflow: hidden
    display: table
    margin: 10px auto
    background-color: white
    padding: 20px
    page-break-after: always


@page
    margin: 0 !important

@media print
    body
        padding: 0 !important
        margin: 0 !important
        background-color: transparent !important
        *
            visibility: hidden
        .phanterpwa-simple-media-print
            position: absolute
            left: 0
            top: 0
            .media-print-visible
                visibility: visible !important
                *
                    visibility: visible !important
                thead
                    height: 20px
                    display: table-header-group

                tfoot
                    height: 20px
                    display: table-footer-group

                tr
                    page-break-inside: avoid

                tbody
                    page-break-inside: avoid
                    page-break-after: auto
                .media-print-inside-invisible
                    visibility: hidden !important
                    display: none
                    *
                        visibility: hidden !important
            .media-print-invisible
                visibility: hidden !important
                *
                    visibility: hidden !important
                .media-print-inside-visible
                    visibility: visible !important
                    *
                        visibility: visible !important
                    thead
                        height: 20px
                        display: table-header-group

                    tfoot
                        height: 20px
                        display: table-footer-group

                    tr
                        page-break-inside: avoid

                    tbody
                        page-break-inside: avoid
                        page-break-after: auto
        .phanterpwa-media-print-container
            table
                page-break-inside: auto
                visibility: visible
            tr
                page-break-inside: avoid
                visibility: visible
            tbody
                page-break-inside: avoid
                page-break-after: auto
                visibility: visible
            thead
                display: table-header-group
                visibility: visible
            tfoot
                display: table-footer-group
                visibility: visible
            .media-print-visible
                visibility: visible !important
                *
                    visibility: visible !important
            .media-print-inside-invisible
                visibility: hidden
                *
                    visibility: hidden !important

            position: absolute
            left: 0
            top: 0

            .continuos
                width: 100%
                visibility: visible
                *
                    visibility: visible
            .dont_cut_inside
                break-inside: avoid

        .phanterpwa-media-print
            table
                page-break-inside: auto
            tr
                page-break-inside: avoid
            tbody
                page-break-inside: avoid
                page-break-after: auto
            thead
                display: table-header-group
            tfoot
                display: table-footer-group

            padding: 0
            margin: 0
            .a4
                margin: 0
                padding: 0
                visibility: visible
                width: 210mm
                height: 297mm
                *
                    visibility: visible
            .dont_cut_inside
                break-inside: avoid
