# -*- coding: utf-8 -*-
# author: PhanterJR<junior.conex@gmail.com>
# license: MIT

__author__ = "PhanterJR<junior.conex@gmail.com>"
__version__ = "14.4.2"
__install_requeriments__ = [
    "psutil",
    "tornado",
    "libsass",
    "transcrypt==3.9.0",
    "pillow",
    "itsdangerous==2.1.2",
    "requests-oauthlib",
    "pydal>=19.04",
    "passlib",
    "appdirs>=1.4.3"
]
