
.phanterpwa-captchasvg-0101
    .fil0
        fill: #990000
    .fil1
        fill: #1F1A17
        fill-rule: nonzero

.phanterpwa-captchasvg-0102
    .fil0
        fill: #990000
    .fil1
        fill: white
        fill-rule: nonzero

.phanterpwa-captchasvg-0103
    .fil0
        fill: #990000
    .fil1
        fill: #009933
        fill-rule: nonzero

.phanterpwa-captchasvg-0201
    .fil0
        fill: #FFCC00
    .fil1
        fill: #1F1A17
        fill-rule: nonzero

.phanterpwa-captchasvg-0202
    .fil0
        fill: #FFCC00
    .fil1
        fill: white
        fill-rule: nonzero

.phanterpwa-captchasvg-0203
    .fil0
        fill: #FFCC00
    .fil1
        fill: #009933
        fill-rule: nonzero

.phanterpwa-captchasvg-0301
    .fil0
        fill: #6699FF
    .fil1
        fill: #1F1A17
        fill-rule: nonzero

.phanterpwa-captchasvg-0302
    .fil0
        fill: #6699FF
    .fil1
        fill: white
        fill-rule: nonzero

.phanterpwa-captchasvg-0303
    .fil0
        fill: #6699FF
    .fil1
        fill: #009933
        fill-rule: nonzero
