.promo-container
    a
        text-decoration: none
        color: $components
    &.link
        &:hover
            .promo-icon
                color: $attention
    text-align: center
    padding: 20px
    .promo-icon
        font-size: 6rem
.title_page_container
    display: table
    margin: 0 0 30px 0
    padding: 8px
    width: 100%
    text-transform: uppercase
    font-size: 1.6rem
    text-align: center
    &.card
        border-radius: 0px
        min-height: 58px
        @media #{$width1}
            min-height: auto
        @media #{$width2}
            min-height: auto
    h1, h2, h3, h4, h5
        margin: 0