.phanterpwa-xsection-container
	position: relative
	display: block
	padding: 10px 0
	width: 100%
	overflow: auto

	.phanterpwa-xsection
		border: 1px solid #d7d7d7
		overflow: auto
		border-radius: 5px
		width: 100%
		padding: 15px 0px 10px 0px
		.phanterpwa-xsection-menu_buttom
			position: absolute
			top: 15px
			right: 7px
		&>label
			padding: 5px
			color: $components
			position: absolute
			top: 0
			left: 15px
			background-color: white
			padding-left: 10px
			margin: 0
			border: 1px solid #d7d7d7
			padding-right: 10px
			border-radius: 5px
			font-size: 0.8rem
		&>hr
			border: 0.5px solid #eeeded

		.phanterpwa-xsection-title
			width: 100%
			text-align: center
			color: white
			background: #4e6eaa
			height: 51px
			margin-top: -16px
			padding: 16px 20px

	&.has_menu_button
		.phanterpwa-xsection
			padding: 50px 0px 10px 0px
		.phanterpwa-xsection-title
			margin-top: -50px

	&.e-padding_20
		.phanterpwa-xsection
			padding: 20px
	&.e-padding_10
		.phanterpwa-xsection
			padding: 10px

.phanterpwa-component-modal-container
	.phanterpwa-xsection-container
		.phanterpwa-xsection
			border: 1px solid $material_level06
			&>label
				border: 1px solid $material_level06
				background-color: $material_level03

.phanterpwa-card-panel-control-content
	.phanterpwa-xsection-container
		.phanterpwa-xsection
			border: 1px solid $material_level06
			&>label
				border: 1px solid $material_level06
				background-color: $material_level03