.phanterpwa-widget-captcha-container
    text-align: center
    height: 131px
    width: 300px
    overflow-x: hidden
    overflow-y: hidden
    margin: 0px auto 0px auto
    border: none
    padding: 0px
    .preloader-wrapper
        width: 131px !important
        height: 131px !important
    @media #{$width1}
        overflow-x: auto
        overflow-y: hidden
        margin: 2px auto 2px auto
        border: 1px #d5d3d0 solid    
    @media #{$width2}
        overflow-x: auto
        overflow-y: hidden
        margin: 2px auto 2px auto
        border: 1px #d5d3d0 solid

    .captcha-container, .captcha-options-container 
        width: auto 

    .captcha-option-11 
        .fil0 
            fill: #6699FF 
        .fil1 
            fill: #1F1A17 
            fill-rule: nonzero 
    .captcha-option-12 
        .fil0 
            fill: #FFCC00 
        .fil1 
            fill: white 
            fill-rule: nonzero 
    .captcha-option-13 
        .fil0 
            fill: #990000 
        .fil1 
            fill: white 
            fill-rule: nonzero 
    .captcha-option-14 
        .fil1 
            fill: #009933 
        .fil0 
            fill: fuchsia 
    .captcha-option-21 
        .fil0 
            fill: fuchsia 
        .fil1 
            fill: #7F7F7F 
            fill-rule: nonzero 
    .captcha-option-22 
        .fil0 
            fill: #6699FF 
        .fil1 
            fill: #1F1A17 
            fill-rule: nonzero 
    .captcha-option-23 
        .fil0 
            fill: #FFCC00 
        .fil1 
            fill: #009933 
            fill-rule: nonzero 
    .captcha-option-24 
        .fil1 
            fill: black 
        .fil0 
            fill: #990000 
    .captcha-option-31 
        .fil0 
            fill: #990000 
        .fil1 
            fill: #1F1A17 
            fill-rule: nonzero 
    .captcha-option-32 
        .fil0 
            fill: fuchsia 
        .fil1 
            fill: #009933 
            fill-rule: nonzero 
    .captcha-option-33 
        .fil0 
            fill: #6699FF 
        .fil1 
            fill: #7F7F7F 
            fill-rule: nonzero 
    .captcha-option-34 
        .fil1 
            fill: #7F7F7F 
        .fil0 
            fill: #FFCC00 
    .captcha-option-41 
        .fil0 
            fill: #FFCC00 
        .fil1 
            fill: #009933 
            fill-rule: nonzero 
    .captcha-option-42 
        .fil0 
            fill: #990000 
        .fil1 
            fill: #7F7F7F 
            fill-rule: nonzero 
    .captcha-option-43 
        .fil0 
            fill: fuchsia 
        .fil1 
            fill: white 
            fill-rule: nonzero 
    .captcha-option-44 
        .fil0 
            fill: #6699FF 
        .fil1 
            fill: white 
    .captcha-ok-svg-container 
        padding-top: 10px 
        padding-bottom: 10px 
        .fil0 
            fill: green 
        .fil1 
            fill: white 
            fill-rule: nonzero 
    .captcha-question-container 
        font-size: 0.8rem 
        min-height: 36px 
        text-transform: uppercase 
    #captcha-ok-svg-container-login 
        display: none 
    .captcha-container 
        width: 300px 
        margin-left: auto 
        margin-right: auto 
        padding: 10px 
        text-align: center 
    .captcha-options-container 
        display: block 
        width: 240px 
        margin-left: auto 
        margin-right: auto 
        @media #{$width1}
            width: 200px
        @media #{$width2}
            width: 200px
    .captcha-option-container 
        float: left 
        padding: 5px 
        width: 60px 
        display: inline-block 
        padding: 5px 
        float: none 
        display: inline-block 
        padding: 5px 
        float: none 
        @media #{$width1}
            width: 50px
        @media #{$width2}
            width: 50px
    .captcha-option
        transition: box-shadow 0.2s ease
        svg
            transition: width 0.2s ease, margin-top, margin-top 0.2s ease
        &:hover 
            background-color: #e4e4e4 
            border-radius: 2px 
            box-shadow: 0px 0px 5px inset 
            svg 
                width: 90% 
                margin-top: 4px 
    .captcha-option-svg 
        svg 
            width: 100% 
            height: auto 
    .captcha_reload_conection_button
        color: red
        i
            font-size: 36pt

    .phanterpwa-captchasvg-0101
        .fil0
            fill: #990000
        .fil1
            fill: #1F1A17
            fill-rule: nonzero

    .phanterpwa-captchasvg-0102
        .fil0
            fill: #990000
        .fil1
            fill: white
            fill-rule: nonzero

    .phanterpwa-captchasvg-0103
        .fil0
            fill: #990000
        .fil1
            fill: #009933
            fill-rule: nonzero

    .phanterpwa-captchasvg-0201
        .fil0
            fill: #FFCC00
        .fil1
            fill: #1F1A17
            fill-rule: nonzero

    .phanterpwa-captchasvg-0202
        .fil0
            fill: #FFCC00
        .fil1
            fill: white
            fill-rule: nonzero

    .phanterpwa-captchasvg-0203
        .fil0
            fill: #FFCC00
        .fil1
            fill: #009933
            fill-rule: nonzero

    .phanterpwa-captchasvg-0301
        .fil0
            fill: #6699FF
        .fil1
            fill: #1F1A17
            fill-rule: nonzero

    .phanterpwa-captchasvg-0302
        .fil0
            fill: #6699FF
        .fil1
            fill: white
            fill-rule: nonzero

    .phanterpwa-captchasvg-0303
        .fil0
            fill: #6699FF
        .fil1
            fill: #009933
            fill-rule: nonzero


.phanterpwa-captchasvg-free
    .phanterpwa-captchasvg-0101
        .fil0
            fill: #990000
        .fil1
            fill: #1F1A17
            fill-rule: nonzero

    .phanterpwa-captchasvg-0102
        .fil0
            fill: #990000
        .fil1
            fill: white
            fill-rule: nonzero

    .phanterpwa-captchasvg-0103
        .fil0
            fill: #990000
        .fil1
            fill: #009933
            fill-rule: nonzero

    .phanterpwa-captchasvg-0201
        .fil0
            fill: #FFCC00
        .fil1
            fill: #1F1A17
            fill-rule: nonzero

    .phanterpwa-captchasvg-0202
        .fil0
            fill: #FFCC00
        .fil1
            fill: white
            fill-rule: nonzero

    .phanterpwa-captchasvg-0203
        .fil0
            fill: #FFCC00
        .fil1
            fill: #009933
            fill-rule: nonzero

    .phanterpwa-captchasvg-0301
        .fil0
            fill: #6699FF
        .fil1
            fill: #1F1A17
            fill-rule: nonzero

    .phanterpwa-captchasvg-0302
        .fil0
            fill: #6699FF
        .fil1
            fill: white
            fill-rule: nonzero

    .phanterpwa-captchasvg-0303
        .fil0
            fill: #6699FF
        .fil1
            fill: #009933
            fill-rule: nonzero