.icon_awesome-wrapper
    font-size: 3rem
    padding: 6px
    text-align: center
    .icon_button
        width: 5rem
        height: 5rem
        line-height: 5rem

    .icon_awesome-label
        font-size: 1rem
        text-align: center
        max-height: 4rem
        overflow: hidden
        table
            width: 100%
            height: 4rem


    .icon_awesome-button
        &:hover
            color: orange
            cursor: pointer
    .icon_awesome-panel-container
        position: relative
    .icon_awesome_panel_wrapper 
        position: absolute
        padding: 10px
        font-size: 1rem
        display: block
        z-index: 900
.icon_awesome_panel 
    padding-right: 40px
    line-height: 40px
    position: relative
    width: 300px
    .label_and_input
        position: relative
        padding-left: 70px
        label
            position: absolute
            text-align: right
            width: 67px
            left: 0
            line-height: 40px
        input
            width: 100%
    .copy_buttom
        position: absolute
        right: 14px
        top: 0
        cursor: pointer
        &:hover
            color: orange
