$COLOR: red
$CONTAINERWIDTH: 60px
$CONTAINERHEIGHT: 60px
$BORDERRADIUS: 0
.example_container
  *
    box-sizing: border-box
  .my_square
    display: inline-block
    position: relative
    width: $CONTAINERWIDTH
    height: $CONTAINERHEIGHT
    .other_square
      width: 100%
      height: 100%
      border-radius: $BORDERRADIUS