.basic-profile-user-name, .form-profile-user-name
    font-size: 2rem
    @media #{$width1}
        font-size: 0.95rem
    @media #{$width2}
        font-size: 1rem
    @media #{$width3}
        font-size: 1.2rem
    @media #{$width4}
        font-size: 1.5rem
.basic-profile-user-img-container img, .form-image-user-img img
    width: 100%
    height: auto

.form-image-user-container
    position: relative
    width: 100%
    height: 200px
    min-height: 200px
    @media #{$width1}
        height: 130px
        min-height: 130px
    @media #{$width2}
        height: 150px
        min-height: 150px
    @media #{$width3}
        height: 170px
        min-height: 170px
    @media #{$width4}
        height: 200px
        min-height: 200px
#modal_user_login #form-login-image-user-container, #modal_user_login #form-login-button-other-user-container
    display: none

.form-profile-user-info
    text-align: center

#modal_user_login.show_current_user #phanterpwa-container-input-text-email, #modal_user_login.show_current_user #form-login-input-tittle-display_hidden-switch
    display: none

.basic-profile-user-img-container, .form-image-user-img
    height: 100px
    width: 100px
    overflow: hidden
    border-radius: 100%
    margin-left: auto
    margin-right: auto
    @media #{$width1}
        height: 90px
        width: 90px
    @media #{$width2}
        height: 90px
        width: 90px
    @media #{$width3}
        height: 100px
        width: 100px
    @media #{$width4}
        height: 110px
        width: 110px
.phanterpwa-form-buttons-container
    display: block
    width: 100%
    text-align: center

.phanterpwa-form-captcha-ajax-container
    text-align: center
    padding-top: 10px
    min-height: 120px

.basic-profile-user-role, .form-profile-user-role
    font-size: 0.7rem
    color: grey

.basic-profile-user-email, .form-profile-user-email
    font-size: 0.9rem
    color: grey

.buttons-form-container
    text-align: center

#form-profile
    padding: 20px 0
    #profile-image-user-container
        padding-top: 20px
        padding-bottom: 10px
        .phanterpwa-centralizer-wrapper
            margin: auto
            box-shadow: 1px 1px 5px gray
            border-radius: 2px
    .profile_inputs_container
        padding-top: 20px
        padding-bottom: 10px

.phanterpwa-jsonform
    padding: 10px

#modal_user_login.show_current_user #form-login-image-user-container, #modal_user_login.show_current_user #form-login-button-other-user-container
    display: block

.phanterpwa-widget-form-group
    overflow: auto
    width: 100%

.lock-container
    padding: 30px 20px 10px 20px

.phanterpwa-widget-form-separator
    background-color: gray
    padding: 10px
    text-align: center
    font-weight: bold
    color: white
    margin-top: 20px
    margin-bottom: 10px

