.logo_fil0
    fill: $grey_darken_1
    fill-rule: nonzero
.logo_fil1
    fill: $grey_darken_1
    fill-rule: nonzero
.logo_fil3
    fill: $yellow_lighten_3
    fill-rule: nonzero
.logo_fil2
    fill: url("#logo_id0")
.logo-svg
    height: 58px
    width: auto
    font-size: 1rem
    line-height: normal
    padding: 10px
.logo-svg svg 
    width: auto
    height: 100%

