// phanterpwa-components

.phanterpwa-components-preloaders-circle_button
    .preloader-wrapper
        width: 40px
        height: 40px
        overflow: hidden
        .circle_button_animation
            width: 40px
            height: 40px
            border: 4px solid white
            border-top: 4px solid orange
            border-radius: 50%
            animation: spin 1s linear infinite

@keyframes spin
    0%
        transform: rotate(0deg)
    100%
        transform: rotate(360deg)