/* SASS Source Code (MAIN FILE): C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\phanterpwa.sass */
/* start change programmatically */
/* end change programmatically */
@font-face {
  font-family: "Segoe UI Mono W01 Regular";
  src: url("/static/0.0.1/fonts/88bdc70ec292cc29451932f9875b378a.eot");
  src: url("/static/0.0.1/fonts/88bdc70ec292cc29451932f9875b378a.eot?#iefix") format("embedded-opentype"), url("/static/0.0.1/fonts/88bdc70ec292cc29451932f9875b378a.woff2") format("woff2"), url("/static/0.0.1/fonts/88bdc70ec292cc29451932f9875b378a.woff") format("woff"), url("/static/0.0.1/fonts/88bdc70ec292cc29451932f9875b378a.ttf") format("truetype"), url("/static/0.0.1/fonts/88bdc70ec292cc29451932f9875b378a.svg#Segoe UI Mono W01 Regular") format("svg");
}

@font-face {
  font-family: "Segoe UI";
  font-style: normal;
  font-weight: normal;
  src: local("Segoe UI Regular"), url("/static/0.0.1/fonts/Segoe UI.woff") format("woff");
}

@font-face {
  font-family: "Segoe UI Italic";
  font-style: normal;
  font-weight: normal;
  src: local("Segoe UI Italic"), url("/static/0.0.1/fonts/Segoe UI Italic.woff") format("woff");
}

@font-face {
  font-family: "Segoe UI Bold";
  font-style: normal;
  font-weight: normal;
  src: local("Segoe UI Bold"), url("/static/0.0.1/fonts/Segoe UI Bold.woff") format("woff");
}

@font-face {
  font-family: "Segoe UI Bold Italic";
  font-style: normal;
  font-weight: normal;
  src: local("Segoe UI Bold Italic"), url("/static/0.0.1/fonts/Segoe UI Bold Italic.woff") format("woff");
}

/* roboto-thin - latin */
@font-face {
  font-family: "Roboto Thin";
  font-style: normal;
  font-weight: 400;
  src: url("/static/0.0.1/fonts/Roboto-Thin.eot");
  src: local("Roboto Thin"), local("Roboto-Regular"), url("/static/0.0.1/fonts/Roboto-Thin.eot?#iefix") format("embedded-opentype"), url("/static/0.0.1/fonts/Roboto-Thin.woff2") format("woff2"), url("/static/0.0.1/fonts/Roboto-Thin.woff") format("woff"), url("/static/0.0.1/fonts/Roboto-Thin.ttf") format("truetype");
}

/* roboto-regular - latin */
@font-face {
  font-family: "Roboto";
  font-style: normal;
  font-weight: 400;
  src: url("/static/0.0.1/fonts/roboto-v20-latin-regular.eot");
  src: local("Roboto"), local("Roboto-Regular"), url("/static/0.0.1/fonts/roboto-v20-latin-regular.eot?#iefix") format("embedded-opentype"), url("/static/0.0.1/fonts/roboto-v20-latin-regular.woff2") format("woff2"), url("/static/0.0.1/fonts/roboto-v20-latin-regular.woff") format("woff"), url("/static/0.0.1/fonts/roboto-v20-latin-regular.ttf") format("truetype"), url("/static/0.0.1/fonts/roboto-v20-latin-regular.svg#Roboto") format("svg");
}

/* latin-ext */
@font-face {
  font-family: "Roboto Mono";
  font-style: normal;
  font-weight: 400;
  src: url("/static/0.0.1/fonts/L0x5DF4xlVMF-BfR8bXMIjhFq3-OXg.woff2") format("woff2");
  unicode-range: U+0100-024F, U+0259, U+1E00-1EFF, U+2020, U+20A0-20AB, U+20AD-20CF, U+2113, U+2C60-2C7F, U+A720-A7FF;
}

/* latin */
@font-face {
  font-family: "Roboto Mono";
  font-style: normal;
  font-weight: 400;
  src: url("/static/0.0.1/fonts/L0x5DF4xlVMF-BfR8bXMIjhLq38.woff2") format("woff2");
  unicode-range: U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+2000-206F, U+2074, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD;
}

/* latin-ext */
@font-face {
  font-family: "Cousine";
  font-style: normal;
  font-weight: 400;
  src: url("/static/0.0.1/fonts/d6lIkaiiRdih4SpP_SovzAbt.woff2") format("woff2");
  unicode-range: U+0100-024F, U+0259, U+1E00-1EFF, U+2020, U+20A0-20AB, U+20AD-20CF, U+2113, U+2C60-2C7F, U+A720-A7FF;
}

/* latin */
@font-face {
  font-family: "Cousine";
  font-style: normal;
  font-weight: 400;
  src: url("/static/0.0.1/fonts/d6lIkaiiRdih4SpP_SQvzA.woff2") format("woff2");
  unicode-range: U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+2000-206F, U+2074, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD;
}

* {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}

.p-logo {
  float: left;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .p-logo {
    width: 70%;
    text-align: center;
    float: none;
    margin: auto;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .p-logo {
    width: 70%;
    text-align: center;
    float: none;
    margin: auto;
  }
}

@media screen and (min-width: 600px) and (max-width: 901px) {
  .p-logo {
    width: 70%;
    text-align: center;
    float: none;
    margin: auto;
  }
}

.hidden_on_all_width, .hidden {
  display: none;
}

body {
  background-color: #e0e0e0;
  transition: background-color 1s;
  display: flex;
  min-height: 100vh;
  height: 100vh;
  flex-direction: column;
  padding-top: 64px;
  color: rgba(0, 0, 0, 0.8);
  font-family: -apple-system,BlinkMacSystemFont,"Segoe UI",Oxygen-Sans,Ubuntu,Cantarell,"Helvetica Neue",sans-serif;
}

body.p-lock {
  background-color: #424242;
}

h1, h2, h3, h4, h5, h6 {
  font-family: "Roboto Thin","Segoe UI Italic",-apple-system,BlinkMacSystemFont,Roboto,Oxygen-Sans,Ubuntu,Cantarell,"Helvetica Neue",sans-serif;
}

nav {
  width: 100%;
  background-color: #181818;
  color: #E8E8E8;
  height: 64px;
}

main {
  flex: 1 0 auto;
}

hr {
  width: 100%;
  border: transparent;
  color: #4e6eaa;
  border-bottom: 1px solid;
  margin: 20px 0 20px 0;
}

.card h1, .card h2, .card h3, .card h4, .card h4, .card h5, .card h6 {
  text-align: center;
  color: #4e6eaa;
}

.link {
  cursor: pointer;
}

.link:hover {
  color: orange;
}

#main-container {
  flex: 1 0 auto;
  height: 100%;
  overflow: auto;
}

#main-container > h2 {
  text-align: center;
}

.main-nav {
  position: fixed;
  z-index: 1005;
  top: 0;
  box-shadow: 0px 2px 5px black;
}

.masked_input, .p-monospace, .phanterpwa-monospace {
  font-family: Cousine;
  letter-spacing: -1px;
}

.background-empresa {
  background-image: url("/static/0.0.1/images/background.jpg");
  width: 100%;
  min-height: 450px;
  height: 100%;
  background-size: 100% auto;
  background-position: center;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .background-empresa {
    background-size: auto 100%;
    background-repeat: repeat;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .background-empresa {
    background-size: auto 110%;
    background-repeat: repeat;
  }
}

.container, .p-container, .phanterpwa-container {
  width: 70%;
  max-width: 1366px;
  margin: auto;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .container, .p-container, .phanterpwa-container {
    width: 95%;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .container, .p-container, .phanterpwa-container {
    width: 95%;
  }
}

@media screen and (min-width: 600px) and (max-width: 901px) {
  .container, .p-container, .phanterpwa-container {
    width: 90%;
  }
}

@media screen and (min-width: 900px) and (max-width: 1201px) {
  .container, .p-container, .phanterpwa-container {
    width: 80%;
  }
}

@media screen and (min-width: 1200px) and (max-width: 1501px) {
  .container, .p-container, .phanterpwa-container {
    width: 80%;
  }
}

@media screen and (min-width: 1500px) and (max-width: 1801px) {
  .container, .p-container, .phanterpwa-container {
    width: 70%;
  }
}

@media screen and (min-width: 1800px) and (max-width: 2100px) {
  .container, .p-container, .phanterpwa-container {
    width: 70%;
  }
}

@media screen and (min-width: 2100px) {
  .container, .p-container, .phanterpwa-container {
    width: 70%;
  }
}

.phanterpwa-fixed-fulldisplay {
  position: fixed;
  height: 100%;
  width: 100%;
  top: 0;
  left: 0;
  background-color: rgba(0, 0, 0, 0.7);
  z-index: 900;
}

.phanterpwa-centralizer-wrapper {
  display: table;
  height: 100%;
  width: 100%;
  margin: auto;
}

.phanterpwa-centralizer-wrapper .phanterpwa-centralizer-vertical {
  display: table-row;
}

.phanterpwa-centralizer-wrapper .phanterpwa-centralizer-vertical .phanterpwa-centralizer-horizontal {
  display: table-cell;
  vertical-align: middle;
}

.phanterpwa-centralizer-wrapper .phanterpwa-centralizer-vertical .phanterpwa-centralizer-horizontal .phanterpwa-centralizer-center {
  text-align: center;
  position: relative;
}

.phanterpwa-container .warnings-container {
  min-height: 429px;
  padding: 50px 20px;
  text-align: center;
  margin-bottom: 30px;
}

.phanterpwa-container .warnings-container .button-container {
  margin-top: 20px;
}

.phanterpwa-container .content-warnings {
  text-align: center;
  color: red;
  font-size: 2rem;
}

.phanterpwa-container .image-warnings-container {
  max-width: 256px;
  height: 227px;
  margin: auto;
}

.phanterpwa-container .image-warnings-container img {
  width: 100%;
}

.main-footer, .p-footer, .phanterpwa-footer {
  padding: 0;
  z-index: 1003;
  background-color: #212121;
  padding-top: 20px;
}

.main-footer .footer-copyright, .p-footer .footer-copyright, .phanterpwa-footer .footer-copyright {
  line-height: 3rem;
  background-color: #424141;
  padding: 0 20px;
}

.main-footer .footer-copyright a, .p-footer .footer-copyright a, .phanterpwa-footer .footer-copyright a {
  text-decoration: none;
  color: #c8c364;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\alert_top_activation.sass */
#phanterpwa-component-topslide-wrapper {
  display: none;
  position: fixed;
  width: 100%;
  left: 0;
  padding-left: 50px;
  z-index: 2;
  box-shadow: 0px 0px 6px black;
  transition: padding-left 1s ease;
  background-color: #E8E8E8;
}

#phanterpwa-component-topslide-wrapper .phanterpwa-component-topslide-close.link {
  text-align: right;
  margin: 5px 12px;
}

#phanterpwa-component-topslide-wrapper .phanterpwa-component-topslide-close.link:hover {
  color: orange;
}

#phanterpwa-component-topslide-wrapper .phanterpwa-auth_user-activation-container {
  text-align: center;
  padding: 20px;
  position: relative;
}

#phanterpwa-component-topslide-wrapper .phanterpwa-auth_user-activation-text {
  color: red;
}

#phanterpwa-component-topslide-wrapper .phanterpwa-auth_user-activation-phone-center {
  display: block;
  width: 100%;
}

#phanterpwa-component-topslide-wrapper .phanterpwa-auth_user-activation-phone-center .phanterpwa-auth_user-activation-action-activate, #phanterpwa-component-topslide-wrapper .phanterpwa-auth_user-activation-phone-center .phanterpwa-auth_user-activation-action-send {
  width: 50%;
  float: left;
  text-align: center;
}

#phanterpwa-component-topslide-wrapper .phanterpwa-auth_user-activation-phone-center .phanterpwa-auth_user-activation-action-activate .phanterpwa-materialize-button-form-container, #phanterpwa-component-topslide-wrapper .phanterpwa-auth_user-activation-phone-center .phanterpwa-auth_user-activation-action-send .phanterpwa-materialize-button-form-container {
  width: auto;
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  #phanterpwa-component-topslide-wrapper {
    padding-left: 0px;
  }
  #phanterpwa-component-topslide-wrapper .phanterpwa-auth_user-activation-phone-center {
    display: block;
    width: 100%;
  }
  #phanterpwa-component-topslide-wrapper .phanterpwa-auth_user-activation-phone-center .phanterpwa-auth_user-activation-action-activate, #phanterpwa-component-topslide-wrapper .phanterpwa-auth_user-activation-phone-center .phanterpwa-auth_user-activation-action-send {
    float: none;
    width: 100%;
  }
  #phanterpwa-component-topslide-wrapper .phanterpwa-auth_user-activation-phone-center .phanterpwa-auth_user-activation-action-activate .phanterpwa-materialize-button-form-container, #phanterpwa-component-topslide-wrapper .phanterpwa-auth_user-activation-phone-center .phanterpwa-auth_user-activation-action-send .phanterpwa-materialize-button-form-container {
    width: 100% !important;
  }
}

@media screen and (min-width: 0) and (max-width: 301px) {
  #phanterpwa-component-topslide-wrapper {
    padding-left: 0px;
  }
  #phanterpwa-component-topslide-wrapper .phanterpwa-auth_user-activation-phone-center {
    display: block;
    width: 100%;
  }
  #phanterpwa-component-topslide-wrapper .phanterpwa-auth_user-activation-phone-center .phanterpwa-auth_user-activation-action-activate, #phanterpwa-component-topslide-wrapper .phanterpwa-auth_user-activation-phone-center .phanterpwa-auth_user-activation-action-send {
    float: none;
    width: 100%;
  }
  #phanterpwa-component-topslide-wrapper .phanterpwa-auth_user-activation-phone-center .phanterpwa-auth_user-activation-action-activate .phanterpwa-materialize-button-form-container, #phanterpwa-component-topslide-wrapper .phanterpwa-auth_user-activation-phone-center .phanterpwa-auth_user-activation-action-send .phanterpwa-materialize-button-form-container {
    width: 100% !important;
  }
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\breascrumbs.sass */
.phanterpwa-breadcrumb-wrapper {
  display: block;
  line-height: 2rem;
  font-size: 1rem;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .phanterpwa-breadcrumb-wrapper {
    font-size: 1rem;
    padding-top: 2px;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .phanterpwa-breadcrumb-wrapper {
    font-size: 1rem;
    padding-top: 2px;
  }
}

.phanterpwa-breadcrumb-wrapper .phanterpwa-breadcrumb {
  float: left;
  line-height: 2rem;
}

.phanterpwa-breadcrumb-wrapper .phanterpwa-breadcrumb.link {
  color: #4e6eaa;
}

.phanterpwa-breadcrumb-wrapper .phanterpwa-breadcrumb.link:hover {
  color: orange;
}

.phanterpwa-breadcrumb-wrapper .phanterpwa-breadcrumb::before {
  content: '\f054';
  color: black;
  vertical-align: top;
  display: inline-block;
  font-family: 'Font Awesome 5 Free';
  font-weight: 900;
  font-style: normal;
  margin: 1px 5px 0 5px;
  font-size: 1.1rem;
  -webkit-font-smoothing: antialiased;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .phanterpwa-breadcrumb-wrapper .phanterpwa-breadcrumb::before {
    font-size: 1.1rem;
    display: none;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .phanterpwa-breadcrumb-wrapper .phanterpwa-breadcrumb::before {
    font-size: 1.1rem;
    display: none;
  }
}

.phanterpwa-breadcrumb-wrapper .phanterpwa-breadcrumb:first-child::before {
  display: none;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .phanterpwa-breadcrumb-wrapper .phanterpwa-breadcrumb {
    display: none;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .phanterpwa-breadcrumb-wrapper .phanterpwa-breadcrumb {
    display: none;
  }
}

.phanterpwa-breadcrumb-wrapper .phanterpwa-breadcrumb:last-child {
  display: block;
  color: #4e6eaa;
  font-size: 2rem;
  line-height: 2rem;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .phanterpwa-breadcrumb-wrapper .phanterpwa-breadcrumb:last-child {
    font-size: 1.5rem;
    line-height: 1.5rem;
    margin-top: -5px;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .phanterpwa-breadcrumb-wrapper .phanterpwa-breadcrumb:last-child {
    font-size: 1.5rem;
    line-height: 1.5rem;
    margin-top: -5px;
  }
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\buttons.sass */
.btn-icon, .icon_button {
  margin-left: auto;
  margin-right: auto;
  height: 40px;
  width: 40px;
  border-radius: 5px;
  line-height: 40px;
  text-align: center;
  border-radius: 40px;
  cursor: pointer;
  transition: color 0.3s ease, box-shadow 0.3s ease;
}

.btn-icon:hover, .icon_button:hover {
  box-shadow: 0px 0px 3px gray;
  color: orange;
  transition: color 0.3s ease, box-shadow 0.3s ease;
}

button.btn {
  border: none !important;
}

.btn, .button {
  margin: 5px;
  padding: 5px 10px 7px 10px;
  background-color: #4e6eaa;
  border-radius: 10px;
  position: relative;
  background-image: linear-gradient(black, #4e6eaa);
  color: white;
  transition: background-image 0.3s ease;
  text-align: center;
}

.btn > i, .button > i {
  margin-right: 10px;
}

.btn::after, .button::after {
  content: "";
  position: absolute;
  width: calc(100% - 8px);
  left: 4px;
  background-color: rgba(255, 255, 255, 0.2);
  height: 10px;
  border-radius: 6px 6px 1px 1px;
  top: 3px;
  background-image: linear-gradient(rgba(255, 255, 255, 0.42), rgba(255, 255, 255, 0));
}

.btn:hover, .button:hover {
  background-image: linear-gradient(black, #62d062);
}

.btn[disabled], .button[disabled] {
  background-image: linear-gradient(#cacaca, gainsboro);
}

.btn[disabled]:hover, .button[disabled]:hover {
  color: white;
  background-image: linear-gradient(#cacaca, gainsboro);
}

.phanterpwa-widget-form-form_button-container, .button-container {
  padding: 2px 2px;
  display: inline-block;
  width: auto;
  max-width: 300px;
  margin-left: auto;
  margin-right: auto;
}

.phanterpwa-widget-form-form_button-container .btn, .button-container .btn {
  width: 100%;
  width: 100%;
}

.phanterpwa-widget-form-form_button-container .btn-form, .phanterpwa-widget-form-form_button-container .button-form, .button-container .btn-form, .button-container .button-form {
  display: inline-block;
  width: 100%;
  width: 100%;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\capcha_preload.sass */
.phanterpwa-components-preloaders-android * {
  box-sizing: border-box;
}

.phanterpwa-components-preloaders-android .left {
  float: left !important;
}

.phanterpwa-components-preloaders-android .right {
  float: right !important;
}

.phanterpwa-components-preloaders-android .preloader-wrapper {
  display: inline-block;
  position: relative;
  width: 200px;
  height: 200px;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_android {
  width: 100%;
  height: 100%;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-layer {
  position: absolute;
  width: 100%;
  height: 100%;
  opacity: 0;
  border-color: blue;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-one,
.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-one-only {
  border-color: blue;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-two,
.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-two-only {
  border-color: red;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-three,
.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-three-only {
  border-color: #f4b400;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-four,
.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-four-only {
  border-color: green;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_gap-patch {
  position: absolute;
  top: 0;
  left: 45%;
  width: 10%;
  height: 100%;
  overflow: hidden;
  border-color: inherit;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_gap-patch .phanterpwa_circle {
  width: 1000%;
  left: -450%;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_circle_clipper {
  display: inline-block;
  position: relative;
  width: 50%;
  height: 100%;
  overflow: hidden;
  border-color: inherit;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_circle_clipper .phanterpwa_circle {
  width: 200%;
  height: 100%;
  border-width: 10px;
  border-style: solid;
  border-color: inherit;
  border-bottom-color: transparent !important;
  border-radius: 50%;
  -webkit-animation: none;
  animation: none;
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_circle_clipper.left .phanterpwa_circle {
  left: 0;
  border-right-color: transparent !important;
  -webkit-transform: rotate(129deg);
  transform: rotate(129deg);
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_circle_clipper.right .phanterpwa_circle {
  left: -100%;
  border-left-color: transparent !important;
  -webkit-transform: rotate(-129deg);
  transform: rotate(-129deg);
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .phanterpwa_android {
  opacity: 1;
  -webkit-animation: container-rotate 1568ms linear infinite;
  animation: container-rotate 1568ms linear infinite;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-one {
  -webkit-animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_one_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_one_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-two {
  -webkit-animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_two_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_two_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-three {
  -webkit-animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_three_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_three_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-four {
  -webkit-animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_four_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_four_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer,
.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-one-only,
.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-two-only,
.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-three-only,
.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-four-only {
  opacity: 1;
  -webkit-animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .phanterpwa_circle_clipper.left .phanterpwa_circle {
  -webkit-animation: left-spin 1333ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: left-spin 1333ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .phanterpwa_circle_clipper.right .phanterpwa_circle {
  -webkit-animation: right-spin 1333ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: right-spin 1333ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

@-webkit-keyframes container-rotate {
  to {
    -webkit-transform: rotate(360deg);
  }
}

@keyframes container-rotate {
  to {
    transform: rotate(360deg);
  }
}

@-webkit-keyframes phanterpwa_android_rotate_animate {
  12.5% {
    webkit-transform: rotate(135deg);
  }
  25% {
    webkit-transform: rotate(270deg);
  }
  37.5% {
    webkit-transform: rotate(405deg);
  }
  50% {
    webkit-transform: rotate(540deg);
  }
  62.5% {
    webkit-transform: rotate(675deg);
  }
  75% {
    webkit-transform: rotate(810deg);
  }
  87.5% {
    webkit-transform: rotate(945deg);
  }
  to {
    webkit-transform: rotate(1080deg);
  }
}

@keyframes phanterpwa_android_rotate_animate {
  12.5% {
    transform: rotate(135deg);
  }
  25% {
    transform: rotate(270deg);
  }
  37.5% {
    transform: rotate(405deg);
  }
  50% {
    transform: rotate(540deg);
  }
  62.5% {
    transform: rotate(675deg);
  }
  75% {
    transform: rotate(810deg);
  }
  87.5% {
    transform: rotate(945deg);
  }
  to {
    transform: rotate(1080deg);
  }
}

@-webkit-keyframes phanterpwa_android_one_fade {
  from {
    opacity: 1;
  }
  25% {
    opacity: 1;
  }
  26% {
    opacity: 0;
  }
  89% {
    opacity: 0;
  }
  90% {
    opacity: 1;
  }
  100% {
    opacity: 1;
  }
}

@keyframes phanterpwa_android_one_fade {
  from {
    opacity: 1;
  }
  25% {
    opacity: 1;
  }
  26% {
    opacity: 0;
  }
  89% {
    opacity: 0;
  }
  90% {
    opacity: 1;
  }
  100% {
    opacity: 1;
  }
}

@-webkit-keyframes phanterpwa_android_two_fade {
  from {
    opacity: 0;
  }
  15% {
    opacity: 0;
  }
  25% {
    opacity: 1;
  }
  50% {
    opacity: 1;
  }
  51% {
    opacity: 0;
  }
}

@keyframes phanterpwa_android_two_fade {
  from {
    opacity: 0;
  }
  15% {
    opacity: 0;
  }
  25% {
    opacity: 1;
  }
  50% {
    opacity: 1;
  }
  51% {
    opacity: 0;
  }
}

@-webkit-keyframes phanterpwa_android_three_fade {
  from {
    opacity: 0;
  }
  40% {
    opacity: 0;
  }
  50% {
    opacity: 1;
  }
  75% {
    opacity: 1;
  }
  76% {
    opacity: 0;
  }
}

@keyframes phanterpwa_android_three_fade {
  from {
    opacity: 0;
  }
  40% {
    opacity: 0;
  }
  50% {
    opacity: 1;
  }
  75% {
    opacity: 1;
  }
  76% {
    opacity: 0;
  }
}

@-webkit-keyframes phanterpwa_android_four_fade {
  from {
    opacity: 0;
  }
  65% {
    opacity: 0;
  }
  75% {
    opacity: 1;
  }
  90% {
    opacity: 1;
  }
  100% {
    opacity: 0;
  }
}

@keyframes phanterpwa_android_four_fade {
  from {
    opacity: 0;
  }
  65% {
    opacity: 0;
  }
  75% {
    opacity: 1;
  }
  90% {
    opacity: 1;
  }
  100% {
    opacity: 0;
  }
}

@-webkit-keyframes left-spin {
  from {
    -webkit-transform: rotate(130deg);
  }
  50% {
    -webkit-transform: rotate(-5deg);
  }
  to {
    -webkit-transform: rotate(130deg);
  }
}

@keyframes left-spin {
  from {
    transform: rotate(130deg);
  }
  50% {
    transform: rotate(-5deg);
  }
  to {
    transform: rotate(130deg);
  }
}

@-webkit-keyframes right-spin {
  from {
    -webkit-transform: rotate(-130deg);
  }
  50% {
    -webkit-transform: rotate(5deg);
  }
  to {
    -webkit-transform: rotate(-130deg);
  }
}

@keyframes right-spin {
  from {
    transform: rotate(-130deg);
  }
  50% {
    transform: rotate(5deg);
  }
  to {
    transform: rotate(-130deg);
  }
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\captcha.sass */
.phanterpwa-widget-captcha-container {
  text-align: center;
  height: 131px;
  width: 300px;
  overflow-x: hidden;
  overflow-y: hidden;
  margin: 0px auto 0px auto;
  border: none;
  padding: 0px;
}

.phanterpwa-widget-captcha-container .preloader-wrapper {
  width: 131px !important;
  height: 131px !important;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .phanterpwa-widget-captcha-container {
    overflow-x: auto;
    overflow-y: hidden;
    margin: 2px auto 2px auto;
    border: 1px #d5d3d0 solid;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .phanterpwa-widget-captcha-container {
    overflow-x: auto;
    overflow-y: hidden;
    margin: 2px auto 2px auto;
    border: 1px #d5d3d0 solid;
  }
}

.phanterpwa-widget-captcha-container .captcha-container, .phanterpwa-widget-captcha-container .captcha-options-container {
  width: auto;
}

.phanterpwa-widget-captcha-container .captcha-option-11 .fil0 {
  fill: #6699FF;
}

.phanterpwa-widget-captcha-container .captcha-option-11 .fil1 {
  fill: #1F1A17;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .captcha-option-12 .fil0 {
  fill: #FFCC00;
}

.phanterpwa-widget-captcha-container .captcha-option-12 .fil1 {
  fill: white;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .captcha-option-13 .fil0 {
  fill: #990000;
}

.phanterpwa-widget-captcha-container .captcha-option-13 .fil1 {
  fill: white;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .captcha-option-14 .fil1 {
  fill: #009933;
}

.phanterpwa-widget-captcha-container .captcha-option-14 .fil0 {
  fill: fuchsia;
}

.phanterpwa-widget-captcha-container .captcha-option-21 .fil0 {
  fill: fuchsia;
}

.phanterpwa-widget-captcha-container .captcha-option-21 .fil1 {
  fill: #7F7F7F;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .captcha-option-22 .fil0 {
  fill: #6699FF;
}

.phanterpwa-widget-captcha-container .captcha-option-22 .fil1 {
  fill: #1F1A17;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .captcha-option-23 .fil0 {
  fill: #FFCC00;
}

.phanterpwa-widget-captcha-container .captcha-option-23 .fil1 {
  fill: #009933;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .captcha-option-24 .fil1 {
  fill: black;
}

.phanterpwa-widget-captcha-container .captcha-option-24 .fil0 {
  fill: #990000;
}

.phanterpwa-widget-captcha-container .captcha-option-31 .fil0 {
  fill: #990000;
}

.phanterpwa-widget-captcha-container .captcha-option-31 .fil1 {
  fill: #1F1A17;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .captcha-option-32 .fil0 {
  fill: fuchsia;
}

.phanterpwa-widget-captcha-container .captcha-option-32 .fil1 {
  fill: #009933;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .captcha-option-33 .fil0 {
  fill: #6699FF;
}

.phanterpwa-widget-captcha-container .captcha-option-33 .fil1 {
  fill: #7F7F7F;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .captcha-option-34 .fil1 {
  fill: #7F7F7F;
}

.phanterpwa-widget-captcha-container .captcha-option-34 .fil0 {
  fill: #FFCC00;
}

.phanterpwa-widget-captcha-container .captcha-option-41 .fil0 {
  fill: #FFCC00;
}

.phanterpwa-widget-captcha-container .captcha-option-41 .fil1 {
  fill: #009933;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .captcha-option-42 .fil0 {
  fill: #990000;
}

.phanterpwa-widget-captcha-container .captcha-option-42 .fil1 {
  fill: #7F7F7F;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .captcha-option-43 .fil0 {
  fill: fuchsia;
}

.phanterpwa-widget-captcha-container .captcha-option-43 .fil1 {
  fill: white;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .captcha-option-44 .fil0 {
  fill: #6699FF;
}

.phanterpwa-widget-captcha-container .captcha-option-44 .fil1 {
  fill: white;
}

.phanterpwa-widget-captcha-container .captcha-ok-svg-container {
  padding-top: 10px;
  padding-bottom: 10px;
}

.phanterpwa-widget-captcha-container .captcha-ok-svg-container .fil0 {
  fill: green;
}

.phanterpwa-widget-captcha-container .captcha-ok-svg-container .fil1 {
  fill: white;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .captcha-question-container {
  font-size: 0.8rem;
  min-height: 36px;
  text-transform: uppercase;
}

.phanterpwa-widget-captcha-container #captcha-ok-svg-container-login {
  display: none;
}

.phanterpwa-widget-captcha-container .captcha-container {
  width: 300px;
  margin-left: auto;
  margin-right: auto;
  padding: 10px;
  text-align: center;
}

.phanterpwa-widget-captcha-container .captcha-options-container {
  display: block;
  width: 240px;
  margin-left: auto;
  margin-right: auto;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .phanterpwa-widget-captcha-container .captcha-options-container {
    width: 200px;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .phanterpwa-widget-captcha-container .captcha-options-container {
    width: 200px;
  }
}

.phanterpwa-widget-captcha-container .captcha-option-container {
  float: left;
  padding: 5px;
  width: 60px;
  display: inline-block;
  padding: 5px;
  float: none;
  display: inline-block;
  padding: 5px;
  float: none;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .phanterpwa-widget-captcha-container .captcha-option-container {
    width: 50px;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .phanterpwa-widget-captcha-container .captcha-option-container {
    width: 50px;
  }
}

.phanterpwa-widget-captcha-container .captcha-option {
  transition: box-shadow 0.2s ease;
}

.phanterpwa-widget-captcha-container .captcha-option svg {
  transition: width 0.2s ease, margin-top, margin-top 0.2s ease;
}

.phanterpwa-widget-captcha-container .captcha-option:hover {
  background-color: #e4e4e4;
  border-radius: 2px;
  box-shadow: 0px 0px 5px inset;
}

.phanterpwa-widget-captcha-container .captcha-option:hover svg {
  width: 90%;
  margin-top: 4px;
}

.phanterpwa-widget-captcha-container .captcha-option-svg svg {
  width: 100%;
  height: auto;
}

.phanterpwa-widget-captcha-container .captcha_reload_conection_button {
  color: red;
}

.phanterpwa-widget-captcha-container .captcha_reload_conection_button i {
  font-size: 36pt;
}

.phanterpwa-widget-captcha-container .phanterpwa-captchasvg-0101 .fil0 {
  fill: #990000;
}

.phanterpwa-widget-captcha-container .phanterpwa-captchasvg-0101 .fil1 {
  fill: #1F1A17;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .phanterpwa-captchasvg-0102 .fil0 {
  fill: #990000;
}

.phanterpwa-widget-captcha-container .phanterpwa-captchasvg-0102 .fil1 {
  fill: white;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .phanterpwa-captchasvg-0103 .fil0 {
  fill: #990000;
}

.phanterpwa-widget-captcha-container .phanterpwa-captchasvg-0103 .fil1 {
  fill: #009933;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .phanterpwa-captchasvg-0201 .fil0 {
  fill: #FFCC00;
}

.phanterpwa-widget-captcha-container .phanterpwa-captchasvg-0201 .fil1 {
  fill: #1F1A17;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .phanterpwa-captchasvg-0202 .fil0 {
  fill: #FFCC00;
}

.phanterpwa-widget-captcha-container .phanterpwa-captchasvg-0202 .fil1 {
  fill: white;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .phanterpwa-captchasvg-0203 .fil0 {
  fill: #FFCC00;
}

.phanterpwa-widget-captcha-container .phanterpwa-captchasvg-0203 .fil1 {
  fill: #009933;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .phanterpwa-captchasvg-0301 .fil0 {
  fill: #6699FF;
}

.phanterpwa-widget-captcha-container .phanterpwa-captchasvg-0301 .fil1 {
  fill: #1F1A17;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .phanterpwa-captchasvg-0302 .fil0 {
  fill: #6699FF;
}

.phanterpwa-widget-captcha-container .phanterpwa-captchasvg-0302 .fil1 {
  fill: white;
  fill-rule: nonzero;
}

.phanterpwa-widget-captcha-container .phanterpwa-captchasvg-0303 .fil0 {
  fill: #6699FF;
}

.phanterpwa-widget-captcha-container .phanterpwa-captchasvg-0303 .fil1 {
  fill: #009933;
  fill-rule: nonzero;
}

.phanterpwa-captchasvg-free .phanterpwa-captchasvg-0101 .fil0 {
  fill: #990000;
}

.phanterpwa-captchasvg-free .phanterpwa-captchasvg-0101 .fil1 {
  fill: #1F1A17;
  fill-rule: nonzero;
}

.phanterpwa-captchasvg-free .phanterpwa-captchasvg-0102 .fil0 {
  fill: #990000;
}

.phanterpwa-captchasvg-free .phanterpwa-captchasvg-0102 .fil1 {
  fill: white;
  fill-rule: nonzero;
}

.phanterpwa-captchasvg-free .phanterpwa-captchasvg-0103 .fil0 {
  fill: #990000;
}

.phanterpwa-captchasvg-free .phanterpwa-captchasvg-0103 .fil1 {
  fill: #009933;
  fill-rule: nonzero;
}

.phanterpwa-captchasvg-free .phanterpwa-captchasvg-0201 .fil0 {
  fill: #FFCC00;
}

.phanterpwa-captchasvg-free .phanterpwa-captchasvg-0201 .fil1 {
  fill: #1F1A17;
  fill-rule: nonzero;
}

.phanterpwa-captchasvg-free .phanterpwa-captchasvg-0202 .fil0 {
  fill: #FFCC00;
}

.phanterpwa-captchasvg-free .phanterpwa-captchasvg-0202 .fil1 {
  fill: white;
  fill-rule: nonzero;
}

.phanterpwa-captchasvg-free .phanterpwa-captchasvg-0203 .fil0 {
  fill: #FFCC00;
}

.phanterpwa-captchasvg-free .phanterpwa-captchasvg-0203 .fil1 {
  fill: #009933;
  fill-rule: nonzero;
}

.phanterpwa-captchasvg-free .phanterpwa-captchasvg-0301 .fil0 {
  fill: #6699FF;
}

.phanterpwa-captchasvg-free .phanterpwa-captchasvg-0301 .fil1 {
  fill: #1F1A17;
  fill-rule: nonzero;
}

.phanterpwa-captchasvg-free .phanterpwa-captchasvg-0302 .fil0 {
  fill: #6699FF;
}

.phanterpwa-captchasvg-free .phanterpwa-captchasvg-0302 .fil1 {
  fill: white;
  fill-rule: nonzero;
}

.phanterpwa-captchasvg-free .phanterpwa-captchasvg-0303 .fil0 {
  fill: #6699FF;
}

.phanterpwa-captchasvg-free .phanterpwa-captchasvg-0303 .fil1 {
  fill: #009933;
  fill-rule: nonzero;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\captcha_preload_image_3x4.sass */
.phanterpwa-components-preloaders-explosion * {
  box-sizing: border-box;
}

.phanterpwa-components-preloaders-explosion .left {
  float: left !important;
}

.phanterpwa-components-preloaders-explosion .right {
  float: right !important;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper {
  display: inline-block;
  position: relative;
  width: 60px;
  height: 60px;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper .phanterpwa_explosion {
  width: 16%;
  height: 16%;
  background-color: red;
  position: absolute;
  left: 40%;
  top: 40%;
  border-radius: 0%;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper .phanterpwa_explosion:nth-child(2n+0) {
  margin-right: 0px;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper .phanterpwa_explosion_container {
  width: 100%;
  height: 100%;
  position: relative;
  transform: rotate(45deg);
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper.enabled .phanterpwa_explosion_one {
  background-color: red;
  -webkit-animation: phanterpwa_explosion_one_animate 2s infinite;
  animation: phanterpwa_explosion_one_animate 2s infinite;
  -webkit-animation-delay: 0.2s;
  animation-delay: 0.2s;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper.enabled .phanterpwa_explosion_two {
  background-color: red;
  -webkit-animation: phanterpwa_explosion_two_animate 2s infinite;
  animation: phanterpwa_explosion_two_animate 2s infinite;
  -webkit-animation-delay: 0.3s;
  animation-delay: 0.3s;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper.enabled .phanterpwa_explosion_three {
  background-color: red;
  -webkit-animation: phanterpwa_explosion_three_animate 2s infinite;
  animation: phanterpwa_explosion_three_animate 2s infinite;
  -webkit-animation-delay: 0.4s;
  animation-delay: 0.4s;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper.enabled .phanterpwa_explosion_four {
  background-color: red;
  -webkit-animation: phanterpwa_explosion_four_animate 2s infinite;
  animation: phanterpwa_explosion_four_animate 2s infinite;
  -webkit-animation-delay: 0.5s;
  animation-delay: 0.5s;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper.enabled .phanterpwa_explosion_five {
  background-color: red;
  -webkit-animation: phanterpwa_explosion_five_animate 2s infinite;
  animation: phanterpwa_explosion_five_animate 2s infinite;
  -webkit-animation-delay: 0.6s;
  animation-delay: 0.6s;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper.enabled .phanterpwa_explosion_six {
  background-color: red;
  -webkit-animation: phanterpwa_explosion_six_animate 2s infinite;
  animation: phanterpwa_explosion_six_animate 2s infinite;
  -webkit-animation-delay: 0.7s;
  animation-delay: 0.7s;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper.enabled .phanterpwa_explosion_seven {
  background-color: red;
  -webkit-animation: phanterpwa_explosion_seven_animate 2s infinite;
  animation: phanterpwa_explosion_seven_animate 2s infinite;
  -webkit-animation-delay: 0.8s;
  animation-delay: 0.8s;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper.enabled .phanterpwa_explosion_eight {
  background-color: red;
  -webkit-animation: phanterpwa_explosion_eight_animate 2s infinite;
  animation: phanterpwa_explosion_eight_animate 2s infinite;
  animation--webkit-animation-delay: 0.9s;
  animation-delay: 0.9s;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper.enabled .phanterpwa_explosion_big {
  background-color: red;
  position: absolute;
  width: 16%;
  height: 16%;
  left: 40%;
  top: 40%;
  -webkit-animation: phanterpwa_explosion_big_animate 2s infinite;
  animation: phanterpwa_explosion_big_animate 2s infinite;
  -webkit-animation-delay: 0.5s;
  animation-delay: 1.5s;
}

@-webkit-keyframes phanterpwa_explosion_big_animate {
  40% {
    width: 50%;
    height: 50%;
    top: 27%;
    left: 27%;
  }
}

@-webkit-keyframes phanterpwa_explosion_one_animate {
  60% {
    top: 42%;
    left: 0%;
  }
}

@-webkit-keyframes phanterpwa_explosion_two_animate {
  60% {
    top: 0;
    left: 0;
  }
}

@-webkit-keyframes phanterpwa_explosion_three_animate {
  60% {
    top: 0%;
    left: 42%;
  }
}

@-webkit-keyframes phanterpwa_explosion_four_animate {
  60% {
    top: 0%;
    left: 84%;
  }
}

@-webkit-keyframes phanterpwa_explosion_five_animate {
  60% {
    top: 42%;
    left: 84%;
  }
}

@-webkit-keyframes phanterpwa_explosion_six_animate {
  60% {
    top: 84%;
    left: 84%;
  }
}

@-webkit-keyframes phanterpwa_explosion_seven_animate {
  60% {
    top: 84%;
    left: 42%;
  }
}

@-webkit-keyframes phanterpwa_explosion_eight_animate {
  60% {
    top: 84%;
    left: 0%;
  }
}

@keyframes phanterpwa_explosion_big_animate {
  40% {
    width: 50%;
    height: 50%;
    top: 27%;
    left: 27%;
  }
}

@keyframes phanterpwa_explosion_one_animate {
  60% {
    top: 42%;
    left: 0%;
  }
}

@keyframes phanterpwa_explosion_two_animate {
  60% {
    top: 0;
    left: 0;
  }
}

@keyframes phanterpwa_explosion_three_animate {
  60% {
    top: 0%;
    left: 42%;
  }
}

@keyframes phanterpwa_explosion_four_animate {
  60% {
    top: 0%;
    left: 84%;
  }
}

@keyframes phanterpwa_explosion_five_animate {
  60% {
    top: 42%;
    left: 84%;
  }
}

@keyframes phanterpwa_explosion_six_animate {
  60% {
    top: 84%;
    left: 84%;
  }
}

@keyframes phanterpwa_explosion_seven_animate {
  60% {
    top: 84%;
    left: 42%;
  }
}

@keyframes phanterpwa_explosion_eight_animate {
  60% {
    top: 84%;
    left: 0%;
  }
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\card.sass */
.card {
  background-color: white;
  box-shadow: 0px 1px 2px grey;
  overflow: auto;
}

.p-input-materialize input {
  border: none;
  width: 100%;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\cmp_auth_user.sass */
body .cmp_user_login-container {
  opacity: 1;
  max-width: 300px;
  min-width: 100px;
  transition: opacity ease 1s, max-width ease-in 1s,  min-width ease-in 1s;
}

body.phanterpwa-lock .cmp_user_login-container {
  cursor: none;
  opacity: 0;
  max-width: 0;
  min-width: 0;
  transition: opacity ease 1s, max-width ease-in 1s,  min-width ease-in 1s;
}

#layout-auth_user-container {
  text-align: center;
}

#layout-auth_user-container .container_preloader_button_auth_user {
  min-width: 200px;
  text-align: center;
  height: 64px;
  padding-top: 20px;
}

#layout-auth_user-container .container_preloader_button_auth_user .preloader-wrapper {
  width: 50px;
  height: 17px;
}

.cmd-bar_user-expands {
  position: absolute;
}

.cmp-bar_user-name-role-container {
  float: left;
  margin-left: 10px;
}

.cmp-bar_user-name {
  font-size: 0.9rem;
}

.cmp-bar_user-name-role {
  line-height: 60px;
  padding-top: 16px;
}

.cmp-bar_user-name-role div {
  line-height: 1.2rem;
}

.cmp-bar_user-name-role .cmp-bar_user-role {
  font-size: 0.8rem;
  color: grey;
}

@keyframes fadeInFromNoneAndMargin {
  0% {
    display: block;
    opacity: 0;
    margin-top: 20px;
  }
  1% {
    display: block;
    opacity: 0;
    margin-top: 20px;
  }
  100% {
    display: block;
    opacity: 1;
    margin-top: 0px;
  }
}

#phanterpwa-component-auth_user-container {
  min-width: 175px;
}

#phanterpwa-component-auth_user-container #toggle-cmp-bar_user {
  min-height: 64px;
  max-height: 64px;
}

#phanterpwa-component-auth_user-container .cmp-bar_user-name-role #user_role_login {
  text-transform: uppercase;
  font-size: 0.5rem;
}

#phanterpwa-component-auth_user-container .phanterpwa-component-auth_user-button-toggle {
  min-height: 64px;
  max-height: 64px;
  min-width: 200px;
}

#phanterpwa-component-auth_user-container .phanterpwa-component-auth_user-button-toggle .phanterpwa-component-auth_user-nologin-wrapper {
  text-align: center;
  color: #c8c364;
  display: block;
  min-height: 64px;
  max-height: 64px;
  padding-left: 20px;
  padding-right: 20px;
  transition: background-color 0.3s ease;
}

#phanterpwa-component-auth_user-container .phanterpwa-component-auth_user-button-toggle .phanterpwa-component-auth_user-nologin-wrapper .phanterpwa-component-auth_user-nologin-led_and_start-wrapper {
  display: inline-block;
}

#phanterpwa-component-auth_user-container .phanterpwa-component-auth_user-button-toggle .phanterpwa-component-auth_user-nologin-wrapper .phanterpwa-component-auth_user-nologin-led_and_start-wrapper .phanterpwa-component-auth_user-nologin-led_and_start {
  display: table;
  padding-top: 20px;
}

#phanterpwa-component-auth_user-container .phanterpwa-component-auth_user-button-toggle .phanterpwa-component-auth_user-nologin-wrapper .phanterpwa-component-auth_user-nologin-led_and_start-wrapper .phanterpwa-component-auth_user-nologin-led_and_start .phanterpwa-component-auth_user-nologin-led {
  float: left;
}

#phanterpwa-component-auth_user-container .phanterpwa-component-auth_user-button-toggle .phanterpwa-component-auth_user-nologin-wrapper .phanterpwa-component-auth_user-nologin-led_and_start-wrapper .phanterpwa-component-auth_user-nologin-led_and_start .phanterpwa-component-auth_user-nologin-led .led {
  width: 0.5rem;
  height: 0.5rem;
  background-color: #A0A0A0;
  border-radius: 100%;
  margin: 7px 12px 7px 0px;
  transition: box-shadow 0.2s ease-in;
}

#phanterpwa-component-auth_user-container .phanterpwa-component-auth_user-button-toggle .phanterpwa-component-auth_user-nologin-wrapper .phanterpwa-component-auth_user-nologin-led_and_start-wrapper .phanterpwa-component-auth_user-nologin-led_and_start .phanterpwa-component-auth_user-nologin-start {
  float: left;
}

#phanterpwa-component-auth_user-container .phanterpwa-component-auth_user-button-toggle #user_first_and_last_name_login {
  color: white;
}

#phanterpwa-component-auth_user-container .phanterpwa-component-auth_user-button-toggle.enabled {
  background-color: black;
}

#phanterpwa-component-auth_user-container .phanterpwa-component-auth_user-button-toggle.link:hover {
  background-color: black;
  color: white;
}

#phanterpwa-component-auth_user-container .phanterpwa-component-auth_user-button-toggle.link:hover #user_role_login {
  color: #c8c364;
}

#phanterpwa-component-auth_user-container .phanterpwa-component-auth_user-button-toggle.link:hover #user_first_and_last_name_login {
  color: orange;
}

#phanterpwa-component-auth_user-container .phanterpwa-component-auth_user-button-toggle-options {
  display: none;
  box-shadow: 0px 1px 3px #888888;
  color: #888888;
  background-color: white;
  line-height: 3rem;
  margin-top: 20px;
  opacity: 0;
}

#phanterpwa-component-auth_user-container.enabled .phanterpwa-component-auth_user-button-toggle {
  background-color: black;
  color: #E0E0E0;
}

#phanterpwa-component-auth_user-container.enabled > .phanterpwa-component-auth_user-button-toggle-options {
  display: block;
  margin-top: 0px;
  opacity: 1;
  animation-name: fadeInFromNoneAndMargin;
  animation-iteration-count: 1;
  animation-timing-function: ease-in;
  animation-duration: 0.5s;
}

#phanterpwa-component-auth_user-container.enabled .led {
  background-color: #62d062 !important;
  box-shadow: 0px 0px 10px #62d062;
}

#phanterpwa-component-auth_user-container .cmd-bar_user-expand-icon {
  position: absolute;
  top: 2px;
  left: 0;
  line-height: 1rem !important;
  height: 1rem !important;
}

#phanterpwa-component-auth_user-container .led {
  width: 0.5rem;
  height: 0.5rem;
  background-color: #A0A0A0;
  border-radius: 100%;
  margin-top: 10px;
  margin-left: 10px;
  transition: box-shadow 0.2s ease-in;
}

#phanterpwa-component-auth_user-container .cmd-bar-start {
  float: left;
  margin-left: 0px;
  padding: 20px 0 0 0;
  min-width: 140px;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  #phanterpwa-component-auth_user-container {
    display: none !important;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  #phanterpwa-component-auth_user-container {
    display: none !important;
  }
}

@media screen and (min-width: 600px) and (max-width: 901px) {
  #phanterpwa-component-auth_user-container {
    display: none !important;
  }
}

.cmd-bar_user-expand-container {
  display: inline-block;
  position: relative;
  width: 19px;
  padding: 20px 0 0 3px;
  float: left;
}

.component-auth_user-option {
  padding-left: 20px;
  padding-right: 20px;
  display: block;
}

.component-auth_user-option.link:hover {
  background-color: #A0A0A0;
  color: white;
  transition: background-color 0.2s ease-in;
}

.cmp-bar_user-info-container {
  display: inline-block;
  max-height: 64px;
}

.cmp-bar_user-info {
  display: inline-block;
  padding-left: 6px;
  padding-right: 1px;
  line-height: normal;
}

.cmp-bar_user-img-container {
  float: left;
  padding-top: 15px;
  height: 64px;
}

.cmp-bar_user-img {
  width: 34px;
  height: 34px;
  padding: 0px;
  line-height: unset;
  background-color: #A0A0A0;
  border-radius: 34px;
  overflow: hidden;
}

.cmp-bar_user-img img {
  width: 100%;
  height: auto;
  display: block;
  z-index: auto;
}

.option-label-menu {
  text-align: left;
  position: relative;
  padding-left: 30px;
}

.option-label-menu i {
  font-size: 1rem;
  padding: 0;
  line-height: 3rem;
  height: auto;
  position: absolute;
  left: 0;
}

.phanterpwa-component-left_bar-image-user-container {
  border-radius: 100%;
  width: 40px;
  height: 40px;
  padding: 4px;
  float: left;
}

.phanterpwa-component-left_bar-image-user {
  width: 100%;
  height: 100%;
  background-color: #A0A0A0;
  background-color-border-radius: 100%;
  overflow: hidden;
}

.phanterpwa-component-left_bar-image-user img {
  width: 100%;
  height: auto;
}

body.p-lock #phanterpwa-component-left_bar {
  width: 0;
  transition: width 1s ease;
}

body.p-lock #phanterpwa-component-left_bar-main_button.enabled {
  text-shadow: 0px 0px 4px #62d062;
  color: #62d062;
}

#phanterpwa-component-left_bar-main_button {
  min-height: 64px;
  max-height: 64px;
  float: left;
  padding: 0 10px 0 10px;
  width: 50px;
  text-align: center;
}

#phanterpwa-component-left_bar-main_button i {
  font-size: 20pt;
  padding-top: 18px;
}

#phanterpwa-component-left_bar-main_button.enabled {
  text-shadow: 0px 0px 4px #62d062;
  color: #62d062;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  #phanterpwa-component-left_bar-main_button.enabled_submenu {
    text-shadow: 0px 0px 4px #62d062;
    color: #62d062;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  #phanterpwa-component-left_bar-main_button.enabled_submenu {
    text-shadow: 0px 0px 4px #62d062;
    color: #62d062;
  }
}

@media screen and (min-width: 600px) and (max-width: 901px) {
  #phanterpwa-component-left_bar-main_button.enabled_submenu {
    text-shadow: 0px 0px 4px #62d062;
    color: #62d062;
  }
}

@media screen and (min-width: 900px) and (max-width: 1201px) {
  #phanterpwa-component-left_bar-main_button.enabled_submenu {
    text-shadow: 0px 0px 4px #62d062;
    color: #62d062;
  }
}

.phanterpwa-component-left_bar-menu_button-wrapper:hover .phanterpwa-component-left_bar-icon-container {
  color: orange;
}

.phanterpwa-component-left_bar-menu_button-wrapper:hover .phanterpwa-component-left_bar-label {
  color: orange;
}

.main-and-left_bar {
  flex: 1 0 auto;
  padding-left: 50px;
  position: relative;
  transition: padding-left 1s ease;
  min-height: 450px;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .main-and-left_bar {
    padding-left: 0px;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .main-and-left_bar {
    padding-left: 0px;
  }
}

.phanterpwa-component-left_bar-wrapper {
  width: 50px;
  flex: 1 0 auto;
  height: 100%;
  background-color: #181818;
  position: fixed;
  left: 0;
  max-width: 200px;
  z-index: 20;
  top: 64px;
  text-overflow: ellipsis;
  white-space: nowrap;
  transition: box-shadow 1s, -webkit-box-shadow 1s, width 1s ease, left 1s ease;
  line-height: 1.2rem;
  overflow-y: auto;
  overflow-x: hidden;
}

.phanterpwa-component-left_bar-wrapper .phanterpwa-component-left_bar-menu, .phanterpwa-component-left_bar-wrapper .phanterpwa-component-left_bar-button {
  display: table;
}

.phanterpwa-component-left_bar-wrapper .phanterpwa-component-left_bar-menu i, .phanterpwa-component-left_bar-wrapper .phanterpwa-component-left_bar-button i {
  font-size: 1.5rem;
  margin-top: 0.25rem;
}

.phanterpwa-component-left_bar-wrapper .phanterpwa-component-left_bar-menu:hover, .phanterpwa-component-left_bar-wrapper .phanterpwa-component-left_bar-button:hover {
  color: orange;
}

.phanterpwa-component-left_bar-wrapper .phanterpwa-component-left_bar-menu_button-wrapper-auth_user {
  display: none;
}

.phanterpwa-component-left_bar-wrapper .phanterpwa-component-left_bar-label {
  float: left;
  margin-top: 11px;
  margin-left: 10px;
  width: 127px;
  max-width: 127px;
  text-align: left;
  opacity: 0;
  transition: opacity 1s ease;
  overflow: hidden;
  color: #888888;
}

.phanterpwa-component-left_bar-wrapper .phanterpwa-component-left_bar-image-user {
  border-radius: 100%;
  margin-left: 5px;
  transition: margin-left 1s ease;
}

.phanterpwa-component-left_bar-wrapper .phanterpwa-component-left_bar-icon-container {
  border-radius: 100%;
  height: 40px;
  width: 40px;
  padding: 4px;
  margin-left: 5px;
  color: #888888;
  float: left;
  text-align: center;
  transition: margin-left 1s ease, color 0.3s ease;
}

.phanterpwa-component-left_bar-wrapper .phanterpwa-component-left_bar-submenu-button {
  margin-bottom: 5px;
  margin-left: 0px;
  transition: margin-left 1s ease, margin-bottom 1s ease;
}

.phanterpwa-component-left_bar-wrapper.enabled {
  overflow-x: auto;
  width: 90%;
  box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.14), 0 3px 1px -2px rgba(0, 0, 0, 0.12), 0 1px 5px 0 rgba(0, 0, 0, 0.2);
  transition: box-shadow 1s, -webkit-box-shadow 1s, width 1s ease;
}

.phanterpwa-component-left_bar-wrapper.enabled .phanterpwa-component-left_bar-label {
  opacity: 1;
  width: 127px;
  max-width: 127px;
}

.phanterpwa-component-left_bar-wrapper.enabled .phanterpwa-component-left_bar-image-user {
  margin-left: 0px;
}

.phanterpwa-component-left_bar-wrapper.enabled .phanterpwa-component-left_bar-icon-container {
  margin-left: 0px;
}

.phanterpwa-component-left_bar-wrapper.enabled_submenu .phanterpwa-component-left_bar-submenu-button {
  margin-bottom: 11px;
  margin-left: 15px;
}

.phanterpwa-component-left_bar-wrapper .phanterpwa-component-left_bar-submenu-button:hover {
  color: orange;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .phanterpwa-component-left_bar-wrapper {
    left: -50px;
  }
  .phanterpwa-component-left_bar-wrapper .phanterpwa-component-left_bar-menu_button-wrapper-auth_user {
    display: block;
  }
  .phanterpwa-component-left_bar-wrapper.enabled {
    left: 0px;
  }
  .phanterpwa-component-left_bar-wrapper.enabled_submenu {
    left: 0px;
    overflow-x: auto;
    width: 90%;
    box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.14), 0 3px 1px -2px rgba(0, 0, 0, 0.12), 0 1px 5px 0 rgba(0, 0, 0, 0.2);
    transition: box-shadow 1s, -webkit-box-shadow 1s, width 1s ease;
  }
  .phanterpwa-component-left_bar-wrapper.enabled_submenu .phanterpwa-component-left_bar-label {
    opacity: 1;
    width: 127px;
    max-width: 127px;
  }
  .phanterpwa-component-left_bar-wrapper.enabled_submenu .phanterpwa-component-left_bar-icon-container {
    margin-left: 0px;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .phanterpwa-component-left_bar-wrapper {
    left: -50px;
  }
  .phanterpwa-component-left_bar-wrapper .phanterpwa-component-left_bar-menu_button-wrapper-auth_user {
    display: block;
  }
  .phanterpwa-component-left_bar-wrapper.enabled {
    left: 0px;
  }
  .phanterpwa-component-left_bar-wrapper.enabled_submenu {
    left: 0px;
    overflow-x: auto;
    width: 90%;
    box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.14), 0 3px 1px -2px rgba(0, 0, 0, 0.12), 0 1px 5px 0 rgba(0, 0, 0, 0.2);
    transition: box-shadow 1s, -webkit-box-shadow 1s, width 1s ease;
  }
  .phanterpwa-component-left_bar-wrapper.enabled_submenu .phanterpwa-component-left_bar-label {
    opacity: 1;
    width: 127px;
    max-width: 127px;
  }
  .phanterpwa-component-left_bar-wrapper.enabled_submenu .phanterpwa-component-left_bar-icon-container {
    margin-left: 0px;
  }
}

@media screen and (min-width: 600px) and (max-width: 901px) {
  .phanterpwa-component-left_bar-wrapper .phanterpwa-component-left_bar-menu_button-wrapper-auth_user {
    display: block;
  }
  .phanterpwa-component-left_bar-wrapper.enabled_submenu {
    overflow-x: auto;
    width: 90%;
    box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.14), 0 3px 1px -2px rgba(0, 0, 0, 0.12), 0 1px 5px 0 rgba(0, 0, 0, 0.2);
    transition: box-shadow 1s, -webkit-box-shadow 1s, width 1s ease;
  }
  .phanterpwa-component-left_bar-wrapper.enabled_submenu .phanterpwa-component-left_bar-label {
    opacity: 1;
    width: 127px;
    max-width: 127px;
  }
  .phanterpwa-component-left_bar-wrapper.enabled_submenu .phanterpwa-component-left_bar-icon-container {
    margin-left: 0px;
  }
}

.phanterpwa-component-left_bar-menu_button-wrapper {
  width: 200px;
}

.phanterpwa-component-left_bar-menu_button-wrapper .phanterpwa-component-left_bar-submenu-container {
  overflow: hidden;
  transition: max-height 1s ease;
  height: auto;
  max-height: 0;
}

.phanterpwa-component-left_bar-menu_button-wrapper.enabled .phanterpwa-component-left_bar-submenu-container {
  max-height: 80px;
}

.phanterpwa-component-left_bar-submenu-button .phanterpwa-component-left_bar-submenu-icon-container {
  padding: 0px 0px 0px 16px;
  color: #888888;
  float: left;
  margin: 0;
  margin-right: 5px;
}

.phanterpwa-component-left_bar-submenu-button .phanterpwa-component-left_bar-submenu-icon-container.enabled {
  background-color: white;
  color: black;
}

.phanterpwa-component-left_bar-submenu-button .phanterpwa-component-left_bar-submenu-label {
  color: #888888;
}

.phanterpwa-component-left_bar-submenu-button:hover .phanterpwa-component-left_bar-submenu-icon-container, .phanterpwa-component-left_bar-submenu-button:hover .phanterpwa-component-left_bar-submenu-label {
  color: orange;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\datetimepicker.sass */
.phanterpwa-fixed-fulldisplay {
  transition: box-shadow 0.5s, -webkit-box-shadow 0.5s, width 0.5s ease, opacity 0.5s ease;
  opacity: 0;
  overflow: auto;
}

.phanterpwa-fixed-fulldisplay.enabled {
  opacity: 1;
  box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.14), 0 3px 1px -2px rgba(0, 0, 0, 0.12), 0 1px 5px 0 rgba(0, 0, 0, 0.2);
}

.phanterpwa-fixed-fulldisplay.enabled .phanterpwa_datetimepicker_container {
  margin-top: 0px;
}

.phanterpwa_datetimepicker_container {
  margin: auto;
  width: 600px;
  background-color: #4e6eaa;
  position: relative;
  margin-top: -30px;
  transition: margin-top 0.5s ease;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .phanterpwa_datetimepicker_container {
    width: 300px;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .phanterpwa_datetimepicker_container {
    width: 300px;
  }
}

.phanterpwa_datetimepicker_container .phanterpwa_datetimepicker_summary_container {
  position: absolute;
  right: 0;
  top: 0;
  width: 280px;
  padding: 10px;
  color: white;
  font-size: 2.5rem;
  text-align: center;
}

.phanterpwa_datetimepicker_container .phanterpwa_datetimepicker_summary_container .phanterpwa-models-close {
  font-size: 1rem;
  width: 30px;
  height: 30px;
  padding-right: 0px;
  padding-top: 3px;
  position: absolute;
  top: 6px;
  right: 6px;
  cursor: pointer;
}

.phanterpwa_datetimepicker_container .phanterpwa_datetimepicker_summary_container .phanterpwa-models-close:hover {
  color: orange;
}

.phanterpwa_datetimepicker_container .phanterpwa_datetimepicker_summary_container .phanterpwa_datetimepicker_summary_year {
  font-size: 6rem;
  line-height: 6rem;
  text-align: center;
}

.phanterpwa_datetimepicker_container .phanterpwa_datetimepicker_summary_container .phanterpwa_datetimepicker_summary_hour, .phanterpwa_datetimepicker_container .phanterpwa_datetimepicker_summary_container .phanterpwa_datetimepicker_summary_week {
  font-size: 1.5rem;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .phanterpwa_datetimepicker_container .phanterpwa_datetimepicker_summary_container {
    width: 300px;
    position: relative;
  }
  .phanterpwa_datetimepicker_container .phanterpwa_datetimepicker_summary_container .phanterpwa_datetimepicker_summary_year, .phanterpwa_datetimepicker_container .phanterpwa_datetimepicker_summary_container .phanterpwa_datetimepicker_summary_day_and_month {
    font-size: 2rem;
    line-height: 2rem;
  }
  .phanterpwa_datetimepicker_container .phanterpwa_datetimepicker_summary_container .phanterpwa_datetimepicker_summary_hour {
    font-size: 0.9rem;
  }
  .phanterpwa_datetimepicker_container .phanterpwa_datetimepicker_summary_container .phanterpwa_datetimepicker_summary_week {
    display: none;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .phanterpwa_datetimepicker_container .phanterpwa_datetimepicker_summary_container {
    width: 300px;
    position: relative;
  }
  .phanterpwa_datetimepicker_container .phanterpwa_datetimepicker_summary_container .phanterpwa_datetimepicker_summary_year, .phanterpwa_datetimepicker_container .phanterpwa_datetimepicker_summary_container .phanterpwa_datetimepicker_summary_day_and_month {
    font-size: 2rem;
    line-height: 2rem;
  }
  .phanterpwa_datetimepicker_container .phanterpwa_datetimepicker_summary_container .phanterpwa_datetimepicker_summary_hour {
    font-size: 0.9rem;
  }
  .phanterpwa_datetimepicker_container .phanterpwa_datetimepicker_summary_container .phanterpwa_datetimepicker_summary_week {
    display: none;
  }
}

.phanterpwa_datetimepicker_wrapper {
  width: 320px;
  height: 400px;
  padding: 10px;
  position: relative;
  background-color: white;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .phanterpwa_datetimepicker_wrapper {
    width: 300px;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .phanterpwa_datetimepicker_wrapper {
    width: 300px;
  }
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_button {
  cursor: pointer;
  height: 30px;
  margin: auto;
  padding-top: 4px;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_button:hover {
  color: orange;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_previusnext {
  float: left;
  width: 15%;
  text-align: center;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_secondlist_second, .phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_minutelist_minute {
  float: left;
  width: 16.66666%;
  text-align: center;
  height: 34px;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_secondlist_second.selected, .phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_minutelist_minute.selected {
  color: #d1d1d1;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_hourlist_hour {
  width: 33.33333%;
  float: left;
  text-align: center;
  height: 42px;
  padding-top: 12px;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_hourlist_hour.selected {
  color: #d1d1d1;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_time_container {
  display: block;
  width: 100%;
  position: absolute;
  bottom: 10px;
  left: 0;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_time_container hr {
  border: 1px solid #e8e8e8;
  width: 95%;
  margin: auto;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_time_container .phanterpwa_datetimepicker_time_col {
  width: 33.33333%;
  padding-left: 5px;
  padding-right: 5px;
  float: left;
  text-align: center;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_time_container .phanterpwa_datetimepicker_time_col.label {
  font-size: 0.7rem;
  margin-top: -8px;
  color: gray;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_time_container .phanterpwa_datetimepicker_time_col .phanterpwa_datetimepicker_previusnext {
  width: 30%;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_time_container .phanterpwa_datetimepicker_time_col .phanterpwa_datetimepicker_hour, .phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_time_container .phanterpwa_datetimepicker_time_col .phanterpwa_datetimepicker_minute, .phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_time_container .phanterpwa_datetimepicker_time_col .phanterpwa_datetimepicker_second {
  width: 40%;
  float: left;
  text-align: center;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_year, .phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_month {
  float: left;
  width: 70%;
  text-align: center;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_row {
  width: 100%;
  display: block;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_weekday, .phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_day_empty, .phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_day {
  float: left;
  width: 14.285714285714285714285714285714%;
  text-align: center;
  min-height: 30px;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_weekday .phanterpwa_datetimepicker_unit, .phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_day .phanterpwa_datetimepicker_unit {
  width: 42px;
  height: 42px;
  margin: auto;
  padding-top: 9px;
  transition: width 0.5s ease;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_weekday .phanterpwa_datetimepicker_unit, .phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_day .phanterpwa_datetimepicker_unit {
    width: 42px;
    height: 42px;
    transition: width 0.5s ease;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_weekday .phanterpwa_datetimepicker_unit, .phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_day .phanterpwa_datetimepicker_unit {
    width: 40px;
    height: 40px;
    transition: width 0.5s ease;
  }
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_day .phanterpwa_datetimepicker_unit {
  border-radius: 100%;
  cursor: pointer;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_day.phanterpwa_datetimepicker_its_now .phanterpwa_datetimepicker_unit {
  border: solid 2px #d1d1d1;
  padding-top: 7px;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_day.phanterpwa_datetimepicker_selected .phanterpwa_datetimepicker_unit {
  background-color: #4e6eaa;
  color: white;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_day:hover .phanterpwa_datetimepicker_unit {
  border: solid 2px orange;
  padding-top: 7px;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_monthlist_container {
  text-align: center;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_monthlist_container .phanterpwa_datetimepicker_monthlist_month {
  width: 33.333333%;
  float: left;
  height: 70px;
  padding-top: 25px;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_monthlist_container .phanterpwa_datetimepicker_monthlist_month.selected {
  color: #d1d1d1;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_yearlist_container {
  text-align: center;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_yearlist_container .phanterpwa_datetimepicker_yearlist_year {
  width: 20%;
  float: left;
  height: 44px;
  padding-top: 13px;
}

.phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_yearlist_container .phanterpwa_datetimepicker_yearlist_year.selected {
  color: #d1d1d1;
}

.phanterpwa_datetimepicker_container .phanterpwa_datetimepicker_wrapper {
  height: 400px;
}

.phanterpwa_datetimepicker_container.phanterpwa_datetimepicker_is_not_datetime .phanterpwa_datetimepicker_summary_hour {
  display: none;
}

.phanterpwa_datetimepicker_container.phanterpwa_datetimepicker_is_not_datetime .phanterpwa_datetimepicker_wrapper {
  height: 380px;
}

.phanterpwa_datetimepicker_container.phanterpwa_datetimepicker_is_not_datetime .phanterpwa_datetimepicker_wrapper .phanterpwa_datetimepicker_time_container {
  display: none;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\developer.sass */
#p-developer_toolbar {
  width: 100%;
}

#p-developer_toolbar .phanterpwa-developer_toolbar-objects {
  width: 100%;
}

#p-developer_toolbar .phanterpwa-developer_toolbar-objects .phanterpwa-developer_toolbar {
  color: #4e6eaa;
  padding: 20px;
  margin-bottom: 5px;
  overflow: auto;
}

#p-developer_toolbar .phanterpwa-developer_toolbar-objects .phanterpwa-developer_toolbar h2 {
  margin: 0px;
  color: gray;
}

#p-developer_toolbar .phanterpwa-developer_toolbar-objects .phanterpwa-developer_toolbar textarea {
  width: 100%;
  min-height: 300px;
}

#p-developer_toolbar .button_developer {
  position: fixed;
  width: 300px;
  right: auto;
  left: calc(50% - 150px);
  background-color: red;
  border-radius: 10px 10px 0 0;
  margin: auto;
  margin-bottom: 0;
  bottom: 0;
  padding: 5px;
  text-align: center;
  z-index: 2000;
  cursor: pointer;
}

#p-developer_toolbar .button_developer:hover {
  color: orange;
}

#p-developer_toolbar .button_developer .current-size {
  display: inline-block;
}

#p-developer_toolbar .button_developer .current-grid-width {
  display: inline-block;
}

#p-developer_toolbar .button_developer .current-grid-width::before {
  content: "width\+";
}

@media screen and (min-width: 0) and (max-width: 301px) {
  #p-developer_toolbar .button_developer .current-grid-width::before {
    content: "width1";
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  #p-developer_toolbar .button_developer .current-grid-width::before {
    content: "width2";
  }
}

@media screen and (min-width: 600px) and (max-width: 901px) {
  #p-developer_toolbar .button_developer .current-grid-width::before {
    content: "width3";
  }
}

@media screen and (min-width: 900px) and (max-width: 1201px) {
  #p-developer_toolbar .button_developer .current-grid-width::before {
    content: "width4";
  }
}

@media screen and (min-width: 1200px) and (max-width: 1501px) {
  #p-developer_toolbar .button_developer .current-grid-width::before {
    content: "width5";
  }
}

@media screen and (min-width: 1500px) and (max-width: 1801px) {
  #p-developer_toolbar .button_developer .current-grid-width::before {
    content: "width6";
  }
}

@media screen and (min-width: 1800px) and (max-width: 2100px) {
  #p-developer_toolbar .button_developer .current-grid-width::before {
    content: "width7";
  }
}

@media screen and (min-width: 2100px) {
  #p-developer_toolbar .button_developer .current-grid-width::before {
    content: "width8";
  }
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\easy.sass */
.e-padding_10 {
  padding: 10px !important;
}

.e-padding_10 hr {
  width: calc(100% + 20px);
  margin-left: -10px;
  color: white;
  outline: none;
  background-color: gray;
  margin-bottom: 20px;
}

.e-padding_10.phanterpwa-container-section .phanterpwa-section > label {
  top: 5px;
  left: 20px;
}

.e-padding_bottom_10 {
  padding-bottom: 10px !important;
}

.e-padding_top_10 {
  padding-top: 10px !important;
}

.e-padding_left_10 {
  padding-left: 10px !important;
}

.e-padding_right_10 {
  margin-right: 10px !important;
}

.e-padding_20 {
  padding: 20px !important;
}

.e-padding_20 hr {
  width: calc(100% + 40px);
  margin-left: -20px;
}

.e-padding_20.phanterpwa-container-section .phanterpwa-section > label {
  top: 10px;
  left: 30px;
}

.e-padding_bottom_20 {
  padding-bottom: 20px !important;
}

.e-padding_top_20 {
  padding-top: 20px !important;
}

.e-padding_left_20 {
  padding-left: 20px !important;
}

.e-padding_right_20 {
  padding-right: 20px !important;
}

.e-display_hidden {
  display: none !important;
}

.e-overflow_hidden {
  overflow: hidden !important;
}

.e-text_align_center {
  text-align: center !important;
}

.e-float_left {
  float: left !important;
}

.e-float_right {
  float: right !important;
}

.e-title-alert {
  text-align: center !important;
  color: red !important;
}

.e-margin_20 {
  margin: 20px;
}

.e-margin_bottom_20 {
  margin-bottom: 20px !important;
}

.e-margin_top_20 {
  margin-top: 20px !important;
}

.e-margin_left_20 {
  margin-left: 20px !important;
}

.e-margin_right_20 {
  margin-right: 20px !important;
}

.e-margin_10 {
  margin: 10px !important;
}

.e-margin_bottom_10 {
  margin-bottom: 10px !important;
}

.e-margin_top_10 {
  margin-top: 10px !important;
}

.e-margin_left_10 {
  margin-left: 10px !important;
}

.e-margin_right_10 {
  margin-right: 10px !important;
}

.e-tagger-wrapper {
  overflow: hidden;
  text-overflow: ellipsis;
  border: 1px solid #4e6eaa;
  margin: 2px;
  border-radius: 5px;
  line-height: 2rem;
}

.e-tagger-wrapper strong {
  background-color: #4e6eaa;
  padding: 0px 10px;
  color: white;
  height: 2rem;
  display: block;
  float: left;
}

.e-tagger-wrapper span {
  padding: 6px;
  white-space: nowrap;
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .e-tagger-wrapper strong {
    float: none;
  }
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .e-tagger-wrapper strong {
    float: none;
  }
}

.e-tagger-wrapper .e-tagger-button {
  float: right;
  font-size: 0.7rem;
  width: 2rem;
  height: 2rem;
  text-align: center;
  color: #4e6eaa;
}

.e-hidden {
  display: none;
}

.e-link {
  cursor: pointer;
}

.e-link:hover {
  color: orange;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\flash.sass */
body #phanterpwa-flash-container {
  position: fixed;
  z-index: 3000;
  right: 10%;
  top: 74px;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  body #phanterpwa-flash-container {
    width: 100%;
    top: auto;
    bottom: 0px;
    right: auto;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  body #phanterpwa-flash-container {
    width: 100%;
    top: auto;
    bottom: 0px;
    right: auto;
    text-align: center;
  }
}

@media screen and (min-width: 600px) and (max-width: 901px) {
  body #phanterpwa-flash-container {
    top: auto;
    bottom: 30px;
    text-align: center;
  }
}

body #phanterpwa-flash-container .phanterpwa-flash-wrapper {
  background-color: #303030;
  color: #c8c364;
  padding: 10px 20px;
  border-radius: 5px;
  margin-bottom: 5px;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  body #phanterpwa-flash-container .phanterpwa-flash-wrapper {
    width: 100%;
    text-align: center;
    float: none;
    margin: auto;
    border-radius: 0px;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  body #phanterpwa-flash-container .phanterpwa-flash-wrapper {
    width: 100%;
    text-align: center;
    float: none;
    margin: auto;
    border-radius: 0px;
  }
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\forms.sass */
.basic-profile-user-name, .form-profile-user-name {
  font-size: 2rem;
}

.basic-profile-user-img-container img, .form-image-user-img img {
  width: 100%;
  height: auto;
}

.form-image-user-container {
  position: relative;
  width: 100%;
  min-height: 200px;
}

#modal_user_login #form-login-image-user-container, #modal_user_login #form-login-button-other-user-container {
  display: none;
}

.form-profile-user-info {
  text-align: center;
}

#modal_user_login.show_current_user #phanterpwa-container-input-text-email, #modal_user_login.show_current_user #form-login-input-tittle-display_hidden-switch {
  display: none;
}

.basic-profile-user-img-container, .form-image-user-img {
  height: 145px;
  width: 145px;
  overflow: hidden;
  border-radius: 100%;
  margin-left: auto;
  margin-right: auto;
}

.phanterpwa-form-buttons-container {
  display: block;
  width: 100%;
  text-align: center;
}

.phanterpwa-form-captcha-ajax-container {
  text-align: center;
  padding-top: 10px;
  min-height: 120px;
}

.basic-profile-user-role, .form-profile-user-role {
  font-size: 0.8rem;
  color: grey;
}

.buttons-form-container {
  text-align: center;
}

#form-profile {
  padding: 20px 0;
}

#form-profile #profile-image-user-container {
  padding-top: 20px;
  padding-bottom: 10px;
}

#form-profile #profile-image-user-container .phanterpwa-centralizer-wrapper {
  margin: auto;
  box-shadow: 1px 1px 5px gray;
  border-radius: 2px;
}

#form-profile .profile_inputs_container {
  padding-top: 20px;
  padding-bottom: 10px;
}

.phanterpwa-jsonform {
  padding: 10px;
}

#modal_user_login.show_current_user #form-login-image-user-container, #modal_user_login.show_current_user #form-login-button-other-user-container {
  display: block;
}

.phanterpwa-widget-form-group {
  overflow: auto;
  width: 100%;
}

.lock-container {
  padding: 30px 20px 10px 20px;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\modal.sass */
@keyframes modalInFromNone {
  0% {
    display: none;
    padding-top: 0px;
    opacity: 0;
  }
  1% {
    display: block;
    padding-top: 0px;
    opacity: 0;
  }
  100% {
    display: block;
    padding-top: 30px;
    opacity: 1;
  }
}

.phanterpwa-component-modal-container .phanterpwa-component-modal-wrapper {
  position: relative;
  min-height: 190px;
  max-height: 95%;
  overflow: hidden;
  background-color: white;
  padding-top: 80px;
  padding-bottom: 80px;
}

.phanterpwa-component-modal-container .phanterpwa-component-modal-wrapper .phanterpwa-component-modal-header-container {
  position: absolute;
  top: 0;
  width: 100%;
  padding: 10px 50px;
  height: 80px;
  box-shadow: 1px 0px 4px #888888;
}

.phanterpwa-component-modal-container .phanterpwa-component-modal-wrapper .phanterpwa-component-modal-header-container .phanterpwa-component-modal-close {
  position: absolute;
  right: 0;
  top: 0;
  width: 50px;
  height: 50px;
  text-align: center;
  font-size: 1.5rem;
  padding-top: 5px;
}

.phanterpwa-component-modal-container .phanterpwa-component-modal-wrapper .phanterpwa-component-modal-header-container .phanterpwa-component-modal-close:hover {
  color: orange;
}

.phanterpwa-component-modal-container .phanterpwa-component-modal-wrapper .phanterpwa-component-modal-header-container .phanterpwa-component-modal-title {
  font-size: 1.5rem;
  text-transform: uppercase;
  text-align: center;
}

.phanterpwa-component-modal-container .phanterpwa-component-modal-wrapper .phanterpwa-component-modal-content-container {
  width: 100%;
  padding: 10px 10px;
  min-height: 50px;
  overflow: auto;
  height: 100%;
  background-color: #E8E8E8;
}

.phanterpwa-component-modal-container .phanterpwa-component-modal-wrapper .phanterpwa-component-modal-footer-container {
  position: absolute;
  bottom: 0;
  width: 100%;
  padding: 10px 10px;
  height: 80px;
  box-shadow: 1px 0px 4px #888888;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .phanterpwa-component-modal-container .phanterpwa-component-modal-wrapper .phanterpwa-component-modal-footer-container {
    padding: 0;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .phanterpwa-component-modal-container .phanterpwa-component-modal-wrapper .phanterpwa-component-modal-footer-container {
    padding: 0;
  }
}

.phanterpwa-component-modal-container.phanterpwa-fixed-fulldisplay {
  z-index: 1006;
  opacity: 0;
  display: none;
  padding-top: 0px;
  animation: modalInFromNone 0.5s;
}

.phanterpwa-component-modal-container.phanterpwa-fixed-fulldisplay.enabled {
  padding-top: 30px;
  display: block;
  opacity: 1;
  transition: opacity 0.5s ease, padding-top 0.5s ease;
}

.phanterpwa-component-modal-container.phanterpwa-fixed-fulldisplay.enabled.closing {
  padding-top: 0px;
  opacity: 0;
}

.phanterpwa-component-modal-container.phanterpwa-fixed-fulldisplay.enabled .phanterpwa-component-modal-wrapper {
  margin-top: -30px;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\modal_login.sass */
.phanterpwa-component-modal-content-container .phanterpwa-auth_user-form-inputs #form-login-image-user-container {
  display: none;
}

.phanterpwa-component-modal-content-container .phanterpwa-auth_user-form-inputs #form-login-button-other-user-container {
  display: none;
}

.phanterpwa-component-modal-content-container .phanterpwa-auth_user-form-inputs .phanterpwa-modal-login-social-buttons-container {
  text-align: center;
  display: block;
}

.phanterpwa-component-modal-content-container .phanterpwa-auth_user-form-inputs .phanterpwa-modal-login-social-buttons-container .btn-social_login-wrapper {
  display: inline-block;
}

.phanterpwa-component-modal-content-container .phanterpwa-auth_user-form-inputs.has_auth_user #phanterpwa-widget-login-email {
  display: none;
}

.phanterpwa-component-modal-content-container .phanterpwa-auth_user-form-inputs.has_auth_user #form-login-image-user-container {
  display: block;
}

.phanterpwa-component-modal-content-container .phanterpwa-auth_user-form-inputs.has_auth_user #form-login-button-other-user-container {
  display: block;
}

.phanterpwa-component-modal-content-container .phanterpwa-auth_user-form-inputs.has_auth_user .phanterpwa-modal-login-social-buttons-container {
  display: none;
}

.phanterpwa-component-modal-content-container .phanterpwa-auth_user-form-inputs.has_auth_user .phanterpwa-modal-login-auth_user_logged_by_social_login-buttons-container {
  display: none;
}

.phanterpwa-component-modal-content-container .phanterpwa-auth_user-form-inputs.has_auth_user.auth_user_logged_by_social_login #phanterpwa-widget-wrapper-login-password, .phanterpwa-component-modal-content-container .phanterpwa-auth_user-form-inputs.has_auth_user.auth_user_logged_by_social_login #phanterpwa-widget-wrapper-login-remember_me {
  display: none;
}

.phanterpwa-component-modal-content-container .phanterpwa-auth_user-form-inputs.has_auth_user.auth_user_logged_by_social_login .phanterpwa-modal-login-auth_user_logged_by_social_login-buttons-container {
  display: block;
}

.phanterpwa-component-modal-footer-container .hidden_on_not_has_auth_user {
  display: none;
}

.phanterpwa-component-modal-footer-container .hidden_on_its_social_login {
  display: unset;
}

.phanterpwa-component-modal-footer-container .has_auth_user.its_social_login #phanterpwa-widget-login-captcha-container {
  display: none;
}

.phanterpwa-component-modal-footer-container .has_auth_user.its_social_login .hidden_on_its_social_login {
  display: none;
}

.phanterpwa-component-modal-footer-container .has_auth_user.its_social_login .hidden_on_not_has_auth_user {
  display: unset;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\phanterpwa_gallery.sass */
.phanterpwa-gallery-upload-image-button {
  color: #4e6eaa;
  text-align: center;
}

.phanterpwa-gallery-upload-image-button i {
  font-size: 3rem;
}

.phanterpwa-gallery-upload-input-file {
  display: none;
}

.phanterpwa-gallery-upload-image-simple-view-buttons {
  padding: 2px 4px;
  background-color: rgba(0, 0, 0, 0.7);
  position: absolute;
  right: 5px;
  top: 5px;
  color: white;
  border-radius: 1px;
}

.phanterpwa-gallery-upload-image-simple-view-buttons i {
  cursor: pointer;
}

.phanterpwa-gallery-upload-image-simple-view-buttons i:hover {
  color: orange;
}

.phanterpwa-gallery-upload-image-button i {
  cursor: pointer;
}

.phanterpwa-gallery-upload-image-button i:hover {
  color: orange;
}

.phanterpwa-gallery-wrapper {
  position: relative;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\phanterpwa_gallery_cutter.sass */
/*PhanterGallery*/
.phanterpwa-gallery-container-main {
  width: 100%;
}

@keyframes fadeInFromNone {
  0% {
    display: none;
    opacity: 0;
  }
  1% {
    display: block;
    opacity: 0;
  }
  100% {
    display: block;
    opacity: 1;
  }
}

.phanterpwa-gallery-object {
  width: 170px;
  margin-left: auto;
  margin-right: auto;
  text-align: center;
  padding: 10px;
  background-color: orange;
  border-radius: 12px;
  text-transform: uppercase;
  cursor: pointer;
  min-height: 103px;
  display: none;
  opacity: 0;
  animation: fadeInFromNone 0.5s;
}

.phanterpwa-gallery-object.enable {
  display: block;
  opacity: 1;
  transition: opacity 0.5s;
}

.phanterpwa-gallery-upload-image-button .phanterwebsvg.upload-cloud {
  font-size: 50px;
  color: red;
}

.phanterpwa-gallery-panel-cutter-container {
  display: none;
  position: fixed;
  top: -20px;
  left: 0;
  width: 100%;
  height: 100%;
  overflow: hidden;
}

.phanterpwa-gallery-panel-cutter-container.enable {
  display: block;
  position: fixed;
  top: 0;
  opacity: 1;
  animation-name: fadeInOpacity;
  animation-iteration-count: 1;
  animation-timing-function: ease-in;
  animation-duration: 0.5s;
}

@keyframes fadeInOpacity {
  0% {
    opacity: 0;
    top: -20px;
  }
  100% {
    opacity: 1;
    top: 0px;
  }
}

.phanterpwa-gallery-panel-cutter-container.close {
  display: block;
  position: fixed;
  top: 0;
  height: 0;
  transition: height 1s cubic-bezier(1, 0.04, 0.38, 0.82);
}

.phanterpwa-gallery-cutter-background {
  position: absolute;
  height: 100%;
}

.phanterpwa-gallery-cutter-pad {
  position: absolute;
  width: 100%;
  height: 100%;
}

.phanterpwa-gallery-cutter-shadow {
  position: absolute;
  width: 100%;
  height: 100%;
  background-color: rgba(0, 0, 0, 0.7);
}

.phanterpwa-gallery-panel-cutter {
  position: absolute;
  width: 100%;
  height: 100%;
}

.phanterpwa-gallery-cutter-zoom-container {
  background-color: rgba(255, 252, 252, 0.8);
  width: 100%;
  height: 50px;
  position: absolute;
  bottom: 0;
}

.phanterpwa-gallery-cutter-zoom-controls {
  width: 300px;
  margin-left: auto;
  margin-right: auto;
  height: 50px;
  padding: 10px 36px 10px 36px;
  position: relative;
}

.phanterpwa-gallery-cutter-zoom-controls .image-decrease {
  position: absolute;
  left: 11px;
  top: 16px;
  font-size: 1.2rem;
}

.phanterpwa-gallery-cutter-zoom-controls .image-increase {
  position: absolute;
  right: -1px;
  top: 11px;
  font-size: 2rem;
}

.phanterwebsvg.image-increase {
  position: absolute;
  right: 0;
  font-size: 30px;
  top: 10px;
}

.phanterwebsvg.image-decrease {
  position: absolute;
  left: 0;
  font-size: 30px;
  top: 10px;
}

.phanterpwa-gallery-cutter-zoom-control-container {
  width: 100%;
  height: 30px;
  background: grey;
  border-radius: 15px;
  overflow: hidden;
  position: relative;
}

.phanterpwa-gallery-cutter-zoom-control {
  width: 50px;
  height: 30px;
  background-color: black;
  border-radius: 30px;
  left: 89px;
  position: absolute;
}

.phanterpwa-gallery-cutter-zoom-control:hover {
  background-color: orange;
}

.phanterpwa-gallery-cutter-zoom-control:active {
  background-color: red;
}

.phanterpwa-gallery-cutter-control {
  text-align: center;
}

.phanterpwa-gallery-cutter-control i {
  font-size: 40px;
  cursor: pointer;
}

.phanterpwa-gallery-cutter-control.enable {
  color: red;
}

.phanterpwa-gallery-cutter-control:hover {
  color: orange;
}

.phanterpwa-gallery-cutter-shadow.enable {
  background-color: black;
}

.phanterpwa-gallery-cutter-controls-container {
  position: absolute;
  right: 5px;
  top: 5px;
  background-color: rgba(255, 255, 255, 0.8);
  padding: 10px;
}

.phanterpwa-gallery-panel-cutter-size-container {
  position: absolute;
  box-shadow: 0px 0px 5px #b0b0b0, 0px 0px 10px #323232;
}

.phanterpwa-gallery-panel-cutter-image {
  position: absolute;
}

.phanterpwa-gallery-upload-area-progress {
  display: none;
}

.phanterpwa-gallery-upload-area-progress.enable {
  display: block;
}

.phanterpwa-gallery-upload-messages {
  text-transform: uppercase;
  text-align: center;
  color: red;
}

.phanterpwa-gallery-target-view-container {
  display: none;
  box-shadow: 1px 1px 8px grey;
  margin-left: auto;
  margin-right: auto;
  position: relative;
  background-color: black;
  opacity: 0;
  animation: fadeInFromNone;
}

.phanterpwa-gallery-target-view-container.enable {
  display: block;
  opacity: 1;
  transition: opacity 0.5s;
}

.phanterpwa-gallery-imagecuted-controls {
  position: absolute;
  background-color: rgba(255, 255, 255, 0.71);
  top: 5px;
  right: 5px;
  padding: 5px;
}

.phanterpwa-gallery-imagecuted-control i {
  font-size: 18pt;
}

.phanterpwa-gallery-imagecuted-control:hover i {
  cursor: pointer;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\progressbar.sass */
.main-progress-bar-container {
  height: 7px;
  width: 100%;
}

.main-progress-bar-container {
  position: fixed;
  bottom: 0px;
}

.phanterpwa-main_progressbar-wrapper {
  display: none;
}

.phanterpwa-main_progressbar-wrapper > .phanterpwa-main_progressbar {
  margin: 0px;
  height: 10px;
  position: relative;
  display: block;
  width: 100%;
  background-color: #acece6;
  border-radius: 2px;
  background-clip: padding-box;
  overflow: hidden;
}

.phanterpwa-main_progressbar-wrapper > .phanterpwa-main_progressbar .phanterpwa-main_progressbar-movement {
  background-color: #4e6eaa;
}

.phanterpwa-main_progressbar-wrapper > .phanterpwa-main_progressbar .phanterpwa-main_progressbar-movement:before {
  content: '';
  position: absolute;
  background-color: inherit;
  top: 0;
  left: 0;
  bottom: 0;
  will-change: left, right;
  -webkit-animation: none;
  animation: none;
}

.phanterpwa-main_progressbar-wrapper > .phanterpwa-main_progressbar .phanterpwa-main_progressbar-movement:after {
  content: '';
  position: absolute;
  background-color: inherit;
  top: 0;
  left: 0;
  bottom: 0;
  will-change: left, right;
  -webkit-animation: none;
  animation: none;
  -webkit-animation-delay: 1.15s;
  animation-delay: 1.15s;
}

.phanterpwa-main_progressbar-wrapper.enabled {
  display: block;
}

.phanterpwa-main_progressbar-wrapper.enabled > .phanterpwa-main_progressbar .phanterpwa-main_progressbar-movement {
  background-color: #4e6eaa;
}

.phanterpwa-main_progressbar-wrapper.enabled > .phanterpwa-main_progressbar .phanterpwa-main_progressbar-movement:before {
  -webkit-animation: phanterpwa_progressbar-movement 2.1s cubic-bezier(0.65, 0.815, 0.735, 0.395) infinite;
  animation: phanterpwa_progressbar-movement 2.1s cubic-bezier(0.65, 0.815, 0.735, 0.395) infinite;
}

.phanterpwa-main_progressbar-wrapper.enabled > .phanterpwa-main_progressbar .phanterpwa-main_progressbar-movement:after {
  -webkit-animation: phanterpwa_progressbar-movement-short 2.1s cubic-bezier(0.165, 0.84, 0.44, 1) infinite;
  animation: phanterpwa_progressbar-movement-short 2.1s cubic-bezier(0.165, 0.84, 0.44, 1) infinite;
  -webkit-animation-delay: 1.15s;
  animation-delay: 1.15s;
}

@-webkit-keyframes phanterpwa_progressbar-movement {
  0% {
    left: -35%;
    right: 100%;
  }
  60% {
    left: 100%;
    right: -90%;
  }
  100% {
    left: 100%;
    right: -90%;
  }
}

@keyframes phanterpwa_progressbar-movement {
  0% {
    left: -35%;
    right: 100%;
  }
  60% {
    left: 100%;
    right: -90%;
  }
  100% {
    left: 100%;
    right: -90%;
  }
}

@-webkit-keyframes phanterpwa_progressbar-movement-short {
  0% {
    left: -200%;
    right: 100%;
  }
  60% {
    left: 107%;
    right: -8%;
  }
  100% {
    left: 107%;
    right: -8%;
  }
}

@keyframes phanterpwa_progressbar-movement-short {
  0% {
    left: -200%;
    right: 100%;
  }
  60% {
    left: 107%;
    right: -8%;
  }
  100% {
    left: 107%;
    right: -8%;
  }
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\pseudomodal.sass */
.phanterpwa-component-pseudomodal-container {
  position: absolute;
  z-index: 900;
  top: 0;
}

.phanterpwa-component-pseudomodal-container .phanterpwa-component-pseudomodal-wrapper {
  position: relative;
}

.phanterpwa-component-pseudomodal-container .phanterpwa-component-pseudomodal-wrapper .phanterpwa-component-pseudomodal-content-wrapper {
  position: absolute;
  width: auto;
  box-shadow: 0px 1px 6px gray;
  max-height: 50%;
  display: table;
  overflow: auto;
}

.phanterpwa-component-pseudomodal-container .phanterpwa-component-pseudomodal-wrapper .phanterpwa-component-pseudomodal-content-wrapper .phanterpwa-component-pseudomodal-content {
  width: 100%;
  overflow: auto;
  max-height: 200px;
  box-shadow: 1px 1px 4px gray;
  background-color: white;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\run_points.sass */
.phanterpwa-components-preloaders-run_points * {
  box-sizing: border-box;
}

.phanterpwa-components-preloaders-run_points .left {
  float: left !important;
}

.phanterpwa-components-preloaders-run_points .right {
  float: right !important;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper {
  display: inline-block;
  position: relative;
  width: 100px;
  height: 40px;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper .phanterpwa_run_points_container {
  width: 100%;
  height: 100%;
  position: relative;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper .phanterpwa_run_points {
  width: 20%;
  height: 50%;
  -moz-border-radius: 100%;
  -webkit-border-radius: 100%;
  border-radius: 100%;
  position: absolute;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_one_content {
  -webkit-animation: phanterpwa_run_points 2s linear infinite;
  animation: phanterpwa_run_points 2s linear infinite;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_one_content .phanterpwa_run_points_one {
  background-color: red;
  left: 10%;
  top: 10%;
  border-radius: 100%;
  width: 80%;
  height: 80%;
  position: absolute;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_two_content {
  -webkit-animation: phanterpwa_run_points 2s linear infinite -.4s;
  animation: phanterpwa_run_points 2s linear infinite -.4s;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_two_content .phanterpwa_run_points_two {
  background-color: blue;
  left: 10%;
  top: 10%;
  border-radius: 100%;
  width: 80%;
  height: 80%;
  position: absolute;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_three_content {
  -webkit-animation: phanterpwa_run_points 2s linear infinite -.8s;
  animation: phanterpwa_run_points 2s linear infinite -.8s;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_three_content .phanterpwa_run_points_three {
  background-color: yellow;
  left: 10%;
  top: 10%;
  border-radius: 100%;
  width: 80%;
  height: 80%;
  position: absolute;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_four_content {
  -webkit-animation: phanterpwa_run_points 2s linear infinite -1.2s;
  animation: phanterpwa_run_points 2s linear infinite -1.2s;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_four_content .phanterpwa_run_points_four {
  background-color: pink;
  left: 10%;
  top: 10%;
  border-radius: 100%;
  width: 80%;
  height: 80%;
  position: absolute;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_five_content {
  -webkit-animation: phanterpwa_run_points 2s linear infinite -1.6s;
  animation: phanterpwa_run_points 2s linear infinite -1.6s;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_five_content .phanterpwa_run_points_five {
  background-color: green;
  left: 10%;
  top: 10%;
  border-radius: 100%;
  width: 80%;
  height: 80%;
  position: absolute;
}

@keyframes phanterpwa_run_points {
  0% {
    width: 20%;
    left: 80%;
    top: 50%;
  }
  80% {
    width: 20%;
    left: 0;
    top: 50%;
  }
  85% {
    left: 0;
    top: 10%;
    width: 20%;
    height: 50%;
  }
  88% {
    top: 20%;
    width: 40%;
    height: 30%;
  }
  92.5% {
    top: 20%;
    width: 40%;
    height: 30%;
  }
  97% {
    top: 20%;
    left: 80%;
    width: 20%;
    height: 30%;
  }
  100% {
    width: 20%;
    left: 80%;
    top: 50%;
  }
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\sections.sass */
.phanterpwa-xsection-container {
  position: relative;
  display: block;
  padding: 10px 0;
  width: 100%;
  overflow: auto;
}

.phanterpwa-xsection-container .phanterpwa-xsection {
  padding: 0 10px;
  border: 1px solid #d7d7d7;
  overflow: auto;
  border-radius: 5px;
  width: 100%;
  padding: 15px 0px 10px 0px;
}

.phanterpwa-xsection-container .phanterpwa-xsection > label {
  padding: 5px;
  color: #4e6eaa;
  position: absolute;
  top: 0;
  left: 15px;
  background-color: white;
  padding-left: 10px;
  margin: 0;
  border: 1px solid #d7d7d7;
  padding-right: 10px;
  border-radius: 5px;
  font-size: 0.8rem;
}

.phanterpwa-xsection-container .phanterpwa-xsection > hr {
  border: 0.5px solid #eeeded;
}

.phanterpwa-xsection-container.e-padding_20 .phanterpwa-xsection {
  padding: 20px;
}

.phanterpwa-xsection-container.e-padding_10 .phanterpwa-xsection {
  padding: 10px;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\snippet.sass */
.phanterpwa-snippet-centralizer {
  min-height: 20px;
  height: 20px;
  overflow: hidden;
  transition: height 1s ease;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\widgets.sass */
.phanterpwa-widget {
  position: relative;
  display: block;
  width: 100%;
  padding: 5px;
}

.phanterpwa-widget .phanterpwa-widget-icon-wrapper {
  color: white;
}

.phanterpwa-widget .phanterpwa-widget-check {
  position: absolute;
  top: 30px;
  right: 7px;
  font-size: 0.7rem;
  color: #62d062;
}

.phanterpwa-widget .phanterpwa-widget-wrapper .phanterpwa-widget-check {
  color: #62d062;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.no_pass .phanterpwa-widget-check {
  color: #E8E8E8;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.has_error .phanterpwa-widget-check {
  color: red;
}

.phanterpwa-widget.phanterpwa-widget-reference .phanterpwa-widget-check {
  right: 21px;
}

.phanterpwa-widget.phanterpwa-widget-select .phanterpwa-widget-check {
  right: 21px;
  cursor: pointer;
}

.phanterpwa-widget.phanterpwa-widget-multselect .phanterpwa-widget-multselect-caret {
  cursor: pointer;
}

.phanterpwa-widget.phanterpwa-widget-multselect .phanterpwa-widget-check {
  cursor: pointer;
}

.phanterpwa-widget.phanterpwa-widget-has_icon_button {
  padding-right: 40px;
}

.phanterpwa-widget.phanterpwa-widget-has_icon_button .phanterpwa-widget-check {
  right: 47px;
}

.phanterpwa-widget.phanterpwa-widget-has_icon_button.phanterpwa-widget-reference .phanterpwa-widget-check {
  right: 61px;
}

.phanterpwa-widget.phanterpwa-widget-has_icon_button.phanterpwa-widget-select .phanterpwa-widget-check {
  right: 61px;
}

.phanterpwa-widget.phanterpwa-widget-has_icon_button .phanterpwa-widget-icon_button {
  position: absolute;
  width: 35px;
  height: 35px;
  border-radius: 35px;
  padding: 0;
  top: 20px;
  right: 0;
}

.phanterpwa-widget.phanterpwa-widget-menubox {
  text-align: center;
  padding: 0;
}

.phanterpwa-widget {
  padding-right: 5px;
}

.phanterpwa-widget input, .phanterpwa-widget textarea {
  color: rgba(0, 0, 0, 0.8);
}

.phanterpwa-widget input:focus, .phanterpwa-widget textarea:focus {
  outline-offset: unset !important;
  outline: unset !important;
}

.phanterpwa-widget .phanterpwa-widget-wrapper {
  padding-bottom: 10px;
}

.phanterpwa-widget .phanterpwa-widget-wrapper .phanterpwa-widget-message_error {
  position: absolute;
  width: 100%;
  text-align: center;
  font-size: 0.7rem;
  color: red;
  overflow: hidden;
  height: 0;
  transition: height 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.has_error .phanterpwa-widget-message_error {
  height: auto;
  transition: height 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.has_icon {
  padding-right: 45px;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.has_icon .phanterpwa-widget-check {
  right: 45px;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.has_icon .phanterpwa-widget-icon-wrapper {
  width: 40px;
  height: 40px;
  position: absolute;
  right: 0;
  top: 4px;
  background-color: #4e6eaa;
  text-align: center;
  line-height: 40px;
  border-radius: 29px;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.has_mask input, .phanterpwa-widget .phanterpwa-widget-wrapper.its_numbers input {
  font-family: "Segoe UI Mono W01 Regular";
  letter-spacing: -0.08rem;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.its_disabled input[disabled] {
  color: gray;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.its_disabled hr.material-widgets-animation-offfocus {
  border-bottom: 1px dotted;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-checkbox-wrapper {
  text-align: left;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-checkbox-wrapper .phanterpwa-widget-checkbox-input {
  display: none;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-checkbox-wrapper .phanterpwa-widget-checkbox-checkbox {
  cursor: pointer;
  float: left;
  margin-left: 10px;
  margin-right: 10px;
  width: 25px;
  position: relative;
  height: 25px;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-checkbox-wrapper .phanterpwa-widget-checkbox-checkbox .phanterpwa-widget-checkbox-option-container {
  position: absolute;
  width: 25px;
  height: 25px;
  border: 1px solid;
  top: 0;
  border-radius: 5px;
  border: 1px solid gray;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-checkbox-wrapper .phanterpwa-widget-checkbox-checkbox .phanterpwa-widget-checkbox-true {
  opacity: 0;
  width: 0;
  text-align: center;
  line-height: 25px;
  overflow: hidden;
  transition: width 0.5s ease, opacity 0.5s ease;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-radio-wrapper label {
  cursor: pointer;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-radio-wrapper .phanterpwa-widget-radio-input {
  display: none;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-radio-wrapper .phanterpwa-widget-radio-radio {
  cursor: pointer;
  float: left;
  margin-left: 10px;
  margin-right: 10px;
  width: 25px;
  position: relative;
  height: 25px;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-radio-wrapper .phanterpwa-widget-radio-radio .phanterpwa-widget-radio-option-container {
  position: absolute;
  width: 25px;
  height: 25px;
  top: 0;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.has_true .phanterpwa-widget-checkbox-checkbox .phanterpwa-widget-checkbox-option-container {
  border: 1px solid #4e6eaa;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.has_true .phanterpwa-widget-checkbox-checkbox .phanterpwa-widget-checkbox-true {
  opacity: 1;
  width: 100%;
  text-align: center;
  color: #4e6eaa;
  transition: width 0.5s ease, opacity 0.5s ease;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.has_true .phanterpwa-widget-radio-radio .phanterpwa-widget-radio-true {
  color: #4e6eaa;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.focus label {
  color: #4e6eaa;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-wear-material .phanterpwa-widget-checkbox-checkbox .phanterpwa-widget-checkbox-option-container {
  border: 2px solid;
  opacity: 1;
  transition: all 0.5s ease;
  bottom: 0px;
  top: auto;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-wear-material .phanterpwa-widget-checkbox-checkbox .phanterpwa-widget-checkbox-true {
  opacity: 0;
  width: 100%;
  text-align: left;
  color: #4e6eaa;
  font-size: 1.4rem;
  left: -2px;
  overflow: hidden;
  transform: rotate(45deg);
  bottom: -8px;
  transition: all 0.5s ease;
  position: absolute;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-wear-material.has_true .phanterpwa-widget-checkbox-checkbox .phanterpwa-widget-checkbox-option-container {
  opacity: 0;
  transition: all 0.5s ease;
  transform: rotate(-45deg);
  position: absolute;
  bottom: 7px;
  top: auto;
  left: -2px;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-wear-material.has_true .phanterpwa-widget-checkbox-checkbox .phanterpwa-widget-checkbox-true {
  font-size: 1.4rem;
  opacity: 1;
  width: 100%;
  overflow: hidden;
  text-align: left;
  transform: rotate(0deg);
  transition: all 0.5s ease;
  bottom: 1px;
  position: absolute;
  left: 2px;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material {
  position: relative;
  padding-top: 10px;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material input, .phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material textarea, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material input, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material textarea, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material input, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material textarea, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material input, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material textarea {
  width: 100%;
  width: calc(100% - 15px);
  padding: 5px;
  background-color: transparent;
  border: none;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material input::placeholder, .phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material textarea::placeholder, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material input::placeholder, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material textarea::placeholder, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material input::placeholder, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material textarea::placeholder, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material input::placeholder, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material textarea::placeholder {
  color: rgba(128, 128, 128, 0);
  opacity: 0;
  transition: opacity 0.3s ease, color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material textarea, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material textarea, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material textarea, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material textarea {
  min-height: 2rem;
  width: 100% !important;
  transition: height 0.3s ease;
  resize: none;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material label, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material label, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material label, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material label {
  left: 5px;
  bottom: 20px;
  position: absolute;
  color: #888888;
  font-size: 1rem;
  transition: bottom 0.3s ease, color 0.3s ease, font-size 0.3s ease, left 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material .material-widgets-animation-onfocus, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material .material-widgets-animation-onfocus, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material .material-widgets-animation-onfocus, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material .material-widgets-animation-onfocus {
  width: 0;
  transition: width 0.3s ease;
  margin: auto;
  bottom: -1px;
  height: 2px;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material hr.material-widgets-animation-offfocus, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material hr.material-widgets-animation-offfocus, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material hr.material-widgets-animation-offfocus, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material hr.material-widgets-animation-offfocus {
  border-bottom: 1px solid #888888;
  width: 100% !important;
  margin: 0 !important;
  transition: border-bottom 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material .phanterpwa-widget-check, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material .phanterpwa-widget-check, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material .phanterpwa-widget-check, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material .phanterpwa-widget-check {
  position: absolute;
  top: 13px;
  right: 5px;
  font-size: 0.7rem;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material.has_icon .phanterpwa-widget-check, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material.has_icon .phanterpwa-widget-check, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material.has_icon .phanterpwa-widget-check, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material.has_icon .phanterpwa-widget-check {
  right: 50px;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material.has_icon .phanterpwa-widget-icon-wrapper, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material.has_icon .phanterpwa-widget-icon-wrapper, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material.has_icon .phanterpwa-widget-icon-wrapper, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material.has_icon .phanterpwa-widget-icon-wrapper {
  border-radius: 100%;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material.has_value label, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material.has_value label, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material.has_value label, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material.has_value label {
  left: 5px;
  bottom: 39px;
  position: absolute;
  color: #4e6eaa;
  font-size: 0.8rem;
  transition: bottom 0.3s ease, color 0.3s ease, font-size 0.3s ease, left 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material.focus input::placeholder, .phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material.focus textarea::placeholder, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material.focus input::placeholder, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material.focus textarea::placeholder, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material.focus input::placeholder, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material.focus textarea::placeholder, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material.focus input::placeholder, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material.focus textarea::placeholder {
  color: #E0E0E0;
  opacity: 1;
  transition: opacity 0.3s ease, color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material.focus label, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material.focus label, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material.focus label, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material.focus label {
  left: 5px;
  bottom: 39px;
  position: absolute;
  color: #4e6eaa;
  font-size: 0.8rem;
  transition: bottom 0.3s ease, color 0.3s ease, font-size 0.3s ease, left 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material.focus .material-widgets-animation-onfocus, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material.focus .material-widgets-animation-onfocus, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material.focus .material-widgets-animation-onfocus, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material.focus .material-widgets-animation-onfocus {
  width: 100%;
  height: 2px;
  background-color: #4e6eaa;
  transition: width 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material.focus hr.material-widgets-animation-offfocus, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material.focus hr.material-widgets-animation-offfocus, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material.focus hr.material-widgets-animation-offfocus, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material.focus hr.material-widgets-animation-offfocus {
  border-bottom: 1px solid #4e6eaa;
  transition: border-bottom 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material.has_error label, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material.has_error label, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material.has_error label, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material.has_error label {
  color: red;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material.has_error .material-widgets-animation-onfocus, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material.has_error .material-widgets-animation-onfocus, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material.has_error .material-widgets-animation-onfocus, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material.has_error .material-widgets-animation-onfocus {
  width: 100%;
  height: 2px;
  background-color: red;
  transition: width 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-material.has_error .phanterpwa-widget-message_error, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material.has_error .phanterpwa-widget-message_error, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-material.has_error .phanterpwa-widget-message_error, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material.has_error .phanterpwa-widget-message_error {
  height: auto;
  transition: height 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-shadows, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-shadows, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-shadows, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-shadows {
  position: relative;
  padding-top: 5px;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-shadows.has_label, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-shadows.has_label, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-shadows.has_label, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-shadows.has_label {
  padding: 22px 5px;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-shadows input, .phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-shadows textarea, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-shadows input, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-shadows textarea, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-shadows input, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-shadows textarea, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-shadows input, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-shadows textarea {
  border-radius: 5px;
  line-height: 1.5rem;
  width: 100%;
  padding: 5px;
  background-color: white;
  border: 1px solid #888888;
  transition: box-shadow 0.3s ease, border, 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-shadows label, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-shadows label, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-shadows label, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-shadows label {
  left: 5px;
  top: 0px;
  position: absolute;
  color: #888888;
  font-size: 1rem;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-check, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-check, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-check, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-check {
  position: absolute;
  top: 32px;
  right: 13px;
  font-size: 0.7rem;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-check label, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-check label, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-check label, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-check label {
  color: #4e6eaa;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-shadows.focus input, .phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-shadows.focus textarea, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-shadows.focus input, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-shadows.focus textarea, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-shadows.focus input, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-shadows.focus textarea, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-shadows.focus input, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-shadows.focus textarea {
  box-shadow: 0px 0px 5px #4e6eaa;
  border: 1px solid #4e6eaa;
  transition: box-shadow 0.3s ease, border, 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-shadows.focus label, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-shadows.focus label, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-shadows.focus label, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-shadows.focus label {
  color: #4e6eaa;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-elegant, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-elegant, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-elegant, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-elegant {
  position: relative;
  padding-top: 5px;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-elegant.has_label, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-elegant.has_label, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-elegant.has_label, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-elegant.has_label {
  padding: 22px 5px;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-elegant input, .phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-elegant textarea, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-elegant input, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-elegant textarea, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-elegant input, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-elegant textarea, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-elegant input, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-elegant textarea {
  border-radius: 5px;
  line-height: 2rem;
  width: 100%;
  padding: 10px;
  background-color: white;
  border: 1px solid #888888;
  transition: border 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-elegant input::placeholder, .phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-elegant textarea::placeholder, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-elegant input::placeholder, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-elegant textarea::placeholder, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-elegant input::placeholder, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-elegant textarea::placeholder, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-elegant input::placeholder, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-elegant textarea::placeholder {
  text-transform: uppercase;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-elegant label, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-elegant label, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-elegant label, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-elegant label {
  left: 5px;
  top: 0px;
  position: absolute;
  color: #888888;
  font-size: 1rem;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-check, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-check, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-check, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-check {
  position: absolute;
  top: 24px;
  right: 11px;
  font-size: 0.7rem;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-elegant.has_value label, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-elegant.has_value label, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-elegant.has_value label, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-elegant.has_value label {
  color: #4e6eaa;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-elegant.focus input, .phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-elegant.focus textarea, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-elegant.focus input, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-elegant.focus textarea, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-elegant.focus input, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-elegant.focus textarea, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-elegant.focus input, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-elegant.focus textarea {
  border: 1px solid #4e6eaa;
  transition: box-shadow 0.3s ease, border, 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-input-wrapper.phanterpwa-widget-wear-elegant.focus label, .phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-elegant.focus label, .phanterpwa-widget .phanterpwa-widget-inert-wrapper.phanterpwa-widget-wear-elegant.focus label, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-elegant.focus label {
  color: #4e6eaa;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material label, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material label {
  top: 20px;
  bottom: auto;
  transition: top 0.3s ease, color 0.3s ease, font-size 0.3s ease, left 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material.focus label, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material.focus label {
  top: -3px;
  bottom: auto;
  transition: top 0.3s ease, color 0.3s ease, font-size 0.3s ease, left 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-textarea-wrapper.phanterpwa-widget-wear-material.has_value label, .phanterpwa-widget .phanterpwa-widget-list_string-wrapper.phanterpwa-widget-wear-material.has_value label {
  top: -3px;
  bottom: auto;
  transition: top 0.3s ease, color 0.3s ease, font-size 0.3s ease, left 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper .phanterpwa-widget-select-li-title {
  color: white;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material {
  position: relative;
  padding-top: 10px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material .phanterpwa-widget-select-touchpad {
  position: absolute;
  width: 100%;
  top: 0;
  left: 0;
  height: 100;
  min-height: 50px;
  cursor: pointer;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material .phanterpwa-widget-select-touchpad:focus {
  outline: none;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material input {
  cursor: pointer;
  width: 100%;
  width: calc(100% - 38px);
  padding: 5px;
  background-color: transparent;
  border: none;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material input::placeholder {
  color: #E0E0E0;
  opacity: 1;
  transition: opacity 0.3s ease, color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material input[enabled] {
  color: rgba(0, 0, 0, 0.8);
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material label {
  cursor: pointer;
  left: 5px;
  bottom: 20px;
  position: absolute;
  color: #888888;
  font-size: 1rem;
  transition: bottom 0.3s ease, color 0.3s ease, font-size 0.3s ease, left 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material select {
  display: none;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material .material-widgets-animation-onfocus {
  width: 0;
  transition: width 0.3s ease;
  margin: auto;
  bottom: -1px;
  height: 2px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material hr.material-widgets-animation-offfocus {
  border-bottom: 1px solid #888888;
  width: 100% !important;
  margin: 0 !important;
  transition: border-bottom 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material .phanterpwa-widget-select-caret {
  position: absolute;
  top: 12px;
  right: 13px;
  font-size: 1rem;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material .phanterpwa-widget-check {
  position: absolute;
  top: 12px;
  right: 30px;
  font-size: 0.7rem;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material.has_icon .phanterpwa-widget-select-caret {
  right: 53px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material.has_icon .phanterpwa-widget-check {
  right: 70px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material.has_icon .phanterpwa-widget-select-touchpad {
  width: calc(100% - 40px);
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material.has_value label {
  left: 5px;
  bottom: 39px;
  position: absolute;
  color: #4e6eaa;
  font-size: 0.8rem;
  transition: bottom 0.3s ease, color 0.3s ease, font-size 0.3s ease, left 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material.has_label input::placeholder {
  color: rgba(128, 128, 128, 0);
  opacity: 0;
  transition: opacity 0.3s ease, color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material.focus input::placeholder {
  color: #E0E0E0;
  opacity: 1;
  transition: opacity 0.3s ease, color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material.focus label {
  left: 5px;
  bottom: 39px;
  position: absolute;
  color: #4e6eaa;
  font-size: 0.8rem;
  transition: bottom 0.3s ease, color 0.3s ease, font-size 0.3s ease, left 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material.focus .material-widgets-animation-onfocus {
  width: 100%;
  height: 2px;
  background-color: #4e6eaa;
  transition: width 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material.focus hr.material-widgets-animation-offfocus {
  border-bottom: 1px solid #4e6eaa;
  transition: border-bottom 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material.pre_focus label {
  color: #4e6eaa;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material.pre_focus .material-widgets-animation-onfocus {
  width: 100%;
  height: 2px;
  background-color: #4e6eaa;
  transition: width 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-material.pre_focus hr.material-widgets-animation-offfocus {
  border-bottom: 1px solid #4e6eaa;
  transition: border-bottom 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows {
  position: relative;
  padding-top: 5px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-select-touchpad {
  position: absolute;
  width: 100%;
  top: 7px;
  left: 0;
  height: 100;
  min-height: 36px;
  cursor: pointer;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-select-touchpad:focus {
  outline: none;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-select-caret {
  position: absolute;
  top: 12px;
  right: 13px;
  font-size: 1rem;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-check {
  position: absolute;
  top: 16px;
  right: 30px;
  font-size: 0.7rem;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-icon-wrapper {
  top: 4px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows.has_label {
  padding: 22px 0px 10px 5px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows.has_label .phanterpwa-widget-select-caret {
  position: absolute;
  top: 30px;
  right: 13px;
  font-size: 1rem;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows.has_label .phanterpwa-widget-check {
  position: absolute;
  top: 32px;
  right: 30px;
  font-size: 0.7rem;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows.has_label .phanterpwa-widget-icon-wrapper {
  top: 20px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows.has_label .phanterpwa-widget-select-touchpad {
  top: 9px;
  min-height: 50px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows.has_icon {
  padding-right: 45px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows.has_icon .phanterpwa-widget-select-touchpad {
  width: calc(100% - 40px);
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows.has_icon .phanterpwa-widget-icon-wrapper {
  width: 40px;
  height: 40px;
  position: absolute;
  right: 0;
  background-color: #4e6eaa;
  text-align: center;
  line-height: 40px;
  border-radius: 29px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows.has_icon .phanterpwa-widget-select-caret {
  right: 53px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows.has_icon .phanterpwa-widget-check {
  right: 70px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows input {
  border-radius: 5px;
  line-height: 1.5rem;
  width: 100%;
  padding: 5px;
  background-color: white;
  border: 1px solid #888888;
  transition: box-shadow 0.3s ease, border, 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows label {
  left: 5px;
  top: 0px;
  position: absolute;
  color: #888888;
  font-size: 1rem;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows select {
  display: none;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows.has_value label {
  color: #4e6eaa;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows.focus input {
  box-shadow: 0px 0px 5px #4e6eaa;
  border: 1px solid #4e6eaa;
  transition: box-shadow 0.3s ease, border, 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows.focus label {
  color: #4e6eaa;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows.pre_focus input {
  box-shadow: 0px 0px 5px #4e6eaa;
  border: 1px solid #4e6eaa;
  transition: box-shadow 0.3s ease, border, 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-shadows.pre_focus label {
  color: #4e6eaa;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant {
  position: relative;
  padding-top: 5px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-select-touchpad {
  position: absolute;
  width: 100%;
  top: 6px;
  left: 0;
  height: 100;
  min-height: 52px;
  cursor: pointer;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-select-touchpad:focus {
  outline: none;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-icon-wrapper {
  top: 15px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-select-caret {
  top: 24px;
  right: 7px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-check {
  top: 7px;
  right: 7px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant.has_label {
  padding: 22px 0px 10px 5px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant.has_label .phanterpwa-widget-select-touchpad {
  top: 7px;
  min-height: 67px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant.has_label .phanterpwa-widget-select-caret {
  top: 42px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant.has_label .phanterpwa-widget-check {
  top: 25px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant.has_label .phanterpwa-widget-icon-wrapper {
  top: 29px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant input {
  border-radius: 5px;
  line-height: 2rem;
  width: 100%;
  padding: 10px;
  background-color: white;
  border: 1px solid #888888;
  transition: border 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant input::placeholder {
  text-transform: uppercase;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant label {
  left: 5px;
  top: 0px;
  position: absolute;
  color: #888888;
  font-size: 1rem;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant select {
  display: none;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-select-caret {
  position: absolute;
  font-size: 1rem;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-check {
  position: absolute;
  font-size: 0.7rem;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant.has_icon {
  padding-right: 45px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant.has_icon .phanterpwa-widget-select-touchpad {
  width: calc(100% - 40px);
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant.has_icon .phanterpwa-widget-icon-wrapper {
  width: 40px;
  height: 40px;
  position: absolute;
  right: 0;
  background-color: #4e6eaa;
  text-align: center;
  line-height: 40px;
  border-radius: 29px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant.has_icon .phanterpwa-widget-select-caret {
  right: 53px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant.has_icon .phanterpwa-widget-check {
  right: 53px;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant.has_value label {
  color: #4e6eaa;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant.focus input {
  border: 1px solid #4e6eaa;
  transition: box-shadow 0.3s ease, border, 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant.focus label {
  color: #4e6eaa;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant.pre_focus input {
  border: 1px solid #4e6eaa;
  transition: box-shadow 0.3s ease, border, 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-select-wrapper.phanterpwa-widget-wear-elegant.pre_focus label {
  color: #4e6eaa;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper .phanterpwa-widget-multselect-li-title {
  color: white;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material {
  position: relative;
  padding-top: 10px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material .phanterpwa-widget-multselect-touchpad {
  position: absolute;
  width: 100%;
  top: 0;
  left: 0;
  height: 100;
  min-height: 50px;
  cursor: pointer;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material .phanterpwa-widget-multselect-touchpad:focus {
  outline: none;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material input {
  cursor: pointer;
  width: 100%;
  width: calc(100% - 38px);
  padding: 5px;
  background-color: transparent;
  border: none;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material input::placeholder {
  color: rgba(128, 128, 128, 0);
  opacity: 0;
  transition: opacity 0.3s ease, color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material input[enabled] {
  color: rgba(0, 0, 0, 0.8);
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material label {
  cursor: pointer;
  left: 5px;
  top: 15px;
  position: absolute;
  color: #888888;
  font-size: 1rem;
  transition: bottom 0.3s ease, color 0.3s ease, font-size 0.3s ease, left 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material select {
  display: none;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material .material-widgets-animation-onfocus {
  width: 0;
  transition: width 0.3s ease;
  margin: auto;
  bottom: -1px;
  height: 2px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material hr.material-widgets-animation-offfocus {
  border-bottom: 1px solid #888888;
  width: 100% !important;
  margin: 0 !important;
  transition: border-bottom 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material .phanterpwa-widget-multselect-caret {
  position: absolute;
  top: 12px;
  right: 13px;
  font-size: 1rem;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material .phanterpwa-widget-check {
  position: absolute;
  top: 12px;
  right: 30px;
  font-size: 0.7rem;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material.has_icon .phanterpwa-widget-multselect-touchpad {
  width: calc(100% - 40px);
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material.has_icon .phanterpwa-widget-multselect-caret {
  right: 53px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material.has_icon .phanterpwa-widget-check {
  right: 70px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material.has_value label {
  left: 5px;
  top: 0px;
  position: absolute;
  color: #4e6eaa;
  font-size: 0.8rem;
  transition: top 0.3s ease, color 0.3s ease, font-size 0.3s ease, left 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material.focus input::placeholder {
  color: #E0E0E0;
  opacity: 1;
  transition: opacity 0.3s ease, color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material.focus label {
  left: 5px;
  top: 0px;
  bottom: auto;
  position: absolute;
  color: #4e6eaa;
  font-size: 0.8rem;
  transition: top 0.3s ease, color 0.3s ease, font-size 0.3s ease, left 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material.focus .material-widgets-animation-onfocus {
  width: 100%;
  height: 2px;
  background-color: #4e6eaa;
  transition: width 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material.focus hr.material-widgets-animation-offfocus {
  border-bottom: 1px solid #4e6eaa;
  transition: border-bottom 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material.pre_focus label {
  color: #4e6eaa;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material.pre_focus .material-widgets-animation-onfocus {
  width: 100%;
  height: 2px;
  background-color: #4e6eaa;
  transition: width 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-material.pre_focus hr.material-widgets-animation-offfocus {
  border-bottom: 1px solid #4e6eaa;
  transition: border-bottom 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows {
  position: relative;
  padding-top: 5px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-multselect-touchpad {
  position: absolute;
  width: 100%;
  top: 5px;
  left: 0;
  height: 36px;
  min-height: 36px;
  cursor: pointer;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-multselect-touchpad:focus {
  outline: none;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-multselect-caret {
  top: 15px;
  right: 13px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-check {
  top: 17px;
  right: 30px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-icon-wrapper {
  top: 3px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows.has_label {
  padding: 22px 0 10px 5px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows.has_label .phanterpwa-widget-multselect-touchpad {
  top: 9px;
  height: 50px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows.has_label .phanterpwa-widget-multselect-caret {
  top: 30px;
  right: 13px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows.has_label .phanterpwa-widget-check {
  top: 32px;
  right: 30px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows.has_label .phanterpwa-widget-icon-wrapper {
  top: 20px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows input {
  border-radius: 5px;
  line-height: 1.5rem;
  width: 100%;
  padding: 5px;
  background-color: white;
  border: 1px solid #888888;
  transition: box-shadow 0.3s ease, border, 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows label {
  left: 5px;
  top: 0px;
  position: absolute;
  color: #888888;
  font-size: 1rem;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows select {
  display: none;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-multselect-caret {
  position: absolute;
  font-size: 1rem;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-check {
  position: absolute;
  font-size: 0.7rem;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows.has_icon {
  padding-right: 45px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows.has_icon .phanterpwa-widget-multselect-touchpad {
  width: calc(100% - 40px);
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows.has_icon .phanterpwa-widget-icon-wrapper {
  width: 40px;
  height: 40px;
  position: absolute;
  right: 0;
  background-color: #4e6eaa;
  text-align: center;
  line-height: 40px;
  border-radius: 29px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows.has_icon .phanterpwa-widget-multselect-caret {
  right: 53px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows.has_icon .phanterpwa-widget-check {
  right: 70px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows.has_value label {
  color: #4e6eaa;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows.focus input {
  box-shadow: 0px 0px 5px #4e6eaa;
  border: 1px solid #4e6eaa;
  transition: box-shadow 0.3s ease, border, 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows.focus label {
  color: #4e6eaa;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows.pre_focus input {
  box-shadow: 0px 0px 5px #4e6eaa;
  border: 1px solid #4e6eaa;
  transition: box-shadow 0.3s ease, border, 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-shadows.pre_focus label {
  color: #4e6eaa;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant {
  position: relative;
  padding-top: 5px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-multselect-touchpad {
  position: absolute;
  width: 100%;
  top: 6px;
  left: 0;
  min-height: 54px;
  cursor: pointer;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-multselect-touchpad:focus {
  outline: none;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-multselect-caret {
  top: 24px;
  right: 7px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-check {
  top: 7px;
  right: 7px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-icon-wrapper {
  top: 15px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant.has_label {
  padding: 22px 0 10px 5px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant.has_label .phanterpwa-widget-multselect-caret {
  top: 40px;
  right: 7px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant.has_label .phanterpwa-widget-check {
  top: 24px;
  right: 7px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant.has_label .phanterpwa-widget-icon-wrapper {
  top: 27px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant.has_label .phanterpwa-widget-multselect-touchpad {
  top: 6px;
  height: 70px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant input {
  border-radius: 5px;
  line-height: 2rem;
  width: 100%;
  padding: 10px;
  background-color: white;
  border: 1px solid #888888;
  transition: border 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant input::placeholder {
  text-transform: uppercase;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant label {
  left: 5px;
  top: 0px;
  position: absolute;
  color: #888888;
  font-size: 1rem;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant select {
  display: none;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-multselect-caret {
  position: absolute;
  font-size: 1rem;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-check {
  position: absolute;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant.has_icon {
  padding-right: 45px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant.has_icon .phanterpwa-widget-multselect-touchpad {
  width: calc(100% - 40px);
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant.has_icon .phanterpwa-widget-icon-wrapper {
  width: 40px;
  height: 40px;
  position: absolute;
  right: 0;
  background-color: #4e6eaa;
  text-align: center;
  line-height: 40px;
  border-radius: 29px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant.has_icon .phanterpwa-widget-multselect-caret {
  right: 52px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant.has_icon .phanterpwa-widget-check {
  right: 52px;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant.has_value label {
  color: #4e6eaa;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant.focus input {
  border: 1px solid #4e6eaa;
  transition: box-shadow 0.3s ease, border, 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant.focus label {
  color: #4e6eaa;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant.pre_focus input {
  border: 1px solid #4e6eaa;
  transition: box-shadow 0.3s ease, border, 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-multselect-wrapper.phanterpwa-widget-wear-elegant.pre_focus label {
  color: #4e6eaa;
  transition: color 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-wrapper .phanterpwa-widget-multselect-values {
  min-height: 29px;
  overflow: auto;
  padding-right: 40px;
}

.phanterpwa-widget .phanterpwa-widget-wrapper .phanterpwa-widget-multselect-values:focus {
  outline: none;
}

.phanterpwa-widget .phanterpwa-widget-wrapper .phanterpwa-widget-multselect-values .phanterpwa-widget-multselect-value-content {
  float: left;
  padding: 0px 22px 0px 10px;
  position: relative;
  line-height: 20px;
  text-align: center;
  margin: 5px 5px 2px 5px;
  background-color: #4e6eaa;
  border-radius: 20px;
  color: white;
  font-size: 12pt;
}

.phanterpwa-widget .phanterpwa-widget-wrapper .phanterpwa-widget-multselect-values .phanterpwa-widget-multselect-value-content:focus {
  outline: none;
  color: orange;
}

.phanterpwa-widget .phanterpwa-widget-wrapper .phanterpwa-widget-multselect-values .phanterpwa-widget-multselect-value-content:focus .phanterpwa-widget-multselect-value-icon_close {
  color: orange;
}

.phanterpwa-widget .phanterpwa-widget-wrapper .phanterpwa-widget-multselect-values .phanterpwa-widget-multselect-value-content .phanterpwa-widget-multselect-value {
  padding-bottom: 2px;
}

.phanterpwa-widget .phanterpwa-widget-wrapper .phanterpwa-widget-multselect-values .phanterpwa-widget-multselect-value-content .phanterpwa-widget-multselect-value-icon_close {
  position: absolute;
  top: 1px;
  right: 0px;
  width: 20px;
  color: white;
  cursor: pointer;
  height: 20px;
  line-height: 20px;
  font-size: 9pt;
}

.phanterpwa-widget .phanterpwa-widget-wrapper .phanterpwa-widget-multselect-values .phanterpwa-widget-multselect-value-content .phanterpwa-widget-multselect-value-icon_close:hover {
  color: orange;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-wear-shadows .phanterpwa-widget-multselect-values {
  border-radius: 5px;
  line-height: 1.5rem;
  min-height: 36px;
  width: 100%;
  padding: 5px 37px 5px 5px;
  background-color: white;
  border: 1px solid #888888;
  transition: box-shadow 0.3s ease, border, 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-wear-shadows.focus .phanterpwa-widget-multselect-values {
  box-shadow: 0px 0px 5px #4e6eaa;
  border: 1px solid #4e6eaa;
  transition: box-shadow 0.3s ease, border, 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-wear-shadows.pre_focus .phanterpwa-widget-multselect-values {
  box-shadow: 0px 0px 5px #4e6eaa;
  border: 1px solid #4e6eaa;
  transition: box-shadow 0.3s ease, border, 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-multselect-values {
  border-radius: 5px;
  line-height: 2rem;
  min-height: 54px;
  width: 100%;
  padding: 10px 17px 10px 10px;
  background-color: white;
  border: 1px solid #888888;
  transition: border 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-wear-elegant .phanterpwa-widget-multselect-values::placeholder {
  text-transform: uppercase;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-wear-elegant.focus .phanterpwa-widget-multselect-values {
  border: 1px solid #4e6eaa;
  transition: box-shadow 0.3s ease, border, 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-wrapper.phanterpwa-widget-wear-elegant.pre_focus .phanterpwa-widget-multselect-values {
  border: 1px solid #4e6eaa;
  transition: box-shadow 0.3s ease, border, 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper .phanterpwa-widget-list_string-list_values {
  width: 100%;
  height: 32px;
  overflow: hidden;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper .phanterpwa-widget-list_string-list_values:focus {
  outline: none;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper .phanterpwa-widget-list_string-list_values .phanterpwa-widget-list_string-value-content {
  float: left;
  padding: 0px 40px 0px 15px;
  position: relative;
  line-height: 40px;
  text-align: center;
  margin: 5px;
  background-color: #4e6eaa;
  border-radius: 20px;
  color: white;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper .phanterpwa-widget-list_string-list_values .phanterpwa-widget-list_string-value-content:focus {
  outline: none;
  color: orange;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper .phanterpwa-widget-list_string-list_values .phanterpwa-widget-list_string-value-content:focus .phanterpwa-widget-list_string-value-icon_close {
  color: orange;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper .phanterpwa-widget-list_string-list_values .phanterpwa-widget-list_string-value-content .phanterpwa-widget-list_string-value-icon_close {
  position: absolute;
  top: 0px;
  right: 0px;
  width: 40px;
  color: white;
  cursor: pointer;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper .phanterpwa-widget-list_string-list_values .phanterpwa-widget-list_string-value-content .phanterpwa-widget-list_string-value-icon_close:hover {
  color: orange;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper .phanterpwa-widget-list_string-list_values .phanterpwa-widget-list_string-plus_icon-container {
  opacity: 0;
  float: left;
  padding: 0px 40px 0px 0px;
  position: relative;
  line-height: 40px;
  margin: 5px;
  height: 40px;
  width: 40px;
  background-color: #D3D3D3;
  color: #4e6eaa;
  border-radius: 20px;
  transition: width 0.3s ease, opacity 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper .phanterpwa-widget-list_string-list_values .phanterpwa-widget-list_string-plus_icon-container.has_input {
  padding: 0px 40px 0px 15px;
  min-width: 100px;
  width: 200px;
  transition: width 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper .phanterpwa-widget-list_string-list_values .phanterpwa-widget-list_string-plus_icon-container .icon_button {
  position: absolute;
  position: absolute;
  top: 0px;
  right: 0px;
  width: 40px;
  cursor: pointer;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper .phanterpwa-widget-list_string-list_values .phanterpwa-widget-list_string-plus_icon-container .icon_button:hover {
  color: orange;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper .phanterpwa-widget-list_string-list_values .phanterpwa-widget-list_string-plus_icon-container:focus {
  outline: none;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper .phanterpwa-widget-list_string-list_values .phanterpwa-widget-list_string-plus_icon-container:focus .icon_button {
  color: orange;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper .phanterpwa-widget-list_string-list_predefinitions_values {
  display: table;
  width: 100%;
  background-color: #E0E0E0;
  border-radius: 0px 0px 10px 10px;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper .phanterpwa-widget-list_string-list_predefinitions_values .phanterpwa-widget-list_string-value-predefinition-content {
  float: left;
  padding: 0px 40px 0px 15px;
  position: relative;
  line-height: 40px;
  text-align: center;
  margin: 5px;
  background-color: #C0C0C0;
  border-radius: 20px;
  color: #4e6eaa;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper .phanterpwa-widget-list_string-list_predefinitions_values .phanterpwa-widget-list_string-value-predefinition-content:focus {
  outline: none;
  color: orange;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper .phanterpwa-widget-list_string-list_predefinitions_values .phanterpwa-widget-list_string-value-predefinition-content .phanterpwa-widget-list_string-value-icon_plus_predifinition {
  position: absolute;
  top: 0px;
  right: 0px;
  width: 40px;
  color: #4e6eaa;
  cursor: pointer;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper .phanterpwa-widget-list_string-list_predefinitions_values .phanterpwa-widget-list_string-value-predefinition-content .phanterpwa-widget-list_string-value-icon_plus_predifinition:hover {
  color: orange;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper.focus .phanterpwa-widget-list_string-list_values {
  min-height: 50px;
  height: auto;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper.focus .phanterpwa-widget-list_string-plus_icon-container {
  opacity: 1;
  transition: width 0.3s ease, opacity 0.3s ease;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper.has_value .phanterpwa-widget-list_string-list_values {
  height: auto;
}

.phanterpwa-widget .phanterpwa-widget-list_string-wrapper.has_value .phanterpwa-widget-list_string-plus_icon-container {
  opacity: 1;
  transition: width 0.3s ease, opacity 0.3s ease;
}

table.phanterpwa-widget-select-options-wrapper {
  width: 100%;
  padding: 0;
  margin: 0;
  border: none;
  border-collapse: collapse;
  border-spacing: 0;
}

table.phanterpwa-widget-select-options-wrapper td {
  position: relative;
  padding-right: 50px;
  height: 3rem;
  padding-left: 20px;
}

table.phanterpwa-widget-select-options-wrapper td.phanterpwa-widget-select-li-title {
  text-align: center;
  background-color: #4e6eaa;
  color: white;
  height: 2rem;
}

table.phanterpwa-widget-select-options-wrapper td.phanterpwa-widget-select-li-title .phanterpwa-widget-select-placeholder {
  opacity: 1;
  transition: opacity 0.5s ease;
}

table.phanterpwa-widget-select-options-wrapper td.phanterpwa-widget-select-li-title.has_editable {
  height: 3rem;
}

table.phanterpwa-widget-select-options-wrapper td.phanterpwa-widget-select-li-title.editable_enabled .phanterpwa-widget-select-placeholder {
  opacity: 0;
  transition: opacity 0.5s ease;
}

table.phanterpwa-widget-select-options-wrapper td.phanterpwa-widget-select-li-option {
  cursor: pointer;
  border-top: 1px solid #E0E0E0;
}

table.phanterpwa-widget-select-options-wrapper td.phanterpwa-widget-select-li-option.selected {
  color: #4e6eaa;
}

table.phanterpwa-widget-select-options-wrapper td.phanterpwa-widget-select-li-option:hover {
  color: orange !important;
}

table.phanterpwa-widget-select-options-wrapper td.phanterpwa-widget-select-li-option.empty span {
  color: #E8E8E8;
}

table.phanterpwa-widget-select-options-wrapper td::marker {
  content: "";
}

table.phanterpwa-widget-select-options-wrapper td .phanterpwa-widget-select-li-icon {
  position: absolute;
  right: 16px;
  top: 0;
  line-height: 3rem;
}

table.phanterpwa-widget-select-options-wrapper td .phanterpwa-widget-select-li-icon.phanterpwa-widget-select-li-icon_plus {
  line-height: unset;
}

table.phanterpwa-widget-select-options-wrapper td .phanterpwa-widget-select-li-icon_plus-wrapper {
  position: absolute;
  width: 100%;
  overflow: hidden;
  transition: width 0.5s ease;
  top: 0;
  right: 0;
}

table.phanterpwa-widget-select-options-wrapper td .phanterpwa-widget-select-li-icon_plus-wrapper .phanterpwa-widget-select-li-input-editable-wrapper {
  position: relative;
  padding-left: 35px;
  transition: width 0.5s ease;
  width: 0px;
  margin-right: 0;
  margin-left: auto;
}

table.phanterpwa-widget-select-options-wrapper td .phanterpwa-widget-select-li-icon_plus-wrapper .phanterpwa-widget-select-li-input-editable-wrapper i {
  line-height: 3rem;
}

table.phanterpwa-widget-select-options-wrapper td .phanterpwa-widget-select-li-icon_plus-wrapper .phanterpwa-widget-select-li-input-editable-wrapper .phanterpwa-widget-select-li-icon_plus {
  left: -5px;
  position: absolute;
  line-height: 3rem;
  right: auto;
  width: 35px;
  transform: rotate(0deg);
  transition: all 0.5s ease;
}

table.phanterpwa-widget-select-options-wrapper td .phanterpwa-widget-select-li-icon_plus-wrapper .phanterpwa-widget-select-li-input-editable-wrapper .phanterpwa-widget-select-li-input-editable {
  width: 100%;
  margin-right: 0;
  margin-left: auto;
  position: relative;
  transition: width 0.5s ease;
  overflow: hidden;
}

table.phanterpwa-widget-select-options-wrapper td .phanterpwa-widget-select-li-icon_plus-wrapper .phanterpwa-widget-select-li-input-editable-wrapper .phanterpwa-widget-select-li-input-editable input {
  border: none;
  border-bottom: 1px solid #4e6eaa;
  width: 100%;
  line-height: 3rem;
  height: 3rem;
}

table.phanterpwa-widget-select-options-wrapper td .phanterpwa-widget-select-li-icon_plus-wrapper .phanterpwa-widget-select-li-input-editable-wrapper .phanterpwa-widget-select-li-input-editable .phanterpwa-widget-select-li-icon_confirm {
  color: white;
  line-height: 3rem;
  position: absolute;
  top: 0;
  right: 5px;
  width: 35px;
  transition: color 0.5s ease;
}

table.phanterpwa-widget-select-options-wrapper td .phanterpwa-widget-select-li-icon_plus-wrapper.enabled .phanterpwa-widget-select-li-input-editable-wrapper {
  width: 100%;
}

table.phanterpwa-widget-select-options-wrapper td .phanterpwa-widget-select-li-icon_plus-wrapper.enabled .phanterpwa-widget-select-li-input-editable-wrapper .phanterpwa-widget-select-li-icon_plus {
  left: 0;
  color: red;
  transform: rotate(45deg);
  transition: all 0.5s ease;
}

table.phanterpwa-widget-select-options-wrapper td .phanterpwa-widget-select-li-icon_plus-wrapper.enabled .phanterpwa-widget-select-li-input-editable-wrapper .phanterpwa-widget-select-li-icon_confirm {
  color: #62d062;
  position: absolute;
  top: 0;
  right: 5px;
  width: 35px;
  transition: color 0.5s ease;
}

table.phanterpwa-widget-multselect-options-wrapper {
  width: 100%;
  padding: 0;
  margin: 0;
  border: none;
  border-collapse: collapse;
  border-spacing: 0;
}

table.phanterpwa-widget-multselect-options-wrapper td {
  position: relative;
  padding-right: 50px;
  height: 3rem;
  padding-left: 20px;
}

table.phanterpwa-widget-multselect-options-wrapper td.phanterpwa-widget-multselect-li-title {
  text-align: center;
  background-color: #4e6eaa;
  color: white;
  height: 2rem;
}

table.phanterpwa-widget-multselect-options-wrapper td.phanterpwa-widget-multselect-li-title.has_editable {
  height: 3rem;
}

table.phanterpwa-widget-multselect-options-wrapper td.phanterpwa-widget-multselect-li-title.editable_enabled .phanterpwa-widget-multselect-placeholder {
  opacity: 0;
  transition: opacity 0.5s ease;
}

table.phanterpwa-widget-multselect-options-wrapper td.phanterpwa-widget-multselect-li-option {
  cursor: pointer;
  border-top: 1px solid #E0E0E0;
}

table.phanterpwa-widget-multselect-options-wrapper td.phanterpwa-widget-multselect-li-option.selected {
  color: #4e6eaa;
}

table.phanterpwa-widget-multselect-options-wrapper td.phanterpwa-widget-multselect-li-option:hover {
  color: orange !important;
}

table.phanterpwa-widget-multselect-options-wrapper td.phanterpwa-widget-multselect-li-option.empty span {
  color: #E8E8E8;
}

table.phanterpwa-widget-multselect-options-wrapper td::marker {
  content: "";
}

table.phanterpwa-widget-multselect-options-wrapper td .phanterpwa-widget-multselect-li-icon {
  position: absolute;
  right: 16px;
  top: 0;
  line-height: 3rem;
}

table.phanterpwa-widget-multselect-options-wrapper td .phanterpwa-widget-multselect-li-icon.phanterpwa-widget-multselect-li-icon_plus {
  line-height: unset;
}

table.phanterpwa-widget-multselect-options-wrapper td .phanterpwa-widget-multselect-li-icon_plus-wrapper {
  position: absolute;
  width: 100%;
  overflow: hidden;
  transition: width 0.5s ease;
  top: 0;
  right: 0;
}

table.phanterpwa-widget-multselect-options-wrapper td .phanterpwa-widget-multselect-li-icon_plus-wrapper .phanterpwa-widget-multselect-li-input-editable-wrapper {
  position: relative;
  padding-left: 35px;
  transition: width 0.5s ease;
  width: 0px;
  margin-right: 0;
  margin-left: auto;
}

table.phanterpwa-widget-multselect-options-wrapper td .phanterpwa-widget-multselect-li-icon_plus-wrapper .phanterpwa-widget-multselect-li-input-editable-wrapper i {
  line-height: 3rem;
}

table.phanterpwa-widget-multselect-options-wrapper td .phanterpwa-widget-multselect-li-icon_plus-wrapper .phanterpwa-widget-multselect-li-input-editable-wrapper .phanterpwa-widget-multselect-li-icon_plus {
  left: -5px;
  position: absolute;
  line-height: 3rem;
  right: auto;
  width: 35px;
  transform: rotate(0deg);
  transition: all 0.5s ease;
}

table.phanterpwa-widget-multselect-options-wrapper td .phanterpwa-widget-multselect-li-icon_plus-wrapper .phanterpwa-widget-multselect-li-input-editable-wrapper .phanterpwa-widget-multselect-li-input-editable {
  width: 100%;
  margin-right: 0;
  margin-left: auto;
  position: relative;
  transition: width 0.5s ease;
  overflow: hidden;
}

table.phanterpwa-widget-multselect-options-wrapper td .phanterpwa-widget-multselect-li-icon_plus-wrapper .phanterpwa-widget-multselect-li-input-editable-wrapper .phanterpwa-widget-multselect-li-input-editable input {
  border: none;
  border-bottom: 1px solid #4e6eaa;
  width: 100%;
  line-height: 3rem;
  height: 3rem;
}

table.phanterpwa-widget-multselect-options-wrapper td .phanterpwa-widget-multselect-li-icon_plus-wrapper .phanterpwa-widget-multselect-li-input-editable-wrapper .phanterpwa-widget-multselect-li-input-editable .phanterpwa-widget-multselect-li-icon_confirm {
  color: white;
  position: absolute;
  top: 0;
  right: 5px;
  width: 35px;
  transition: color 0.5s ease;
}

table.phanterpwa-widget-multselect-options-wrapper td .phanterpwa-widget-multselect-li-icon_plus-wrapper.enabled .phanterpwa-widget-multselect-li-input-editable-wrapper {
  width: 100%;
}

table.phanterpwa-widget-multselect-options-wrapper td .phanterpwa-widget-multselect-li-icon_plus-wrapper.enabled .phanterpwa-widget-multselect-li-input-editable-wrapper .phanterpwa-widget-multselect-li-icon_plus {
  left: 0;
  color: red;
  transform: rotate(45deg);
  transition: all 0.5s ease;
}

table.phanterpwa-widget-multselect-options-wrapper td .phanterpwa-widget-multselect-li-icon_plus-wrapper.enabled .phanterpwa-widget-multselect-li-input-editable-wrapper .phanterpwa-widget-multselect-li-icon_confirm {
  color: #62d062;
  position: absolute;
  top: 0;
  right: 5px;
  width: 35px;
  transition: color 0.5s ease;
}

.phanterpwa-widget-menubox-options-content .phanterpwa-widget-menubox-option {
  min-width: 150px;
  height: 40px;
  line-height: 20px;
  width: 100%;
  white-space: nowrap;
  color: #4e6eaa;
  text-align: center;
  margin: 0;
  padding: 10px 20px;
  cursor: pointer;
}

.phanterpwa-widget-menubox-options-content .phanterpwa-widget-menubox-option:hover {
  background-color: #e0e0e0;
}

.phanterpwa-widget-menubox-options-content ul {
  margin: 0;
  padding: 0;
}

.phanterpwa-widget-menubox-options-content ul > li {
  color: #4e6eaa;
  text-align: center;
  margin: 0;
  padding: 10px 20px;
  cursor: pointer;
}

.phanterpwa-widget-menubox-options-content ul > li::marker {
  content: "";
}

.phanterpwa-widget-menubox-options-content ul > li:hover {
  background-color: #e0e0e0;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\components\xtable.sass */
.phanterpwa-widget.phanterpwa-widget-table-container {
  padding: 0;
  overflow: auto;
  width: 100%;
  border: solid 1px rgba(51, 51, 51, 0.12);
  border-radius: 10px;
}

.phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table {
  display: table;
  min-width: 100%;
  margin: 0;
  padding: 0;
  border-collapse: collapse;
  border-spacing: 0;
}

.phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table td, .phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table th, .phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table tr, .phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table table, .phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table thead, .phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table tbody, .phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table tfoot {
  margin: 0;
  padding: 0;
  background: none;
  border-collapse: collapse;
  border-spacing: 0;
}

.phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table tr[draggable="true"] {
  cursor: grab;
}

.phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table tr[draggable="true"]:hover {
  background: #f0f0f0;
}

.phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table tr[draggable="true"]:active {
  cursor: grabbing;
}

.phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table tr.phanterpwa-widget-table-head, .phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table tr.phanterpwa-widget-table-data, .phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table tr.phanterpwa-widget-table-pagination {
  display: table-row;
  position: static;
}

.phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table td, .phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table th {
  text-align: left;
  padding: 5px 10px;
  height: 40px;
}

.phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table td:first-child, .phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table th:first-child {
  padding-left: 20px;
}

.phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table td:last-child, .phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table th:last-child {
  padding-right: 20px;
}

.phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table td {
  border-top: solid 1px rgba(51, 51, 51, 0.12);
}

.phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table td.phanterpwa-widget-table-head-td {
  display: table-cell;
}

.phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table th.phanterpwa-widget-table-head-th {
  display: table-cell;
}

.phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table th.phanterpwa-widget-table-head-th .icon_button {
  color: #4e6eaa;
}

.phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table th.phanterpwa-widget-table-head-th .phanterpwa-widget-table-head-th-sortable {
  position: relative;
  display: table;
  width: max-content;
  cursor: pointer;
  color: #4e6eaa;
}

.phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table th.phanterpwa-widget-table-head-th.enabled {
  text-shadow: 0px 0px 5px #4e6eaa;
}

.phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table th.phanterpwa-widget-table-head-th.enabled.asc .phanterpwa-widget-table-head-th-sortable::after {
  content: "\f0d8";
  font-family: "Font Awesome 5 Free";
  font-weight: 900;
  position: absolute;
  width: 100%;
  bottom: -15px;
  left: 0;
  text-align: center;
}

.phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table th.phanterpwa-widget-table-head-th.enabled.desc .phanterpwa-widget-table-head-th-sortable::after {
  content: "\f0d7";
  font-family: "Font Awesome 5 Free";
  font-weight: 900;
  position: absolute;
  width: 100%;
  bottom: -15px;
  left: 0;
  text-align: center;
}

.phanterpwa-widget.phanterpwa-widget-table-container .phanterpwa-widget-table th.phanterpwa-widget-table-head-th .phanterpwa-widget-table-head-th-sortable::hover {
  color: orange;
}

.phanterpwa-widget-pagination-container {
  min-width: 210px;
  text-align: center;
  margin: auto;
  display: table;
  color: #4e6eaa;
}

.phanterpwa-widget-pagination-container .phanterpwa-widget-pagination-first_button, .phanterpwa-widget-pagination-container .phanterpwa-widget-pagination-previous_button, .phanterpwa-widget-pagination-container .phanterpwa-widget-pagination-next_button, .phanterpwa-widget-pagination-container .phanterpwa-widget-pagination-last_button {
  display: inline-block;
  width: 30px;
  height: 30px;
  line-height: 30px;
  text-align: center;
  cursor: pointer;
}

.phanterpwa-widget-pagination-container .phanterpwa-widget-pagination-first_button:hover, .phanterpwa-widget-pagination-container .phanterpwa-widget-pagination-previous_button:hover, .phanterpwa-widget-pagination-container .phanterpwa-widget-pagination-next_button:hover, .phanterpwa-widget-pagination-container .phanterpwa-widget-pagination-last_button:hover {
  color: orange;
}

.phanterpwa-widget-pagination-container .phanterpwa-widget-pagination-first_button.disabled, .phanterpwa-widget-pagination-container .phanterpwa-widget-pagination-previous_button.disabled, .phanterpwa-widget-pagination-container .phanterpwa-widget-pagination-next_button.disabled, .phanterpwa-widget-pagination-container .phanterpwa-widget-pagination-last_button.disabled {
  color: #D3D3D3;
}

.phanterpwa-widget-pagination-container .phanterpwa-widget-pagination-first_button.disabled:hover, .phanterpwa-widget-pagination-container .phanterpwa-widget-pagination-previous_button.disabled:hover, .phanterpwa-widget-pagination-container .phanterpwa-widget-pagination-next_button.disabled:hover, .phanterpwa-widget-pagination-container .phanterpwa-widget-pagination-last_button.disabled:hover {
  color: #D3D3D3;
}

.phanterpwa-widget-pagination-container .phanterpwa-widget-pagination-status_button {
  display: inline-block;
  line-height: 30px;
  height: 30px;
  padding: 0 5px;
  cursor: pointer;
}

.phanterpwa-widget-pagination-container .phanterpwa-widget-pagination-status_button:hover {
  color: orange;
}

.phanterpwa-widget-pagination-pages_panel {
  padding: 5px;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\display\grid.sass */
@media screen and (min-width: 0) and (max-width: 301px) {
  .hidden_on_width1, .show_on_width2, .show_on_width3, .show_on_width4, .show_on_width5, .show_on_width6, .show_on_width7, .show_on_width8 {
    display: none !important;
  }
  .p-hidden_on_width1, .p-show_on_width2, .p-show_on_width3, .p-show_on_width4, .p-show_on_width5, .p-show_on_width6, .p-show_on_width7, .p-show_on_width8 {
    display: none !important;
  }
  .phanterpwa-hidden_on_width1, .phanterpwa-show_on_width2, .phanterpwa-show_on_width3, .phanterpwa-show_on_width4, .phanterpwa-show_on_width5, .phanterpwa-show_on_width6, .phanterpwa-show_on_width7, .phanterpwa-show_on_width8 {
    display: none !important;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .show_on_width1, .hidden_on_width2, .show_on_width3, .show_on_width4, .show_on_width5, .show_on_width6, .show_on_width7, .show_on_width8 {
    display: none !important;
  }
  .p-show_on_width1, .p-hidden_on_width2, .p-show_on_width3, .p-show_on_width4, .p-show_on_width5, .p-show_on_width6, .p-show_on_width7, .p-show_on_width8 {
    display: none !important;
  }
  .phanterpwa-show_on_width1, .phanterpwa-hidden_on_width2, .phanterpwa-show_on_width3, .phanterpwa-show_on_width4, .phanterpwa-show_on_width5, .phanterpwa-show_on_width6, .phanterpwa-show_on_width7, .phanterpwa-show_on_width8 {
    display: none !important;
  }
}

@media screen and (min-width: 600px) and (max-width: 901px) {
  .show_on_width1, .show_on_width2, .hidden_on_width3, .show_on_width4, .show_on_width5, .show_on_width6, .show_on_width7, .show_on_width8 {
    display: none !important;
  }
  .p-show_on_width1, .p-show_on_width2, .p-hidden_on_width3, .p-show_on_width4, .p-show_on_width5, .p-show_on_width6, .p-show_on_width7, .p-show_on_width8 {
    display: none !important;
  }
  .phanterpwa-show_on_width1, .phanterpwa-show_on_width2, .phanterpwa-hidden_on_width3, .phanterpwa-show_on_width4, .phanterpwa-show_on_width5, .phanterpwa-show_on_width6, .phanterpwa-show_on_width7, .phanterpwa-show_on_width8 {
    display: none !important;
  }
}

@media screen and (min-width: 900px) and (max-width: 1201px) {
  .show_on_width1, .show_on_width2, .show_on_width3, .hidden_on_width4, .show_on_width5, .show_on_width6, .show_on_width7, .show_on_width8 {
    display: none !important;
  }
  .p-show_on_width1, .p-show_on_width2, .p-show_on_width3, .p-hidden_on_width4, .p-show_on_width5, .p-show_on_width6, .p-show_on_width7, .p-show_on_width8 {
    display: none !important;
  }
  .phanterpwa-show_on_width1, .phanterpwa-show_on_width2, .phanterpwa-show_on_width3, .phanterpwa-hidden_on_width4, .phanterpwa-show_on_width5, .phanterpwa-show_on_width6, .phanterpwa-show_on_width7, .phanterpwa-show_on_width8 {
    display: none !important;
  }
}

@media screen and (min-width: 1200px) and (max-width: 1501px) {
  .show_on_width1, .show_on_width2, .show_on_width3, .show_on_width4, .hidden_on_width5, .show_on_width6, .show_on_width7, .show_on_width8 {
    display: none !important;
  }
  .p-show_on_width1, .p-show_on_width2, .p-show_on_width3, .p-show_on_width4, .p-hidden_on_width5, .p-show_on_width6, .p-show_on_width7, .p-show_on_width8 {
    display: none !important;
  }
  .phanterpwa-show_on_width1, .phanterpwa-show_on_width2, .phanterpwa-show_on_width3, .phanterpwa-show_on_width4, .phanterpwa-hidden_on_width5, .phanterpwa-show_on_width6, .phanterpwa-show_on_width7, .phanterpwa-show_on_width8 {
    display: none !important;
  }
}

@media screen and (min-width: 1500px) and (max-width: 1801px) {
  .show_on_width1, .show_on_width2, .show_on_width3, .show_on_width4, .show_on_width5, .hidden_on_width6, .show_on_width7, .show_on_width8 {
    display: none !important;
  }
  .p-show_on_width1, .p-show_on_width2, .p-show_on_width3, .p-show_on_width4, .p-show_on_width5, .p-hidden_on_width6, .p-show_on_width7, .p-show_on_width8 {
    display: none !important;
  }
  .phanterpwa-show_on_width1, .phanterpwa-show_on_width2, .phanterpwa-show_on_width3, .phanterpwa-show_on_width4, .phanterpwa-show_on_width5, .phanterpwa-hidden_on_width6, .phanterpwa-show_on_width7, .phanterpwa-show_on_width8 {
    display: none !important;
  }
}

@media screen and (min-width: 1800px) and (max-width: 2100px) {
  .show_on_width1, .show_on_width2, .show_on_width3, .show_on_width4, .show_on_width5, .show_on_width6, .hidden_on_width7, .show_on_width8 {
    display: none !important;
  }
  .p-show_on_width1, .p-show_on_width2, .p-show_on_width3, .p-show_on_width4, .p-show_on_width5, .p-show_on_width6, .p-hidden_on_width7, .p-show_on_width8 {
    display: none !important;
  }
  .phanterpwa-show_on_width1, .phanterpwa-show_on_width2, .phanterpwa-show_on_width3, .phanterpwa-show_on_width4, .phanterpwa-show_on_width5, .phanterpwa-show_on_width6, .phanterpwa-hidden_on_width7, .phanterpwa-show_on_width8 {
    display: none !important;
  }
}

@media screen and (min-width: 2100px) {
  .show_on_width1, .show_on_width2, .show_on_width3, .show_on_width4, .show_on_width5, .show_on_width6, .show_on_width7, .hidden_on_width8 {
    display: none !important;
  }
  .p-show_on_width1, .p-show_on_width2, .p-show_on_width3, .p-show_on_width4, .p-show_on_width5, .p-show_on_width6, .p-show_on_width7, .p-hidden_on_width8 {
    display: none !important;
  }
  .phanterpwa-show_on_width1, .phanterpwa-show_on_width2, .phanterpwa-show_on_width3, .phanterpwa-show_on_width4, .phanterpwa-show_on_width5, .phanterpwa-show_on_width6, .phanterpwa-show_on_width7, .phanterpwa-hidden_on_width8 {
    display: none !important;
  }
}

.row, .p-row, .phanterpwa-row {
  margin: 0px;
  margin-left: auto;
  margin-right: auto;
  width: 100%;
}

.row::after, .p-row::after, .phanterpwa-row::after {
  content: "";
  display: block;
  clear: both;
}

.row .col, .row .p-col, .row .phanterpwa-col, .p-row .col, .p-row .p-col, .p-row .phanterpwa-col, .phanterpwa-row .col, .phanterpwa-row .p-col, .phanterpwa-row .phanterpwa-col {
  float: left;
  margin-left: auto;
  left: auto;
  right: auto;
  margin-right: auto;
  min-height: 1px;
}

.row .col.w1p1, .row .p-col.w1p1, .row .phanterpwa-col.w1p1, .p-row .col.w1p1, .p-row .p-col.w1p1, .p-row .phanterpwa-col.w1p1, .phanterpwa-row .col.w1p1, .phanterpwa-row .p-col.w1p1, .phanterpwa-row .phanterpwa-col.w1p1 {
  width: 1%;
}

.row .col.w1p2, .row .p-col.w1p2, .row .phanterpwa-col.w1p2, .p-row .col.w1p2, .p-row .p-col.w1p2, .p-row .phanterpwa-col.w1p2, .phanterpwa-row .col.w1p2, .phanterpwa-row .p-col.w1p2, .phanterpwa-row .phanterpwa-col.w1p2 {
  width: 2%;
}

.row .col.w1p3, .row .p-col.w1p3, .row .phanterpwa-col.w1p3, .p-row .col.w1p3, .p-row .p-col.w1p3, .p-row .phanterpwa-col.w1p3, .phanterpwa-row .col.w1p3, .phanterpwa-row .p-col.w1p3, .phanterpwa-row .phanterpwa-col.w1p3 {
  width: 3%;
}

.row .col.w1p4, .row .p-col.w1p4, .row .phanterpwa-col.w1p4, .p-row .col.w1p4, .p-row .p-col.w1p4, .p-row .phanterpwa-col.w1p4, .phanterpwa-row .col.w1p4, .phanterpwa-row .p-col.w1p4, .phanterpwa-row .phanterpwa-col.w1p4 {
  width: 4%;
}

.row .col.w1p5, .row .p-col.w1p5, .row .phanterpwa-col.w1p5, .p-row .col.w1p5, .p-row .p-col.w1p5, .p-row .phanterpwa-col.w1p5, .phanterpwa-row .col.w1p5, .phanterpwa-row .p-col.w1p5, .phanterpwa-row .phanterpwa-col.w1p5 {
  width: 5%;
}

.row .col.w1p6, .row .p-col.w1p6, .row .phanterpwa-col.w1p6, .p-row .col.w1p6, .p-row .p-col.w1p6, .p-row .phanterpwa-col.w1p6, .phanterpwa-row .col.w1p6, .phanterpwa-row .p-col.w1p6, .phanterpwa-row .phanterpwa-col.w1p6 {
  width: 6%;
}

.row .col.w1p7, .row .p-col.w1p7, .row .phanterpwa-col.w1p7, .p-row .col.w1p7, .p-row .p-col.w1p7, .p-row .phanterpwa-col.w1p7, .phanterpwa-row .col.w1p7, .phanterpwa-row .p-col.w1p7, .phanterpwa-row .phanterpwa-col.w1p7 {
  width: 7%;
}

.row .col.w1p8, .row .p-col.w1p8, .row .phanterpwa-col.w1p8, .p-row .col.w1p8, .p-row .p-col.w1p8, .p-row .phanterpwa-col.w1p8, .phanterpwa-row .col.w1p8, .phanterpwa-row .p-col.w1p8, .phanterpwa-row .phanterpwa-col.w1p8 {
  width: 8%;
}

.row .col.w1p9, .row .p-col.w1p9, .row .phanterpwa-col.w1p9, .p-row .col.w1p9, .p-row .p-col.w1p9, .p-row .phanterpwa-col.w1p9, .phanterpwa-row .col.w1p9, .phanterpwa-row .p-col.w1p9, .phanterpwa-row .phanterpwa-col.w1p9 {
  width: 9%;
}

.row .col.w1p10, .row .p-col.w1p10, .row .phanterpwa-col.w1p10, .p-row .col.w1p10, .p-row .p-col.w1p10, .p-row .phanterpwa-col.w1p10, .phanterpwa-row .col.w1p10, .phanterpwa-row .p-col.w1p10, .phanterpwa-row .phanterpwa-col.w1p10 {
  width: 10%;
}

.row .col.w1p11, .row .p-col.w1p11, .row .phanterpwa-col.w1p11, .p-row .col.w1p11, .p-row .p-col.w1p11, .p-row .phanterpwa-col.w1p11, .phanterpwa-row .col.w1p11, .phanterpwa-row .p-col.w1p11, .phanterpwa-row .phanterpwa-col.w1p11 {
  width: 11%;
}

.row .col.w1p12, .row .p-col.w1p12, .row .phanterpwa-col.w1p12, .p-row .col.w1p12, .p-row .p-col.w1p12, .p-row .phanterpwa-col.w1p12, .phanterpwa-row .col.w1p12, .phanterpwa-row .p-col.w1p12, .phanterpwa-row .phanterpwa-col.w1p12 {
  width: 12%;
}

.row .col.w1p13, .row .p-col.w1p13, .row .phanterpwa-col.w1p13, .p-row .col.w1p13, .p-row .p-col.w1p13, .p-row .phanterpwa-col.w1p13, .phanterpwa-row .col.w1p13, .phanterpwa-row .p-col.w1p13, .phanterpwa-row .phanterpwa-col.w1p13 {
  width: 13%;
}

.row .col.w1p14, .row .p-col.w1p14, .row .phanterpwa-col.w1p14, .p-row .col.w1p14, .p-row .p-col.w1p14, .p-row .phanterpwa-col.w1p14, .phanterpwa-row .col.w1p14, .phanterpwa-row .p-col.w1p14, .phanterpwa-row .phanterpwa-col.w1p14 {
  width: 14%;
}

.row .col.w1p15, .row .p-col.w1p15, .row .phanterpwa-col.w1p15, .p-row .col.w1p15, .p-row .p-col.w1p15, .p-row .phanterpwa-col.w1p15, .phanterpwa-row .col.w1p15, .phanterpwa-row .p-col.w1p15, .phanterpwa-row .phanterpwa-col.w1p15 {
  width: 15%;
}

.row .col.w1p16, .row .p-col.w1p16, .row .phanterpwa-col.w1p16, .p-row .col.w1p16, .p-row .p-col.w1p16, .p-row .phanterpwa-col.w1p16, .phanterpwa-row .col.w1p16, .phanterpwa-row .p-col.w1p16, .phanterpwa-row .phanterpwa-col.w1p16 {
  width: 16%;
}

.row .col.w1p17, .row .p-col.w1p17, .row .phanterpwa-col.w1p17, .p-row .col.w1p17, .p-row .p-col.w1p17, .p-row .phanterpwa-col.w1p17, .phanterpwa-row .col.w1p17, .phanterpwa-row .p-col.w1p17, .phanterpwa-row .phanterpwa-col.w1p17 {
  width: 17%;
}

.row .col.w1p18, .row .p-col.w1p18, .row .phanterpwa-col.w1p18, .p-row .col.w1p18, .p-row .p-col.w1p18, .p-row .phanterpwa-col.w1p18, .phanterpwa-row .col.w1p18, .phanterpwa-row .p-col.w1p18, .phanterpwa-row .phanterpwa-col.w1p18 {
  width: 18%;
}

.row .col.w1p19, .row .p-col.w1p19, .row .phanterpwa-col.w1p19, .p-row .col.w1p19, .p-row .p-col.w1p19, .p-row .phanterpwa-col.w1p19, .phanterpwa-row .col.w1p19, .phanterpwa-row .p-col.w1p19, .phanterpwa-row .phanterpwa-col.w1p19 {
  width: 19%;
}

.row .col.w1p20, .row .p-col.w1p20, .row .phanterpwa-col.w1p20, .p-row .col.w1p20, .p-row .p-col.w1p20, .p-row .phanterpwa-col.w1p20, .phanterpwa-row .col.w1p20, .phanterpwa-row .p-col.w1p20, .phanterpwa-row .phanterpwa-col.w1p20 {
  width: 20%;
}

.row .col.w1p21, .row .p-col.w1p21, .row .phanterpwa-col.w1p21, .p-row .col.w1p21, .p-row .p-col.w1p21, .p-row .phanterpwa-col.w1p21, .phanterpwa-row .col.w1p21, .phanterpwa-row .p-col.w1p21, .phanterpwa-row .phanterpwa-col.w1p21 {
  width: 21%;
}

.row .col.w1p22, .row .p-col.w1p22, .row .phanterpwa-col.w1p22, .p-row .col.w1p22, .p-row .p-col.w1p22, .p-row .phanterpwa-col.w1p22, .phanterpwa-row .col.w1p22, .phanterpwa-row .p-col.w1p22, .phanterpwa-row .phanterpwa-col.w1p22 {
  width: 22%;
}

.row .col.w1p23, .row .p-col.w1p23, .row .phanterpwa-col.w1p23, .p-row .col.w1p23, .p-row .p-col.w1p23, .p-row .phanterpwa-col.w1p23, .phanterpwa-row .col.w1p23, .phanterpwa-row .p-col.w1p23, .phanterpwa-row .phanterpwa-col.w1p23 {
  width: 23%;
}

.row .col.w1p24, .row .p-col.w1p24, .row .phanterpwa-col.w1p24, .p-row .col.w1p24, .p-row .p-col.w1p24, .p-row .phanterpwa-col.w1p24, .phanterpwa-row .col.w1p24, .phanterpwa-row .p-col.w1p24, .phanterpwa-row .phanterpwa-col.w1p24 {
  width: 24%;
}

.row .col.w1p25, .row .p-col.w1p25, .row .phanterpwa-col.w1p25, .p-row .col.w1p25, .p-row .p-col.w1p25, .p-row .phanterpwa-col.w1p25, .phanterpwa-row .col.w1p25, .phanterpwa-row .p-col.w1p25, .phanterpwa-row .phanterpwa-col.w1p25 {
  width: 25%;
}

.row .col.w1p26, .row .p-col.w1p26, .row .phanterpwa-col.w1p26, .p-row .col.w1p26, .p-row .p-col.w1p26, .p-row .phanterpwa-col.w1p26, .phanterpwa-row .col.w1p26, .phanterpwa-row .p-col.w1p26, .phanterpwa-row .phanterpwa-col.w1p26 {
  width: 26%;
}

.row .col.w1p27, .row .p-col.w1p27, .row .phanterpwa-col.w1p27, .p-row .col.w1p27, .p-row .p-col.w1p27, .p-row .phanterpwa-col.w1p27, .phanterpwa-row .col.w1p27, .phanterpwa-row .p-col.w1p27, .phanterpwa-row .phanterpwa-col.w1p27 {
  width: 27%;
}

.row .col.w1p28, .row .p-col.w1p28, .row .phanterpwa-col.w1p28, .p-row .col.w1p28, .p-row .p-col.w1p28, .p-row .phanterpwa-col.w1p28, .phanterpwa-row .col.w1p28, .phanterpwa-row .p-col.w1p28, .phanterpwa-row .phanterpwa-col.w1p28 {
  width: 28%;
}

.row .col.w1p29, .row .p-col.w1p29, .row .phanterpwa-col.w1p29, .p-row .col.w1p29, .p-row .p-col.w1p29, .p-row .phanterpwa-col.w1p29, .phanterpwa-row .col.w1p29, .phanterpwa-row .p-col.w1p29, .phanterpwa-row .phanterpwa-col.w1p29 {
  width: 29%;
}

.row .col.w1p30, .row .p-col.w1p30, .row .phanterpwa-col.w1p30, .p-row .col.w1p30, .p-row .p-col.w1p30, .p-row .phanterpwa-col.w1p30, .phanterpwa-row .col.w1p30, .phanterpwa-row .p-col.w1p30, .phanterpwa-row .phanterpwa-col.w1p30 {
  width: 30%;
}

.row .col.w1p31, .row .p-col.w1p31, .row .phanterpwa-col.w1p31, .p-row .col.w1p31, .p-row .p-col.w1p31, .p-row .phanterpwa-col.w1p31, .phanterpwa-row .col.w1p31, .phanterpwa-row .p-col.w1p31, .phanterpwa-row .phanterpwa-col.w1p31 {
  width: 31%;
}

.row .col.w1p32, .row .p-col.w1p32, .row .phanterpwa-col.w1p32, .p-row .col.w1p32, .p-row .p-col.w1p32, .p-row .phanterpwa-col.w1p32, .phanterpwa-row .col.w1p32, .phanterpwa-row .p-col.w1p32, .phanterpwa-row .phanterpwa-col.w1p32 {
  width: 32%;
}

.row .col.w1p33, .row .p-col.w1p33, .row .phanterpwa-col.w1p33, .p-row .col.w1p33, .p-row .p-col.w1p33, .p-row .phanterpwa-col.w1p33, .phanterpwa-row .col.w1p33, .phanterpwa-row .p-col.w1p33, .phanterpwa-row .phanterpwa-col.w1p33 {
  width: 33%;
}

.row .col.w1p34, .row .p-col.w1p34, .row .phanterpwa-col.w1p34, .p-row .col.w1p34, .p-row .p-col.w1p34, .p-row .phanterpwa-col.w1p34, .phanterpwa-row .col.w1p34, .phanterpwa-row .p-col.w1p34, .phanterpwa-row .phanterpwa-col.w1p34 {
  width: 34%;
}

.row .col.w1p35, .row .p-col.w1p35, .row .phanterpwa-col.w1p35, .p-row .col.w1p35, .p-row .p-col.w1p35, .p-row .phanterpwa-col.w1p35, .phanterpwa-row .col.w1p35, .phanterpwa-row .p-col.w1p35, .phanterpwa-row .phanterpwa-col.w1p35 {
  width: 35%;
}

.row .col.w1p36, .row .p-col.w1p36, .row .phanterpwa-col.w1p36, .p-row .col.w1p36, .p-row .p-col.w1p36, .p-row .phanterpwa-col.w1p36, .phanterpwa-row .col.w1p36, .phanterpwa-row .p-col.w1p36, .phanterpwa-row .phanterpwa-col.w1p36 {
  width: 36%;
}

.row .col.w1p37, .row .p-col.w1p37, .row .phanterpwa-col.w1p37, .p-row .col.w1p37, .p-row .p-col.w1p37, .p-row .phanterpwa-col.w1p37, .phanterpwa-row .col.w1p37, .phanterpwa-row .p-col.w1p37, .phanterpwa-row .phanterpwa-col.w1p37 {
  width: 37%;
}

.row .col.w1p38, .row .p-col.w1p38, .row .phanterpwa-col.w1p38, .p-row .col.w1p38, .p-row .p-col.w1p38, .p-row .phanterpwa-col.w1p38, .phanterpwa-row .col.w1p38, .phanterpwa-row .p-col.w1p38, .phanterpwa-row .phanterpwa-col.w1p38 {
  width: 38%;
}

.row .col.w1p39, .row .p-col.w1p39, .row .phanterpwa-col.w1p39, .p-row .col.w1p39, .p-row .p-col.w1p39, .p-row .phanterpwa-col.w1p39, .phanterpwa-row .col.w1p39, .phanterpwa-row .p-col.w1p39, .phanterpwa-row .phanterpwa-col.w1p39 {
  width: 39%;
}

.row .col.w1p40, .row .p-col.w1p40, .row .phanterpwa-col.w1p40, .p-row .col.w1p40, .p-row .p-col.w1p40, .p-row .phanterpwa-col.w1p40, .phanterpwa-row .col.w1p40, .phanterpwa-row .p-col.w1p40, .phanterpwa-row .phanterpwa-col.w1p40 {
  width: 40%;
}

.row .col.w1p41, .row .p-col.w1p41, .row .phanterpwa-col.w1p41, .p-row .col.w1p41, .p-row .p-col.w1p41, .p-row .phanterpwa-col.w1p41, .phanterpwa-row .col.w1p41, .phanterpwa-row .p-col.w1p41, .phanterpwa-row .phanterpwa-col.w1p41 {
  width: 41%;
}

.row .col.w1p42, .row .p-col.w1p42, .row .phanterpwa-col.w1p42, .p-row .col.w1p42, .p-row .p-col.w1p42, .p-row .phanterpwa-col.w1p42, .phanterpwa-row .col.w1p42, .phanterpwa-row .p-col.w1p42, .phanterpwa-row .phanterpwa-col.w1p42 {
  width: 42%;
}

.row .col.w1p43, .row .p-col.w1p43, .row .phanterpwa-col.w1p43, .p-row .col.w1p43, .p-row .p-col.w1p43, .p-row .phanterpwa-col.w1p43, .phanterpwa-row .col.w1p43, .phanterpwa-row .p-col.w1p43, .phanterpwa-row .phanterpwa-col.w1p43 {
  width: 43%;
}

.row .col.w1p44, .row .p-col.w1p44, .row .phanterpwa-col.w1p44, .p-row .col.w1p44, .p-row .p-col.w1p44, .p-row .phanterpwa-col.w1p44, .phanterpwa-row .col.w1p44, .phanterpwa-row .p-col.w1p44, .phanterpwa-row .phanterpwa-col.w1p44 {
  width: 44%;
}

.row .col.w1p45, .row .p-col.w1p45, .row .phanterpwa-col.w1p45, .p-row .col.w1p45, .p-row .p-col.w1p45, .p-row .phanterpwa-col.w1p45, .phanterpwa-row .col.w1p45, .phanterpwa-row .p-col.w1p45, .phanterpwa-row .phanterpwa-col.w1p45 {
  width: 45%;
}

.row .col.w1p46, .row .p-col.w1p46, .row .phanterpwa-col.w1p46, .p-row .col.w1p46, .p-row .p-col.w1p46, .p-row .phanterpwa-col.w1p46, .phanterpwa-row .col.w1p46, .phanterpwa-row .p-col.w1p46, .phanterpwa-row .phanterpwa-col.w1p46 {
  width: 46%;
}

.row .col.w1p47, .row .p-col.w1p47, .row .phanterpwa-col.w1p47, .p-row .col.w1p47, .p-row .p-col.w1p47, .p-row .phanterpwa-col.w1p47, .phanterpwa-row .col.w1p47, .phanterpwa-row .p-col.w1p47, .phanterpwa-row .phanterpwa-col.w1p47 {
  width: 47%;
}

.row .col.w1p48, .row .p-col.w1p48, .row .phanterpwa-col.w1p48, .p-row .col.w1p48, .p-row .p-col.w1p48, .p-row .phanterpwa-col.w1p48, .phanterpwa-row .col.w1p48, .phanterpwa-row .p-col.w1p48, .phanterpwa-row .phanterpwa-col.w1p48 {
  width: 48%;
}

.row .col.w1p49, .row .p-col.w1p49, .row .phanterpwa-col.w1p49, .p-row .col.w1p49, .p-row .p-col.w1p49, .p-row .phanterpwa-col.w1p49, .phanterpwa-row .col.w1p49, .phanterpwa-row .p-col.w1p49, .phanterpwa-row .phanterpwa-col.w1p49 {
  width: 49%;
}

.row .col.w1p50, .row .p-col.w1p50, .row .phanterpwa-col.w1p50, .p-row .col.w1p50, .p-row .p-col.w1p50, .p-row .phanterpwa-col.w1p50, .phanterpwa-row .col.w1p50, .phanterpwa-row .p-col.w1p50, .phanterpwa-row .phanterpwa-col.w1p50 {
  width: 50%;
}

.row .col.w1p51, .row .p-col.w1p51, .row .phanterpwa-col.w1p51, .p-row .col.w1p51, .p-row .p-col.w1p51, .p-row .phanterpwa-col.w1p51, .phanterpwa-row .col.w1p51, .phanterpwa-row .p-col.w1p51, .phanterpwa-row .phanterpwa-col.w1p51 {
  width: 51%;
}

.row .col.w1p52, .row .p-col.w1p52, .row .phanterpwa-col.w1p52, .p-row .col.w1p52, .p-row .p-col.w1p52, .p-row .phanterpwa-col.w1p52, .phanterpwa-row .col.w1p52, .phanterpwa-row .p-col.w1p52, .phanterpwa-row .phanterpwa-col.w1p52 {
  width: 52%;
}

.row .col.w1p53, .row .p-col.w1p53, .row .phanterpwa-col.w1p53, .p-row .col.w1p53, .p-row .p-col.w1p53, .p-row .phanterpwa-col.w1p53, .phanterpwa-row .col.w1p53, .phanterpwa-row .p-col.w1p53, .phanterpwa-row .phanterpwa-col.w1p53 {
  width: 53%;
}

.row .col.w1p54, .row .p-col.w1p54, .row .phanterpwa-col.w1p54, .p-row .col.w1p54, .p-row .p-col.w1p54, .p-row .phanterpwa-col.w1p54, .phanterpwa-row .col.w1p54, .phanterpwa-row .p-col.w1p54, .phanterpwa-row .phanterpwa-col.w1p54 {
  width: 54%;
}

.row .col.w1p55, .row .p-col.w1p55, .row .phanterpwa-col.w1p55, .p-row .col.w1p55, .p-row .p-col.w1p55, .p-row .phanterpwa-col.w1p55, .phanterpwa-row .col.w1p55, .phanterpwa-row .p-col.w1p55, .phanterpwa-row .phanterpwa-col.w1p55 {
  width: 55%;
}

.row .col.w1p56, .row .p-col.w1p56, .row .phanterpwa-col.w1p56, .p-row .col.w1p56, .p-row .p-col.w1p56, .p-row .phanterpwa-col.w1p56, .phanterpwa-row .col.w1p56, .phanterpwa-row .p-col.w1p56, .phanterpwa-row .phanterpwa-col.w1p56 {
  width: 56%;
}

.row .col.w1p57, .row .p-col.w1p57, .row .phanterpwa-col.w1p57, .p-row .col.w1p57, .p-row .p-col.w1p57, .p-row .phanterpwa-col.w1p57, .phanterpwa-row .col.w1p57, .phanterpwa-row .p-col.w1p57, .phanterpwa-row .phanterpwa-col.w1p57 {
  width: 57%;
}

.row .col.w1p58, .row .p-col.w1p58, .row .phanterpwa-col.w1p58, .p-row .col.w1p58, .p-row .p-col.w1p58, .p-row .phanterpwa-col.w1p58, .phanterpwa-row .col.w1p58, .phanterpwa-row .p-col.w1p58, .phanterpwa-row .phanterpwa-col.w1p58 {
  width: 58%;
}

.row .col.w1p59, .row .p-col.w1p59, .row .phanterpwa-col.w1p59, .p-row .col.w1p59, .p-row .p-col.w1p59, .p-row .phanterpwa-col.w1p59, .phanterpwa-row .col.w1p59, .phanterpwa-row .p-col.w1p59, .phanterpwa-row .phanterpwa-col.w1p59 {
  width: 59%;
}

.row .col.w1p60, .row .p-col.w1p60, .row .phanterpwa-col.w1p60, .p-row .col.w1p60, .p-row .p-col.w1p60, .p-row .phanterpwa-col.w1p60, .phanterpwa-row .col.w1p60, .phanterpwa-row .p-col.w1p60, .phanterpwa-row .phanterpwa-col.w1p60 {
  width: 60%;
}

.row .col.w1p61, .row .p-col.w1p61, .row .phanterpwa-col.w1p61, .p-row .col.w1p61, .p-row .p-col.w1p61, .p-row .phanterpwa-col.w1p61, .phanterpwa-row .col.w1p61, .phanterpwa-row .p-col.w1p61, .phanterpwa-row .phanterpwa-col.w1p61 {
  width: 61%;
}

.row .col.w1p62, .row .p-col.w1p62, .row .phanterpwa-col.w1p62, .p-row .col.w1p62, .p-row .p-col.w1p62, .p-row .phanterpwa-col.w1p62, .phanterpwa-row .col.w1p62, .phanterpwa-row .p-col.w1p62, .phanterpwa-row .phanterpwa-col.w1p62 {
  width: 62%;
}

.row .col.w1p63, .row .p-col.w1p63, .row .phanterpwa-col.w1p63, .p-row .col.w1p63, .p-row .p-col.w1p63, .p-row .phanterpwa-col.w1p63, .phanterpwa-row .col.w1p63, .phanterpwa-row .p-col.w1p63, .phanterpwa-row .phanterpwa-col.w1p63 {
  width: 63%;
}

.row .col.w1p64, .row .p-col.w1p64, .row .phanterpwa-col.w1p64, .p-row .col.w1p64, .p-row .p-col.w1p64, .p-row .phanterpwa-col.w1p64, .phanterpwa-row .col.w1p64, .phanterpwa-row .p-col.w1p64, .phanterpwa-row .phanterpwa-col.w1p64 {
  width: 64%;
}

.row .col.w1p65, .row .p-col.w1p65, .row .phanterpwa-col.w1p65, .p-row .col.w1p65, .p-row .p-col.w1p65, .p-row .phanterpwa-col.w1p65, .phanterpwa-row .col.w1p65, .phanterpwa-row .p-col.w1p65, .phanterpwa-row .phanterpwa-col.w1p65 {
  width: 65%;
}

.row .col.w1p66, .row .p-col.w1p66, .row .phanterpwa-col.w1p66, .p-row .col.w1p66, .p-row .p-col.w1p66, .p-row .phanterpwa-col.w1p66, .phanterpwa-row .col.w1p66, .phanterpwa-row .p-col.w1p66, .phanterpwa-row .phanterpwa-col.w1p66 {
  width: 66%;
}

.row .col.w1p67, .row .p-col.w1p67, .row .phanterpwa-col.w1p67, .p-row .col.w1p67, .p-row .p-col.w1p67, .p-row .phanterpwa-col.w1p67, .phanterpwa-row .col.w1p67, .phanterpwa-row .p-col.w1p67, .phanterpwa-row .phanterpwa-col.w1p67 {
  width: 67%;
}

.row .col.w1p68, .row .p-col.w1p68, .row .phanterpwa-col.w1p68, .p-row .col.w1p68, .p-row .p-col.w1p68, .p-row .phanterpwa-col.w1p68, .phanterpwa-row .col.w1p68, .phanterpwa-row .p-col.w1p68, .phanterpwa-row .phanterpwa-col.w1p68 {
  width: 68%;
}

.row .col.w1p69, .row .p-col.w1p69, .row .phanterpwa-col.w1p69, .p-row .col.w1p69, .p-row .p-col.w1p69, .p-row .phanterpwa-col.w1p69, .phanterpwa-row .col.w1p69, .phanterpwa-row .p-col.w1p69, .phanterpwa-row .phanterpwa-col.w1p69 {
  width: 69%;
}

.row .col.w1p70, .row .p-col.w1p70, .row .phanterpwa-col.w1p70, .p-row .col.w1p70, .p-row .p-col.w1p70, .p-row .phanterpwa-col.w1p70, .phanterpwa-row .col.w1p70, .phanterpwa-row .p-col.w1p70, .phanterpwa-row .phanterpwa-col.w1p70 {
  width: 70%;
}

.row .col.w1p71, .row .p-col.w1p71, .row .phanterpwa-col.w1p71, .p-row .col.w1p71, .p-row .p-col.w1p71, .p-row .phanterpwa-col.w1p71, .phanterpwa-row .col.w1p71, .phanterpwa-row .p-col.w1p71, .phanterpwa-row .phanterpwa-col.w1p71 {
  width: 71%;
}

.row .col.w1p72, .row .p-col.w1p72, .row .phanterpwa-col.w1p72, .p-row .col.w1p72, .p-row .p-col.w1p72, .p-row .phanterpwa-col.w1p72, .phanterpwa-row .col.w1p72, .phanterpwa-row .p-col.w1p72, .phanterpwa-row .phanterpwa-col.w1p72 {
  width: 72%;
}

.row .col.w1p73, .row .p-col.w1p73, .row .phanterpwa-col.w1p73, .p-row .col.w1p73, .p-row .p-col.w1p73, .p-row .phanterpwa-col.w1p73, .phanterpwa-row .col.w1p73, .phanterpwa-row .p-col.w1p73, .phanterpwa-row .phanterpwa-col.w1p73 {
  width: 73%;
}

.row .col.w1p74, .row .p-col.w1p74, .row .phanterpwa-col.w1p74, .p-row .col.w1p74, .p-row .p-col.w1p74, .p-row .phanterpwa-col.w1p74, .phanterpwa-row .col.w1p74, .phanterpwa-row .p-col.w1p74, .phanterpwa-row .phanterpwa-col.w1p74 {
  width: 74%;
}

.row .col.w1p75, .row .p-col.w1p75, .row .phanterpwa-col.w1p75, .p-row .col.w1p75, .p-row .p-col.w1p75, .p-row .phanterpwa-col.w1p75, .phanterpwa-row .col.w1p75, .phanterpwa-row .p-col.w1p75, .phanterpwa-row .phanterpwa-col.w1p75 {
  width: 75%;
}

.row .col.w1p76, .row .p-col.w1p76, .row .phanterpwa-col.w1p76, .p-row .col.w1p76, .p-row .p-col.w1p76, .p-row .phanterpwa-col.w1p76, .phanterpwa-row .col.w1p76, .phanterpwa-row .p-col.w1p76, .phanterpwa-row .phanterpwa-col.w1p76 {
  width: 76%;
}

.row .col.w1p77, .row .p-col.w1p77, .row .phanterpwa-col.w1p77, .p-row .col.w1p77, .p-row .p-col.w1p77, .p-row .phanterpwa-col.w1p77, .phanterpwa-row .col.w1p77, .phanterpwa-row .p-col.w1p77, .phanterpwa-row .phanterpwa-col.w1p77 {
  width: 77%;
}

.row .col.w1p78, .row .p-col.w1p78, .row .phanterpwa-col.w1p78, .p-row .col.w1p78, .p-row .p-col.w1p78, .p-row .phanterpwa-col.w1p78, .phanterpwa-row .col.w1p78, .phanterpwa-row .p-col.w1p78, .phanterpwa-row .phanterpwa-col.w1p78 {
  width: 78%;
}

.row .col.w1p79, .row .p-col.w1p79, .row .phanterpwa-col.w1p79, .p-row .col.w1p79, .p-row .p-col.w1p79, .p-row .phanterpwa-col.w1p79, .phanterpwa-row .col.w1p79, .phanterpwa-row .p-col.w1p79, .phanterpwa-row .phanterpwa-col.w1p79 {
  width: 79%;
}

.row .col.w1p80, .row .p-col.w1p80, .row .phanterpwa-col.w1p80, .p-row .col.w1p80, .p-row .p-col.w1p80, .p-row .phanterpwa-col.w1p80, .phanterpwa-row .col.w1p80, .phanterpwa-row .p-col.w1p80, .phanterpwa-row .phanterpwa-col.w1p80 {
  width: 80%;
}

.row .col.w1p81, .row .p-col.w1p81, .row .phanterpwa-col.w1p81, .p-row .col.w1p81, .p-row .p-col.w1p81, .p-row .phanterpwa-col.w1p81, .phanterpwa-row .col.w1p81, .phanterpwa-row .p-col.w1p81, .phanterpwa-row .phanterpwa-col.w1p81 {
  width: 81%;
}

.row .col.w1p82, .row .p-col.w1p82, .row .phanterpwa-col.w1p82, .p-row .col.w1p82, .p-row .p-col.w1p82, .p-row .phanterpwa-col.w1p82, .phanterpwa-row .col.w1p82, .phanterpwa-row .p-col.w1p82, .phanterpwa-row .phanterpwa-col.w1p82 {
  width: 82%;
}

.row .col.w1p83, .row .p-col.w1p83, .row .phanterpwa-col.w1p83, .p-row .col.w1p83, .p-row .p-col.w1p83, .p-row .phanterpwa-col.w1p83, .phanterpwa-row .col.w1p83, .phanterpwa-row .p-col.w1p83, .phanterpwa-row .phanterpwa-col.w1p83 {
  width: 83%;
}

.row .col.w1p84, .row .p-col.w1p84, .row .phanterpwa-col.w1p84, .p-row .col.w1p84, .p-row .p-col.w1p84, .p-row .phanterpwa-col.w1p84, .phanterpwa-row .col.w1p84, .phanterpwa-row .p-col.w1p84, .phanterpwa-row .phanterpwa-col.w1p84 {
  width: 84%;
}

.row .col.w1p85, .row .p-col.w1p85, .row .phanterpwa-col.w1p85, .p-row .col.w1p85, .p-row .p-col.w1p85, .p-row .phanterpwa-col.w1p85, .phanterpwa-row .col.w1p85, .phanterpwa-row .p-col.w1p85, .phanterpwa-row .phanterpwa-col.w1p85 {
  width: 85%;
}

.row .col.w1p86, .row .p-col.w1p86, .row .phanterpwa-col.w1p86, .p-row .col.w1p86, .p-row .p-col.w1p86, .p-row .phanterpwa-col.w1p86, .phanterpwa-row .col.w1p86, .phanterpwa-row .p-col.w1p86, .phanterpwa-row .phanterpwa-col.w1p86 {
  width: 86%;
}

.row .col.w1p87, .row .p-col.w1p87, .row .phanterpwa-col.w1p87, .p-row .col.w1p87, .p-row .p-col.w1p87, .p-row .phanterpwa-col.w1p87, .phanterpwa-row .col.w1p87, .phanterpwa-row .p-col.w1p87, .phanterpwa-row .phanterpwa-col.w1p87 {
  width: 87%;
}

.row .col.w1p88, .row .p-col.w1p88, .row .phanterpwa-col.w1p88, .p-row .col.w1p88, .p-row .p-col.w1p88, .p-row .phanterpwa-col.w1p88, .phanterpwa-row .col.w1p88, .phanterpwa-row .p-col.w1p88, .phanterpwa-row .phanterpwa-col.w1p88 {
  width: 88%;
}

.row .col.w1p89, .row .p-col.w1p89, .row .phanterpwa-col.w1p89, .p-row .col.w1p89, .p-row .p-col.w1p89, .p-row .phanterpwa-col.w1p89, .phanterpwa-row .col.w1p89, .phanterpwa-row .p-col.w1p89, .phanterpwa-row .phanterpwa-col.w1p89 {
  width: 89%;
}

.row .col.w1p90, .row .p-col.w1p90, .row .phanterpwa-col.w1p90, .p-row .col.w1p90, .p-row .p-col.w1p90, .p-row .phanterpwa-col.w1p90, .phanterpwa-row .col.w1p90, .phanterpwa-row .p-col.w1p90, .phanterpwa-row .phanterpwa-col.w1p90 {
  width: 90%;
}

.row .col.w1p91, .row .p-col.w1p91, .row .phanterpwa-col.w1p91, .p-row .col.w1p91, .p-row .p-col.w1p91, .p-row .phanterpwa-col.w1p91, .phanterpwa-row .col.w1p91, .phanterpwa-row .p-col.w1p91, .phanterpwa-row .phanterpwa-col.w1p91 {
  width: 91%;
}

.row .col.w1p92, .row .p-col.w1p92, .row .phanterpwa-col.w1p92, .p-row .col.w1p92, .p-row .p-col.w1p92, .p-row .phanterpwa-col.w1p92, .phanterpwa-row .col.w1p92, .phanterpwa-row .p-col.w1p92, .phanterpwa-row .phanterpwa-col.w1p92 {
  width: 92%;
}

.row .col.w1p93, .row .p-col.w1p93, .row .phanterpwa-col.w1p93, .p-row .col.w1p93, .p-row .p-col.w1p93, .p-row .phanterpwa-col.w1p93, .phanterpwa-row .col.w1p93, .phanterpwa-row .p-col.w1p93, .phanterpwa-row .phanterpwa-col.w1p93 {
  width: 93%;
}

.row .col.w1p94, .row .p-col.w1p94, .row .phanterpwa-col.w1p94, .p-row .col.w1p94, .p-row .p-col.w1p94, .p-row .phanterpwa-col.w1p94, .phanterpwa-row .col.w1p94, .phanterpwa-row .p-col.w1p94, .phanterpwa-row .phanterpwa-col.w1p94 {
  width: 94%;
}

.row .col.w1p95, .row .p-col.w1p95, .row .phanterpwa-col.w1p95, .p-row .col.w1p95, .p-row .p-col.w1p95, .p-row .phanterpwa-col.w1p95, .phanterpwa-row .col.w1p95, .phanterpwa-row .p-col.w1p95, .phanterpwa-row .phanterpwa-col.w1p95 {
  width: 95%;
}

.row .col.w1p96, .row .p-col.w1p96, .row .phanterpwa-col.w1p96, .p-row .col.w1p96, .p-row .p-col.w1p96, .p-row .phanterpwa-col.w1p96, .phanterpwa-row .col.w1p96, .phanterpwa-row .p-col.w1p96, .phanterpwa-row .phanterpwa-col.w1p96 {
  width: 96%;
}

.row .col.w1p97, .row .p-col.w1p97, .row .phanterpwa-col.w1p97, .p-row .col.w1p97, .p-row .p-col.w1p97, .p-row .phanterpwa-col.w1p97, .phanterpwa-row .col.w1p97, .phanterpwa-row .p-col.w1p97, .phanterpwa-row .phanterpwa-col.w1p97 {
  width: 97%;
}

.row .col.w1p98, .row .p-col.w1p98, .row .phanterpwa-col.w1p98, .p-row .col.w1p98, .p-row .p-col.w1p98, .p-row .phanterpwa-col.w1p98, .phanterpwa-row .col.w1p98, .phanterpwa-row .p-col.w1p98, .phanterpwa-row .phanterpwa-col.w1p98 {
  width: 98%;
}

.row .col.w1p99, .row .p-col.w1p99, .row .phanterpwa-col.w1p99, .p-row .col.w1p99, .p-row .p-col.w1p99, .p-row .phanterpwa-col.w1p99, .phanterpwa-row .col.w1p99, .phanterpwa-row .p-col.w1p99, .phanterpwa-row .phanterpwa-col.w1p99 {
  width: 99%;
}

.row .col.w1p100, .row .p-col.w1p100, .row .phanterpwa-col.w1p100, .p-row .col.w1p100, .p-row .p-col.w1p100, .p-row .phanterpwa-col.w1p100, .phanterpwa-row .col.w1p100, .phanterpwa-row .p-col.w1p100, .phanterpwa-row .phanterpwa-col.w1p100 {
  width: 100%;
}

@media screen and (min-width: 300px) {
  .row .col.w2p1, .row .p-col.w2p1, .row .phanterpwa-col.w2p1, .p-row .col.w2p1, .p-row .p-col.w2p1, .p-row .phanterpwa-col.w2p1, .phanterpwa-row .col.w2p1, .phanterpwa-row .p-col.w2p1, .phanterpwa-row .phanterpwa-col.w2p1 {
    width: 1%;
  }
  .row .col.w2p2, .row .p-col.w2p2, .row .phanterpwa-col.w2p2, .p-row .col.w2p2, .p-row .p-col.w2p2, .p-row .phanterpwa-col.w2p2, .phanterpwa-row .col.w2p2, .phanterpwa-row .p-col.w2p2, .phanterpwa-row .phanterpwa-col.w2p2 {
    width: 2%;
  }
  .row .col.w2p3, .row .p-col.w2p3, .row .phanterpwa-col.w2p3, .p-row .col.w2p3, .p-row .p-col.w2p3, .p-row .phanterpwa-col.w2p3, .phanterpwa-row .col.w2p3, .phanterpwa-row .p-col.w2p3, .phanterpwa-row .phanterpwa-col.w2p3 {
    width: 3%;
  }
  .row .col.w2p4, .row .p-col.w2p4, .row .phanterpwa-col.w2p4, .p-row .col.w2p4, .p-row .p-col.w2p4, .p-row .phanterpwa-col.w2p4, .phanterpwa-row .col.w2p4, .phanterpwa-row .p-col.w2p4, .phanterpwa-row .phanterpwa-col.w2p4 {
    width: 4%;
  }
  .row .col.w2p5, .row .p-col.w2p5, .row .phanterpwa-col.w2p5, .p-row .col.w2p5, .p-row .p-col.w2p5, .p-row .phanterpwa-col.w2p5, .phanterpwa-row .col.w2p5, .phanterpwa-row .p-col.w2p5, .phanterpwa-row .phanterpwa-col.w2p5 {
    width: 5%;
  }
  .row .col.w2p6, .row .p-col.w2p6, .row .phanterpwa-col.w2p6, .p-row .col.w2p6, .p-row .p-col.w2p6, .p-row .phanterpwa-col.w2p6, .phanterpwa-row .col.w2p6, .phanterpwa-row .p-col.w2p6, .phanterpwa-row .phanterpwa-col.w2p6 {
    width: 6%;
  }
  .row .col.w2p7, .row .p-col.w2p7, .row .phanterpwa-col.w2p7, .p-row .col.w2p7, .p-row .p-col.w2p7, .p-row .phanterpwa-col.w2p7, .phanterpwa-row .col.w2p7, .phanterpwa-row .p-col.w2p7, .phanterpwa-row .phanterpwa-col.w2p7 {
    width: 7%;
  }
  .row .col.w2p8, .row .p-col.w2p8, .row .phanterpwa-col.w2p8, .p-row .col.w2p8, .p-row .p-col.w2p8, .p-row .phanterpwa-col.w2p8, .phanterpwa-row .col.w2p8, .phanterpwa-row .p-col.w2p8, .phanterpwa-row .phanterpwa-col.w2p8 {
    width: 8%;
  }
  .row .col.w2p9, .row .p-col.w2p9, .row .phanterpwa-col.w2p9, .p-row .col.w2p9, .p-row .p-col.w2p9, .p-row .phanterpwa-col.w2p9, .phanterpwa-row .col.w2p9, .phanterpwa-row .p-col.w2p9, .phanterpwa-row .phanterpwa-col.w2p9 {
    width: 9%;
  }
  .row .col.w2p10, .row .p-col.w2p10, .row .phanterpwa-col.w2p10, .p-row .col.w2p10, .p-row .p-col.w2p10, .p-row .phanterpwa-col.w2p10, .phanterpwa-row .col.w2p10, .phanterpwa-row .p-col.w2p10, .phanterpwa-row .phanterpwa-col.w2p10 {
    width: 10%;
  }
  .row .col.w2p11, .row .p-col.w2p11, .row .phanterpwa-col.w2p11, .p-row .col.w2p11, .p-row .p-col.w2p11, .p-row .phanterpwa-col.w2p11, .phanterpwa-row .col.w2p11, .phanterpwa-row .p-col.w2p11, .phanterpwa-row .phanterpwa-col.w2p11 {
    width: 11%;
  }
  .row .col.w2p12, .row .p-col.w2p12, .row .phanterpwa-col.w2p12, .p-row .col.w2p12, .p-row .p-col.w2p12, .p-row .phanterpwa-col.w2p12, .phanterpwa-row .col.w2p12, .phanterpwa-row .p-col.w2p12, .phanterpwa-row .phanterpwa-col.w2p12 {
    width: 12%;
  }
  .row .col.w2p13, .row .p-col.w2p13, .row .phanterpwa-col.w2p13, .p-row .col.w2p13, .p-row .p-col.w2p13, .p-row .phanterpwa-col.w2p13, .phanterpwa-row .col.w2p13, .phanterpwa-row .p-col.w2p13, .phanterpwa-row .phanterpwa-col.w2p13 {
    width: 13%;
  }
  .row .col.w2p14, .row .p-col.w2p14, .row .phanterpwa-col.w2p14, .p-row .col.w2p14, .p-row .p-col.w2p14, .p-row .phanterpwa-col.w2p14, .phanterpwa-row .col.w2p14, .phanterpwa-row .p-col.w2p14, .phanterpwa-row .phanterpwa-col.w2p14 {
    width: 14%;
  }
  .row .col.w2p15, .row .p-col.w2p15, .row .phanterpwa-col.w2p15, .p-row .col.w2p15, .p-row .p-col.w2p15, .p-row .phanterpwa-col.w2p15, .phanterpwa-row .col.w2p15, .phanterpwa-row .p-col.w2p15, .phanterpwa-row .phanterpwa-col.w2p15 {
    width: 15%;
  }
  .row .col.w2p16, .row .p-col.w2p16, .row .phanterpwa-col.w2p16, .p-row .col.w2p16, .p-row .p-col.w2p16, .p-row .phanterpwa-col.w2p16, .phanterpwa-row .col.w2p16, .phanterpwa-row .p-col.w2p16, .phanterpwa-row .phanterpwa-col.w2p16 {
    width: 16%;
  }
  .row .col.w2p17, .row .p-col.w2p17, .row .phanterpwa-col.w2p17, .p-row .col.w2p17, .p-row .p-col.w2p17, .p-row .phanterpwa-col.w2p17, .phanterpwa-row .col.w2p17, .phanterpwa-row .p-col.w2p17, .phanterpwa-row .phanterpwa-col.w2p17 {
    width: 17%;
  }
  .row .col.w2p18, .row .p-col.w2p18, .row .phanterpwa-col.w2p18, .p-row .col.w2p18, .p-row .p-col.w2p18, .p-row .phanterpwa-col.w2p18, .phanterpwa-row .col.w2p18, .phanterpwa-row .p-col.w2p18, .phanterpwa-row .phanterpwa-col.w2p18 {
    width: 18%;
  }
  .row .col.w2p19, .row .p-col.w2p19, .row .phanterpwa-col.w2p19, .p-row .col.w2p19, .p-row .p-col.w2p19, .p-row .phanterpwa-col.w2p19, .phanterpwa-row .col.w2p19, .phanterpwa-row .p-col.w2p19, .phanterpwa-row .phanterpwa-col.w2p19 {
    width: 19%;
  }
  .row .col.w2p20, .row .p-col.w2p20, .row .phanterpwa-col.w2p20, .p-row .col.w2p20, .p-row .p-col.w2p20, .p-row .phanterpwa-col.w2p20, .phanterpwa-row .col.w2p20, .phanterpwa-row .p-col.w2p20, .phanterpwa-row .phanterpwa-col.w2p20 {
    width: 20%;
  }
  .row .col.w2p21, .row .p-col.w2p21, .row .phanterpwa-col.w2p21, .p-row .col.w2p21, .p-row .p-col.w2p21, .p-row .phanterpwa-col.w2p21, .phanterpwa-row .col.w2p21, .phanterpwa-row .p-col.w2p21, .phanterpwa-row .phanterpwa-col.w2p21 {
    width: 21%;
  }
  .row .col.w2p22, .row .p-col.w2p22, .row .phanterpwa-col.w2p22, .p-row .col.w2p22, .p-row .p-col.w2p22, .p-row .phanterpwa-col.w2p22, .phanterpwa-row .col.w2p22, .phanterpwa-row .p-col.w2p22, .phanterpwa-row .phanterpwa-col.w2p22 {
    width: 22%;
  }
  .row .col.w2p23, .row .p-col.w2p23, .row .phanterpwa-col.w2p23, .p-row .col.w2p23, .p-row .p-col.w2p23, .p-row .phanterpwa-col.w2p23, .phanterpwa-row .col.w2p23, .phanterpwa-row .p-col.w2p23, .phanterpwa-row .phanterpwa-col.w2p23 {
    width: 23%;
  }
  .row .col.w2p24, .row .p-col.w2p24, .row .phanterpwa-col.w2p24, .p-row .col.w2p24, .p-row .p-col.w2p24, .p-row .phanterpwa-col.w2p24, .phanterpwa-row .col.w2p24, .phanterpwa-row .p-col.w2p24, .phanterpwa-row .phanterpwa-col.w2p24 {
    width: 24%;
  }
  .row .col.w2p25, .row .p-col.w2p25, .row .phanterpwa-col.w2p25, .p-row .col.w2p25, .p-row .p-col.w2p25, .p-row .phanterpwa-col.w2p25, .phanterpwa-row .col.w2p25, .phanterpwa-row .p-col.w2p25, .phanterpwa-row .phanterpwa-col.w2p25 {
    width: 25%;
  }
  .row .col.w2p26, .row .p-col.w2p26, .row .phanterpwa-col.w2p26, .p-row .col.w2p26, .p-row .p-col.w2p26, .p-row .phanterpwa-col.w2p26, .phanterpwa-row .col.w2p26, .phanterpwa-row .p-col.w2p26, .phanterpwa-row .phanterpwa-col.w2p26 {
    width: 26%;
  }
  .row .col.w2p27, .row .p-col.w2p27, .row .phanterpwa-col.w2p27, .p-row .col.w2p27, .p-row .p-col.w2p27, .p-row .phanterpwa-col.w2p27, .phanterpwa-row .col.w2p27, .phanterpwa-row .p-col.w2p27, .phanterpwa-row .phanterpwa-col.w2p27 {
    width: 27%;
  }
  .row .col.w2p28, .row .p-col.w2p28, .row .phanterpwa-col.w2p28, .p-row .col.w2p28, .p-row .p-col.w2p28, .p-row .phanterpwa-col.w2p28, .phanterpwa-row .col.w2p28, .phanterpwa-row .p-col.w2p28, .phanterpwa-row .phanterpwa-col.w2p28 {
    width: 28%;
  }
  .row .col.w2p29, .row .p-col.w2p29, .row .phanterpwa-col.w2p29, .p-row .col.w2p29, .p-row .p-col.w2p29, .p-row .phanterpwa-col.w2p29, .phanterpwa-row .col.w2p29, .phanterpwa-row .p-col.w2p29, .phanterpwa-row .phanterpwa-col.w2p29 {
    width: 29%;
  }
  .row .col.w2p30, .row .p-col.w2p30, .row .phanterpwa-col.w2p30, .p-row .col.w2p30, .p-row .p-col.w2p30, .p-row .phanterpwa-col.w2p30, .phanterpwa-row .col.w2p30, .phanterpwa-row .p-col.w2p30, .phanterpwa-row .phanterpwa-col.w2p30 {
    width: 30%;
  }
  .row .col.w2p31, .row .p-col.w2p31, .row .phanterpwa-col.w2p31, .p-row .col.w2p31, .p-row .p-col.w2p31, .p-row .phanterpwa-col.w2p31, .phanterpwa-row .col.w2p31, .phanterpwa-row .p-col.w2p31, .phanterpwa-row .phanterpwa-col.w2p31 {
    width: 31%;
  }
  .row .col.w2p32, .row .p-col.w2p32, .row .phanterpwa-col.w2p32, .p-row .col.w2p32, .p-row .p-col.w2p32, .p-row .phanterpwa-col.w2p32, .phanterpwa-row .col.w2p32, .phanterpwa-row .p-col.w2p32, .phanterpwa-row .phanterpwa-col.w2p32 {
    width: 32%;
  }
  .row .col.w2p33, .row .p-col.w2p33, .row .phanterpwa-col.w2p33, .p-row .col.w2p33, .p-row .p-col.w2p33, .p-row .phanterpwa-col.w2p33, .phanterpwa-row .col.w2p33, .phanterpwa-row .p-col.w2p33, .phanterpwa-row .phanterpwa-col.w2p33 {
    width: 33%;
  }
  .row .col.w2p34, .row .p-col.w2p34, .row .phanterpwa-col.w2p34, .p-row .col.w2p34, .p-row .p-col.w2p34, .p-row .phanterpwa-col.w2p34, .phanterpwa-row .col.w2p34, .phanterpwa-row .p-col.w2p34, .phanterpwa-row .phanterpwa-col.w2p34 {
    width: 34%;
  }
  .row .col.w2p35, .row .p-col.w2p35, .row .phanterpwa-col.w2p35, .p-row .col.w2p35, .p-row .p-col.w2p35, .p-row .phanterpwa-col.w2p35, .phanterpwa-row .col.w2p35, .phanterpwa-row .p-col.w2p35, .phanterpwa-row .phanterpwa-col.w2p35 {
    width: 35%;
  }
  .row .col.w2p36, .row .p-col.w2p36, .row .phanterpwa-col.w2p36, .p-row .col.w2p36, .p-row .p-col.w2p36, .p-row .phanterpwa-col.w2p36, .phanterpwa-row .col.w2p36, .phanterpwa-row .p-col.w2p36, .phanterpwa-row .phanterpwa-col.w2p36 {
    width: 36%;
  }
  .row .col.w2p37, .row .p-col.w2p37, .row .phanterpwa-col.w2p37, .p-row .col.w2p37, .p-row .p-col.w2p37, .p-row .phanterpwa-col.w2p37, .phanterpwa-row .col.w2p37, .phanterpwa-row .p-col.w2p37, .phanterpwa-row .phanterpwa-col.w2p37 {
    width: 37%;
  }
  .row .col.w2p38, .row .p-col.w2p38, .row .phanterpwa-col.w2p38, .p-row .col.w2p38, .p-row .p-col.w2p38, .p-row .phanterpwa-col.w2p38, .phanterpwa-row .col.w2p38, .phanterpwa-row .p-col.w2p38, .phanterpwa-row .phanterpwa-col.w2p38 {
    width: 38%;
  }
  .row .col.w2p39, .row .p-col.w2p39, .row .phanterpwa-col.w2p39, .p-row .col.w2p39, .p-row .p-col.w2p39, .p-row .phanterpwa-col.w2p39, .phanterpwa-row .col.w2p39, .phanterpwa-row .p-col.w2p39, .phanterpwa-row .phanterpwa-col.w2p39 {
    width: 39%;
  }
  .row .col.w2p40, .row .p-col.w2p40, .row .phanterpwa-col.w2p40, .p-row .col.w2p40, .p-row .p-col.w2p40, .p-row .phanterpwa-col.w2p40, .phanterpwa-row .col.w2p40, .phanterpwa-row .p-col.w2p40, .phanterpwa-row .phanterpwa-col.w2p40 {
    width: 40%;
  }
  .row .col.w2p41, .row .p-col.w2p41, .row .phanterpwa-col.w2p41, .p-row .col.w2p41, .p-row .p-col.w2p41, .p-row .phanterpwa-col.w2p41, .phanterpwa-row .col.w2p41, .phanterpwa-row .p-col.w2p41, .phanterpwa-row .phanterpwa-col.w2p41 {
    width: 41%;
  }
  .row .col.w2p42, .row .p-col.w2p42, .row .phanterpwa-col.w2p42, .p-row .col.w2p42, .p-row .p-col.w2p42, .p-row .phanterpwa-col.w2p42, .phanterpwa-row .col.w2p42, .phanterpwa-row .p-col.w2p42, .phanterpwa-row .phanterpwa-col.w2p42 {
    width: 42%;
  }
  .row .col.w2p43, .row .p-col.w2p43, .row .phanterpwa-col.w2p43, .p-row .col.w2p43, .p-row .p-col.w2p43, .p-row .phanterpwa-col.w2p43, .phanterpwa-row .col.w2p43, .phanterpwa-row .p-col.w2p43, .phanterpwa-row .phanterpwa-col.w2p43 {
    width: 43%;
  }
  .row .col.w2p44, .row .p-col.w2p44, .row .phanterpwa-col.w2p44, .p-row .col.w2p44, .p-row .p-col.w2p44, .p-row .phanterpwa-col.w2p44, .phanterpwa-row .col.w2p44, .phanterpwa-row .p-col.w2p44, .phanterpwa-row .phanterpwa-col.w2p44 {
    width: 44%;
  }
  .row .col.w2p45, .row .p-col.w2p45, .row .phanterpwa-col.w2p45, .p-row .col.w2p45, .p-row .p-col.w2p45, .p-row .phanterpwa-col.w2p45, .phanterpwa-row .col.w2p45, .phanterpwa-row .p-col.w2p45, .phanterpwa-row .phanterpwa-col.w2p45 {
    width: 45%;
  }
  .row .col.w2p46, .row .p-col.w2p46, .row .phanterpwa-col.w2p46, .p-row .col.w2p46, .p-row .p-col.w2p46, .p-row .phanterpwa-col.w2p46, .phanterpwa-row .col.w2p46, .phanterpwa-row .p-col.w2p46, .phanterpwa-row .phanterpwa-col.w2p46 {
    width: 46%;
  }
  .row .col.w2p47, .row .p-col.w2p47, .row .phanterpwa-col.w2p47, .p-row .col.w2p47, .p-row .p-col.w2p47, .p-row .phanterpwa-col.w2p47, .phanterpwa-row .col.w2p47, .phanterpwa-row .p-col.w2p47, .phanterpwa-row .phanterpwa-col.w2p47 {
    width: 47%;
  }
  .row .col.w2p48, .row .p-col.w2p48, .row .phanterpwa-col.w2p48, .p-row .col.w2p48, .p-row .p-col.w2p48, .p-row .phanterpwa-col.w2p48, .phanterpwa-row .col.w2p48, .phanterpwa-row .p-col.w2p48, .phanterpwa-row .phanterpwa-col.w2p48 {
    width: 48%;
  }
  .row .col.w2p49, .row .p-col.w2p49, .row .phanterpwa-col.w2p49, .p-row .col.w2p49, .p-row .p-col.w2p49, .p-row .phanterpwa-col.w2p49, .phanterpwa-row .col.w2p49, .phanterpwa-row .p-col.w2p49, .phanterpwa-row .phanterpwa-col.w2p49 {
    width: 49%;
  }
  .row .col.w2p50, .row .p-col.w2p50, .row .phanterpwa-col.w2p50, .p-row .col.w2p50, .p-row .p-col.w2p50, .p-row .phanterpwa-col.w2p50, .phanterpwa-row .col.w2p50, .phanterpwa-row .p-col.w2p50, .phanterpwa-row .phanterpwa-col.w2p50 {
    width: 50%;
  }
  .row .col.w2p51, .row .p-col.w2p51, .row .phanterpwa-col.w2p51, .p-row .col.w2p51, .p-row .p-col.w2p51, .p-row .phanterpwa-col.w2p51, .phanterpwa-row .col.w2p51, .phanterpwa-row .p-col.w2p51, .phanterpwa-row .phanterpwa-col.w2p51 {
    width: 51%;
  }
  .row .col.w2p52, .row .p-col.w2p52, .row .phanterpwa-col.w2p52, .p-row .col.w2p52, .p-row .p-col.w2p52, .p-row .phanterpwa-col.w2p52, .phanterpwa-row .col.w2p52, .phanterpwa-row .p-col.w2p52, .phanterpwa-row .phanterpwa-col.w2p52 {
    width: 52%;
  }
  .row .col.w2p53, .row .p-col.w2p53, .row .phanterpwa-col.w2p53, .p-row .col.w2p53, .p-row .p-col.w2p53, .p-row .phanterpwa-col.w2p53, .phanterpwa-row .col.w2p53, .phanterpwa-row .p-col.w2p53, .phanterpwa-row .phanterpwa-col.w2p53 {
    width: 53%;
  }
  .row .col.w2p54, .row .p-col.w2p54, .row .phanterpwa-col.w2p54, .p-row .col.w2p54, .p-row .p-col.w2p54, .p-row .phanterpwa-col.w2p54, .phanterpwa-row .col.w2p54, .phanterpwa-row .p-col.w2p54, .phanterpwa-row .phanterpwa-col.w2p54 {
    width: 54%;
  }
  .row .col.w2p55, .row .p-col.w2p55, .row .phanterpwa-col.w2p55, .p-row .col.w2p55, .p-row .p-col.w2p55, .p-row .phanterpwa-col.w2p55, .phanterpwa-row .col.w2p55, .phanterpwa-row .p-col.w2p55, .phanterpwa-row .phanterpwa-col.w2p55 {
    width: 55%;
  }
  .row .col.w2p56, .row .p-col.w2p56, .row .phanterpwa-col.w2p56, .p-row .col.w2p56, .p-row .p-col.w2p56, .p-row .phanterpwa-col.w2p56, .phanterpwa-row .col.w2p56, .phanterpwa-row .p-col.w2p56, .phanterpwa-row .phanterpwa-col.w2p56 {
    width: 56%;
  }
  .row .col.w2p57, .row .p-col.w2p57, .row .phanterpwa-col.w2p57, .p-row .col.w2p57, .p-row .p-col.w2p57, .p-row .phanterpwa-col.w2p57, .phanterpwa-row .col.w2p57, .phanterpwa-row .p-col.w2p57, .phanterpwa-row .phanterpwa-col.w2p57 {
    width: 57%;
  }
  .row .col.w2p58, .row .p-col.w2p58, .row .phanterpwa-col.w2p58, .p-row .col.w2p58, .p-row .p-col.w2p58, .p-row .phanterpwa-col.w2p58, .phanterpwa-row .col.w2p58, .phanterpwa-row .p-col.w2p58, .phanterpwa-row .phanterpwa-col.w2p58 {
    width: 58%;
  }
  .row .col.w2p59, .row .p-col.w2p59, .row .phanterpwa-col.w2p59, .p-row .col.w2p59, .p-row .p-col.w2p59, .p-row .phanterpwa-col.w2p59, .phanterpwa-row .col.w2p59, .phanterpwa-row .p-col.w2p59, .phanterpwa-row .phanterpwa-col.w2p59 {
    width: 59%;
  }
  .row .col.w2p60, .row .p-col.w2p60, .row .phanterpwa-col.w2p60, .p-row .col.w2p60, .p-row .p-col.w2p60, .p-row .phanterpwa-col.w2p60, .phanterpwa-row .col.w2p60, .phanterpwa-row .p-col.w2p60, .phanterpwa-row .phanterpwa-col.w2p60 {
    width: 60%;
  }
  .row .col.w2p61, .row .p-col.w2p61, .row .phanterpwa-col.w2p61, .p-row .col.w2p61, .p-row .p-col.w2p61, .p-row .phanterpwa-col.w2p61, .phanterpwa-row .col.w2p61, .phanterpwa-row .p-col.w2p61, .phanterpwa-row .phanterpwa-col.w2p61 {
    width: 61%;
  }
  .row .col.w2p62, .row .p-col.w2p62, .row .phanterpwa-col.w2p62, .p-row .col.w2p62, .p-row .p-col.w2p62, .p-row .phanterpwa-col.w2p62, .phanterpwa-row .col.w2p62, .phanterpwa-row .p-col.w2p62, .phanterpwa-row .phanterpwa-col.w2p62 {
    width: 62%;
  }
  .row .col.w2p63, .row .p-col.w2p63, .row .phanterpwa-col.w2p63, .p-row .col.w2p63, .p-row .p-col.w2p63, .p-row .phanterpwa-col.w2p63, .phanterpwa-row .col.w2p63, .phanterpwa-row .p-col.w2p63, .phanterpwa-row .phanterpwa-col.w2p63 {
    width: 63%;
  }
  .row .col.w2p64, .row .p-col.w2p64, .row .phanterpwa-col.w2p64, .p-row .col.w2p64, .p-row .p-col.w2p64, .p-row .phanterpwa-col.w2p64, .phanterpwa-row .col.w2p64, .phanterpwa-row .p-col.w2p64, .phanterpwa-row .phanterpwa-col.w2p64 {
    width: 64%;
  }
  .row .col.w2p65, .row .p-col.w2p65, .row .phanterpwa-col.w2p65, .p-row .col.w2p65, .p-row .p-col.w2p65, .p-row .phanterpwa-col.w2p65, .phanterpwa-row .col.w2p65, .phanterpwa-row .p-col.w2p65, .phanterpwa-row .phanterpwa-col.w2p65 {
    width: 65%;
  }
  .row .col.w2p66, .row .p-col.w2p66, .row .phanterpwa-col.w2p66, .p-row .col.w2p66, .p-row .p-col.w2p66, .p-row .phanterpwa-col.w2p66, .phanterpwa-row .col.w2p66, .phanterpwa-row .p-col.w2p66, .phanterpwa-row .phanterpwa-col.w2p66 {
    width: 66%;
  }
  .row .col.w2p67, .row .p-col.w2p67, .row .phanterpwa-col.w2p67, .p-row .col.w2p67, .p-row .p-col.w2p67, .p-row .phanterpwa-col.w2p67, .phanterpwa-row .col.w2p67, .phanterpwa-row .p-col.w2p67, .phanterpwa-row .phanterpwa-col.w2p67 {
    width: 67%;
  }
  .row .col.w2p68, .row .p-col.w2p68, .row .phanterpwa-col.w2p68, .p-row .col.w2p68, .p-row .p-col.w2p68, .p-row .phanterpwa-col.w2p68, .phanterpwa-row .col.w2p68, .phanterpwa-row .p-col.w2p68, .phanterpwa-row .phanterpwa-col.w2p68 {
    width: 68%;
  }
  .row .col.w2p69, .row .p-col.w2p69, .row .phanterpwa-col.w2p69, .p-row .col.w2p69, .p-row .p-col.w2p69, .p-row .phanterpwa-col.w2p69, .phanterpwa-row .col.w2p69, .phanterpwa-row .p-col.w2p69, .phanterpwa-row .phanterpwa-col.w2p69 {
    width: 69%;
  }
  .row .col.w2p70, .row .p-col.w2p70, .row .phanterpwa-col.w2p70, .p-row .col.w2p70, .p-row .p-col.w2p70, .p-row .phanterpwa-col.w2p70, .phanterpwa-row .col.w2p70, .phanterpwa-row .p-col.w2p70, .phanterpwa-row .phanterpwa-col.w2p70 {
    width: 70%;
  }
  .row .col.w2p71, .row .p-col.w2p71, .row .phanterpwa-col.w2p71, .p-row .col.w2p71, .p-row .p-col.w2p71, .p-row .phanterpwa-col.w2p71, .phanterpwa-row .col.w2p71, .phanterpwa-row .p-col.w2p71, .phanterpwa-row .phanterpwa-col.w2p71 {
    width: 71%;
  }
  .row .col.w2p72, .row .p-col.w2p72, .row .phanterpwa-col.w2p72, .p-row .col.w2p72, .p-row .p-col.w2p72, .p-row .phanterpwa-col.w2p72, .phanterpwa-row .col.w2p72, .phanterpwa-row .p-col.w2p72, .phanterpwa-row .phanterpwa-col.w2p72 {
    width: 72%;
  }
  .row .col.w2p73, .row .p-col.w2p73, .row .phanterpwa-col.w2p73, .p-row .col.w2p73, .p-row .p-col.w2p73, .p-row .phanterpwa-col.w2p73, .phanterpwa-row .col.w2p73, .phanterpwa-row .p-col.w2p73, .phanterpwa-row .phanterpwa-col.w2p73 {
    width: 73%;
  }
  .row .col.w2p74, .row .p-col.w2p74, .row .phanterpwa-col.w2p74, .p-row .col.w2p74, .p-row .p-col.w2p74, .p-row .phanterpwa-col.w2p74, .phanterpwa-row .col.w2p74, .phanterpwa-row .p-col.w2p74, .phanterpwa-row .phanterpwa-col.w2p74 {
    width: 74%;
  }
  .row .col.w2p75, .row .p-col.w2p75, .row .phanterpwa-col.w2p75, .p-row .col.w2p75, .p-row .p-col.w2p75, .p-row .phanterpwa-col.w2p75, .phanterpwa-row .col.w2p75, .phanterpwa-row .p-col.w2p75, .phanterpwa-row .phanterpwa-col.w2p75 {
    width: 75%;
  }
  .row .col.w2p76, .row .p-col.w2p76, .row .phanterpwa-col.w2p76, .p-row .col.w2p76, .p-row .p-col.w2p76, .p-row .phanterpwa-col.w2p76, .phanterpwa-row .col.w2p76, .phanterpwa-row .p-col.w2p76, .phanterpwa-row .phanterpwa-col.w2p76 {
    width: 76%;
  }
  .row .col.w2p77, .row .p-col.w2p77, .row .phanterpwa-col.w2p77, .p-row .col.w2p77, .p-row .p-col.w2p77, .p-row .phanterpwa-col.w2p77, .phanterpwa-row .col.w2p77, .phanterpwa-row .p-col.w2p77, .phanterpwa-row .phanterpwa-col.w2p77 {
    width: 77%;
  }
  .row .col.w2p78, .row .p-col.w2p78, .row .phanterpwa-col.w2p78, .p-row .col.w2p78, .p-row .p-col.w2p78, .p-row .phanterpwa-col.w2p78, .phanterpwa-row .col.w2p78, .phanterpwa-row .p-col.w2p78, .phanterpwa-row .phanterpwa-col.w2p78 {
    width: 78%;
  }
  .row .col.w2p79, .row .p-col.w2p79, .row .phanterpwa-col.w2p79, .p-row .col.w2p79, .p-row .p-col.w2p79, .p-row .phanterpwa-col.w2p79, .phanterpwa-row .col.w2p79, .phanterpwa-row .p-col.w2p79, .phanterpwa-row .phanterpwa-col.w2p79 {
    width: 79%;
  }
  .row .col.w2p80, .row .p-col.w2p80, .row .phanterpwa-col.w2p80, .p-row .col.w2p80, .p-row .p-col.w2p80, .p-row .phanterpwa-col.w2p80, .phanterpwa-row .col.w2p80, .phanterpwa-row .p-col.w2p80, .phanterpwa-row .phanterpwa-col.w2p80 {
    width: 80%;
  }
  .row .col.w2p81, .row .p-col.w2p81, .row .phanterpwa-col.w2p81, .p-row .col.w2p81, .p-row .p-col.w2p81, .p-row .phanterpwa-col.w2p81, .phanterpwa-row .col.w2p81, .phanterpwa-row .p-col.w2p81, .phanterpwa-row .phanterpwa-col.w2p81 {
    width: 81%;
  }
  .row .col.w2p82, .row .p-col.w2p82, .row .phanterpwa-col.w2p82, .p-row .col.w2p82, .p-row .p-col.w2p82, .p-row .phanterpwa-col.w2p82, .phanterpwa-row .col.w2p82, .phanterpwa-row .p-col.w2p82, .phanterpwa-row .phanterpwa-col.w2p82 {
    width: 82%;
  }
  .row .col.w2p83, .row .p-col.w2p83, .row .phanterpwa-col.w2p83, .p-row .col.w2p83, .p-row .p-col.w2p83, .p-row .phanterpwa-col.w2p83, .phanterpwa-row .col.w2p83, .phanterpwa-row .p-col.w2p83, .phanterpwa-row .phanterpwa-col.w2p83 {
    width: 83%;
  }
  .row .col.w2p84, .row .p-col.w2p84, .row .phanterpwa-col.w2p84, .p-row .col.w2p84, .p-row .p-col.w2p84, .p-row .phanterpwa-col.w2p84, .phanterpwa-row .col.w2p84, .phanterpwa-row .p-col.w2p84, .phanterpwa-row .phanterpwa-col.w2p84 {
    width: 84%;
  }
  .row .col.w2p85, .row .p-col.w2p85, .row .phanterpwa-col.w2p85, .p-row .col.w2p85, .p-row .p-col.w2p85, .p-row .phanterpwa-col.w2p85, .phanterpwa-row .col.w2p85, .phanterpwa-row .p-col.w2p85, .phanterpwa-row .phanterpwa-col.w2p85 {
    width: 85%;
  }
  .row .col.w2p86, .row .p-col.w2p86, .row .phanterpwa-col.w2p86, .p-row .col.w2p86, .p-row .p-col.w2p86, .p-row .phanterpwa-col.w2p86, .phanterpwa-row .col.w2p86, .phanterpwa-row .p-col.w2p86, .phanterpwa-row .phanterpwa-col.w2p86 {
    width: 86%;
  }
  .row .col.w2p87, .row .p-col.w2p87, .row .phanterpwa-col.w2p87, .p-row .col.w2p87, .p-row .p-col.w2p87, .p-row .phanterpwa-col.w2p87, .phanterpwa-row .col.w2p87, .phanterpwa-row .p-col.w2p87, .phanterpwa-row .phanterpwa-col.w2p87 {
    width: 87%;
  }
  .row .col.w2p88, .row .p-col.w2p88, .row .phanterpwa-col.w2p88, .p-row .col.w2p88, .p-row .p-col.w2p88, .p-row .phanterpwa-col.w2p88, .phanterpwa-row .col.w2p88, .phanterpwa-row .p-col.w2p88, .phanterpwa-row .phanterpwa-col.w2p88 {
    width: 88%;
  }
  .row .col.w2p89, .row .p-col.w2p89, .row .phanterpwa-col.w2p89, .p-row .col.w2p89, .p-row .p-col.w2p89, .p-row .phanterpwa-col.w2p89, .phanterpwa-row .col.w2p89, .phanterpwa-row .p-col.w2p89, .phanterpwa-row .phanterpwa-col.w2p89 {
    width: 89%;
  }
  .row .col.w2p90, .row .p-col.w2p90, .row .phanterpwa-col.w2p90, .p-row .col.w2p90, .p-row .p-col.w2p90, .p-row .phanterpwa-col.w2p90, .phanterpwa-row .col.w2p90, .phanterpwa-row .p-col.w2p90, .phanterpwa-row .phanterpwa-col.w2p90 {
    width: 90%;
  }
  .row .col.w2p91, .row .p-col.w2p91, .row .phanterpwa-col.w2p91, .p-row .col.w2p91, .p-row .p-col.w2p91, .p-row .phanterpwa-col.w2p91, .phanterpwa-row .col.w2p91, .phanterpwa-row .p-col.w2p91, .phanterpwa-row .phanterpwa-col.w2p91 {
    width: 91%;
  }
  .row .col.w2p92, .row .p-col.w2p92, .row .phanterpwa-col.w2p92, .p-row .col.w2p92, .p-row .p-col.w2p92, .p-row .phanterpwa-col.w2p92, .phanterpwa-row .col.w2p92, .phanterpwa-row .p-col.w2p92, .phanterpwa-row .phanterpwa-col.w2p92 {
    width: 92%;
  }
  .row .col.w2p93, .row .p-col.w2p93, .row .phanterpwa-col.w2p93, .p-row .col.w2p93, .p-row .p-col.w2p93, .p-row .phanterpwa-col.w2p93, .phanterpwa-row .col.w2p93, .phanterpwa-row .p-col.w2p93, .phanterpwa-row .phanterpwa-col.w2p93 {
    width: 93%;
  }
  .row .col.w2p94, .row .p-col.w2p94, .row .phanterpwa-col.w2p94, .p-row .col.w2p94, .p-row .p-col.w2p94, .p-row .phanterpwa-col.w2p94, .phanterpwa-row .col.w2p94, .phanterpwa-row .p-col.w2p94, .phanterpwa-row .phanterpwa-col.w2p94 {
    width: 94%;
  }
  .row .col.w2p95, .row .p-col.w2p95, .row .phanterpwa-col.w2p95, .p-row .col.w2p95, .p-row .p-col.w2p95, .p-row .phanterpwa-col.w2p95, .phanterpwa-row .col.w2p95, .phanterpwa-row .p-col.w2p95, .phanterpwa-row .phanterpwa-col.w2p95 {
    width: 95%;
  }
  .row .col.w2p96, .row .p-col.w2p96, .row .phanterpwa-col.w2p96, .p-row .col.w2p96, .p-row .p-col.w2p96, .p-row .phanterpwa-col.w2p96, .phanterpwa-row .col.w2p96, .phanterpwa-row .p-col.w2p96, .phanterpwa-row .phanterpwa-col.w2p96 {
    width: 96%;
  }
  .row .col.w2p97, .row .p-col.w2p97, .row .phanterpwa-col.w2p97, .p-row .col.w2p97, .p-row .p-col.w2p97, .p-row .phanterpwa-col.w2p97, .phanterpwa-row .col.w2p97, .phanterpwa-row .p-col.w2p97, .phanterpwa-row .phanterpwa-col.w2p97 {
    width: 97%;
  }
  .row .col.w2p98, .row .p-col.w2p98, .row .phanterpwa-col.w2p98, .p-row .col.w2p98, .p-row .p-col.w2p98, .p-row .phanterpwa-col.w2p98, .phanterpwa-row .col.w2p98, .phanterpwa-row .p-col.w2p98, .phanterpwa-row .phanterpwa-col.w2p98 {
    width: 98%;
  }
  .row .col.w2p99, .row .p-col.w2p99, .row .phanterpwa-col.w2p99, .p-row .col.w2p99, .p-row .p-col.w2p99, .p-row .phanterpwa-col.w2p99, .phanterpwa-row .col.w2p99, .phanterpwa-row .p-col.w2p99, .phanterpwa-row .phanterpwa-col.w2p99 {
    width: 99%;
  }
  .row .col.w2p100, .row .p-col.w2p100, .row .phanterpwa-col.w2p100, .p-row .col.w2p100, .p-row .p-col.w2p100, .p-row .phanterpwa-col.w2p100, .phanterpwa-row .col.w2p100, .phanterpwa-row .p-col.w2p100, .phanterpwa-row .phanterpwa-col.w2p100 {
    width: 100%;
  }
}

@media screen and (min-width: 600px) {
  .row .col.w3p1, .row .p-col.w3p1, .row .phanterpwa-col.w3p1, .p-row .col.w3p1, .p-row .p-col.w3p1, .p-row .phanterpwa-col.w3p1, .phanterpwa-row .col.w3p1, .phanterpwa-row .p-col.w3p1, .phanterpwa-row .phanterpwa-col.w3p1 {
    width: 1%;
  }
  .row .col.w3p2, .row .p-col.w3p2, .row .phanterpwa-col.w3p2, .p-row .col.w3p2, .p-row .p-col.w3p2, .p-row .phanterpwa-col.w3p2, .phanterpwa-row .col.w3p2, .phanterpwa-row .p-col.w3p2, .phanterpwa-row .phanterpwa-col.w3p2 {
    width: 2%;
  }
  .row .col.w3p3, .row .p-col.w3p3, .row .phanterpwa-col.w3p3, .p-row .col.w3p3, .p-row .p-col.w3p3, .p-row .phanterpwa-col.w3p3, .phanterpwa-row .col.w3p3, .phanterpwa-row .p-col.w3p3, .phanterpwa-row .phanterpwa-col.w3p3 {
    width: 3%;
  }
  .row .col.w3p4, .row .p-col.w3p4, .row .phanterpwa-col.w3p4, .p-row .col.w3p4, .p-row .p-col.w3p4, .p-row .phanterpwa-col.w3p4, .phanterpwa-row .col.w3p4, .phanterpwa-row .p-col.w3p4, .phanterpwa-row .phanterpwa-col.w3p4 {
    width: 4%;
  }
  .row .col.w3p5, .row .p-col.w3p5, .row .phanterpwa-col.w3p5, .p-row .col.w3p5, .p-row .p-col.w3p5, .p-row .phanterpwa-col.w3p5, .phanterpwa-row .col.w3p5, .phanterpwa-row .p-col.w3p5, .phanterpwa-row .phanterpwa-col.w3p5 {
    width: 5%;
  }
  .row .col.w3p6, .row .p-col.w3p6, .row .phanterpwa-col.w3p6, .p-row .col.w3p6, .p-row .p-col.w3p6, .p-row .phanterpwa-col.w3p6, .phanterpwa-row .col.w3p6, .phanterpwa-row .p-col.w3p6, .phanterpwa-row .phanterpwa-col.w3p6 {
    width: 6%;
  }
  .row .col.w3p7, .row .p-col.w3p7, .row .phanterpwa-col.w3p7, .p-row .col.w3p7, .p-row .p-col.w3p7, .p-row .phanterpwa-col.w3p7, .phanterpwa-row .col.w3p7, .phanterpwa-row .p-col.w3p7, .phanterpwa-row .phanterpwa-col.w3p7 {
    width: 7%;
  }
  .row .col.w3p8, .row .p-col.w3p8, .row .phanterpwa-col.w3p8, .p-row .col.w3p8, .p-row .p-col.w3p8, .p-row .phanterpwa-col.w3p8, .phanterpwa-row .col.w3p8, .phanterpwa-row .p-col.w3p8, .phanterpwa-row .phanterpwa-col.w3p8 {
    width: 8%;
  }
  .row .col.w3p9, .row .p-col.w3p9, .row .phanterpwa-col.w3p9, .p-row .col.w3p9, .p-row .p-col.w3p9, .p-row .phanterpwa-col.w3p9, .phanterpwa-row .col.w3p9, .phanterpwa-row .p-col.w3p9, .phanterpwa-row .phanterpwa-col.w3p9 {
    width: 9%;
  }
  .row .col.w3p10, .row .p-col.w3p10, .row .phanterpwa-col.w3p10, .p-row .col.w3p10, .p-row .p-col.w3p10, .p-row .phanterpwa-col.w3p10, .phanterpwa-row .col.w3p10, .phanterpwa-row .p-col.w3p10, .phanterpwa-row .phanterpwa-col.w3p10 {
    width: 10%;
  }
  .row .col.w3p11, .row .p-col.w3p11, .row .phanterpwa-col.w3p11, .p-row .col.w3p11, .p-row .p-col.w3p11, .p-row .phanterpwa-col.w3p11, .phanterpwa-row .col.w3p11, .phanterpwa-row .p-col.w3p11, .phanterpwa-row .phanterpwa-col.w3p11 {
    width: 11%;
  }
  .row .col.w3p12, .row .p-col.w3p12, .row .phanterpwa-col.w3p12, .p-row .col.w3p12, .p-row .p-col.w3p12, .p-row .phanterpwa-col.w3p12, .phanterpwa-row .col.w3p12, .phanterpwa-row .p-col.w3p12, .phanterpwa-row .phanterpwa-col.w3p12 {
    width: 12%;
  }
  .row .col.w3p13, .row .p-col.w3p13, .row .phanterpwa-col.w3p13, .p-row .col.w3p13, .p-row .p-col.w3p13, .p-row .phanterpwa-col.w3p13, .phanterpwa-row .col.w3p13, .phanterpwa-row .p-col.w3p13, .phanterpwa-row .phanterpwa-col.w3p13 {
    width: 13%;
  }
  .row .col.w3p14, .row .p-col.w3p14, .row .phanterpwa-col.w3p14, .p-row .col.w3p14, .p-row .p-col.w3p14, .p-row .phanterpwa-col.w3p14, .phanterpwa-row .col.w3p14, .phanterpwa-row .p-col.w3p14, .phanterpwa-row .phanterpwa-col.w3p14 {
    width: 14%;
  }
  .row .col.w3p15, .row .p-col.w3p15, .row .phanterpwa-col.w3p15, .p-row .col.w3p15, .p-row .p-col.w3p15, .p-row .phanterpwa-col.w3p15, .phanterpwa-row .col.w3p15, .phanterpwa-row .p-col.w3p15, .phanterpwa-row .phanterpwa-col.w3p15 {
    width: 15%;
  }
  .row .col.w3p16, .row .p-col.w3p16, .row .phanterpwa-col.w3p16, .p-row .col.w3p16, .p-row .p-col.w3p16, .p-row .phanterpwa-col.w3p16, .phanterpwa-row .col.w3p16, .phanterpwa-row .p-col.w3p16, .phanterpwa-row .phanterpwa-col.w3p16 {
    width: 16%;
  }
  .row .col.w3p17, .row .p-col.w3p17, .row .phanterpwa-col.w3p17, .p-row .col.w3p17, .p-row .p-col.w3p17, .p-row .phanterpwa-col.w3p17, .phanterpwa-row .col.w3p17, .phanterpwa-row .p-col.w3p17, .phanterpwa-row .phanterpwa-col.w3p17 {
    width: 17%;
  }
  .row .col.w3p18, .row .p-col.w3p18, .row .phanterpwa-col.w3p18, .p-row .col.w3p18, .p-row .p-col.w3p18, .p-row .phanterpwa-col.w3p18, .phanterpwa-row .col.w3p18, .phanterpwa-row .p-col.w3p18, .phanterpwa-row .phanterpwa-col.w3p18 {
    width: 18%;
  }
  .row .col.w3p19, .row .p-col.w3p19, .row .phanterpwa-col.w3p19, .p-row .col.w3p19, .p-row .p-col.w3p19, .p-row .phanterpwa-col.w3p19, .phanterpwa-row .col.w3p19, .phanterpwa-row .p-col.w3p19, .phanterpwa-row .phanterpwa-col.w3p19 {
    width: 19%;
  }
  .row .col.w3p20, .row .p-col.w3p20, .row .phanterpwa-col.w3p20, .p-row .col.w3p20, .p-row .p-col.w3p20, .p-row .phanterpwa-col.w3p20, .phanterpwa-row .col.w3p20, .phanterpwa-row .p-col.w3p20, .phanterpwa-row .phanterpwa-col.w3p20 {
    width: 20%;
  }
  .row .col.w3p21, .row .p-col.w3p21, .row .phanterpwa-col.w3p21, .p-row .col.w3p21, .p-row .p-col.w3p21, .p-row .phanterpwa-col.w3p21, .phanterpwa-row .col.w3p21, .phanterpwa-row .p-col.w3p21, .phanterpwa-row .phanterpwa-col.w3p21 {
    width: 21%;
  }
  .row .col.w3p22, .row .p-col.w3p22, .row .phanterpwa-col.w3p22, .p-row .col.w3p22, .p-row .p-col.w3p22, .p-row .phanterpwa-col.w3p22, .phanterpwa-row .col.w3p22, .phanterpwa-row .p-col.w3p22, .phanterpwa-row .phanterpwa-col.w3p22 {
    width: 22%;
  }
  .row .col.w3p23, .row .p-col.w3p23, .row .phanterpwa-col.w3p23, .p-row .col.w3p23, .p-row .p-col.w3p23, .p-row .phanterpwa-col.w3p23, .phanterpwa-row .col.w3p23, .phanterpwa-row .p-col.w3p23, .phanterpwa-row .phanterpwa-col.w3p23 {
    width: 23%;
  }
  .row .col.w3p24, .row .p-col.w3p24, .row .phanterpwa-col.w3p24, .p-row .col.w3p24, .p-row .p-col.w3p24, .p-row .phanterpwa-col.w3p24, .phanterpwa-row .col.w3p24, .phanterpwa-row .p-col.w3p24, .phanterpwa-row .phanterpwa-col.w3p24 {
    width: 24%;
  }
  .row .col.w3p25, .row .p-col.w3p25, .row .phanterpwa-col.w3p25, .p-row .col.w3p25, .p-row .p-col.w3p25, .p-row .phanterpwa-col.w3p25, .phanterpwa-row .col.w3p25, .phanterpwa-row .p-col.w3p25, .phanterpwa-row .phanterpwa-col.w3p25 {
    width: 25%;
  }
  .row .col.w3p26, .row .p-col.w3p26, .row .phanterpwa-col.w3p26, .p-row .col.w3p26, .p-row .p-col.w3p26, .p-row .phanterpwa-col.w3p26, .phanterpwa-row .col.w3p26, .phanterpwa-row .p-col.w3p26, .phanterpwa-row .phanterpwa-col.w3p26 {
    width: 26%;
  }
  .row .col.w3p27, .row .p-col.w3p27, .row .phanterpwa-col.w3p27, .p-row .col.w3p27, .p-row .p-col.w3p27, .p-row .phanterpwa-col.w3p27, .phanterpwa-row .col.w3p27, .phanterpwa-row .p-col.w3p27, .phanterpwa-row .phanterpwa-col.w3p27 {
    width: 27%;
  }
  .row .col.w3p28, .row .p-col.w3p28, .row .phanterpwa-col.w3p28, .p-row .col.w3p28, .p-row .p-col.w3p28, .p-row .phanterpwa-col.w3p28, .phanterpwa-row .col.w3p28, .phanterpwa-row .p-col.w3p28, .phanterpwa-row .phanterpwa-col.w3p28 {
    width: 28%;
  }
  .row .col.w3p29, .row .p-col.w3p29, .row .phanterpwa-col.w3p29, .p-row .col.w3p29, .p-row .p-col.w3p29, .p-row .phanterpwa-col.w3p29, .phanterpwa-row .col.w3p29, .phanterpwa-row .p-col.w3p29, .phanterpwa-row .phanterpwa-col.w3p29 {
    width: 29%;
  }
  .row .col.w3p30, .row .p-col.w3p30, .row .phanterpwa-col.w3p30, .p-row .col.w3p30, .p-row .p-col.w3p30, .p-row .phanterpwa-col.w3p30, .phanterpwa-row .col.w3p30, .phanterpwa-row .p-col.w3p30, .phanterpwa-row .phanterpwa-col.w3p30 {
    width: 30%;
  }
  .row .col.w3p31, .row .p-col.w3p31, .row .phanterpwa-col.w3p31, .p-row .col.w3p31, .p-row .p-col.w3p31, .p-row .phanterpwa-col.w3p31, .phanterpwa-row .col.w3p31, .phanterpwa-row .p-col.w3p31, .phanterpwa-row .phanterpwa-col.w3p31 {
    width: 31%;
  }
  .row .col.w3p32, .row .p-col.w3p32, .row .phanterpwa-col.w3p32, .p-row .col.w3p32, .p-row .p-col.w3p32, .p-row .phanterpwa-col.w3p32, .phanterpwa-row .col.w3p32, .phanterpwa-row .p-col.w3p32, .phanterpwa-row .phanterpwa-col.w3p32 {
    width: 32%;
  }
  .row .col.w3p33, .row .p-col.w3p33, .row .phanterpwa-col.w3p33, .p-row .col.w3p33, .p-row .p-col.w3p33, .p-row .phanterpwa-col.w3p33, .phanterpwa-row .col.w3p33, .phanterpwa-row .p-col.w3p33, .phanterpwa-row .phanterpwa-col.w3p33 {
    width: 33%;
  }
  .row .col.w3p34, .row .p-col.w3p34, .row .phanterpwa-col.w3p34, .p-row .col.w3p34, .p-row .p-col.w3p34, .p-row .phanterpwa-col.w3p34, .phanterpwa-row .col.w3p34, .phanterpwa-row .p-col.w3p34, .phanterpwa-row .phanterpwa-col.w3p34 {
    width: 34%;
  }
  .row .col.w3p35, .row .p-col.w3p35, .row .phanterpwa-col.w3p35, .p-row .col.w3p35, .p-row .p-col.w3p35, .p-row .phanterpwa-col.w3p35, .phanterpwa-row .col.w3p35, .phanterpwa-row .p-col.w3p35, .phanterpwa-row .phanterpwa-col.w3p35 {
    width: 35%;
  }
  .row .col.w3p36, .row .p-col.w3p36, .row .phanterpwa-col.w3p36, .p-row .col.w3p36, .p-row .p-col.w3p36, .p-row .phanterpwa-col.w3p36, .phanterpwa-row .col.w3p36, .phanterpwa-row .p-col.w3p36, .phanterpwa-row .phanterpwa-col.w3p36 {
    width: 36%;
  }
  .row .col.w3p37, .row .p-col.w3p37, .row .phanterpwa-col.w3p37, .p-row .col.w3p37, .p-row .p-col.w3p37, .p-row .phanterpwa-col.w3p37, .phanterpwa-row .col.w3p37, .phanterpwa-row .p-col.w3p37, .phanterpwa-row .phanterpwa-col.w3p37 {
    width: 37%;
  }
  .row .col.w3p38, .row .p-col.w3p38, .row .phanterpwa-col.w3p38, .p-row .col.w3p38, .p-row .p-col.w3p38, .p-row .phanterpwa-col.w3p38, .phanterpwa-row .col.w3p38, .phanterpwa-row .p-col.w3p38, .phanterpwa-row .phanterpwa-col.w3p38 {
    width: 38%;
  }
  .row .col.w3p39, .row .p-col.w3p39, .row .phanterpwa-col.w3p39, .p-row .col.w3p39, .p-row .p-col.w3p39, .p-row .phanterpwa-col.w3p39, .phanterpwa-row .col.w3p39, .phanterpwa-row .p-col.w3p39, .phanterpwa-row .phanterpwa-col.w3p39 {
    width: 39%;
  }
  .row .col.w3p40, .row .p-col.w3p40, .row .phanterpwa-col.w3p40, .p-row .col.w3p40, .p-row .p-col.w3p40, .p-row .phanterpwa-col.w3p40, .phanterpwa-row .col.w3p40, .phanterpwa-row .p-col.w3p40, .phanterpwa-row .phanterpwa-col.w3p40 {
    width: 40%;
  }
  .row .col.w3p41, .row .p-col.w3p41, .row .phanterpwa-col.w3p41, .p-row .col.w3p41, .p-row .p-col.w3p41, .p-row .phanterpwa-col.w3p41, .phanterpwa-row .col.w3p41, .phanterpwa-row .p-col.w3p41, .phanterpwa-row .phanterpwa-col.w3p41 {
    width: 41%;
  }
  .row .col.w3p42, .row .p-col.w3p42, .row .phanterpwa-col.w3p42, .p-row .col.w3p42, .p-row .p-col.w3p42, .p-row .phanterpwa-col.w3p42, .phanterpwa-row .col.w3p42, .phanterpwa-row .p-col.w3p42, .phanterpwa-row .phanterpwa-col.w3p42 {
    width: 42%;
  }
  .row .col.w3p43, .row .p-col.w3p43, .row .phanterpwa-col.w3p43, .p-row .col.w3p43, .p-row .p-col.w3p43, .p-row .phanterpwa-col.w3p43, .phanterpwa-row .col.w3p43, .phanterpwa-row .p-col.w3p43, .phanterpwa-row .phanterpwa-col.w3p43 {
    width: 43%;
  }
  .row .col.w3p44, .row .p-col.w3p44, .row .phanterpwa-col.w3p44, .p-row .col.w3p44, .p-row .p-col.w3p44, .p-row .phanterpwa-col.w3p44, .phanterpwa-row .col.w3p44, .phanterpwa-row .p-col.w3p44, .phanterpwa-row .phanterpwa-col.w3p44 {
    width: 44%;
  }
  .row .col.w3p45, .row .p-col.w3p45, .row .phanterpwa-col.w3p45, .p-row .col.w3p45, .p-row .p-col.w3p45, .p-row .phanterpwa-col.w3p45, .phanterpwa-row .col.w3p45, .phanterpwa-row .p-col.w3p45, .phanterpwa-row .phanterpwa-col.w3p45 {
    width: 45%;
  }
  .row .col.w3p46, .row .p-col.w3p46, .row .phanterpwa-col.w3p46, .p-row .col.w3p46, .p-row .p-col.w3p46, .p-row .phanterpwa-col.w3p46, .phanterpwa-row .col.w3p46, .phanterpwa-row .p-col.w3p46, .phanterpwa-row .phanterpwa-col.w3p46 {
    width: 46%;
  }
  .row .col.w3p47, .row .p-col.w3p47, .row .phanterpwa-col.w3p47, .p-row .col.w3p47, .p-row .p-col.w3p47, .p-row .phanterpwa-col.w3p47, .phanterpwa-row .col.w3p47, .phanterpwa-row .p-col.w3p47, .phanterpwa-row .phanterpwa-col.w3p47 {
    width: 47%;
  }
  .row .col.w3p48, .row .p-col.w3p48, .row .phanterpwa-col.w3p48, .p-row .col.w3p48, .p-row .p-col.w3p48, .p-row .phanterpwa-col.w3p48, .phanterpwa-row .col.w3p48, .phanterpwa-row .p-col.w3p48, .phanterpwa-row .phanterpwa-col.w3p48 {
    width: 48%;
  }
  .row .col.w3p49, .row .p-col.w3p49, .row .phanterpwa-col.w3p49, .p-row .col.w3p49, .p-row .p-col.w3p49, .p-row .phanterpwa-col.w3p49, .phanterpwa-row .col.w3p49, .phanterpwa-row .p-col.w3p49, .phanterpwa-row .phanterpwa-col.w3p49 {
    width: 49%;
  }
  .row .col.w3p50, .row .p-col.w3p50, .row .phanterpwa-col.w3p50, .p-row .col.w3p50, .p-row .p-col.w3p50, .p-row .phanterpwa-col.w3p50, .phanterpwa-row .col.w3p50, .phanterpwa-row .p-col.w3p50, .phanterpwa-row .phanterpwa-col.w3p50 {
    width: 50%;
  }
  .row .col.w3p51, .row .p-col.w3p51, .row .phanterpwa-col.w3p51, .p-row .col.w3p51, .p-row .p-col.w3p51, .p-row .phanterpwa-col.w3p51, .phanterpwa-row .col.w3p51, .phanterpwa-row .p-col.w3p51, .phanterpwa-row .phanterpwa-col.w3p51 {
    width: 51%;
  }
  .row .col.w3p52, .row .p-col.w3p52, .row .phanterpwa-col.w3p52, .p-row .col.w3p52, .p-row .p-col.w3p52, .p-row .phanterpwa-col.w3p52, .phanterpwa-row .col.w3p52, .phanterpwa-row .p-col.w3p52, .phanterpwa-row .phanterpwa-col.w3p52 {
    width: 52%;
  }
  .row .col.w3p53, .row .p-col.w3p53, .row .phanterpwa-col.w3p53, .p-row .col.w3p53, .p-row .p-col.w3p53, .p-row .phanterpwa-col.w3p53, .phanterpwa-row .col.w3p53, .phanterpwa-row .p-col.w3p53, .phanterpwa-row .phanterpwa-col.w3p53 {
    width: 53%;
  }
  .row .col.w3p54, .row .p-col.w3p54, .row .phanterpwa-col.w3p54, .p-row .col.w3p54, .p-row .p-col.w3p54, .p-row .phanterpwa-col.w3p54, .phanterpwa-row .col.w3p54, .phanterpwa-row .p-col.w3p54, .phanterpwa-row .phanterpwa-col.w3p54 {
    width: 54%;
  }
  .row .col.w3p55, .row .p-col.w3p55, .row .phanterpwa-col.w3p55, .p-row .col.w3p55, .p-row .p-col.w3p55, .p-row .phanterpwa-col.w3p55, .phanterpwa-row .col.w3p55, .phanterpwa-row .p-col.w3p55, .phanterpwa-row .phanterpwa-col.w3p55 {
    width: 55%;
  }
  .row .col.w3p56, .row .p-col.w3p56, .row .phanterpwa-col.w3p56, .p-row .col.w3p56, .p-row .p-col.w3p56, .p-row .phanterpwa-col.w3p56, .phanterpwa-row .col.w3p56, .phanterpwa-row .p-col.w3p56, .phanterpwa-row .phanterpwa-col.w3p56 {
    width: 56%;
  }
  .row .col.w3p57, .row .p-col.w3p57, .row .phanterpwa-col.w3p57, .p-row .col.w3p57, .p-row .p-col.w3p57, .p-row .phanterpwa-col.w3p57, .phanterpwa-row .col.w3p57, .phanterpwa-row .p-col.w3p57, .phanterpwa-row .phanterpwa-col.w3p57 {
    width: 57%;
  }
  .row .col.w3p58, .row .p-col.w3p58, .row .phanterpwa-col.w3p58, .p-row .col.w3p58, .p-row .p-col.w3p58, .p-row .phanterpwa-col.w3p58, .phanterpwa-row .col.w3p58, .phanterpwa-row .p-col.w3p58, .phanterpwa-row .phanterpwa-col.w3p58 {
    width: 58%;
  }
  .row .col.w3p59, .row .p-col.w3p59, .row .phanterpwa-col.w3p59, .p-row .col.w3p59, .p-row .p-col.w3p59, .p-row .phanterpwa-col.w3p59, .phanterpwa-row .col.w3p59, .phanterpwa-row .p-col.w3p59, .phanterpwa-row .phanterpwa-col.w3p59 {
    width: 59%;
  }
  .row .col.w3p60, .row .p-col.w3p60, .row .phanterpwa-col.w3p60, .p-row .col.w3p60, .p-row .p-col.w3p60, .p-row .phanterpwa-col.w3p60, .phanterpwa-row .col.w3p60, .phanterpwa-row .p-col.w3p60, .phanterpwa-row .phanterpwa-col.w3p60 {
    width: 60%;
  }
  .row .col.w3p61, .row .p-col.w3p61, .row .phanterpwa-col.w3p61, .p-row .col.w3p61, .p-row .p-col.w3p61, .p-row .phanterpwa-col.w3p61, .phanterpwa-row .col.w3p61, .phanterpwa-row .p-col.w3p61, .phanterpwa-row .phanterpwa-col.w3p61 {
    width: 61%;
  }
  .row .col.w3p62, .row .p-col.w3p62, .row .phanterpwa-col.w3p62, .p-row .col.w3p62, .p-row .p-col.w3p62, .p-row .phanterpwa-col.w3p62, .phanterpwa-row .col.w3p62, .phanterpwa-row .p-col.w3p62, .phanterpwa-row .phanterpwa-col.w3p62 {
    width: 62%;
  }
  .row .col.w3p63, .row .p-col.w3p63, .row .phanterpwa-col.w3p63, .p-row .col.w3p63, .p-row .p-col.w3p63, .p-row .phanterpwa-col.w3p63, .phanterpwa-row .col.w3p63, .phanterpwa-row .p-col.w3p63, .phanterpwa-row .phanterpwa-col.w3p63 {
    width: 63%;
  }
  .row .col.w3p64, .row .p-col.w3p64, .row .phanterpwa-col.w3p64, .p-row .col.w3p64, .p-row .p-col.w3p64, .p-row .phanterpwa-col.w3p64, .phanterpwa-row .col.w3p64, .phanterpwa-row .p-col.w3p64, .phanterpwa-row .phanterpwa-col.w3p64 {
    width: 64%;
  }
  .row .col.w3p65, .row .p-col.w3p65, .row .phanterpwa-col.w3p65, .p-row .col.w3p65, .p-row .p-col.w3p65, .p-row .phanterpwa-col.w3p65, .phanterpwa-row .col.w3p65, .phanterpwa-row .p-col.w3p65, .phanterpwa-row .phanterpwa-col.w3p65 {
    width: 65%;
  }
  .row .col.w3p66, .row .p-col.w3p66, .row .phanterpwa-col.w3p66, .p-row .col.w3p66, .p-row .p-col.w3p66, .p-row .phanterpwa-col.w3p66, .phanterpwa-row .col.w3p66, .phanterpwa-row .p-col.w3p66, .phanterpwa-row .phanterpwa-col.w3p66 {
    width: 66%;
  }
  .row .col.w3p67, .row .p-col.w3p67, .row .phanterpwa-col.w3p67, .p-row .col.w3p67, .p-row .p-col.w3p67, .p-row .phanterpwa-col.w3p67, .phanterpwa-row .col.w3p67, .phanterpwa-row .p-col.w3p67, .phanterpwa-row .phanterpwa-col.w3p67 {
    width: 67%;
  }
  .row .col.w3p68, .row .p-col.w3p68, .row .phanterpwa-col.w3p68, .p-row .col.w3p68, .p-row .p-col.w3p68, .p-row .phanterpwa-col.w3p68, .phanterpwa-row .col.w3p68, .phanterpwa-row .p-col.w3p68, .phanterpwa-row .phanterpwa-col.w3p68 {
    width: 68%;
  }
  .row .col.w3p69, .row .p-col.w3p69, .row .phanterpwa-col.w3p69, .p-row .col.w3p69, .p-row .p-col.w3p69, .p-row .phanterpwa-col.w3p69, .phanterpwa-row .col.w3p69, .phanterpwa-row .p-col.w3p69, .phanterpwa-row .phanterpwa-col.w3p69 {
    width: 69%;
  }
  .row .col.w3p70, .row .p-col.w3p70, .row .phanterpwa-col.w3p70, .p-row .col.w3p70, .p-row .p-col.w3p70, .p-row .phanterpwa-col.w3p70, .phanterpwa-row .col.w3p70, .phanterpwa-row .p-col.w3p70, .phanterpwa-row .phanterpwa-col.w3p70 {
    width: 70%;
  }
  .row .col.w3p71, .row .p-col.w3p71, .row .phanterpwa-col.w3p71, .p-row .col.w3p71, .p-row .p-col.w3p71, .p-row .phanterpwa-col.w3p71, .phanterpwa-row .col.w3p71, .phanterpwa-row .p-col.w3p71, .phanterpwa-row .phanterpwa-col.w3p71 {
    width: 71%;
  }
  .row .col.w3p72, .row .p-col.w3p72, .row .phanterpwa-col.w3p72, .p-row .col.w3p72, .p-row .p-col.w3p72, .p-row .phanterpwa-col.w3p72, .phanterpwa-row .col.w3p72, .phanterpwa-row .p-col.w3p72, .phanterpwa-row .phanterpwa-col.w3p72 {
    width: 72%;
  }
  .row .col.w3p73, .row .p-col.w3p73, .row .phanterpwa-col.w3p73, .p-row .col.w3p73, .p-row .p-col.w3p73, .p-row .phanterpwa-col.w3p73, .phanterpwa-row .col.w3p73, .phanterpwa-row .p-col.w3p73, .phanterpwa-row .phanterpwa-col.w3p73 {
    width: 73%;
  }
  .row .col.w3p74, .row .p-col.w3p74, .row .phanterpwa-col.w3p74, .p-row .col.w3p74, .p-row .p-col.w3p74, .p-row .phanterpwa-col.w3p74, .phanterpwa-row .col.w3p74, .phanterpwa-row .p-col.w3p74, .phanterpwa-row .phanterpwa-col.w3p74 {
    width: 74%;
  }
  .row .col.w3p75, .row .p-col.w3p75, .row .phanterpwa-col.w3p75, .p-row .col.w3p75, .p-row .p-col.w3p75, .p-row .phanterpwa-col.w3p75, .phanterpwa-row .col.w3p75, .phanterpwa-row .p-col.w3p75, .phanterpwa-row .phanterpwa-col.w3p75 {
    width: 75%;
  }
  .row .col.w3p76, .row .p-col.w3p76, .row .phanterpwa-col.w3p76, .p-row .col.w3p76, .p-row .p-col.w3p76, .p-row .phanterpwa-col.w3p76, .phanterpwa-row .col.w3p76, .phanterpwa-row .p-col.w3p76, .phanterpwa-row .phanterpwa-col.w3p76 {
    width: 76%;
  }
  .row .col.w3p77, .row .p-col.w3p77, .row .phanterpwa-col.w3p77, .p-row .col.w3p77, .p-row .p-col.w3p77, .p-row .phanterpwa-col.w3p77, .phanterpwa-row .col.w3p77, .phanterpwa-row .p-col.w3p77, .phanterpwa-row .phanterpwa-col.w3p77 {
    width: 77%;
  }
  .row .col.w3p78, .row .p-col.w3p78, .row .phanterpwa-col.w3p78, .p-row .col.w3p78, .p-row .p-col.w3p78, .p-row .phanterpwa-col.w3p78, .phanterpwa-row .col.w3p78, .phanterpwa-row .p-col.w3p78, .phanterpwa-row .phanterpwa-col.w3p78 {
    width: 78%;
  }
  .row .col.w3p79, .row .p-col.w3p79, .row .phanterpwa-col.w3p79, .p-row .col.w3p79, .p-row .p-col.w3p79, .p-row .phanterpwa-col.w3p79, .phanterpwa-row .col.w3p79, .phanterpwa-row .p-col.w3p79, .phanterpwa-row .phanterpwa-col.w3p79 {
    width: 79%;
  }
  .row .col.w3p80, .row .p-col.w3p80, .row .phanterpwa-col.w3p80, .p-row .col.w3p80, .p-row .p-col.w3p80, .p-row .phanterpwa-col.w3p80, .phanterpwa-row .col.w3p80, .phanterpwa-row .p-col.w3p80, .phanterpwa-row .phanterpwa-col.w3p80 {
    width: 80%;
  }
  .row .col.w3p81, .row .p-col.w3p81, .row .phanterpwa-col.w3p81, .p-row .col.w3p81, .p-row .p-col.w3p81, .p-row .phanterpwa-col.w3p81, .phanterpwa-row .col.w3p81, .phanterpwa-row .p-col.w3p81, .phanterpwa-row .phanterpwa-col.w3p81 {
    width: 81%;
  }
  .row .col.w3p82, .row .p-col.w3p82, .row .phanterpwa-col.w3p82, .p-row .col.w3p82, .p-row .p-col.w3p82, .p-row .phanterpwa-col.w3p82, .phanterpwa-row .col.w3p82, .phanterpwa-row .p-col.w3p82, .phanterpwa-row .phanterpwa-col.w3p82 {
    width: 82%;
  }
  .row .col.w3p83, .row .p-col.w3p83, .row .phanterpwa-col.w3p83, .p-row .col.w3p83, .p-row .p-col.w3p83, .p-row .phanterpwa-col.w3p83, .phanterpwa-row .col.w3p83, .phanterpwa-row .p-col.w3p83, .phanterpwa-row .phanterpwa-col.w3p83 {
    width: 83%;
  }
  .row .col.w3p84, .row .p-col.w3p84, .row .phanterpwa-col.w3p84, .p-row .col.w3p84, .p-row .p-col.w3p84, .p-row .phanterpwa-col.w3p84, .phanterpwa-row .col.w3p84, .phanterpwa-row .p-col.w3p84, .phanterpwa-row .phanterpwa-col.w3p84 {
    width: 84%;
  }
  .row .col.w3p85, .row .p-col.w3p85, .row .phanterpwa-col.w3p85, .p-row .col.w3p85, .p-row .p-col.w3p85, .p-row .phanterpwa-col.w3p85, .phanterpwa-row .col.w3p85, .phanterpwa-row .p-col.w3p85, .phanterpwa-row .phanterpwa-col.w3p85 {
    width: 85%;
  }
  .row .col.w3p86, .row .p-col.w3p86, .row .phanterpwa-col.w3p86, .p-row .col.w3p86, .p-row .p-col.w3p86, .p-row .phanterpwa-col.w3p86, .phanterpwa-row .col.w3p86, .phanterpwa-row .p-col.w3p86, .phanterpwa-row .phanterpwa-col.w3p86 {
    width: 86%;
  }
  .row .col.w3p87, .row .p-col.w3p87, .row .phanterpwa-col.w3p87, .p-row .col.w3p87, .p-row .p-col.w3p87, .p-row .phanterpwa-col.w3p87, .phanterpwa-row .col.w3p87, .phanterpwa-row .p-col.w3p87, .phanterpwa-row .phanterpwa-col.w3p87 {
    width: 87%;
  }
  .row .col.w3p88, .row .p-col.w3p88, .row .phanterpwa-col.w3p88, .p-row .col.w3p88, .p-row .p-col.w3p88, .p-row .phanterpwa-col.w3p88, .phanterpwa-row .col.w3p88, .phanterpwa-row .p-col.w3p88, .phanterpwa-row .phanterpwa-col.w3p88 {
    width: 88%;
  }
  .row .col.w3p89, .row .p-col.w3p89, .row .phanterpwa-col.w3p89, .p-row .col.w3p89, .p-row .p-col.w3p89, .p-row .phanterpwa-col.w3p89, .phanterpwa-row .col.w3p89, .phanterpwa-row .p-col.w3p89, .phanterpwa-row .phanterpwa-col.w3p89 {
    width: 89%;
  }
  .row .col.w3p90, .row .p-col.w3p90, .row .phanterpwa-col.w3p90, .p-row .col.w3p90, .p-row .p-col.w3p90, .p-row .phanterpwa-col.w3p90, .phanterpwa-row .col.w3p90, .phanterpwa-row .p-col.w3p90, .phanterpwa-row .phanterpwa-col.w3p90 {
    width: 90%;
  }
  .row .col.w3p91, .row .p-col.w3p91, .row .phanterpwa-col.w3p91, .p-row .col.w3p91, .p-row .p-col.w3p91, .p-row .phanterpwa-col.w3p91, .phanterpwa-row .col.w3p91, .phanterpwa-row .p-col.w3p91, .phanterpwa-row .phanterpwa-col.w3p91 {
    width: 91%;
  }
  .row .col.w3p92, .row .p-col.w3p92, .row .phanterpwa-col.w3p92, .p-row .col.w3p92, .p-row .p-col.w3p92, .p-row .phanterpwa-col.w3p92, .phanterpwa-row .col.w3p92, .phanterpwa-row .p-col.w3p92, .phanterpwa-row .phanterpwa-col.w3p92 {
    width: 92%;
  }
  .row .col.w3p93, .row .p-col.w3p93, .row .phanterpwa-col.w3p93, .p-row .col.w3p93, .p-row .p-col.w3p93, .p-row .phanterpwa-col.w3p93, .phanterpwa-row .col.w3p93, .phanterpwa-row .p-col.w3p93, .phanterpwa-row .phanterpwa-col.w3p93 {
    width: 93%;
  }
  .row .col.w3p94, .row .p-col.w3p94, .row .phanterpwa-col.w3p94, .p-row .col.w3p94, .p-row .p-col.w3p94, .p-row .phanterpwa-col.w3p94, .phanterpwa-row .col.w3p94, .phanterpwa-row .p-col.w3p94, .phanterpwa-row .phanterpwa-col.w3p94 {
    width: 94%;
  }
  .row .col.w3p95, .row .p-col.w3p95, .row .phanterpwa-col.w3p95, .p-row .col.w3p95, .p-row .p-col.w3p95, .p-row .phanterpwa-col.w3p95, .phanterpwa-row .col.w3p95, .phanterpwa-row .p-col.w3p95, .phanterpwa-row .phanterpwa-col.w3p95 {
    width: 95%;
  }
  .row .col.w3p96, .row .p-col.w3p96, .row .phanterpwa-col.w3p96, .p-row .col.w3p96, .p-row .p-col.w3p96, .p-row .phanterpwa-col.w3p96, .phanterpwa-row .col.w3p96, .phanterpwa-row .p-col.w3p96, .phanterpwa-row .phanterpwa-col.w3p96 {
    width: 96%;
  }
  .row .col.w3p97, .row .p-col.w3p97, .row .phanterpwa-col.w3p97, .p-row .col.w3p97, .p-row .p-col.w3p97, .p-row .phanterpwa-col.w3p97, .phanterpwa-row .col.w3p97, .phanterpwa-row .p-col.w3p97, .phanterpwa-row .phanterpwa-col.w3p97 {
    width: 97%;
  }
  .row .col.w3p98, .row .p-col.w3p98, .row .phanterpwa-col.w3p98, .p-row .col.w3p98, .p-row .p-col.w3p98, .p-row .phanterpwa-col.w3p98, .phanterpwa-row .col.w3p98, .phanterpwa-row .p-col.w3p98, .phanterpwa-row .phanterpwa-col.w3p98 {
    width: 98%;
  }
  .row .col.w3p99, .row .p-col.w3p99, .row .phanterpwa-col.w3p99, .p-row .col.w3p99, .p-row .p-col.w3p99, .p-row .phanterpwa-col.w3p99, .phanterpwa-row .col.w3p99, .phanterpwa-row .p-col.w3p99, .phanterpwa-row .phanterpwa-col.w3p99 {
    width: 99%;
  }
  .row .col.w3p100, .row .p-col.w3p100, .row .phanterpwa-col.w3p100, .p-row .col.w3p100, .p-row .p-col.w3p100, .p-row .phanterpwa-col.w3p100, .phanterpwa-row .col.w3p100, .phanterpwa-row .p-col.w3p100, .phanterpwa-row .phanterpwa-col.w3p100 {
    width: 100%;
  }
}

@media screen and (min-width: 900px) {
  .row .col.w4p1, .row .p-col.w4p1, .row .phanterpwa-col.w4p1, .p-row .col.w4p1, .p-row .p-col.w4p1, .p-row .phanterpwa-col.w4p1, .phanterpwa-row .col.w4p1, .phanterpwa-row .p-col.w4p1, .phanterpwa-row .phanterpwa-col.w4p1 {
    width: 1%;
  }
  .row .col.w4p2, .row .p-col.w4p2, .row .phanterpwa-col.w4p2, .p-row .col.w4p2, .p-row .p-col.w4p2, .p-row .phanterpwa-col.w4p2, .phanterpwa-row .col.w4p2, .phanterpwa-row .p-col.w4p2, .phanterpwa-row .phanterpwa-col.w4p2 {
    width: 2%;
  }
  .row .col.w4p3, .row .p-col.w4p3, .row .phanterpwa-col.w4p3, .p-row .col.w4p3, .p-row .p-col.w4p3, .p-row .phanterpwa-col.w4p3, .phanterpwa-row .col.w4p3, .phanterpwa-row .p-col.w4p3, .phanterpwa-row .phanterpwa-col.w4p3 {
    width: 3%;
  }
  .row .col.w4p4, .row .p-col.w4p4, .row .phanterpwa-col.w4p4, .p-row .col.w4p4, .p-row .p-col.w4p4, .p-row .phanterpwa-col.w4p4, .phanterpwa-row .col.w4p4, .phanterpwa-row .p-col.w4p4, .phanterpwa-row .phanterpwa-col.w4p4 {
    width: 4%;
  }
  .row .col.w4p5, .row .p-col.w4p5, .row .phanterpwa-col.w4p5, .p-row .col.w4p5, .p-row .p-col.w4p5, .p-row .phanterpwa-col.w4p5, .phanterpwa-row .col.w4p5, .phanterpwa-row .p-col.w4p5, .phanterpwa-row .phanterpwa-col.w4p5 {
    width: 5%;
  }
  .row .col.w4p6, .row .p-col.w4p6, .row .phanterpwa-col.w4p6, .p-row .col.w4p6, .p-row .p-col.w4p6, .p-row .phanterpwa-col.w4p6, .phanterpwa-row .col.w4p6, .phanterpwa-row .p-col.w4p6, .phanterpwa-row .phanterpwa-col.w4p6 {
    width: 6%;
  }
  .row .col.w4p7, .row .p-col.w4p7, .row .phanterpwa-col.w4p7, .p-row .col.w4p7, .p-row .p-col.w4p7, .p-row .phanterpwa-col.w4p7, .phanterpwa-row .col.w4p7, .phanterpwa-row .p-col.w4p7, .phanterpwa-row .phanterpwa-col.w4p7 {
    width: 7%;
  }
  .row .col.w4p8, .row .p-col.w4p8, .row .phanterpwa-col.w4p8, .p-row .col.w4p8, .p-row .p-col.w4p8, .p-row .phanterpwa-col.w4p8, .phanterpwa-row .col.w4p8, .phanterpwa-row .p-col.w4p8, .phanterpwa-row .phanterpwa-col.w4p8 {
    width: 8%;
  }
  .row .col.w4p9, .row .p-col.w4p9, .row .phanterpwa-col.w4p9, .p-row .col.w4p9, .p-row .p-col.w4p9, .p-row .phanterpwa-col.w4p9, .phanterpwa-row .col.w4p9, .phanterpwa-row .p-col.w4p9, .phanterpwa-row .phanterpwa-col.w4p9 {
    width: 9%;
  }
  .row .col.w4p10, .row .p-col.w4p10, .row .phanterpwa-col.w4p10, .p-row .col.w4p10, .p-row .p-col.w4p10, .p-row .phanterpwa-col.w4p10, .phanterpwa-row .col.w4p10, .phanterpwa-row .p-col.w4p10, .phanterpwa-row .phanterpwa-col.w4p10 {
    width: 10%;
  }
  .row .col.w4p11, .row .p-col.w4p11, .row .phanterpwa-col.w4p11, .p-row .col.w4p11, .p-row .p-col.w4p11, .p-row .phanterpwa-col.w4p11, .phanterpwa-row .col.w4p11, .phanterpwa-row .p-col.w4p11, .phanterpwa-row .phanterpwa-col.w4p11 {
    width: 11%;
  }
  .row .col.w4p12, .row .p-col.w4p12, .row .phanterpwa-col.w4p12, .p-row .col.w4p12, .p-row .p-col.w4p12, .p-row .phanterpwa-col.w4p12, .phanterpwa-row .col.w4p12, .phanterpwa-row .p-col.w4p12, .phanterpwa-row .phanterpwa-col.w4p12 {
    width: 12%;
  }
  .row .col.w4p13, .row .p-col.w4p13, .row .phanterpwa-col.w4p13, .p-row .col.w4p13, .p-row .p-col.w4p13, .p-row .phanterpwa-col.w4p13, .phanterpwa-row .col.w4p13, .phanterpwa-row .p-col.w4p13, .phanterpwa-row .phanterpwa-col.w4p13 {
    width: 13%;
  }
  .row .col.w4p14, .row .p-col.w4p14, .row .phanterpwa-col.w4p14, .p-row .col.w4p14, .p-row .p-col.w4p14, .p-row .phanterpwa-col.w4p14, .phanterpwa-row .col.w4p14, .phanterpwa-row .p-col.w4p14, .phanterpwa-row .phanterpwa-col.w4p14 {
    width: 14%;
  }
  .row .col.w4p15, .row .p-col.w4p15, .row .phanterpwa-col.w4p15, .p-row .col.w4p15, .p-row .p-col.w4p15, .p-row .phanterpwa-col.w4p15, .phanterpwa-row .col.w4p15, .phanterpwa-row .p-col.w4p15, .phanterpwa-row .phanterpwa-col.w4p15 {
    width: 15%;
  }
  .row .col.w4p16, .row .p-col.w4p16, .row .phanterpwa-col.w4p16, .p-row .col.w4p16, .p-row .p-col.w4p16, .p-row .phanterpwa-col.w4p16, .phanterpwa-row .col.w4p16, .phanterpwa-row .p-col.w4p16, .phanterpwa-row .phanterpwa-col.w4p16 {
    width: 16%;
  }
  .row .col.w4p17, .row .p-col.w4p17, .row .phanterpwa-col.w4p17, .p-row .col.w4p17, .p-row .p-col.w4p17, .p-row .phanterpwa-col.w4p17, .phanterpwa-row .col.w4p17, .phanterpwa-row .p-col.w4p17, .phanterpwa-row .phanterpwa-col.w4p17 {
    width: 17%;
  }
  .row .col.w4p18, .row .p-col.w4p18, .row .phanterpwa-col.w4p18, .p-row .col.w4p18, .p-row .p-col.w4p18, .p-row .phanterpwa-col.w4p18, .phanterpwa-row .col.w4p18, .phanterpwa-row .p-col.w4p18, .phanterpwa-row .phanterpwa-col.w4p18 {
    width: 18%;
  }
  .row .col.w4p19, .row .p-col.w4p19, .row .phanterpwa-col.w4p19, .p-row .col.w4p19, .p-row .p-col.w4p19, .p-row .phanterpwa-col.w4p19, .phanterpwa-row .col.w4p19, .phanterpwa-row .p-col.w4p19, .phanterpwa-row .phanterpwa-col.w4p19 {
    width: 19%;
  }
  .row .col.w4p20, .row .p-col.w4p20, .row .phanterpwa-col.w4p20, .p-row .col.w4p20, .p-row .p-col.w4p20, .p-row .phanterpwa-col.w4p20, .phanterpwa-row .col.w4p20, .phanterpwa-row .p-col.w4p20, .phanterpwa-row .phanterpwa-col.w4p20 {
    width: 20%;
  }
  .row .col.w4p21, .row .p-col.w4p21, .row .phanterpwa-col.w4p21, .p-row .col.w4p21, .p-row .p-col.w4p21, .p-row .phanterpwa-col.w4p21, .phanterpwa-row .col.w4p21, .phanterpwa-row .p-col.w4p21, .phanterpwa-row .phanterpwa-col.w4p21 {
    width: 21%;
  }
  .row .col.w4p22, .row .p-col.w4p22, .row .phanterpwa-col.w4p22, .p-row .col.w4p22, .p-row .p-col.w4p22, .p-row .phanterpwa-col.w4p22, .phanterpwa-row .col.w4p22, .phanterpwa-row .p-col.w4p22, .phanterpwa-row .phanterpwa-col.w4p22 {
    width: 22%;
  }
  .row .col.w4p23, .row .p-col.w4p23, .row .phanterpwa-col.w4p23, .p-row .col.w4p23, .p-row .p-col.w4p23, .p-row .phanterpwa-col.w4p23, .phanterpwa-row .col.w4p23, .phanterpwa-row .p-col.w4p23, .phanterpwa-row .phanterpwa-col.w4p23 {
    width: 23%;
  }
  .row .col.w4p24, .row .p-col.w4p24, .row .phanterpwa-col.w4p24, .p-row .col.w4p24, .p-row .p-col.w4p24, .p-row .phanterpwa-col.w4p24, .phanterpwa-row .col.w4p24, .phanterpwa-row .p-col.w4p24, .phanterpwa-row .phanterpwa-col.w4p24 {
    width: 24%;
  }
  .row .col.w4p25, .row .p-col.w4p25, .row .phanterpwa-col.w4p25, .p-row .col.w4p25, .p-row .p-col.w4p25, .p-row .phanterpwa-col.w4p25, .phanterpwa-row .col.w4p25, .phanterpwa-row .p-col.w4p25, .phanterpwa-row .phanterpwa-col.w4p25 {
    width: 25%;
  }
  .row .col.w4p26, .row .p-col.w4p26, .row .phanterpwa-col.w4p26, .p-row .col.w4p26, .p-row .p-col.w4p26, .p-row .phanterpwa-col.w4p26, .phanterpwa-row .col.w4p26, .phanterpwa-row .p-col.w4p26, .phanterpwa-row .phanterpwa-col.w4p26 {
    width: 26%;
  }
  .row .col.w4p27, .row .p-col.w4p27, .row .phanterpwa-col.w4p27, .p-row .col.w4p27, .p-row .p-col.w4p27, .p-row .phanterpwa-col.w4p27, .phanterpwa-row .col.w4p27, .phanterpwa-row .p-col.w4p27, .phanterpwa-row .phanterpwa-col.w4p27 {
    width: 27%;
  }
  .row .col.w4p28, .row .p-col.w4p28, .row .phanterpwa-col.w4p28, .p-row .col.w4p28, .p-row .p-col.w4p28, .p-row .phanterpwa-col.w4p28, .phanterpwa-row .col.w4p28, .phanterpwa-row .p-col.w4p28, .phanterpwa-row .phanterpwa-col.w4p28 {
    width: 28%;
  }
  .row .col.w4p29, .row .p-col.w4p29, .row .phanterpwa-col.w4p29, .p-row .col.w4p29, .p-row .p-col.w4p29, .p-row .phanterpwa-col.w4p29, .phanterpwa-row .col.w4p29, .phanterpwa-row .p-col.w4p29, .phanterpwa-row .phanterpwa-col.w4p29 {
    width: 29%;
  }
  .row .col.w4p30, .row .p-col.w4p30, .row .phanterpwa-col.w4p30, .p-row .col.w4p30, .p-row .p-col.w4p30, .p-row .phanterpwa-col.w4p30, .phanterpwa-row .col.w4p30, .phanterpwa-row .p-col.w4p30, .phanterpwa-row .phanterpwa-col.w4p30 {
    width: 30%;
  }
  .row .col.w4p31, .row .p-col.w4p31, .row .phanterpwa-col.w4p31, .p-row .col.w4p31, .p-row .p-col.w4p31, .p-row .phanterpwa-col.w4p31, .phanterpwa-row .col.w4p31, .phanterpwa-row .p-col.w4p31, .phanterpwa-row .phanterpwa-col.w4p31 {
    width: 31%;
  }
  .row .col.w4p32, .row .p-col.w4p32, .row .phanterpwa-col.w4p32, .p-row .col.w4p32, .p-row .p-col.w4p32, .p-row .phanterpwa-col.w4p32, .phanterpwa-row .col.w4p32, .phanterpwa-row .p-col.w4p32, .phanterpwa-row .phanterpwa-col.w4p32 {
    width: 32%;
  }
  .row .col.w4p33, .row .p-col.w4p33, .row .phanterpwa-col.w4p33, .p-row .col.w4p33, .p-row .p-col.w4p33, .p-row .phanterpwa-col.w4p33, .phanterpwa-row .col.w4p33, .phanterpwa-row .p-col.w4p33, .phanterpwa-row .phanterpwa-col.w4p33 {
    width: 33%;
  }
  .row .col.w4p34, .row .p-col.w4p34, .row .phanterpwa-col.w4p34, .p-row .col.w4p34, .p-row .p-col.w4p34, .p-row .phanterpwa-col.w4p34, .phanterpwa-row .col.w4p34, .phanterpwa-row .p-col.w4p34, .phanterpwa-row .phanterpwa-col.w4p34 {
    width: 34%;
  }
  .row .col.w4p35, .row .p-col.w4p35, .row .phanterpwa-col.w4p35, .p-row .col.w4p35, .p-row .p-col.w4p35, .p-row .phanterpwa-col.w4p35, .phanterpwa-row .col.w4p35, .phanterpwa-row .p-col.w4p35, .phanterpwa-row .phanterpwa-col.w4p35 {
    width: 35%;
  }
  .row .col.w4p36, .row .p-col.w4p36, .row .phanterpwa-col.w4p36, .p-row .col.w4p36, .p-row .p-col.w4p36, .p-row .phanterpwa-col.w4p36, .phanterpwa-row .col.w4p36, .phanterpwa-row .p-col.w4p36, .phanterpwa-row .phanterpwa-col.w4p36 {
    width: 36%;
  }
  .row .col.w4p37, .row .p-col.w4p37, .row .phanterpwa-col.w4p37, .p-row .col.w4p37, .p-row .p-col.w4p37, .p-row .phanterpwa-col.w4p37, .phanterpwa-row .col.w4p37, .phanterpwa-row .p-col.w4p37, .phanterpwa-row .phanterpwa-col.w4p37 {
    width: 37%;
  }
  .row .col.w4p38, .row .p-col.w4p38, .row .phanterpwa-col.w4p38, .p-row .col.w4p38, .p-row .p-col.w4p38, .p-row .phanterpwa-col.w4p38, .phanterpwa-row .col.w4p38, .phanterpwa-row .p-col.w4p38, .phanterpwa-row .phanterpwa-col.w4p38 {
    width: 38%;
  }
  .row .col.w4p39, .row .p-col.w4p39, .row .phanterpwa-col.w4p39, .p-row .col.w4p39, .p-row .p-col.w4p39, .p-row .phanterpwa-col.w4p39, .phanterpwa-row .col.w4p39, .phanterpwa-row .p-col.w4p39, .phanterpwa-row .phanterpwa-col.w4p39 {
    width: 39%;
  }
  .row .col.w4p40, .row .p-col.w4p40, .row .phanterpwa-col.w4p40, .p-row .col.w4p40, .p-row .p-col.w4p40, .p-row .phanterpwa-col.w4p40, .phanterpwa-row .col.w4p40, .phanterpwa-row .p-col.w4p40, .phanterpwa-row .phanterpwa-col.w4p40 {
    width: 40%;
  }
  .row .col.w4p41, .row .p-col.w4p41, .row .phanterpwa-col.w4p41, .p-row .col.w4p41, .p-row .p-col.w4p41, .p-row .phanterpwa-col.w4p41, .phanterpwa-row .col.w4p41, .phanterpwa-row .p-col.w4p41, .phanterpwa-row .phanterpwa-col.w4p41 {
    width: 41%;
  }
  .row .col.w4p42, .row .p-col.w4p42, .row .phanterpwa-col.w4p42, .p-row .col.w4p42, .p-row .p-col.w4p42, .p-row .phanterpwa-col.w4p42, .phanterpwa-row .col.w4p42, .phanterpwa-row .p-col.w4p42, .phanterpwa-row .phanterpwa-col.w4p42 {
    width: 42%;
  }
  .row .col.w4p43, .row .p-col.w4p43, .row .phanterpwa-col.w4p43, .p-row .col.w4p43, .p-row .p-col.w4p43, .p-row .phanterpwa-col.w4p43, .phanterpwa-row .col.w4p43, .phanterpwa-row .p-col.w4p43, .phanterpwa-row .phanterpwa-col.w4p43 {
    width: 43%;
  }
  .row .col.w4p44, .row .p-col.w4p44, .row .phanterpwa-col.w4p44, .p-row .col.w4p44, .p-row .p-col.w4p44, .p-row .phanterpwa-col.w4p44, .phanterpwa-row .col.w4p44, .phanterpwa-row .p-col.w4p44, .phanterpwa-row .phanterpwa-col.w4p44 {
    width: 44%;
  }
  .row .col.w4p45, .row .p-col.w4p45, .row .phanterpwa-col.w4p45, .p-row .col.w4p45, .p-row .p-col.w4p45, .p-row .phanterpwa-col.w4p45, .phanterpwa-row .col.w4p45, .phanterpwa-row .p-col.w4p45, .phanterpwa-row .phanterpwa-col.w4p45 {
    width: 45%;
  }
  .row .col.w4p46, .row .p-col.w4p46, .row .phanterpwa-col.w4p46, .p-row .col.w4p46, .p-row .p-col.w4p46, .p-row .phanterpwa-col.w4p46, .phanterpwa-row .col.w4p46, .phanterpwa-row .p-col.w4p46, .phanterpwa-row .phanterpwa-col.w4p46 {
    width: 46%;
  }
  .row .col.w4p47, .row .p-col.w4p47, .row .phanterpwa-col.w4p47, .p-row .col.w4p47, .p-row .p-col.w4p47, .p-row .phanterpwa-col.w4p47, .phanterpwa-row .col.w4p47, .phanterpwa-row .p-col.w4p47, .phanterpwa-row .phanterpwa-col.w4p47 {
    width: 47%;
  }
  .row .col.w4p48, .row .p-col.w4p48, .row .phanterpwa-col.w4p48, .p-row .col.w4p48, .p-row .p-col.w4p48, .p-row .phanterpwa-col.w4p48, .phanterpwa-row .col.w4p48, .phanterpwa-row .p-col.w4p48, .phanterpwa-row .phanterpwa-col.w4p48 {
    width: 48%;
  }
  .row .col.w4p49, .row .p-col.w4p49, .row .phanterpwa-col.w4p49, .p-row .col.w4p49, .p-row .p-col.w4p49, .p-row .phanterpwa-col.w4p49, .phanterpwa-row .col.w4p49, .phanterpwa-row .p-col.w4p49, .phanterpwa-row .phanterpwa-col.w4p49 {
    width: 49%;
  }
  .row .col.w4p50, .row .p-col.w4p50, .row .phanterpwa-col.w4p50, .p-row .col.w4p50, .p-row .p-col.w4p50, .p-row .phanterpwa-col.w4p50, .phanterpwa-row .col.w4p50, .phanterpwa-row .p-col.w4p50, .phanterpwa-row .phanterpwa-col.w4p50 {
    width: 50%;
  }
  .row .col.w4p51, .row .p-col.w4p51, .row .phanterpwa-col.w4p51, .p-row .col.w4p51, .p-row .p-col.w4p51, .p-row .phanterpwa-col.w4p51, .phanterpwa-row .col.w4p51, .phanterpwa-row .p-col.w4p51, .phanterpwa-row .phanterpwa-col.w4p51 {
    width: 51%;
  }
  .row .col.w4p52, .row .p-col.w4p52, .row .phanterpwa-col.w4p52, .p-row .col.w4p52, .p-row .p-col.w4p52, .p-row .phanterpwa-col.w4p52, .phanterpwa-row .col.w4p52, .phanterpwa-row .p-col.w4p52, .phanterpwa-row .phanterpwa-col.w4p52 {
    width: 52%;
  }
  .row .col.w4p53, .row .p-col.w4p53, .row .phanterpwa-col.w4p53, .p-row .col.w4p53, .p-row .p-col.w4p53, .p-row .phanterpwa-col.w4p53, .phanterpwa-row .col.w4p53, .phanterpwa-row .p-col.w4p53, .phanterpwa-row .phanterpwa-col.w4p53 {
    width: 53%;
  }
  .row .col.w4p54, .row .p-col.w4p54, .row .phanterpwa-col.w4p54, .p-row .col.w4p54, .p-row .p-col.w4p54, .p-row .phanterpwa-col.w4p54, .phanterpwa-row .col.w4p54, .phanterpwa-row .p-col.w4p54, .phanterpwa-row .phanterpwa-col.w4p54 {
    width: 54%;
  }
  .row .col.w4p55, .row .p-col.w4p55, .row .phanterpwa-col.w4p55, .p-row .col.w4p55, .p-row .p-col.w4p55, .p-row .phanterpwa-col.w4p55, .phanterpwa-row .col.w4p55, .phanterpwa-row .p-col.w4p55, .phanterpwa-row .phanterpwa-col.w4p55 {
    width: 55%;
  }
  .row .col.w4p56, .row .p-col.w4p56, .row .phanterpwa-col.w4p56, .p-row .col.w4p56, .p-row .p-col.w4p56, .p-row .phanterpwa-col.w4p56, .phanterpwa-row .col.w4p56, .phanterpwa-row .p-col.w4p56, .phanterpwa-row .phanterpwa-col.w4p56 {
    width: 56%;
  }
  .row .col.w4p57, .row .p-col.w4p57, .row .phanterpwa-col.w4p57, .p-row .col.w4p57, .p-row .p-col.w4p57, .p-row .phanterpwa-col.w4p57, .phanterpwa-row .col.w4p57, .phanterpwa-row .p-col.w4p57, .phanterpwa-row .phanterpwa-col.w4p57 {
    width: 57%;
  }
  .row .col.w4p58, .row .p-col.w4p58, .row .phanterpwa-col.w4p58, .p-row .col.w4p58, .p-row .p-col.w4p58, .p-row .phanterpwa-col.w4p58, .phanterpwa-row .col.w4p58, .phanterpwa-row .p-col.w4p58, .phanterpwa-row .phanterpwa-col.w4p58 {
    width: 58%;
  }
  .row .col.w4p59, .row .p-col.w4p59, .row .phanterpwa-col.w4p59, .p-row .col.w4p59, .p-row .p-col.w4p59, .p-row .phanterpwa-col.w4p59, .phanterpwa-row .col.w4p59, .phanterpwa-row .p-col.w4p59, .phanterpwa-row .phanterpwa-col.w4p59 {
    width: 59%;
  }
  .row .col.w4p60, .row .p-col.w4p60, .row .phanterpwa-col.w4p60, .p-row .col.w4p60, .p-row .p-col.w4p60, .p-row .phanterpwa-col.w4p60, .phanterpwa-row .col.w4p60, .phanterpwa-row .p-col.w4p60, .phanterpwa-row .phanterpwa-col.w4p60 {
    width: 60%;
  }
  .row .col.w4p61, .row .p-col.w4p61, .row .phanterpwa-col.w4p61, .p-row .col.w4p61, .p-row .p-col.w4p61, .p-row .phanterpwa-col.w4p61, .phanterpwa-row .col.w4p61, .phanterpwa-row .p-col.w4p61, .phanterpwa-row .phanterpwa-col.w4p61 {
    width: 61%;
  }
  .row .col.w4p62, .row .p-col.w4p62, .row .phanterpwa-col.w4p62, .p-row .col.w4p62, .p-row .p-col.w4p62, .p-row .phanterpwa-col.w4p62, .phanterpwa-row .col.w4p62, .phanterpwa-row .p-col.w4p62, .phanterpwa-row .phanterpwa-col.w4p62 {
    width: 62%;
  }
  .row .col.w4p63, .row .p-col.w4p63, .row .phanterpwa-col.w4p63, .p-row .col.w4p63, .p-row .p-col.w4p63, .p-row .phanterpwa-col.w4p63, .phanterpwa-row .col.w4p63, .phanterpwa-row .p-col.w4p63, .phanterpwa-row .phanterpwa-col.w4p63 {
    width: 63%;
  }
  .row .col.w4p64, .row .p-col.w4p64, .row .phanterpwa-col.w4p64, .p-row .col.w4p64, .p-row .p-col.w4p64, .p-row .phanterpwa-col.w4p64, .phanterpwa-row .col.w4p64, .phanterpwa-row .p-col.w4p64, .phanterpwa-row .phanterpwa-col.w4p64 {
    width: 64%;
  }
  .row .col.w4p65, .row .p-col.w4p65, .row .phanterpwa-col.w4p65, .p-row .col.w4p65, .p-row .p-col.w4p65, .p-row .phanterpwa-col.w4p65, .phanterpwa-row .col.w4p65, .phanterpwa-row .p-col.w4p65, .phanterpwa-row .phanterpwa-col.w4p65 {
    width: 65%;
  }
  .row .col.w4p66, .row .p-col.w4p66, .row .phanterpwa-col.w4p66, .p-row .col.w4p66, .p-row .p-col.w4p66, .p-row .phanterpwa-col.w4p66, .phanterpwa-row .col.w4p66, .phanterpwa-row .p-col.w4p66, .phanterpwa-row .phanterpwa-col.w4p66 {
    width: 66%;
  }
  .row .col.w4p67, .row .p-col.w4p67, .row .phanterpwa-col.w4p67, .p-row .col.w4p67, .p-row .p-col.w4p67, .p-row .phanterpwa-col.w4p67, .phanterpwa-row .col.w4p67, .phanterpwa-row .p-col.w4p67, .phanterpwa-row .phanterpwa-col.w4p67 {
    width: 67%;
  }
  .row .col.w4p68, .row .p-col.w4p68, .row .phanterpwa-col.w4p68, .p-row .col.w4p68, .p-row .p-col.w4p68, .p-row .phanterpwa-col.w4p68, .phanterpwa-row .col.w4p68, .phanterpwa-row .p-col.w4p68, .phanterpwa-row .phanterpwa-col.w4p68 {
    width: 68%;
  }
  .row .col.w4p69, .row .p-col.w4p69, .row .phanterpwa-col.w4p69, .p-row .col.w4p69, .p-row .p-col.w4p69, .p-row .phanterpwa-col.w4p69, .phanterpwa-row .col.w4p69, .phanterpwa-row .p-col.w4p69, .phanterpwa-row .phanterpwa-col.w4p69 {
    width: 69%;
  }
  .row .col.w4p70, .row .p-col.w4p70, .row .phanterpwa-col.w4p70, .p-row .col.w4p70, .p-row .p-col.w4p70, .p-row .phanterpwa-col.w4p70, .phanterpwa-row .col.w4p70, .phanterpwa-row .p-col.w4p70, .phanterpwa-row .phanterpwa-col.w4p70 {
    width: 70%;
  }
  .row .col.w4p71, .row .p-col.w4p71, .row .phanterpwa-col.w4p71, .p-row .col.w4p71, .p-row .p-col.w4p71, .p-row .phanterpwa-col.w4p71, .phanterpwa-row .col.w4p71, .phanterpwa-row .p-col.w4p71, .phanterpwa-row .phanterpwa-col.w4p71 {
    width: 71%;
  }
  .row .col.w4p72, .row .p-col.w4p72, .row .phanterpwa-col.w4p72, .p-row .col.w4p72, .p-row .p-col.w4p72, .p-row .phanterpwa-col.w4p72, .phanterpwa-row .col.w4p72, .phanterpwa-row .p-col.w4p72, .phanterpwa-row .phanterpwa-col.w4p72 {
    width: 72%;
  }
  .row .col.w4p73, .row .p-col.w4p73, .row .phanterpwa-col.w4p73, .p-row .col.w4p73, .p-row .p-col.w4p73, .p-row .phanterpwa-col.w4p73, .phanterpwa-row .col.w4p73, .phanterpwa-row .p-col.w4p73, .phanterpwa-row .phanterpwa-col.w4p73 {
    width: 73%;
  }
  .row .col.w4p74, .row .p-col.w4p74, .row .phanterpwa-col.w4p74, .p-row .col.w4p74, .p-row .p-col.w4p74, .p-row .phanterpwa-col.w4p74, .phanterpwa-row .col.w4p74, .phanterpwa-row .p-col.w4p74, .phanterpwa-row .phanterpwa-col.w4p74 {
    width: 74%;
  }
  .row .col.w4p75, .row .p-col.w4p75, .row .phanterpwa-col.w4p75, .p-row .col.w4p75, .p-row .p-col.w4p75, .p-row .phanterpwa-col.w4p75, .phanterpwa-row .col.w4p75, .phanterpwa-row .p-col.w4p75, .phanterpwa-row .phanterpwa-col.w4p75 {
    width: 75%;
  }
  .row .col.w4p76, .row .p-col.w4p76, .row .phanterpwa-col.w4p76, .p-row .col.w4p76, .p-row .p-col.w4p76, .p-row .phanterpwa-col.w4p76, .phanterpwa-row .col.w4p76, .phanterpwa-row .p-col.w4p76, .phanterpwa-row .phanterpwa-col.w4p76 {
    width: 76%;
  }
  .row .col.w4p77, .row .p-col.w4p77, .row .phanterpwa-col.w4p77, .p-row .col.w4p77, .p-row .p-col.w4p77, .p-row .phanterpwa-col.w4p77, .phanterpwa-row .col.w4p77, .phanterpwa-row .p-col.w4p77, .phanterpwa-row .phanterpwa-col.w4p77 {
    width: 77%;
  }
  .row .col.w4p78, .row .p-col.w4p78, .row .phanterpwa-col.w4p78, .p-row .col.w4p78, .p-row .p-col.w4p78, .p-row .phanterpwa-col.w4p78, .phanterpwa-row .col.w4p78, .phanterpwa-row .p-col.w4p78, .phanterpwa-row .phanterpwa-col.w4p78 {
    width: 78%;
  }
  .row .col.w4p79, .row .p-col.w4p79, .row .phanterpwa-col.w4p79, .p-row .col.w4p79, .p-row .p-col.w4p79, .p-row .phanterpwa-col.w4p79, .phanterpwa-row .col.w4p79, .phanterpwa-row .p-col.w4p79, .phanterpwa-row .phanterpwa-col.w4p79 {
    width: 79%;
  }
  .row .col.w4p80, .row .p-col.w4p80, .row .phanterpwa-col.w4p80, .p-row .col.w4p80, .p-row .p-col.w4p80, .p-row .phanterpwa-col.w4p80, .phanterpwa-row .col.w4p80, .phanterpwa-row .p-col.w4p80, .phanterpwa-row .phanterpwa-col.w4p80 {
    width: 80%;
  }
  .row .col.w4p81, .row .p-col.w4p81, .row .phanterpwa-col.w4p81, .p-row .col.w4p81, .p-row .p-col.w4p81, .p-row .phanterpwa-col.w4p81, .phanterpwa-row .col.w4p81, .phanterpwa-row .p-col.w4p81, .phanterpwa-row .phanterpwa-col.w4p81 {
    width: 81%;
  }
  .row .col.w4p82, .row .p-col.w4p82, .row .phanterpwa-col.w4p82, .p-row .col.w4p82, .p-row .p-col.w4p82, .p-row .phanterpwa-col.w4p82, .phanterpwa-row .col.w4p82, .phanterpwa-row .p-col.w4p82, .phanterpwa-row .phanterpwa-col.w4p82 {
    width: 82%;
  }
  .row .col.w4p83, .row .p-col.w4p83, .row .phanterpwa-col.w4p83, .p-row .col.w4p83, .p-row .p-col.w4p83, .p-row .phanterpwa-col.w4p83, .phanterpwa-row .col.w4p83, .phanterpwa-row .p-col.w4p83, .phanterpwa-row .phanterpwa-col.w4p83 {
    width: 83%;
  }
  .row .col.w4p84, .row .p-col.w4p84, .row .phanterpwa-col.w4p84, .p-row .col.w4p84, .p-row .p-col.w4p84, .p-row .phanterpwa-col.w4p84, .phanterpwa-row .col.w4p84, .phanterpwa-row .p-col.w4p84, .phanterpwa-row .phanterpwa-col.w4p84 {
    width: 84%;
  }
  .row .col.w4p85, .row .p-col.w4p85, .row .phanterpwa-col.w4p85, .p-row .col.w4p85, .p-row .p-col.w4p85, .p-row .phanterpwa-col.w4p85, .phanterpwa-row .col.w4p85, .phanterpwa-row .p-col.w4p85, .phanterpwa-row .phanterpwa-col.w4p85 {
    width: 85%;
  }
  .row .col.w4p86, .row .p-col.w4p86, .row .phanterpwa-col.w4p86, .p-row .col.w4p86, .p-row .p-col.w4p86, .p-row .phanterpwa-col.w4p86, .phanterpwa-row .col.w4p86, .phanterpwa-row .p-col.w4p86, .phanterpwa-row .phanterpwa-col.w4p86 {
    width: 86%;
  }
  .row .col.w4p87, .row .p-col.w4p87, .row .phanterpwa-col.w4p87, .p-row .col.w4p87, .p-row .p-col.w4p87, .p-row .phanterpwa-col.w4p87, .phanterpwa-row .col.w4p87, .phanterpwa-row .p-col.w4p87, .phanterpwa-row .phanterpwa-col.w4p87 {
    width: 87%;
  }
  .row .col.w4p88, .row .p-col.w4p88, .row .phanterpwa-col.w4p88, .p-row .col.w4p88, .p-row .p-col.w4p88, .p-row .phanterpwa-col.w4p88, .phanterpwa-row .col.w4p88, .phanterpwa-row .p-col.w4p88, .phanterpwa-row .phanterpwa-col.w4p88 {
    width: 88%;
  }
  .row .col.w4p89, .row .p-col.w4p89, .row .phanterpwa-col.w4p89, .p-row .col.w4p89, .p-row .p-col.w4p89, .p-row .phanterpwa-col.w4p89, .phanterpwa-row .col.w4p89, .phanterpwa-row .p-col.w4p89, .phanterpwa-row .phanterpwa-col.w4p89 {
    width: 89%;
  }
  .row .col.w4p90, .row .p-col.w4p90, .row .phanterpwa-col.w4p90, .p-row .col.w4p90, .p-row .p-col.w4p90, .p-row .phanterpwa-col.w4p90, .phanterpwa-row .col.w4p90, .phanterpwa-row .p-col.w4p90, .phanterpwa-row .phanterpwa-col.w4p90 {
    width: 90%;
  }
  .row .col.w4p91, .row .p-col.w4p91, .row .phanterpwa-col.w4p91, .p-row .col.w4p91, .p-row .p-col.w4p91, .p-row .phanterpwa-col.w4p91, .phanterpwa-row .col.w4p91, .phanterpwa-row .p-col.w4p91, .phanterpwa-row .phanterpwa-col.w4p91 {
    width: 91%;
  }
  .row .col.w4p92, .row .p-col.w4p92, .row .phanterpwa-col.w4p92, .p-row .col.w4p92, .p-row .p-col.w4p92, .p-row .phanterpwa-col.w4p92, .phanterpwa-row .col.w4p92, .phanterpwa-row .p-col.w4p92, .phanterpwa-row .phanterpwa-col.w4p92 {
    width: 92%;
  }
  .row .col.w4p93, .row .p-col.w4p93, .row .phanterpwa-col.w4p93, .p-row .col.w4p93, .p-row .p-col.w4p93, .p-row .phanterpwa-col.w4p93, .phanterpwa-row .col.w4p93, .phanterpwa-row .p-col.w4p93, .phanterpwa-row .phanterpwa-col.w4p93 {
    width: 93%;
  }
  .row .col.w4p94, .row .p-col.w4p94, .row .phanterpwa-col.w4p94, .p-row .col.w4p94, .p-row .p-col.w4p94, .p-row .phanterpwa-col.w4p94, .phanterpwa-row .col.w4p94, .phanterpwa-row .p-col.w4p94, .phanterpwa-row .phanterpwa-col.w4p94 {
    width: 94%;
  }
  .row .col.w4p95, .row .p-col.w4p95, .row .phanterpwa-col.w4p95, .p-row .col.w4p95, .p-row .p-col.w4p95, .p-row .phanterpwa-col.w4p95, .phanterpwa-row .col.w4p95, .phanterpwa-row .p-col.w4p95, .phanterpwa-row .phanterpwa-col.w4p95 {
    width: 95%;
  }
  .row .col.w4p96, .row .p-col.w4p96, .row .phanterpwa-col.w4p96, .p-row .col.w4p96, .p-row .p-col.w4p96, .p-row .phanterpwa-col.w4p96, .phanterpwa-row .col.w4p96, .phanterpwa-row .p-col.w4p96, .phanterpwa-row .phanterpwa-col.w4p96 {
    width: 96%;
  }
  .row .col.w4p97, .row .p-col.w4p97, .row .phanterpwa-col.w4p97, .p-row .col.w4p97, .p-row .p-col.w4p97, .p-row .phanterpwa-col.w4p97, .phanterpwa-row .col.w4p97, .phanterpwa-row .p-col.w4p97, .phanterpwa-row .phanterpwa-col.w4p97 {
    width: 97%;
  }
  .row .col.w4p98, .row .p-col.w4p98, .row .phanterpwa-col.w4p98, .p-row .col.w4p98, .p-row .p-col.w4p98, .p-row .phanterpwa-col.w4p98, .phanterpwa-row .col.w4p98, .phanterpwa-row .p-col.w4p98, .phanterpwa-row .phanterpwa-col.w4p98 {
    width: 98%;
  }
  .row .col.w4p99, .row .p-col.w4p99, .row .phanterpwa-col.w4p99, .p-row .col.w4p99, .p-row .p-col.w4p99, .p-row .phanterpwa-col.w4p99, .phanterpwa-row .col.w4p99, .phanterpwa-row .p-col.w4p99, .phanterpwa-row .phanterpwa-col.w4p99 {
    width: 99%;
  }
  .row .col.w4p100, .row .p-col.w4p100, .row .phanterpwa-col.w4p100, .p-row .col.w4p100, .p-row .p-col.w4p100, .p-row .phanterpwa-col.w4p100, .phanterpwa-row .col.w4p100, .phanterpwa-row .p-col.w4p100, .phanterpwa-row .phanterpwa-col.w4p100 {
    width: 100%;
  }
}

@media screen and (min-width: 1200px) {
  .row .col.w5p1, .row .p-col.w5p1, .row .phanterpwa-col.w5p1, .p-row .col.w5p1, .p-row .p-col.w5p1, .p-row .phanterpwa-col.w5p1, .phanterpwa-row .col.w5p1, .phanterpwa-row .p-col.w5p1, .phanterpwa-row .phanterpwa-col.w5p1 {
    width: 1%;
  }
  .row .col.w5p2, .row .p-col.w5p2, .row .phanterpwa-col.w5p2, .p-row .col.w5p2, .p-row .p-col.w5p2, .p-row .phanterpwa-col.w5p2, .phanterpwa-row .col.w5p2, .phanterpwa-row .p-col.w5p2, .phanterpwa-row .phanterpwa-col.w5p2 {
    width: 2%;
  }
  .row .col.w5p3, .row .p-col.w5p3, .row .phanterpwa-col.w5p3, .p-row .col.w5p3, .p-row .p-col.w5p3, .p-row .phanterpwa-col.w5p3, .phanterpwa-row .col.w5p3, .phanterpwa-row .p-col.w5p3, .phanterpwa-row .phanterpwa-col.w5p3 {
    width: 3%;
  }
  .row .col.w5p4, .row .p-col.w5p4, .row .phanterpwa-col.w5p4, .p-row .col.w5p4, .p-row .p-col.w5p4, .p-row .phanterpwa-col.w5p4, .phanterpwa-row .col.w5p4, .phanterpwa-row .p-col.w5p4, .phanterpwa-row .phanterpwa-col.w5p4 {
    width: 4%;
  }
  .row .col.w5p5, .row .p-col.w5p5, .row .phanterpwa-col.w5p5, .p-row .col.w5p5, .p-row .p-col.w5p5, .p-row .phanterpwa-col.w5p5, .phanterpwa-row .col.w5p5, .phanterpwa-row .p-col.w5p5, .phanterpwa-row .phanterpwa-col.w5p5 {
    width: 5%;
  }
  .row .col.w5p6, .row .p-col.w5p6, .row .phanterpwa-col.w5p6, .p-row .col.w5p6, .p-row .p-col.w5p6, .p-row .phanterpwa-col.w5p6, .phanterpwa-row .col.w5p6, .phanterpwa-row .p-col.w5p6, .phanterpwa-row .phanterpwa-col.w5p6 {
    width: 6%;
  }
  .row .col.w5p7, .row .p-col.w5p7, .row .phanterpwa-col.w5p7, .p-row .col.w5p7, .p-row .p-col.w5p7, .p-row .phanterpwa-col.w5p7, .phanterpwa-row .col.w5p7, .phanterpwa-row .p-col.w5p7, .phanterpwa-row .phanterpwa-col.w5p7 {
    width: 7%;
  }
  .row .col.w5p8, .row .p-col.w5p8, .row .phanterpwa-col.w5p8, .p-row .col.w5p8, .p-row .p-col.w5p8, .p-row .phanterpwa-col.w5p8, .phanterpwa-row .col.w5p8, .phanterpwa-row .p-col.w5p8, .phanterpwa-row .phanterpwa-col.w5p8 {
    width: 8%;
  }
  .row .col.w5p9, .row .p-col.w5p9, .row .phanterpwa-col.w5p9, .p-row .col.w5p9, .p-row .p-col.w5p9, .p-row .phanterpwa-col.w5p9, .phanterpwa-row .col.w5p9, .phanterpwa-row .p-col.w5p9, .phanterpwa-row .phanterpwa-col.w5p9 {
    width: 9%;
  }
  .row .col.w5p10, .row .p-col.w5p10, .row .phanterpwa-col.w5p10, .p-row .col.w5p10, .p-row .p-col.w5p10, .p-row .phanterpwa-col.w5p10, .phanterpwa-row .col.w5p10, .phanterpwa-row .p-col.w5p10, .phanterpwa-row .phanterpwa-col.w5p10 {
    width: 10%;
  }
  .row .col.w5p11, .row .p-col.w5p11, .row .phanterpwa-col.w5p11, .p-row .col.w5p11, .p-row .p-col.w5p11, .p-row .phanterpwa-col.w5p11, .phanterpwa-row .col.w5p11, .phanterpwa-row .p-col.w5p11, .phanterpwa-row .phanterpwa-col.w5p11 {
    width: 11%;
  }
  .row .col.w5p12, .row .p-col.w5p12, .row .phanterpwa-col.w5p12, .p-row .col.w5p12, .p-row .p-col.w5p12, .p-row .phanterpwa-col.w5p12, .phanterpwa-row .col.w5p12, .phanterpwa-row .p-col.w5p12, .phanterpwa-row .phanterpwa-col.w5p12 {
    width: 12%;
  }
  .row .col.w5p13, .row .p-col.w5p13, .row .phanterpwa-col.w5p13, .p-row .col.w5p13, .p-row .p-col.w5p13, .p-row .phanterpwa-col.w5p13, .phanterpwa-row .col.w5p13, .phanterpwa-row .p-col.w5p13, .phanterpwa-row .phanterpwa-col.w5p13 {
    width: 13%;
  }
  .row .col.w5p14, .row .p-col.w5p14, .row .phanterpwa-col.w5p14, .p-row .col.w5p14, .p-row .p-col.w5p14, .p-row .phanterpwa-col.w5p14, .phanterpwa-row .col.w5p14, .phanterpwa-row .p-col.w5p14, .phanterpwa-row .phanterpwa-col.w5p14 {
    width: 14%;
  }
  .row .col.w5p15, .row .p-col.w5p15, .row .phanterpwa-col.w5p15, .p-row .col.w5p15, .p-row .p-col.w5p15, .p-row .phanterpwa-col.w5p15, .phanterpwa-row .col.w5p15, .phanterpwa-row .p-col.w5p15, .phanterpwa-row .phanterpwa-col.w5p15 {
    width: 15%;
  }
  .row .col.w5p16, .row .p-col.w5p16, .row .phanterpwa-col.w5p16, .p-row .col.w5p16, .p-row .p-col.w5p16, .p-row .phanterpwa-col.w5p16, .phanterpwa-row .col.w5p16, .phanterpwa-row .p-col.w5p16, .phanterpwa-row .phanterpwa-col.w5p16 {
    width: 16%;
  }
  .row .col.w5p17, .row .p-col.w5p17, .row .phanterpwa-col.w5p17, .p-row .col.w5p17, .p-row .p-col.w5p17, .p-row .phanterpwa-col.w5p17, .phanterpwa-row .col.w5p17, .phanterpwa-row .p-col.w5p17, .phanterpwa-row .phanterpwa-col.w5p17 {
    width: 17%;
  }
  .row .col.w5p18, .row .p-col.w5p18, .row .phanterpwa-col.w5p18, .p-row .col.w5p18, .p-row .p-col.w5p18, .p-row .phanterpwa-col.w5p18, .phanterpwa-row .col.w5p18, .phanterpwa-row .p-col.w5p18, .phanterpwa-row .phanterpwa-col.w5p18 {
    width: 18%;
  }
  .row .col.w5p19, .row .p-col.w5p19, .row .phanterpwa-col.w5p19, .p-row .col.w5p19, .p-row .p-col.w5p19, .p-row .phanterpwa-col.w5p19, .phanterpwa-row .col.w5p19, .phanterpwa-row .p-col.w5p19, .phanterpwa-row .phanterpwa-col.w5p19 {
    width: 19%;
  }
  .row .col.w5p20, .row .p-col.w5p20, .row .phanterpwa-col.w5p20, .p-row .col.w5p20, .p-row .p-col.w5p20, .p-row .phanterpwa-col.w5p20, .phanterpwa-row .col.w5p20, .phanterpwa-row .p-col.w5p20, .phanterpwa-row .phanterpwa-col.w5p20 {
    width: 20%;
  }
  .row .col.w5p21, .row .p-col.w5p21, .row .phanterpwa-col.w5p21, .p-row .col.w5p21, .p-row .p-col.w5p21, .p-row .phanterpwa-col.w5p21, .phanterpwa-row .col.w5p21, .phanterpwa-row .p-col.w5p21, .phanterpwa-row .phanterpwa-col.w5p21 {
    width: 21%;
  }
  .row .col.w5p22, .row .p-col.w5p22, .row .phanterpwa-col.w5p22, .p-row .col.w5p22, .p-row .p-col.w5p22, .p-row .phanterpwa-col.w5p22, .phanterpwa-row .col.w5p22, .phanterpwa-row .p-col.w5p22, .phanterpwa-row .phanterpwa-col.w5p22 {
    width: 22%;
  }
  .row .col.w5p23, .row .p-col.w5p23, .row .phanterpwa-col.w5p23, .p-row .col.w5p23, .p-row .p-col.w5p23, .p-row .phanterpwa-col.w5p23, .phanterpwa-row .col.w5p23, .phanterpwa-row .p-col.w5p23, .phanterpwa-row .phanterpwa-col.w5p23 {
    width: 23%;
  }
  .row .col.w5p24, .row .p-col.w5p24, .row .phanterpwa-col.w5p24, .p-row .col.w5p24, .p-row .p-col.w5p24, .p-row .phanterpwa-col.w5p24, .phanterpwa-row .col.w5p24, .phanterpwa-row .p-col.w5p24, .phanterpwa-row .phanterpwa-col.w5p24 {
    width: 24%;
  }
  .row .col.w5p25, .row .p-col.w5p25, .row .phanterpwa-col.w5p25, .p-row .col.w5p25, .p-row .p-col.w5p25, .p-row .phanterpwa-col.w5p25, .phanterpwa-row .col.w5p25, .phanterpwa-row .p-col.w5p25, .phanterpwa-row .phanterpwa-col.w5p25 {
    width: 25%;
  }
  .row .col.w5p26, .row .p-col.w5p26, .row .phanterpwa-col.w5p26, .p-row .col.w5p26, .p-row .p-col.w5p26, .p-row .phanterpwa-col.w5p26, .phanterpwa-row .col.w5p26, .phanterpwa-row .p-col.w5p26, .phanterpwa-row .phanterpwa-col.w5p26 {
    width: 26%;
  }
  .row .col.w5p27, .row .p-col.w5p27, .row .phanterpwa-col.w5p27, .p-row .col.w5p27, .p-row .p-col.w5p27, .p-row .phanterpwa-col.w5p27, .phanterpwa-row .col.w5p27, .phanterpwa-row .p-col.w5p27, .phanterpwa-row .phanterpwa-col.w5p27 {
    width: 27%;
  }
  .row .col.w5p28, .row .p-col.w5p28, .row .phanterpwa-col.w5p28, .p-row .col.w5p28, .p-row .p-col.w5p28, .p-row .phanterpwa-col.w5p28, .phanterpwa-row .col.w5p28, .phanterpwa-row .p-col.w5p28, .phanterpwa-row .phanterpwa-col.w5p28 {
    width: 28%;
  }
  .row .col.w5p29, .row .p-col.w5p29, .row .phanterpwa-col.w5p29, .p-row .col.w5p29, .p-row .p-col.w5p29, .p-row .phanterpwa-col.w5p29, .phanterpwa-row .col.w5p29, .phanterpwa-row .p-col.w5p29, .phanterpwa-row .phanterpwa-col.w5p29 {
    width: 29%;
  }
  .row .col.w5p30, .row .p-col.w5p30, .row .phanterpwa-col.w5p30, .p-row .col.w5p30, .p-row .p-col.w5p30, .p-row .phanterpwa-col.w5p30, .phanterpwa-row .col.w5p30, .phanterpwa-row .p-col.w5p30, .phanterpwa-row .phanterpwa-col.w5p30 {
    width: 30%;
  }
  .row .col.w5p31, .row .p-col.w5p31, .row .phanterpwa-col.w5p31, .p-row .col.w5p31, .p-row .p-col.w5p31, .p-row .phanterpwa-col.w5p31, .phanterpwa-row .col.w5p31, .phanterpwa-row .p-col.w5p31, .phanterpwa-row .phanterpwa-col.w5p31 {
    width: 31%;
  }
  .row .col.w5p32, .row .p-col.w5p32, .row .phanterpwa-col.w5p32, .p-row .col.w5p32, .p-row .p-col.w5p32, .p-row .phanterpwa-col.w5p32, .phanterpwa-row .col.w5p32, .phanterpwa-row .p-col.w5p32, .phanterpwa-row .phanterpwa-col.w5p32 {
    width: 32%;
  }
  .row .col.w5p33, .row .p-col.w5p33, .row .phanterpwa-col.w5p33, .p-row .col.w5p33, .p-row .p-col.w5p33, .p-row .phanterpwa-col.w5p33, .phanterpwa-row .col.w5p33, .phanterpwa-row .p-col.w5p33, .phanterpwa-row .phanterpwa-col.w5p33 {
    width: 33%;
  }
  .row .col.w5p34, .row .p-col.w5p34, .row .phanterpwa-col.w5p34, .p-row .col.w5p34, .p-row .p-col.w5p34, .p-row .phanterpwa-col.w5p34, .phanterpwa-row .col.w5p34, .phanterpwa-row .p-col.w5p34, .phanterpwa-row .phanterpwa-col.w5p34 {
    width: 34%;
  }
  .row .col.w5p35, .row .p-col.w5p35, .row .phanterpwa-col.w5p35, .p-row .col.w5p35, .p-row .p-col.w5p35, .p-row .phanterpwa-col.w5p35, .phanterpwa-row .col.w5p35, .phanterpwa-row .p-col.w5p35, .phanterpwa-row .phanterpwa-col.w5p35 {
    width: 35%;
  }
  .row .col.w5p36, .row .p-col.w5p36, .row .phanterpwa-col.w5p36, .p-row .col.w5p36, .p-row .p-col.w5p36, .p-row .phanterpwa-col.w5p36, .phanterpwa-row .col.w5p36, .phanterpwa-row .p-col.w5p36, .phanterpwa-row .phanterpwa-col.w5p36 {
    width: 36%;
  }
  .row .col.w5p37, .row .p-col.w5p37, .row .phanterpwa-col.w5p37, .p-row .col.w5p37, .p-row .p-col.w5p37, .p-row .phanterpwa-col.w5p37, .phanterpwa-row .col.w5p37, .phanterpwa-row .p-col.w5p37, .phanterpwa-row .phanterpwa-col.w5p37 {
    width: 37%;
  }
  .row .col.w5p38, .row .p-col.w5p38, .row .phanterpwa-col.w5p38, .p-row .col.w5p38, .p-row .p-col.w5p38, .p-row .phanterpwa-col.w5p38, .phanterpwa-row .col.w5p38, .phanterpwa-row .p-col.w5p38, .phanterpwa-row .phanterpwa-col.w5p38 {
    width: 38%;
  }
  .row .col.w5p39, .row .p-col.w5p39, .row .phanterpwa-col.w5p39, .p-row .col.w5p39, .p-row .p-col.w5p39, .p-row .phanterpwa-col.w5p39, .phanterpwa-row .col.w5p39, .phanterpwa-row .p-col.w5p39, .phanterpwa-row .phanterpwa-col.w5p39 {
    width: 39%;
  }
  .row .col.w5p40, .row .p-col.w5p40, .row .phanterpwa-col.w5p40, .p-row .col.w5p40, .p-row .p-col.w5p40, .p-row .phanterpwa-col.w5p40, .phanterpwa-row .col.w5p40, .phanterpwa-row .p-col.w5p40, .phanterpwa-row .phanterpwa-col.w5p40 {
    width: 40%;
  }
  .row .col.w5p41, .row .p-col.w5p41, .row .phanterpwa-col.w5p41, .p-row .col.w5p41, .p-row .p-col.w5p41, .p-row .phanterpwa-col.w5p41, .phanterpwa-row .col.w5p41, .phanterpwa-row .p-col.w5p41, .phanterpwa-row .phanterpwa-col.w5p41 {
    width: 41%;
  }
  .row .col.w5p42, .row .p-col.w5p42, .row .phanterpwa-col.w5p42, .p-row .col.w5p42, .p-row .p-col.w5p42, .p-row .phanterpwa-col.w5p42, .phanterpwa-row .col.w5p42, .phanterpwa-row .p-col.w5p42, .phanterpwa-row .phanterpwa-col.w5p42 {
    width: 42%;
  }
  .row .col.w5p43, .row .p-col.w5p43, .row .phanterpwa-col.w5p43, .p-row .col.w5p43, .p-row .p-col.w5p43, .p-row .phanterpwa-col.w5p43, .phanterpwa-row .col.w5p43, .phanterpwa-row .p-col.w5p43, .phanterpwa-row .phanterpwa-col.w5p43 {
    width: 43%;
  }
  .row .col.w5p44, .row .p-col.w5p44, .row .phanterpwa-col.w5p44, .p-row .col.w5p44, .p-row .p-col.w5p44, .p-row .phanterpwa-col.w5p44, .phanterpwa-row .col.w5p44, .phanterpwa-row .p-col.w5p44, .phanterpwa-row .phanterpwa-col.w5p44 {
    width: 44%;
  }
  .row .col.w5p45, .row .p-col.w5p45, .row .phanterpwa-col.w5p45, .p-row .col.w5p45, .p-row .p-col.w5p45, .p-row .phanterpwa-col.w5p45, .phanterpwa-row .col.w5p45, .phanterpwa-row .p-col.w5p45, .phanterpwa-row .phanterpwa-col.w5p45 {
    width: 45%;
  }
  .row .col.w5p46, .row .p-col.w5p46, .row .phanterpwa-col.w5p46, .p-row .col.w5p46, .p-row .p-col.w5p46, .p-row .phanterpwa-col.w5p46, .phanterpwa-row .col.w5p46, .phanterpwa-row .p-col.w5p46, .phanterpwa-row .phanterpwa-col.w5p46 {
    width: 46%;
  }
  .row .col.w5p47, .row .p-col.w5p47, .row .phanterpwa-col.w5p47, .p-row .col.w5p47, .p-row .p-col.w5p47, .p-row .phanterpwa-col.w5p47, .phanterpwa-row .col.w5p47, .phanterpwa-row .p-col.w5p47, .phanterpwa-row .phanterpwa-col.w5p47 {
    width: 47%;
  }
  .row .col.w5p48, .row .p-col.w5p48, .row .phanterpwa-col.w5p48, .p-row .col.w5p48, .p-row .p-col.w5p48, .p-row .phanterpwa-col.w5p48, .phanterpwa-row .col.w5p48, .phanterpwa-row .p-col.w5p48, .phanterpwa-row .phanterpwa-col.w5p48 {
    width: 48%;
  }
  .row .col.w5p49, .row .p-col.w5p49, .row .phanterpwa-col.w5p49, .p-row .col.w5p49, .p-row .p-col.w5p49, .p-row .phanterpwa-col.w5p49, .phanterpwa-row .col.w5p49, .phanterpwa-row .p-col.w5p49, .phanterpwa-row .phanterpwa-col.w5p49 {
    width: 49%;
  }
  .row .col.w5p50, .row .p-col.w5p50, .row .phanterpwa-col.w5p50, .p-row .col.w5p50, .p-row .p-col.w5p50, .p-row .phanterpwa-col.w5p50, .phanterpwa-row .col.w5p50, .phanterpwa-row .p-col.w5p50, .phanterpwa-row .phanterpwa-col.w5p50 {
    width: 50%;
  }
  .row .col.w5p51, .row .p-col.w5p51, .row .phanterpwa-col.w5p51, .p-row .col.w5p51, .p-row .p-col.w5p51, .p-row .phanterpwa-col.w5p51, .phanterpwa-row .col.w5p51, .phanterpwa-row .p-col.w5p51, .phanterpwa-row .phanterpwa-col.w5p51 {
    width: 51%;
  }
  .row .col.w5p52, .row .p-col.w5p52, .row .phanterpwa-col.w5p52, .p-row .col.w5p52, .p-row .p-col.w5p52, .p-row .phanterpwa-col.w5p52, .phanterpwa-row .col.w5p52, .phanterpwa-row .p-col.w5p52, .phanterpwa-row .phanterpwa-col.w5p52 {
    width: 52%;
  }
  .row .col.w5p53, .row .p-col.w5p53, .row .phanterpwa-col.w5p53, .p-row .col.w5p53, .p-row .p-col.w5p53, .p-row .phanterpwa-col.w5p53, .phanterpwa-row .col.w5p53, .phanterpwa-row .p-col.w5p53, .phanterpwa-row .phanterpwa-col.w5p53 {
    width: 53%;
  }
  .row .col.w5p54, .row .p-col.w5p54, .row .phanterpwa-col.w5p54, .p-row .col.w5p54, .p-row .p-col.w5p54, .p-row .phanterpwa-col.w5p54, .phanterpwa-row .col.w5p54, .phanterpwa-row .p-col.w5p54, .phanterpwa-row .phanterpwa-col.w5p54 {
    width: 54%;
  }
  .row .col.w5p55, .row .p-col.w5p55, .row .phanterpwa-col.w5p55, .p-row .col.w5p55, .p-row .p-col.w5p55, .p-row .phanterpwa-col.w5p55, .phanterpwa-row .col.w5p55, .phanterpwa-row .p-col.w5p55, .phanterpwa-row .phanterpwa-col.w5p55 {
    width: 55%;
  }
  .row .col.w5p56, .row .p-col.w5p56, .row .phanterpwa-col.w5p56, .p-row .col.w5p56, .p-row .p-col.w5p56, .p-row .phanterpwa-col.w5p56, .phanterpwa-row .col.w5p56, .phanterpwa-row .p-col.w5p56, .phanterpwa-row .phanterpwa-col.w5p56 {
    width: 56%;
  }
  .row .col.w5p57, .row .p-col.w5p57, .row .phanterpwa-col.w5p57, .p-row .col.w5p57, .p-row .p-col.w5p57, .p-row .phanterpwa-col.w5p57, .phanterpwa-row .col.w5p57, .phanterpwa-row .p-col.w5p57, .phanterpwa-row .phanterpwa-col.w5p57 {
    width: 57%;
  }
  .row .col.w5p58, .row .p-col.w5p58, .row .phanterpwa-col.w5p58, .p-row .col.w5p58, .p-row .p-col.w5p58, .p-row .phanterpwa-col.w5p58, .phanterpwa-row .col.w5p58, .phanterpwa-row .p-col.w5p58, .phanterpwa-row .phanterpwa-col.w5p58 {
    width: 58%;
  }
  .row .col.w5p59, .row .p-col.w5p59, .row .phanterpwa-col.w5p59, .p-row .col.w5p59, .p-row .p-col.w5p59, .p-row .phanterpwa-col.w5p59, .phanterpwa-row .col.w5p59, .phanterpwa-row .p-col.w5p59, .phanterpwa-row .phanterpwa-col.w5p59 {
    width: 59%;
  }
  .row .col.w5p60, .row .p-col.w5p60, .row .phanterpwa-col.w5p60, .p-row .col.w5p60, .p-row .p-col.w5p60, .p-row .phanterpwa-col.w5p60, .phanterpwa-row .col.w5p60, .phanterpwa-row .p-col.w5p60, .phanterpwa-row .phanterpwa-col.w5p60 {
    width: 60%;
  }
  .row .col.w5p61, .row .p-col.w5p61, .row .phanterpwa-col.w5p61, .p-row .col.w5p61, .p-row .p-col.w5p61, .p-row .phanterpwa-col.w5p61, .phanterpwa-row .col.w5p61, .phanterpwa-row .p-col.w5p61, .phanterpwa-row .phanterpwa-col.w5p61 {
    width: 61%;
  }
  .row .col.w5p62, .row .p-col.w5p62, .row .phanterpwa-col.w5p62, .p-row .col.w5p62, .p-row .p-col.w5p62, .p-row .phanterpwa-col.w5p62, .phanterpwa-row .col.w5p62, .phanterpwa-row .p-col.w5p62, .phanterpwa-row .phanterpwa-col.w5p62 {
    width: 62%;
  }
  .row .col.w5p63, .row .p-col.w5p63, .row .phanterpwa-col.w5p63, .p-row .col.w5p63, .p-row .p-col.w5p63, .p-row .phanterpwa-col.w5p63, .phanterpwa-row .col.w5p63, .phanterpwa-row .p-col.w5p63, .phanterpwa-row .phanterpwa-col.w5p63 {
    width: 63%;
  }
  .row .col.w5p64, .row .p-col.w5p64, .row .phanterpwa-col.w5p64, .p-row .col.w5p64, .p-row .p-col.w5p64, .p-row .phanterpwa-col.w5p64, .phanterpwa-row .col.w5p64, .phanterpwa-row .p-col.w5p64, .phanterpwa-row .phanterpwa-col.w5p64 {
    width: 64%;
  }
  .row .col.w5p65, .row .p-col.w5p65, .row .phanterpwa-col.w5p65, .p-row .col.w5p65, .p-row .p-col.w5p65, .p-row .phanterpwa-col.w5p65, .phanterpwa-row .col.w5p65, .phanterpwa-row .p-col.w5p65, .phanterpwa-row .phanterpwa-col.w5p65 {
    width: 65%;
  }
  .row .col.w5p66, .row .p-col.w5p66, .row .phanterpwa-col.w5p66, .p-row .col.w5p66, .p-row .p-col.w5p66, .p-row .phanterpwa-col.w5p66, .phanterpwa-row .col.w5p66, .phanterpwa-row .p-col.w5p66, .phanterpwa-row .phanterpwa-col.w5p66 {
    width: 66%;
  }
  .row .col.w5p67, .row .p-col.w5p67, .row .phanterpwa-col.w5p67, .p-row .col.w5p67, .p-row .p-col.w5p67, .p-row .phanterpwa-col.w5p67, .phanterpwa-row .col.w5p67, .phanterpwa-row .p-col.w5p67, .phanterpwa-row .phanterpwa-col.w5p67 {
    width: 67%;
  }
  .row .col.w5p68, .row .p-col.w5p68, .row .phanterpwa-col.w5p68, .p-row .col.w5p68, .p-row .p-col.w5p68, .p-row .phanterpwa-col.w5p68, .phanterpwa-row .col.w5p68, .phanterpwa-row .p-col.w5p68, .phanterpwa-row .phanterpwa-col.w5p68 {
    width: 68%;
  }
  .row .col.w5p69, .row .p-col.w5p69, .row .phanterpwa-col.w5p69, .p-row .col.w5p69, .p-row .p-col.w5p69, .p-row .phanterpwa-col.w5p69, .phanterpwa-row .col.w5p69, .phanterpwa-row .p-col.w5p69, .phanterpwa-row .phanterpwa-col.w5p69 {
    width: 69%;
  }
  .row .col.w5p70, .row .p-col.w5p70, .row .phanterpwa-col.w5p70, .p-row .col.w5p70, .p-row .p-col.w5p70, .p-row .phanterpwa-col.w5p70, .phanterpwa-row .col.w5p70, .phanterpwa-row .p-col.w5p70, .phanterpwa-row .phanterpwa-col.w5p70 {
    width: 70%;
  }
  .row .col.w5p71, .row .p-col.w5p71, .row .phanterpwa-col.w5p71, .p-row .col.w5p71, .p-row .p-col.w5p71, .p-row .phanterpwa-col.w5p71, .phanterpwa-row .col.w5p71, .phanterpwa-row .p-col.w5p71, .phanterpwa-row .phanterpwa-col.w5p71 {
    width: 71%;
  }
  .row .col.w5p72, .row .p-col.w5p72, .row .phanterpwa-col.w5p72, .p-row .col.w5p72, .p-row .p-col.w5p72, .p-row .phanterpwa-col.w5p72, .phanterpwa-row .col.w5p72, .phanterpwa-row .p-col.w5p72, .phanterpwa-row .phanterpwa-col.w5p72 {
    width: 72%;
  }
  .row .col.w5p73, .row .p-col.w5p73, .row .phanterpwa-col.w5p73, .p-row .col.w5p73, .p-row .p-col.w5p73, .p-row .phanterpwa-col.w5p73, .phanterpwa-row .col.w5p73, .phanterpwa-row .p-col.w5p73, .phanterpwa-row .phanterpwa-col.w5p73 {
    width: 73%;
  }
  .row .col.w5p74, .row .p-col.w5p74, .row .phanterpwa-col.w5p74, .p-row .col.w5p74, .p-row .p-col.w5p74, .p-row .phanterpwa-col.w5p74, .phanterpwa-row .col.w5p74, .phanterpwa-row .p-col.w5p74, .phanterpwa-row .phanterpwa-col.w5p74 {
    width: 74%;
  }
  .row .col.w5p75, .row .p-col.w5p75, .row .phanterpwa-col.w5p75, .p-row .col.w5p75, .p-row .p-col.w5p75, .p-row .phanterpwa-col.w5p75, .phanterpwa-row .col.w5p75, .phanterpwa-row .p-col.w5p75, .phanterpwa-row .phanterpwa-col.w5p75 {
    width: 75%;
  }
  .row .col.w5p76, .row .p-col.w5p76, .row .phanterpwa-col.w5p76, .p-row .col.w5p76, .p-row .p-col.w5p76, .p-row .phanterpwa-col.w5p76, .phanterpwa-row .col.w5p76, .phanterpwa-row .p-col.w5p76, .phanterpwa-row .phanterpwa-col.w5p76 {
    width: 76%;
  }
  .row .col.w5p77, .row .p-col.w5p77, .row .phanterpwa-col.w5p77, .p-row .col.w5p77, .p-row .p-col.w5p77, .p-row .phanterpwa-col.w5p77, .phanterpwa-row .col.w5p77, .phanterpwa-row .p-col.w5p77, .phanterpwa-row .phanterpwa-col.w5p77 {
    width: 77%;
  }
  .row .col.w5p78, .row .p-col.w5p78, .row .phanterpwa-col.w5p78, .p-row .col.w5p78, .p-row .p-col.w5p78, .p-row .phanterpwa-col.w5p78, .phanterpwa-row .col.w5p78, .phanterpwa-row .p-col.w5p78, .phanterpwa-row .phanterpwa-col.w5p78 {
    width: 78%;
  }
  .row .col.w5p79, .row .p-col.w5p79, .row .phanterpwa-col.w5p79, .p-row .col.w5p79, .p-row .p-col.w5p79, .p-row .phanterpwa-col.w5p79, .phanterpwa-row .col.w5p79, .phanterpwa-row .p-col.w5p79, .phanterpwa-row .phanterpwa-col.w5p79 {
    width: 79%;
  }
  .row .col.w5p80, .row .p-col.w5p80, .row .phanterpwa-col.w5p80, .p-row .col.w5p80, .p-row .p-col.w5p80, .p-row .phanterpwa-col.w5p80, .phanterpwa-row .col.w5p80, .phanterpwa-row .p-col.w5p80, .phanterpwa-row .phanterpwa-col.w5p80 {
    width: 80%;
  }
  .row .col.w5p81, .row .p-col.w5p81, .row .phanterpwa-col.w5p81, .p-row .col.w5p81, .p-row .p-col.w5p81, .p-row .phanterpwa-col.w5p81, .phanterpwa-row .col.w5p81, .phanterpwa-row .p-col.w5p81, .phanterpwa-row .phanterpwa-col.w5p81 {
    width: 81%;
  }
  .row .col.w5p82, .row .p-col.w5p82, .row .phanterpwa-col.w5p82, .p-row .col.w5p82, .p-row .p-col.w5p82, .p-row .phanterpwa-col.w5p82, .phanterpwa-row .col.w5p82, .phanterpwa-row .p-col.w5p82, .phanterpwa-row .phanterpwa-col.w5p82 {
    width: 82%;
  }
  .row .col.w5p83, .row .p-col.w5p83, .row .phanterpwa-col.w5p83, .p-row .col.w5p83, .p-row .p-col.w5p83, .p-row .phanterpwa-col.w5p83, .phanterpwa-row .col.w5p83, .phanterpwa-row .p-col.w5p83, .phanterpwa-row .phanterpwa-col.w5p83 {
    width: 83%;
  }
  .row .col.w5p84, .row .p-col.w5p84, .row .phanterpwa-col.w5p84, .p-row .col.w5p84, .p-row .p-col.w5p84, .p-row .phanterpwa-col.w5p84, .phanterpwa-row .col.w5p84, .phanterpwa-row .p-col.w5p84, .phanterpwa-row .phanterpwa-col.w5p84 {
    width: 84%;
  }
  .row .col.w5p85, .row .p-col.w5p85, .row .phanterpwa-col.w5p85, .p-row .col.w5p85, .p-row .p-col.w5p85, .p-row .phanterpwa-col.w5p85, .phanterpwa-row .col.w5p85, .phanterpwa-row .p-col.w5p85, .phanterpwa-row .phanterpwa-col.w5p85 {
    width: 85%;
  }
  .row .col.w5p86, .row .p-col.w5p86, .row .phanterpwa-col.w5p86, .p-row .col.w5p86, .p-row .p-col.w5p86, .p-row .phanterpwa-col.w5p86, .phanterpwa-row .col.w5p86, .phanterpwa-row .p-col.w5p86, .phanterpwa-row .phanterpwa-col.w5p86 {
    width: 86%;
  }
  .row .col.w5p87, .row .p-col.w5p87, .row .phanterpwa-col.w5p87, .p-row .col.w5p87, .p-row .p-col.w5p87, .p-row .phanterpwa-col.w5p87, .phanterpwa-row .col.w5p87, .phanterpwa-row .p-col.w5p87, .phanterpwa-row .phanterpwa-col.w5p87 {
    width: 87%;
  }
  .row .col.w5p88, .row .p-col.w5p88, .row .phanterpwa-col.w5p88, .p-row .col.w5p88, .p-row .p-col.w5p88, .p-row .phanterpwa-col.w5p88, .phanterpwa-row .col.w5p88, .phanterpwa-row .p-col.w5p88, .phanterpwa-row .phanterpwa-col.w5p88 {
    width: 88%;
  }
  .row .col.w5p89, .row .p-col.w5p89, .row .phanterpwa-col.w5p89, .p-row .col.w5p89, .p-row .p-col.w5p89, .p-row .phanterpwa-col.w5p89, .phanterpwa-row .col.w5p89, .phanterpwa-row .p-col.w5p89, .phanterpwa-row .phanterpwa-col.w5p89 {
    width: 89%;
  }
  .row .col.w5p90, .row .p-col.w5p90, .row .phanterpwa-col.w5p90, .p-row .col.w5p90, .p-row .p-col.w5p90, .p-row .phanterpwa-col.w5p90, .phanterpwa-row .col.w5p90, .phanterpwa-row .p-col.w5p90, .phanterpwa-row .phanterpwa-col.w5p90 {
    width: 90%;
  }
  .row .col.w5p91, .row .p-col.w5p91, .row .phanterpwa-col.w5p91, .p-row .col.w5p91, .p-row .p-col.w5p91, .p-row .phanterpwa-col.w5p91, .phanterpwa-row .col.w5p91, .phanterpwa-row .p-col.w5p91, .phanterpwa-row .phanterpwa-col.w5p91 {
    width: 91%;
  }
  .row .col.w5p92, .row .p-col.w5p92, .row .phanterpwa-col.w5p92, .p-row .col.w5p92, .p-row .p-col.w5p92, .p-row .phanterpwa-col.w5p92, .phanterpwa-row .col.w5p92, .phanterpwa-row .p-col.w5p92, .phanterpwa-row .phanterpwa-col.w5p92 {
    width: 92%;
  }
  .row .col.w5p93, .row .p-col.w5p93, .row .phanterpwa-col.w5p93, .p-row .col.w5p93, .p-row .p-col.w5p93, .p-row .phanterpwa-col.w5p93, .phanterpwa-row .col.w5p93, .phanterpwa-row .p-col.w5p93, .phanterpwa-row .phanterpwa-col.w5p93 {
    width: 93%;
  }
  .row .col.w5p94, .row .p-col.w5p94, .row .phanterpwa-col.w5p94, .p-row .col.w5p94, .p-row .p-col.w5p94, .p-row .phanterpwa-col.w5p94, .phanterpwa-row .col.w5p94, .phanterpwa-row .p-col.w5p94, .phanterpwa-row .phanterpwa-col.w5p94 {
    width: 94%;
  }
  .row .col.w5p95, .row .p-col.w5p95, .row .phanterpwa-col.w5p95, .p-row .col.w5p95, .p-row .p-col.w5p95, .p-row .phanterpwa-col.w5p95, .phanterpwa-row .col.w5p95, .phanterpwa-row .p-col.w5p95, .phanterpwa-row .phanterpwa-col.w5p95 {
    width: 95%;
  }
  .row .col.w5p96, .row .p-col.w5p96, .row .phanterpwa-col.w5p96, .p-row .col.w5p96, .p-row .p-col.w5p96, .p-row .phanterpwa-col.w5p96, .phanterpwa-row .col.w5p96, .phanterpwa-row .p-col.w5p96, .phanterpwa-row .phanterpwa-col.w5p96 {
    width: 96%;
  }
  .row .col.w5p97, .row .p-col.w5p97, .row .phanterpwa-col.w5p97, .p-row .col.w5p97, .p-row .p-col.w5p97, .p-row .phanterpwa-col.w5p97, .phanterpwa-row .col.w5p97, .phanterpwa-row .p-col.w5p97, .phanterpwa-row .phanterpwa-col.w5p97 {
    width: 97%;
  }
  .row .col.w5p98, .row .p-col.w5p98, .row .phanterpwa-col.w5p98, .p-row .col.w5p98, .p-row .p-col.w5p98, .p-row .phanterpwa-col.w5p98, .phanterpwa-row .col.w5p98, .phanterpwa-row .p-col.w5p98, .phanterpwa-row .phanterpwa-col.w5p98 {
    width: 98%;
  }
  .row .col.w5p99, .row .p-col.w5p99, .row .phanterpwa-col.w5p99, .p-row .col.w5p99, .p-row .p-col.w5p99, .p-row .phanterpwa-col.w5p99, .phanterpwa-row .col.w5p99, .phanterpwa-row .p-col.w5p99, .phanterpwa-row .phanterpwa-col.w5p99 {
    width: 99%;
  }
  .row .col.w5p100, .row .p-col.w5p100, .row .phanterpwa-col.w5p100, .p-row .col.w5p100, .p-row .p-col.w5p100, .p-row .phanterpwa-col.w5p100, .phanterpwa-row .col.w5p100, .phanterpwa-row .p-col.w5p100, .phanterpwa-row .phanterpwa-col.w5p100 {
    width: 100%;
  }
}

@media screen and (min-width: 1500px) {
  .row .col.w6p1, .row .p-col.w6p1, .row .phanterpwa-col.w6p1, .p-row .col.w6p1, .p-row .p-col.w6p1, .p-row .phanterpwa-col.w6p1, .phanterpwa-row .col.w6p1, .phanterpwa-row .p-col.w6p1, .phanterpwa-row .phanterpwa-col.w6p1 {
    width: 1%;
  }
  .row .col.w6p2, .row .p-col.w6p2, .row .phanterpwa-col.w6p2, .p-row .col.w6p2, .p-row .p-col.w6p2, .p-row .phanterpwa-col.w6p2, .phanterpwa-row .col.w6p2, .phanterpwa-row .p-col.w6p2, .phanterpwa-row .phanterpwa-col.w6p2 {
    width: 2%;
  }
  .row .col.w6p3, .row .p-col.w6p3, .row .phanterpwa-col.w6p3, .p-row .col.w6p3, .p-row .p-col.w6p3, .p-row .phanterpwa-col.w6p3, .phanterpwa-row .col.w6p3, .phanterpwa-row .p-col.w6p3, .phanterpwa-row .phanterpwa-col.w6p3 {
    width: 3%;
  }
  .row .col.w6p4, .row .p-col.w6p4, .row .phanterpwa-col.w6p4, .p-row .col.w6p4, .p-row .p-col.w6p4, .p-row .phanterpwa-col.w6p4, .phanterpwa-row .col.w6p4, .phanterpwa-row .p-col.w6p4, .phanterpwa-row .phanterpwa-col.w6p4 {
    width: 4%;
  }
  .row .col.w6p5, .row .p-col.w6p5, .row .phanterpwa-col.w6p5, .p-row .col.w6p5, .p-row .p-col.w6p5, .p-row .phanterpwa-col.w6p5, .phanterpwa-row .col.w6p5, .phanterpwa-row .p-col.w6p5, .phanterpwa-row .phanterpwa-col.w6p5 {
    width: 5%;
  }
  .row .col.w6p6, .row .p-col.w6p6, .row .phanterpwa-col.w6p6, .p-row .col.w6p6, .p-row .p-col.w6p6, .p-row .phanterpwa-col.w6p6, .phanterpwa-row .col.w6p6, .phanterpwa-row .p-col.w6p6, .phanterpwa-row .phanterpwa-col.w6p6 {
    width: 6%;
  }
  .row .col.w6p7, .row .p-col.w6p7, .row .phanterpwa-col.w6p7, .p-row .col.w6p7, .p-row .p-col.w6p7, .p-row .phanterpwa-col.w6p7, .phanterpwa-row .col.w6p7, .phanterpwa-row .p-col.w6p7, .phanterpwa-row .phanterpwa-col.w6p7 {
    width: 7%;
  }
  .row .col.w6p8, .row .p-col.w6p8, .row .phanterpwa-col.w6p8, .p-row .col.w6p8, .p-row .p-col.w6p8, .p-row .phanterpwa-col.w6p8, .phanterpwa-row .col.w6p8, .phanterpwa-row .p-col.w6p8, .phanterpwa-row .phanterpwa-col.w6p8 {
    width: 8%;
  }
  .row .col.w6p9, .row .p-col.w6p9, .row .phanterpwa-col.w6p9, .p-row .col.w6p9, .p-row .p-col.w6p9, .p-row .phanterpwa-col.w6p9, .phanterpwa-row .col.w6p9, .phanterpwa-row .p-col.w6p9, .phanterpwa-row .phanterpwa-col.w6p9 {
    width: 9%;
  }
  .row .col.w6p10, .row .p-col.w6p10, .row .phanterpwa-col.w6p10, .p-row .col.w6p10, .p-row .p-col.w6p10, .p-row .phanterpwa-col.w6p10, .phanterpwa-row .col.w6p10, .phanterpwa-row .p-col.w6p10, .phanterpwa-row .phanterpwa-col.w6p10 {
    width: 10%;
  }
  .row .col.w6p11, .row .p-col.w6p11, .row .phanterpwa-col.w6p11, .p-row .col.w6p11, .p-row .p-col.w6p11, .p-row .phanterpwa-col.w6p11, .phanterpwa-row .col.w6p11, .phanterpwa-row .p-col.w6p11, .phanterpwa-row .phanterpwa-col.w6p11 {
    width: 11%;
  }
  .row .col.w6p12, .row .p-col.w6p12, .row .phanterpwa-col.w6p12, .p-row .col.w6p12, .p-row .p-col.w6p12, .p-row .phanterpwa-col.w6p12, .phanterpwa-row .col.w6p12, .phanterpwa-row .p-col.w6p12, .phanterpwa-row .phanterpwa-col.w6p12 {
    width: 12%;
  }
  .row .col.w6p13, .row .p-col.w6p13, .row .phanterpwa-col.w6p13, .p-row .col.w6p13, .p-row .p-col.w6p13, .p-row .phanterpwa-col.w6p13, .phanterpwa-row .col.w6p13, .phanterpwa-row .p-col.w6p13, .phanterpwa-row .phanterpwa-col.w6p13 {
    width: 13%;
  }
  .row .col.w6p14, .row .p-col.w6p14, .row .phanterpwa-col.w6p14, .p-row .col.w6p14, .p-row .p-col.w6p14, .p-row .phanterpwa-col.w6p14, .phanterpwa-row .col.w6p14, .phanterpwa-row .p-col.w6p14, .phanterpwa-row .phanterpwa-col.w6p14 {
    width: 14%;
  }
  .row .col.w6p15, .row .p-col.w6p15, .row .phanterpwa-col.w6p15, .p-row .col.w6p15, .p-row .p-col.w6p15, .p-row .phanterpwa-col.w6p15, .phanterpwa-row .col.w6p15, .phanterpwa-row .p-col.w6p15, .phanterpwa-row .phanterpwa-col.w6p15 {
    width: 15%;
  }
  .row .col.w6p16, .row .p-col.w6p16, .row .phanterpwa-col.w6p16, .p-row .col.w6p16, .p-row .p-col.w6p16, .p-row .phanterpwa-col.w6p16, .phanterpwa-row .col.w6p16, .phanterpwa-row .p-col.w6p16, .phanterpwa-row .phanterpwa-col.w6p16 {
    width: 16%;
  }
  .row .col.w6p17, .row .p-col.w6p17, .row .phanterpwa-col.w6p17, .p-row .col.w6p17, .p-row .p-col.w6p17, .p-row .phanterpwa-col.w6p17, .phanterpwa-row .col.w6p17, .phanterpwa-row .p-col.w6p17, .phanterpwa-row .phanterpwa-col.w6p17 {
    width: 17%;
  }
  .row .col.w6p18, .row .p-col.w6p18, .row .phanterpwa-col.w6p18, .p-row .col.w6p18, .p-row .p-col.w6p18, .p-row .phanterpwa-col.w6p18, .phanterpwa-row .col.w6p18, .phanterpwa-row .p-col.w6p18, .phanterpwa-row .phanterpwa-col.w6p18 {
    width: 18%;
  }
  .row .col.w6p19, .row .p-col.w6p19, .row .phanterpwa-col.w6p19, .p-row .col.w6p19, .p-row .p-col.w6p19, .p-row .phanterpwa-col.w6p19, .phanterpwa-row .col.w6p19, .phanterpwa-row .p-col.w6p19, .phanterpwa-row .phanterpwa-col.w6p19 {
    width: 19%;
  }
  .row .col.w6p20, .row .p-col.w6p20, .row .phanterpwa-col.w6p20, .p-row .col.w6p20, .p-row .p-col.w6p20, .p-row .phanterpwa-col.w6p20, .phanterpwa-row .col.w6p20, .phanterpwa-row .p-col.w6p20, .phanterpwa-row .phanterpwa-col.w6p20 {
    width: 20%;
  }
  .row .col.w6p21, .row .p-col.w6p21, .row .phanterpwa-col.w6p21, .p-row .col.w6p21, .p-row .p-col.w6p21, .p-row .phanterpwa-col.w6p21, .phanterpwa-row .col.w6p21, .phanterpwa-row .p-col.w6p21, .phanterpwa-row .phanterpwa-col.w6p21 {
    width: 21%;
  }
  .row .col.w6p22, .row .p-col.w6p22, .row .phanterpwa-col.w6p22, .p-row .col.w6p22, .p-row .p-col.w6p22, .p-row .phanterpwa-col.w6p22, .phanterpwa-row .col.w6p22, .phanterpwa-row .p-col.w6p22, .phanterpwa-row .phanterpwa-col.w6p22 {
    width: 22%;
  }
  .row .col.w6p23, .row .p-col.w6p23, .row .phanterpwa-col.w6p23, .p-row .col.w6p23, .p-row .p-col.w6p23, .p-row .phanterpwa-col.w6p23, .phanterpwa-row .col.w6p23, .phanterpwa-row .p-col.w6p23, .phanterpwa-row .phanterpwa-col.w6p23 {
    width: 23%;
  }
  .row .col.w6p24, .row .p-col.w6p24, .row .phanterpwa-col.w6p24, .p-row .col.w6p24, .p-row .p-col.w6p24, .p-row .phanterpwa-col.w6p24, .phanterpwa-row .col.w6p24, .phanterpwa-row .p-col.w6p24, .phanterpwa-row .phanterpwa-col.w6p24 {
    width: 24%;
  }
  .row .col.w6p25, .row .p-col.w6p25, .row .phanterpwa-col.w6p25, .p-row .col.w6p25, .p-row .p-col.w6p25, .p-row .phanterpwa-col.w6p25, .phanterpwa-row .col.w6p25, .phanterpwa-row .p-col.w6p25, .phanterpwa-row .phanterpwa-col.w6p25 {
    width: 25%;
  }
  .row .col.w6p26, .row .p-col.w6p26, .row .phanterpwa-col.w6p26, .p-row .col.w6p26, .p-row .p-col.w6p26, .p-row .phanterpwa-col.w6p26, .phanterpwa-row .col.w6p26, .phanterpwa-row .p-col.w6p26, .phanterpwa-row .phanterpwa-col.w6p26 {
    width: 26%;
  }
  .row .col.w6p27, .row .p-col.w6p27, .row .phanterpwa-col.w6p27, .p-row .col.w6p27, .p-row .p-col.w6p27, .p-row .phanterpwa-col.w6p27, .phanterpwa-row .col.w6p27, .phanterpwa-row .p-col.w6p27, .phanterpwa-row .phanterpwa-col.w6p27 {
    width: 27%;
  }
  .row .col.w6p28, .row .p-col.w6p28, .row .phanterpwa-col.w6p28, .p-row .col.w6p28, .p-row .p-col.w6p28, .p-row .phanterpwa-col.w6p28, .phanterpwa-row .col.w6p28, .phanterpwa-row .p-col.w6p28, .phanterpwa-row .phanterpwa-col.w6p28 {
    width: 28%;
  }
  .row .col.w6p29, .row .p-col.w6p29, .row .phanterpwa-col.w6p29, .p-row .col.w6p29, .p-row .p-col.w6p29, .p-row .phanterpwa-col.w6p29, .phanterpwa-row .col.w6p29, .phanterpwa-row .p-col.w6p29, .phanterpwa-row .phanterpwa-col.w6p29 {
    width: 29%;
  }
  .row .col.w6p30, .row .p-col.w6p30, .row .phanterpwa-col.w6p30, .p-row .col.w6p30, .p-row .p-col.w6p30, .p-row .phanterpwa-col.w6p30, .phanterpwa-row .col.w6p30, .phanterpwa-row .p-col.w6p30, .phanterpwa-row .phanterpwa-col.w6p30 {
    width: 30%;
  }
  .row .col.w6p31, .row .p-col.w6p31, .row .phanterpwa-col.w6p31, .p-row .col.w6p31, .p-row .p-col.w6p31, .p-row .phanterpwa-col.w6p31, .phanterpwa-row .col.w6p31, .phanterpwa-row .p-col.w6p31, .phanterpwa-row .phanterpwa-col.w6p31 {
    width: 31%;
  }
  .row .col.w6p32, .row .p-col.w6p32, .row .phanterpwa-col.w6p32, .p-row .col.w6p32, .p-row .p-col.w6p32, .p-row .phanterpwa-col.w6p32, .phanterpwa-row .col.w6p32, .phanterpwa-row .p-col.w6p32, .phanterpwa-row .phanterpwa-col.w6p32 {
    width: 32%;
  }
  .row .col.w6p33, .row .p-col.w6p33, .row .phanterpwa-col.w6p33, .p-row .col.w6p33, .p-row .p-col.w6p33, .p-row .phanterpwa-col.w6p33, .phanterpwa-row .col.w6p33, .phanterpwa-row .p-col.w6p33, .phanterpwa-row .phanterpwa-col.w6p33 {
    width: 33%;
  }
  .row .col.w6p34, .row .p-col.w6p34, .row .phanterpwa-col.w6p34, .p-row .col.w6p34, .p-row .p-col.w6p34, .p-row .phanterpwa-col.w6p34, .phanterpwa-row .col.w6p34, .phanterpwa-row .p-col.w6p34, .phanterpwa-row .phanterpwa-col.w6p34 {
    width: 34%;
  }
  .row .col.w6p35, .row .p-col.w6p35, .row .phanterpwa-col.w6p35, .p-row .col.w6p35, .p-row .p-col.w6p35, .p-row .phanterpwa-col.w6p35, .phanterpwa-row .col.w6p35, .phanterpwa-row .p-col.w6p35, .phanterpwa-row .phanterpwa-col.w6p35 {
    width: 35%;
  }
  .row .col.w6p36, .row .p-col.w6p36, .row .phanterpwa-col.w6p36, .p-row .col.w6p36, .p-row .p-col.w6p36, .p-row .phanterpwa-col.w6p36, .phanterpwa-row .col.w6p36, .phanterpwa-row .p-col.w6p36, .phanterpwa-row .phanterpwa-col.w6p36 {
    width: 36%;
  }
  .row .col.w6p37, .row .p-col.w6p37, .row .phanterpwa-col.w6p37, .p-row .col.w6p37, .p-row .p-col.w6p37, .p-row .phanterpwa-col.w6p37, .phanterpwa-row .col.w6p37, .phanterpwa-row .p-col.w6p37, .phanterpwa-row .phanterpwa-col.w6p37 {
    width: 37%;
  }
  .row .col.w6p38, .row .p-col.w6p38, .row .phanterpwa-col.w6p38, .p-row .col.w6p38, .p-row .p-col.w6p38, .p-row .phanterpwa-col.w6p38, .phanterpwa-row .col.w6p38, .phanterpwa-row .p-col.w6p38, .phanterpwa-row .phanterpwa-col.w6p38 {
    width: 38%;
  }
  .row .col.w6p39, .row .p-col.w6p39, .row .phanterpwa-col.w6p39, .p-row .col.w6p39, .p-row .p-col.w6p39, .p-row .phanterpwa-col.w6p39, .phanterpwa-row .col.w6p39, .phanterpwa-row .p-col.w6p39, .phanterpwa-row .phanterpwa-col.w6p39 {
    width: 39%;
  }
  .row .col.w6p40, .row .p-col.w6p40, .row .phanterpwa-col.w6p40, .p-row .col.w6p40, .p-row .p-col.w6p40, .p-row .phanterpwa-col.w6p40, .phanterpwa-row .col.w6p40, .phanterpwa-row .p-col.w6p40, .phanterpwa-row .phanterpwa-col.w6p40 {
    width: 40%;
  }
  .row .col.w6p41, .row .p-col.w6p41, .row .phanterpwa-col.w6p41, .p-row .col.w6p41, .p-row .p-col.w6p41, .p-row .phanterpwa-col.w6p41, .phanterpwa-row .col.w6p41, .phanterpwa-row .p-col.w6p41, .phanterpwa-row .phanterpwa-col.w6p41 {
    width: 41%;
  }
  .row .col.w6p42, .row .p-col.w6p42, .row .phanterpwa-col.w6p42, .p-row .col.w6p42, .p-row .p-col.w6p42, .p-row .phanterpwa-col.w6p42, .phanterpwa-row .col.w6p42, .phanterpwa-row .p-col.w6p42, .phanterpwa-row .phanterpwa-col.w6p42 {
    width: 42%;
  }
  .row .col.w6p43, .row .p-col.w6p43, .row .phanterpwa-col.w6p43, .p-row .col.w6p43, .p-row .p-col.w6p43, .p-row .phanterpwa-col.w6p43, .phanterpwa-row .col.w6p43, .phanterpwa-row .p-col.w6p43, .phanterpwa-row .phanterpwa-col.w6p43 {
    width: 43%;
  }
  .row .col.w6p44, .row .p-col.w6p44, .row .phanterpwa-col.w6p44, .p-row .col.w6p44, .p-row .p-col.w6p44, .p-row .phanterpwa-col.w6p44, .phanterpwa-row .col.w6p44, .phanterpwa-row .p-col.w6p44, .phanterpwa-row .phanterpwa-col.w6p44 {
    width: 44%;
  }
  .row .col.w6p45, .row .p-col.w6p45, .row .phanterpwa-col.w6p45, .p-row .col.w6p45, .p-row .p-col.w6p45, .p-row .phanterpwa-col.w6p45, .phanterpwa-row .col.w6p45, .phanterpwa-row .p-col.w6p45, .phanterpwa-row .phanterpwa-col.w6p45 {
    width: 45%;
  }
  .row .col.w6p46, .row .p-col.w6p46, .row .phanterpwa-col.w6p46, .p-row .col.w6p46, .p-row .p-col.w6p46, .p-row .phanterpwa-col.w6p46, .phanterpwa-row .col.w6p46, .phanterpwa-row .p-col.w6p46, .phanterpwa-row .phanterpwa-col.w6p46 {
    width: 46%;
  }
  .row .col.w6p47, .row .p-col.w6p47, .row .phanterpwa-col.w6p47, .p-row .col.w6p47, .p-row .p-col.w6p47, .p-row .phanterpwa-col.w6p47, .phanterpwa-row .col.w6p47, .phanterpwa-row .p-col.w6p47, .phanterpwa-row .phanterpwa-col.w6p47 {
    width: 47%;
  }
  .row .col.w6p48, .row .p-col.w6p48, .row .phanterpwa-col.w6p48, .p-row .col.w6p48, .p-row .p-col.w6p48, .p-row .phanterpwa-col.w6p48, .phanterpwa-row .col.w6p48, .phanterpwa-row .p-col.w6p48, .phanterpwa-row .phanterpwa-col.w6p48 {
    width: 48%;
  }
  .row .col.w6p49, .row .p-col.w6p49, .row .phanterpwa-col.w6p49, .p-row .col.w6p49, .p-row .p-col.w6p49, .p-row .phanterpwa-col.w6p49, .phanterpwa-row .col.w6p49, .phanterpwa-row .p-col.w6p49, .phanterpwa-row .phanterpwa-col.w6p49 {
    width: 49%;
  }
  .row .col.w6p50, .row .p-col.w6p50, .row .phanterpwa-col.w6p50, .p-row .col.w6p50, .p-row .p-col.w6p50, .p-row .phanterpwa-col.w6p50, .phanterpwa-row .col.w6p50, .phanterpwa-row .p-col.w6p50, .phanterpwa-row .phanterpwa-col.w6p50 {
    width: 50%;
  }
  .row .col.w6p51, .row .p-col.w6p51, .row .phanterpwa-col.w6p51, .p-row .col.w6p51, .p-row .p-col.w6p51, .p-row .phanterpwa-col.w6p51, .phanterpwa-row .col.w6p51, .phanterpwa-row .p-col.w6p51, .phanterpwa-row .phanterpwa-col.w6p51 {
    width: 51%;
  }
  .row .col.w6p52, .row .p-col.w6p52, .row .phanterpwa-col.w6p52, .p-row .col.w6p52, .p-row .p-col.w6p52, .p-row .phanterpwa-col.w6p52, .phanterpwa-row .col.w6p52, .phanterpwa-row .p-col.w6p52, .phanterpwa-row .phanterpwa-col.w6p52 {
    width: 52%;
  }
  .row .col.w6p53, .row .p-col.w6p53, .row .phanterpwa-col.w6p53, .p-row .col.w6p53, .p-row .p-col.w6p53, .p-row .phanterpwa-col.w6p53, .phanterpwa-row .col.w6p53, .phanterpwa-row .p-col.w6p53, .phanterpwa-row .phanterpwa-col.w6p53 {
    width: 53%;
  }
  .row .col.w6p54, .row .p-col.w6p54, .row .phanterpwa-col.w6p54, .p-row .col.w6p54, .p-row .p-col.w6p54, .p-row .phanterpwa-col.w6p54, .phanterpwa-row .col.w6p54, .phanterpwa-row .p-col.w6p54, .phanterpwa-row .phanterpwa-col.w6p54 {
    width: 54%;
  }
  .row .col.w6p55, .row .p-col.w6p55, .row .phanterpwa-col.w6p55, .p-row .col.w6p55, .p-row .p-col.w6p55, .p-row .phanterpwa-col.w6p55, .phanterpwa-row .col.w6p55, .phanterpwa-row .p-col.w6p55, .phanterpwa-row .phanterpwa-col.w6p55 {
    width: 55%;
  }
  .row .col.w6p56, .row .p-col.w6p56, .row .phanterpwa-col.w6p56, .p-row .col.w6p56, .p-row .p-col.w6p56, .p-row .phanterpwa-col.w6p56, .phanterpwa-row .col.w6p56, .phanterpwa-row .p-col.w6p56, .phanterpwa-row .phanterpwa-col.w6p56 {
    width: 56%;
  }
  .row .col.w6p57, .row .p-col.w6p57, .row .phanterpwa-col.w6p57, .p-row .col.w6p57, .p-row .p-col.w6p57, .p-row .phanterpwa-col.w6p57, .phanterpwa-row .col.w6p57, .phanterpwa-row .p-col.w6p57, .phanterpwa-row .phanterpwa-col.w6p57 {
    width: 57%;
  }
  .row .col.w6p58, .row .p-col.w6p58, .row .phanterpwa-col.w6p58, .p-row .col.w6p58, .p-row .p-col.w6p58, .p-row .phanterpwa-col.w6p58, .phanterpwa-row .col.w6p58, .phanterpwa-row .p-col.w6p58, .phanterpwa-row .phanterpwa-col.w6p58 {
    width: 58%;
  }
  .row .col.w6p59, .row .p-col.w6p59, .row .phanterpwa-col.w6p59, .p-row .col.w6p59, .p-row .p-col.w6p59, .p-row .phanterpwa-col.w6p59, .phanterpwa-row .col.w6p59, .phanterpwa-row .p-col.w6p59, .phanterpwa-row .phanterpwa-col.w6p59 {
    width: 59%;
  }
  .row .col.w6p60, .row .p-col.w6p60, .row .phanterpwa-col.w6p60, .p-row .col.w6p60, .p-row .p-col.w6p60, .p-row .phanterpwa-col.w6p60, .phanterpwa-row .col.w6p60, .phanterpwa-row .p-col.w6p60, .phanterpwa-row .phanterpwa-col.w6p60 {
    width: 60%;
  }
  .row .col.w6p61, .row .p-col.w6p61, .row .phanterpwa-col.w6p61, .p-row .col.w6p61, .p-row .p-col.w6p61, .p-row .phanterpwa-col.w6p61, .phanterpwa-row .col.w6p61, .phanterpwa-row .p-col.w6p61, .phanterpwa-row .phanterpwa-col.w6p61 {
    width: 61%;
  }
  .row .col.w6p62, .row .p-col.w6p62, .row .phanterpwa-col.w6p62, .p-row .col.w6p62, .p-row .p-col.w6p62, .p-row .phanterpwa-col.w6p62, .phanterpwa-row .col.w6p62, .phanterpwa-row .p-col.w6p62, .phanterpwa-row .phanterpwa-col.w6p62 {
    width: 62%;
  }
  .row .col.w6p63, .row .p-col.w6p63, .row .phanterpwa-col.w6p63, .p-row .col.w6p63, .p-row .p-col.w6p63, .p-row .phanterpwa-col.w6p63, .phanterpwa-row .col.w6p63, .phanterpwa-row .p-col.w6p63, .phanterpwa-row .phanterpwa-col.w6p63 {
    width: 63%;
  }
  .row .col.w6p64, .row .p-col.w6p64, .row .phanterpwa-col.w6p64, .p-row .col.w6p64, .p-row .p-col.w6p64, .p-row .phanterpwa-col.w6p64, .phanterpwa-row .col.w6p64, .phanterpwa-row .p-col.w6p64, .phanterpwa-row .phanterpwa-col.w6p64 {
    width: 64%;
  }
  .row .col.w6p65, .row .p-col.w6p65, .row .phanterpwa-col.w6p65, .p-row .col.w6p65, .p-row .p-col.w6p65, .p-row .phanterpwa-col.w6p65, .phanterpwa-row .col.w6p65, .phanterpwa-row .p-col.w6p65, .phanterpwa-row .phanterpwa-col.w6p65 {
    width: 65%;
  }
  .row .col.w6p66, .row .p-col.w6p66, .row .phanterpwa-col.w6p66, .p-row .col.w6p66, .p-row .p-col.w6p66, .p-row .phanterpwa-col.w6p66, .phanterpwa-row .col.w6p66, .phanterpwa-row .p-col.w6p66, .phanterpwa-row .phanterpwa-col.w6p66 {
    width: 66%;
  }
  .row .col.w6p67, .row .p-col.w6p67, .row .phanterpwa-col.w6p67, .p-row .col.w6p67, .p-row .p-col.w6p67, .p-row .phanterpwa-col.w6p67, .phanterpwa-row .col.w6p67, .phanterpwa-row .p-col.w6p67, .phanterpwa-row .phanterpwa-col.w6p67 {
    width: 67%;
  }
  .row .col.w6p68, .row .p-col.w6p68, .row .phanterpwa-col.w6p68, .p-row .col.w6p68, .p-row .p-col.w6p68, .p-row .phanterpwa-col.w6p68, .phanterpwa-row .col.w6p68, .phanterpwa-row .p-col.w6p68, .phanterpwa-row .phanterpwa-col.w6p68 {
    width: 68%;
  }
  .row .col.w6p69, .row .p-col.w6p69, .row .phanterpwa-col.w6p69, .p-row .col.w6p69, .p-row .p-col.w6p69, .p-row .phanterpwa-col.w6p69, .phanterpwa-row .col.w6p69, .phanterpwa-row .p-col.w6p69, .phanterpwa-row .phanterpwa-col.w6p69 {
    width: 69%;
  }
  .row .col.w6p70, .row .p-col.w6p70, .row .phanterpwa-col.w6p70, .p-row .col.w6p70, .p-row .p-col.w6p70, .p-row .phanterpwa-col.w6p70, .phanterpwa-row .col.w6p70, .phanterpwa-row .p-col.w6p70, .phanterpwa-row .phanterpwa-col.w6p70 {
    width: 70%;
  }
  .row .col.w6p71, .row .p-col.w6p71, .row .phanterpwa-col.w6p71, .p-row .col.w6p71, .p-row .p-col.w6p71, .p-row .phanterpwa-col.w6p71, .phanterpwa-row .col.w6p71, .phanterpwa-row .p-col.w6p71, .phanterpwa-row .phanterpwa-col.w6p71 {
    width: 71%;
  }
  .row .col.w6p72, .row .p-col.w6p72, .row .phanterpwa-col.w6p72, .p-row .col.w6p72, .p-row .p-col.w6p72, .p-row .phanterpwa-col.w6p72, .phanterpwa-row .col.w6p72, .phanterpwa-row .p-col.w6p72, .phanterpwa-row .phanterpwa-col.w6p72 {
    width: 72%;
  }
  .row .col.w6p73, .row .p-col.w6p73, .row .phanterpwa-col.w6p73, .p-row .col.w6p73, .p-row .p-col.w6p73, .p-row .phanterpwa-col.w6p73, .phanterpwa-row .col.w6p73, .phanterpwa-row .p-col.w6p73, .phanterpwa-row .phanterpwa-col.w6p73 {
    width: 73%;
  }
  .row .col.w6p74, .row .p-col.w6p74, .row .phanterpwa-col.w6p74, .p-row .col.w6p74, .p-row .p-col.w6p74, .p-row .phanterpwa-col.w6p74, .phanterpwa-row .col.w6p74, .phanterpwa-row .p-col.w6p74, .phanterpwa-row .phanterpwa-col.w6p74 {
    width: 74%;
  }
  .row .col.w6p75, .row .p-col.w6p75, .row .phanterpwa-col.w6p75, .p-row .col.w6p75, .p-row .p-col.w6p75, .p-row .phanterpwa-col.w6p75, .phanterpwa-row .col.w6p75, .phanterpwa-row .p-col.w6p75, .phanterpwa-row .phanterpwa-col.w6p75 {
    width: 75%;
  }
  .row .col.w6p76, .row .p-col.w6p76, .row .phanterpwa-col.w6p76, .p-row .col.w6p76, .p-row .p-col.w6p76, .p-row .phanterpwa-col.w6p76, .phanterpwa-row .col.w6p76, .phanterpwa-row .p-col.w6p76, .phanterpwa-row .phanterpwa-col.w6p76 {
    width: 76%;
  }
  .row .col.w6p77, .row .p-col.w6p77, .row .phanterpwa-col.w6p77, .p-row .col.w6p77, .p-row .p-col.w6p77, .p-row .phanterpwa-col.w6p77, .phanterpwa-row .col.w6p77, .phanterpwa-row .p-col.w6p77, .phanterpwa-row .phanterpwa-col.w6p77 {
    width: 77%;
  }
  .row .col.w6p78, .row .p-col.w6p78, .row .phanterpwa-col.w6p78, .p-row .col.w6p78, .p-row .p-col.w6p78, .p-row .phanterpwa-col.w6p78, .phanterpwa-row .col.w6p78, .phanterpwa-row .p-col.w6p78, .phanterpwa-row .phanterpwa-col.w6p78 {
    width: 78%;
  }
  .row .col.w6p79, .row .p-col.w6p79, .row .phanterpwa-col.w6p79, .p-row .col.w6p79, .p-row .p-col.w6p79, .p-row .phanterpwa-col.w6p79, .phanterpwa-row .col.w6p79, .phanterpwa-row .p-col.w6p79, .phanterpwa-row .phanterpwa-col.w6p79 {
    width: 79%;
  }
  .row .col.w6p80, .row .p-col.w6p80, .row .phanterpwa-col.w6p80, .p-row .col.w6p80, .p-row .p-col.w6p80, .p-row .phanterpwa-col.w6p80, .phanterpwa-row .col.w6p80, .phanterpwa-row .p-col.w6p80, .phanterpwa-row .phanterpwa-col.w6p80 {
    width: 80%;
  }
  .row .col.w6p81, .row .p-col.w6p81, .row .phanterpwa-col.w6p81, .p-row .col.w6p81, .p-row .p-col.w6p81, .p-row .phanterpwa-col.w6p81, .phanterpwa-row .col.w6p81, .phanterpwa-row .p-col.w6p81, .phanterpwa-row .phanterpwa-col.w6p81 {
    width: 81%;
  }
  .row .col.w6p82, .row .p-col.w6p82, .row .phanterpwa-col.w6p82, .p-row .col.w6p82, .p-row .p-col.w6p82, .p-row .phanterpwa-col.w6p82, .phanterpwa-row .col.w6p82, .phanterpwa-row .p-col.w6p82, .phanterpwa-row .phanterpwa-col.w6p82 {
    width: 82%;
  }
  .row .col.w6p83, .row .p-col.w6p83, .row .phanterpwa-col.w6p83, .p-row .col.w6p83, .p-row .p-col.w6p83, .p-row .phanterpwa-col.w6p83, .phanterpwa-row .col.w6p83, .phanterpwa-row .p-col.w6p83, .phanterpwa-row .phanterpwa-col.w6p83 {
    width: 83%;
  }
  .row .col.w6p84, .row .p-col.w6p84, .row .phanterpwa-col.w6p84, .p-row .col.w6p84, .p-row .p-col.w6p84, .p-row .phanterpwa-col.w6p84, .phanterpwa-row .col.w6p84, .phanterpwa-row .p-col.w6p84, .phanterpwa-row .phanterpwa-col.w6p84 {
    width: 84%;
  }
  .row .col.w6p85, .row .p-col.w6p85, .row .phanterpwa-col.w6p85, .p-row .col.w6p85, .p-row .p-col.w6p85, .p-row .phanterpwa-col.w6p85, .phanterpwa-row .col.w6p85, .phanterpwa-row .p-col.w6p85, .phanterpwa-row .phanterpwa-col.w6p85 {
    width: 85%;
  }
  .row .col.w6p86, .row .p-col.w6p86, .row .phanterpwa-col.w6p86, .p-row .col.w6p86, .p-row .p-col.w6p86, .p-row .phanterpwa-col.w6p86, .phanterpwa-row .col.w6p86, .phanterpwa-row .p-col.w6p86, .phanterpwa-row .phanterpwa-col.w6p86 {
    width: 86%;
  }
  .row .col.w6p87, .row .p-col.w6p87, .row .phanterpwa-col.w6p87, .p-row .col.w6p87, .p-row .p-col.w6p87, .p-row .phanterpwa-col.w6p87, .phanterpwa-row .col.w6p87, .phanterpwa-row .p-col.w6p87, .phanterpwa-row .phanterpwa-col.w6p87 {
    width: 87%;
  }
  .row .col.w6p88, .row .p-col.w6p88, .row .phanterpwa-col.w6p88, .p-row .col.w6p88, .p-row .p-col.w6p88, .p-row .phanterpwa-col.w6p88, .phanterpwa-row .col.w6p88, .phanterpwa-row .p-col.w6p88, .phanterpwa-row .phanterpwa-col.w6p88 {
    width: 88%;
  }
  .row .col.w6p89, .row .p-col.w6p89, .row .phanterpwa-col.w6p89, .p-row .col.w6p89, .p-row .p-col.w6p89, .p-row .phanterpwa-col.w6p89, .phanterpwa-row .col.w6p89, .phanterpwa-row .p-col.w6p89, .phanterpwa-row .phanterpwa-col.w6p89 {
    width: 89%;
  }
  .row .col.w6p90, .row .p-col.w6p90, .row .phanterpwa-col.w6p90, .p-row .col.w6p90, .p-row .p-col.w6p90, .p-row .phanterpwa-col.w6p90, .phanterpwa-row .col.w6p90, .phanterpwa-row .p-col.w6p90, .phanterpwa-row .phanterpwa-col.w6p90 {
    width: 90%;
  }
  .row .col.w6p91, .row .p-col.w6p91, .row .phanterpwa-col.w6p91, .p-row .col.w6p91, .p-row .p-col.w6p91, .p-row .phanterpwa-col.w6p91, .phanterpwa-row .col.w6p91, .phanterpwa-row .p-col.w6p91, .phanterpwa-row .phanterpwa-col.w6p91 {
    width: 91%;
  }
  .row .col.w6p92, .row .p-col.w6p92, .row .phanterpwa-col.w6p92, .p-row .col.w6p92, .p-row .p-col.w6p92, .p-row .phanterpwa-col.w6p92, .phanterpwa-row .col.w6p92, .phanterpwa-row .p-col.w6p92, .phanterpwa-row .phanterpwa-col.w6p92 {
    width: 92%;
  }
  .row .col.w6p93, .row .p-col.w6p93, .row .phanterpwa-col.w6p93, .p-row .col.w6p93, .p-row .p-col.w6p93, .p-row .phanterpwa-col.w6p93, .phanterpwa-row .col.w6p93, .phanterpwa-row .p-col.w6p93, .phanterpwa-row .phanterpwa-col.w6p93 {
    width: 93%;
  }
  .row .col.w6p94, .row .p-col.w6p94, .row .phanterpwa-col.w6p94, .p-row .col.w6p94, .p-row .p-col.w6p94, .p-row .phanterpwa-col.w6p94, .phanterpwa-row .col.w6p94, .phanterpwa-row .p-col.w6p94, .phanterpwa-row .phanterpwa-col.w6p94 {
    width: 94%;
  }
  .row .col.w6p95, .row .p-col.w6p95, .row .phanterpwa-col.w6p95, .p-row .col.w6p95, .p-row .p-col.w6p95, .p-row .phanterpwa-col.w6p95, .phanterpwa-row .col.w6p95, .phanterpwa-row .p-col.w6p95, .phanterpwa-row .phanterpwa-col.w6p95 {
    width: 95%;
  }
  .row .col.w6p96, .row .p-col.w6p96, .row .phanterpwa-col.w6p96, .p-row .col.w6p96, .p-row .p-col.w6p96, .p-row .phanterpwa-col.w6p96, .phanterpwa-row .col.w6p96, .phanterpwa-row .p-col.w6p96, .phanterpwa-row .phanterpwa-col.w6p96 {
    width: 96%;
  }
  .row .col.w6p97, .row .p-col.w6p97, .row .phanterpwa-col.w6p97, .p-row .col.w6p97, .p-row .p-col.w6p97, .p-row .phanterpwa-col.w6p97, .phanterpwa-row .col.w6p97, .phanterpwa-row .p-col.w6p97, .phanterpwa-row .phanterpwa-col.w6p97 {
    width: 97%;
  }
  .row .col.w6p98, .row .p-col.w6p98, .row .phanterpwa-col.w6p98, .p-row .col.w6p98, .p-row .p-col.w6p98, .p-row .phanterpwa-col.w6p98, .phanterpwa-row .col.w6p98, .phanterpwa-row .p-col.w6p98, .phanterpwa-row .phanterpwa-col.w6p98 {
    width: 98%;
  }
  .row .col.w6p99, .row .p-col.w6p99, .row .phanterpwa-col.w6p99, .p-row .col.w6p99, .p-row .p-col.w6p99, .p-row .phanterpwa-col.w6p99, .phanterpwa-row .col.w6p99, .phanterpwa-row .p-col.w6p99, .phanterpwa-row .phanterpwa-col.w6p99 {
    width: 99%;
  }
  .row .col.w6p100, .row .p-col.w6p100, .row .phanterpwa-col.w6p100, .p-row .col.w6p100, .p-row .p-col.w6p100, .p-row .phanterpwa-col.w6p100, .phanterpwa-row .col.w6p100, .phanterpwa-row .p-col.w6p100, .phanterpwa-row .phanterpwa-col.w6p100 {
    width: 100%;
  }
}

@media screen and (min-width: 1800px) {
  .row .col.w7p1, .row .p-col.w7p1, .row .phanterpwa-col.w7p1, .p-row .col.w7p1, .p-row .p-col.w7p1, .p-row .phanterpwa-col.w7p1, .phanterpwa-row .col.w7p1, .phanterpwa-row .p-col.w7p1, .phanterpwa-row .phanterpwa-col.w7p1 {
    width: 1%;
  }
  .row .col.w7p2, .row .p-col.w7p2, .row .phanterpwa-col.w7p2, .p-row .col.w7p2, .p-row .p-col.w7p2, .p-row .phanterpwa-col.w7p2, .phanterpwa-row .col.w7p2, .phanterpwa-row .p-col.w7p2, .phanterpwa-row .phanterpwa-col.w7p2 {
    width: 2%;
  }
  .row .col.w7p3, .row .p-col.w7p3, .row .phanterpwa-col.w7p3, .p-row .col.w7p3, .p-row .p-col.w7p3, .p-row .phanterpwa-col.w7p3, .phanterpwa-row .col.w7p3, .phanterpwa-row .p-col.w7p3, .phanterpwa-row .phanterpwa-col.w7p3 {
    width: 3%;
  }
  .row .col.w7p4, .row .p-col.w7p4, .row .phanterpwa-col.w7p4, .p-row .col.w7p4, .p-row .p-col.w7p4, .p-row .phanterpwa-col.w7p4, .phanterpwa-row .col.w7p4, .phanterpwa-row .p-col.w7p4, .phanterpwa-row .phanterpwa-col.w7p4 {
    width: 4%;
  }
  .row .col.w7p5, .row .p-col.w7p5, .row .phanterpwa-col.w7p5, .p-row .col.w7p5, .p-row .p-col.w7p5, .p-row .phanterpwa-col.w7p5, .phanterpwa-row .col.w7p5, .phanterpwa-row .p-col.w7p5, .phanterpwa-row .phanterpwa-col.w7p5 {
    width: 5%;
  }
  .row .col.w7p6, .row .p-col.w7p6, .row .phanterpwa-col.w7p6, .p-row .col.w7p6, .p-row .p-col.w7p6, .p-row .phanterpwa-col.w7p6, .phanterpwa-row .col.w7p6, .phanterpwa-row .p-col.w7p6, .phanterpwa-row .phanterpwa-col.w7p6 {
    width: 6%;
  }
  .row .col.w7p7, .row .p-col.w7p7, .row .phanterpwa-col.w7p7, .p-row .col.w7p7, .p-row .p-col.w7p7, .p-row .phanterpwa-col.w7p7, .phanterpwa-row .col.w7p7, .phanterpwa-row .p-col.w7p7, .phanterpwa-row .phanterpwa-col.w7p7 {
    width: 7%;
  }
  .row .col.w7p8, .row .p-col.w7p8, .row .phanterpwa-col.w7p8, .p-row .col.w7p8, .p-row .p-col.w7p8, .p-row .phanterpwa-col.w7p8, .phanterpwa-row .col.w7p8, .phanterpwa-row .p-col.w7p8, .phanterpwa-row .phanterpwa-col.w7p8 {
    width: 8%;
  }
  .row .col.w7p9, .row .p-col.w7p9, .row .phanterpwa-col.w7p9, .p-row .col.w7p9, .p-row .p-col.w7p9, .p-row .phanterpwa-col.w7p9, .phanterpwa-row .col.w7p9, .phanterpwa-row .p-col.w7p9, .phanterpwa-row .phanterpwa-col.w7p9 {
    width: 9%;
  }
  .row .col.w7p10, .row .p-col.w7p10, .row .phanterpwa-col.w7p10, .p-row .col.w7p10, .p-row .p-col.w7p10, .p-row .phanterpwa-col.w7p10, .phanterpwa-row .col.w7p10, .phanterpwa-row .p-col.w7p10, .phanterpwa-row .phanterpwa-col.w7p10 {
    width: 10%;
  }
  .row .col.w7p11, .row .p-col.w7p11, .row .phanterpwa-col.w7p11, .p-row .col.w7p11, .p-row .p-col.w7p11, .p-row .phanterpwa-col.w7p11, .phanterpwa-row .col.w7p11, .phanterpwa-row .p-col.w7p11, .phanterpwa-row .phanterpwa-col.w7p11 {
    width: 11%;
  }
  .row .col.w7p12, .row .p-col.w7p12, .row .phanterpwa-col.w7p12, .p-row .col.w7p12, .p-row .p-col.w7p12, .p-row .phanterpwa-col.w7p12, .phanterpwa-row .col.w7p12, .phanterpwa-row .p-col.w7p12, .phanterpwa-row .phanterpwa-col.w7p12 {
    width: 12%;
  }
  .row .col.w7p13, .row .p-col.w7p13, .row .phanterpwa-col.w7p13, .p-row .col.w7p13, .p-row .p-col.w7p13, .p-row .phanterpwa-col.w7p13, .phanterpwa-row .col.w7p13, .phanterpwa-row .p-col.w7p13, .phanterpwa-row .phanterpwa-col.w7p13 {
    width: 13%;
  }
  .row .col.w7p14, .row .p-col.w7p14, .row .phanterpwa-col.w7p14, .p-row .col.w7p14, .p-row .p-col.w7p14, .p-row .phanterpwa-col.w7p14, .phanterpwa-row .col.w7p14, .phanterpwa-row .p-col.w7p14, .phanterpwa-row .phanterpwa-col.w7p14 {
    width: 14%;
  }
  .row .col.w7p15, .row .p-col.w7p15, .row .phanterpwa-col.w7p15, .p-row .col.w7p15, .p-row .p-col.w7p15, .p-row .phanterpwa-col.w7p15, .phanterpwa-row .col.w7p15, .phanterpwa-row .p-col.w7p15, .phanterpwa-row .phanterpwa-col.w7p15 {
    width: 15%;
  }
  .row .col.w7p16, .row .p-col.w7p16, .row .phanterpwa-col.w7p16, .p-row .col.w7p16, .p-row .p-col.w7p16, .p-row .phanterpwa-col.w7p16, .phanterpwa-row .col.w7p16, .phanterpwa-row .p-col.w7p16, .phanterpwa-row .phanterpwa-col.w7p16 {
    width: 16%;
  }
  .row .col.w7p17, .row .p-col.w7p17, .row .phanterpwa-col.w7p17, .p-row .col.w7p17, .p-row .p-col.w7p17, .p-row .phanterpwa-col.w7p17, .phanterpwa-row .col.w7p17, .phanterpwa-row .p-col.w7p17, .phanterpwa-row .phanterpwa-col.w7p17 {
    width: 17%;
  }
  .row .col.w7p18, .row .p-col.w7p18, .row .phanterpwa-col.w7p18, .p-row .col.w7p18, .p-row .p-col.w7p18, .p-row .phanterpwa-col.w7p18, .phanterpwa-row .col.w7p18, .phanterpwa-row .p-col.w7p18, .phanterpwa-row .phanterpwa-col.w7p18 {
    width: 18%;
  }
  .row .col.w7p19, .row .p-col.w7p19, .row .phanterpwa-col.w7p19, .p-row .col.w7p19, .p-row .p-col.w7p19, .p-row .phanterpwa-col.w7p19, .phanterpwa-row .col.w7p19, .phanterpwa-row .p-col.w7p19, .phanterpwa-row .phanterpwa-col.w7p19 {
    width: 19%;
  }
  .row .col.w7p20, .row .p-col.w7p20, .row .phanterpwa-col.w7p20, .p-row .col.w7p20, .p-row .p-col.w7p20, .p-row .phanterpwa-col.w7p20, .phanterpwa-row .col.w7p20, .phanterpwa-row .p-col.w7p20, .phanterpwa-row .phanterpwa-col.w7p20 {
    width: 20%;
  }
  .row .col.w7p21, .row .p-col.w7p21, .row .phanterpwa-col.w7p21, .p-row .col.w7p21, .p-row .p-col.w7p21, .p-row .phanterpwa-col.w7p21, .phanterpwa-row .col.w7p21, .phanterpwa-row .p-col.w7p21, .phanterpwa-row .phanterpwa-col.w7p21 {
    width: 21%;
  }
  .row .col.w7p22, .row .p-col.w7p22, .row .phanterpwa-col.w7p22, .p-row .col.w7p22, .p-row .p-col.w7p22, .p-row .phanterpwa-col.w7p22, .phanterpwa-row .col.w7p22, .phanterpwa-row .p-col.w7p22, .phanterpwa-row .phanterpwa-col.w7p22 {
    width: 22%;
  }
  .row .col.w7p23, .row .p-col.w7p23, .row .phanterpwa-col.w7p23, .p-row .col.w7p23, .p-row .p-col.w7p23, .p-row .phanterpwa-col.w7p23, .phanterpwa-row .col.w7p23, .phanterpwa-row .p-col.w7p23, .phanterpwa-row .phanterpwa-col.w7p23 {
    width: 23%;
  }
  .row .col.w7p24, .row .p-col.w7p24, .row .phanterpwa-col.w7p24, .p-row .col.w7p24, .p-row .p-col.w7p24, .p-row .phanterpwa-col.w7p24, .phanterpwa-row .col.w7p24, .phanterpwa-row .p-col.w7p24, .phanterpwa-row .phanterpwa-col.w7p24 {
    width: 24%;
  }
  .row .col.w7p25, .row .p-col.w7p25, .row .phanterpwa-col.w7p25, .p-row .col.w7p25, .p-row .p-col.w7p25, .p-row .phanterpwa-col.w7p25, .phanterpwa-row .col.w7p25, .phanterpwa-row .p-col.w7p25, .phanterpwa-row .phanterpwa-col.w7p25 {
    width: 25%;
  }
  .row .col.w7p26, .row .p-col.w7p26, .row .phanterpwa-col.w7p26, .p-row .col.w7p26, .p-row .p-col.w7p26, .p-row .phanterpwa-col.w7p26, .phanterpwa-row .col.w7p26, .phanterpwa-row .p-col.w7p26, .phanterpwa-row .phanterpwa-col.w7p26 {
    width: 26%;
  }
  .row .col.w7p27, .row .p-col.w7p27, .row .phanterpwa-col.w7p27, .p-row .col.w7p27, .p-row .p-col.w7p27, .p-row .phanterpwa-col.w7p27, .phanterpwa-row .col.w7p27, .phanterpwa-row .p-col.w7p27, .phanterpwa-row .phanterpwa-col.w7p27 {
    width: 27%;
  }
  .row .col.w7p28, .row .p-col.w7p28, .row .phanterpwa-col.w7p28, .p-row .col.w7p28, .p-row .p-col.w7p28, .p-row .phanterpwa-col.w7p28, .phanterpwa-row .col.w7p28, .phanterpwa-row .p-col.w7p28, .phanterpwa-row .phanterpwa-col.w7p28 {
    width: 28%;
  }
  .row .col.w7p29, .row .p-col.w7p29, .row .phanterpwa-col.w7p29, .p-row .col.w7p29, .p-row .p-col.w7p29, .p-row .phanterpwa-col.w7p29, .phanterpwa-row .col.w7p29, .phanterpwa-row .p-col.w7p29, .phanterpwa-row .phanterpwa-col.w7p29 {
    width: 29%;
  }
  .row .col.w7p30, .row .p-col.w7p30, .row .phanterpwa-col.w7p30, .p-row .col.w7p30, .p-row .p-col.w7p30, .p-row .phanterpwa-col.w7p30, .phanterpwa-row .col.w7p30, .phanterpwa-row .p-col.w7p30, .phanterpwa-row .phanterpwa-col.w7p30 {
    width: 30%;
  }
  .row .col.w7p31, .row .p-col.w7p31, .row .phanterpwa-col.w7p31, .p-row .col.w7p31, .p-row .p-col.w7p31, .p-row .phanterpwa-col.w7p31, .phanterpwa-row .col.w7p31, .phanterpwa-row .p-col.w7p31, .phanterpwa-row .phanterpwa-col.w7p31 {
    width: 31%;
  }
  .row .col.w7p32, .row .p-col.w7p32, .row .phanterpwa-col.w7p32, .p-row .col.w7p32, .p-row .p-col.w7p32, .p-row .phanterpwa-col.w7p32, .phanterpwa-row .col.w7p32, .phanterpwa-row .p-col.w7p32, .phanterpwa-row .phanterpwa-col.w7p32 {
    width: 32%;
  }
  .row .col.w7p33, .row .p-col.w7p33, .row .phanterpwa-col.w7p33, .p-row .col.w7p33, .p-row .p-col.w7p33, .p-row .phanterpwa-col.w7p33, .phanterpwa-row .col.w7p33, .phanterpwa-row .p-col.w7p33, .phanterpwa-row .phanterpwa-col.w7p33 {
    width: 33%;
  }
  .row .col.w7p34, .row .p-col.w7p34, .row .phanterpwa-col.w7p34, .p-row .col.w7p34, .p-row .p-col.w7p34, .p-row .phanterpwa-col.w7p34, .phanterpwa-row .col.w7p34, .phanterpwa-row .p-col.w7p34, .phanterpwa-row .phanterpwa-col.w7p34 {
    width: 34%;
  }
  .row .col.w7p35, .row .p-col.w7p35, .row .phanterpwa-col.w7p35, .p-row .col.w7p35, .p-row .p-col.w7p35, .p-row .phanterpwa-col.w7p35, .phanterpwa-row .col.w7p35, .phanterpwa-row .p-col.w7p35, .phanterpwa-row .phanterpwa-col.w7p35 {
    width: 35%;
  }
  .row .col.w7p36, .row .p-col.w7p36, .row .phanterpwa-col.w7p36, .p-row .col.w7p36, .p-row .p-col.w7p36, .p-row .phanterpwa-col.w7p36, .phanterpwa-row .col.w7p36, .phanterpwa-row .p-col.w7p36, .phanterpwa-row .phanterpwa-col.w7p36 {
    width: 36%;
  }
  .row .col.w7p37, .row .p-col.w7p37, .row .phanterpwa-col.w7p37, .p-row .col.w7p37, .p-row .p-col.w7p37, .p-row .phanterpwa-col.w7p37, .phanterpwa-row .col.w7p37, .phanterpwa-row .p-col.w7p37, .phanterpwa-row .phanterpwa-col.w7p37 {
    width: 37%;
  }
  .row .col.w7p38, .row .p-col.w7p38, .row .phanterpwa-col.w7p38, .p-row .col.w7p38, .p-row .p-col.w7p38, .p-row .phanterpwa-col.w7p38, .phanterpwa-row .col.w7p38, .phanterpwa-row .p-col.w7p38, .phanterpwa-row .phanterpwa-col.w7p38 {
    width: 38%;
  }
  .row .col.w7p39, .row .p-col.w7p39, .row .phanterpwa-col.w7p39, .p-row .col.w7p39, .p-row .p-col.w7p39, .p-row .phanterpwa-col.w7p39, .phanterpwa-row .col.w7p39, .phanterpwa-row .p-col.w7p39, .phanterpwa-row .phanterpwa-col.w7p39 {
    width: 39%;
  }
  .row .col.w7p40, .row .p-col.w7p40, .row .phanterpwa-col.w7p40, .p-row .col.w7p40, .p-row .p-col.w7p40, .p-row .phanterpwa-col.w7p40, .phanterpwa-row .col.w7p40, .phanterpwa-row .p-col.w7p40, .phanterpwa-row .phanterpwa-col.w7p40 {
    width: 40%;
  }
  .row .col.w7p41, .row .p-col.w7p41, .row .phanterpwa-col.w7p41, .p-row .col.w7p41, .p-row .p-col.w7p41, .p-row .phanterpwa-col.w7p41, .phanterpwa-row .col.w7p41, .phanterpwa-row .p-col.w7p41, .phanterpwa-row .phanterpwa-col.w7p41 {
    width: 41%;
  }
  .row .col.w7p42, .row .p-col.w7p42, .row .phanterpwa-col.w7p42, .p-row .col.w7p42, .p-row .p-col.w7p42, .p-row .phanterpwa-col.w7p42, .phanterpwa-row .col.w7p42, .phanterpwa-row .p-col.w7p42, .phanterpwa-row .phanterpwa-col.w7p42 {
    width: 42%;
  }
  .row .col.w7p43, .row .p-col.w7p43, .row .phanterpwa-col.w7p43, .p-row .col.w7p43, .p-row .p-col.w7p43, .p-row .phanterpwa-col.w7p43, .phanterpwa-row .col.w7p43, .phanterpwa-row .p-col.w7p43, .phanterpwa-row .phanterpwa-col.w7p43 {
    width: 43%;
  }
  .row .col.w7p44, .row .p-col.w7p44, .row .phanterpwa-col.w7p44, .p-row .col.w7p44, .p-row .p-col.w7p44, .p-row .phanterpwa-col.w7p44, .phanterpwa-row .col.w7p44, .phanterpwa-row .p-col.w7p44, .phanterpwa-row .phanterpwa-col.w7p44 {
    width: 44%;
  }
  .row .col.w7p45, .row .p-col.w7p45, .row .phanterpwa-col.w7p45, .p-row .col.w7p45, .p-row .p-col.w7p45, .p-row .phanterpwa-col.w7p45, .phanterpwa-row .col.w7p45, .phanterpwa-row .p-col.w7p45, .phanterpwa-row .phanterpwa-col.w7p45 {
    width: 45%;
  }
  .row .col.w7p46, .row .p-col.w7p46, .row .phanterpwa-col.w7p46, .p-row .col.w7p46, .p-row .p-col.w7p46, .p-row .phanterpwa-col.w7p46, .phanterpwa-row .col.w7p46, .phanterpwa-row .p-col.w7p46, .phanterpwa-row .phanterpwa-col.w7p46 {
    width: 46%;
  }
  .row .col.w7p47, .row .p-col.w7p47, .row .phanterpwa-col.w7p47, .p-row .col.w7p47, .p-row .p-col.w7p47, .p-row .phanterpwa-col.w7p47, .phanterpwa-row .col.w7p47, .phanterpwa-row .p-col.w7p47, .phanterpwa-row .phanterpwa-col.w7p47 {
    width: 47%;
  }
  .row .col.w7p48, .row .p-col.w7p48, .row .phanterpwa-col.w7p48, .p-row .col.w7p48, .p-row .p-col.w7p48, .p-row .phanterpwa-col.w7p48, .phanterpwa-row .col.w7p48, .phanterpwa-row .p-col.w7p48, .phanterpwa-row .phanterpwa-col.w7p48 {
    width: 48%;
  }
  .row .col.w7p49, .row .p-col.w7p49, .row .phanterpwa-col.w7p49, .p-row .col.w7p49, .p-row .p-col.w7p49, .p-row .phanterpwa-col.w7p49, .phanterpwa-row .col.w7p49, .phanterpwa-row .p-col.w7p49, .phanterpwa-row .phanterpwa-col.w7p49 {
    width: 49%;
  }
  .row .col.w7p50, .row .p-col.w7p50, .row .phanterpwa-col.w7p50, .p-row .col.w7p50, .p-row .p-col.w7p50, .p-row .phanterpwa-col.w7p50, .phanterpwa-row .col.w7p50, .phanterpwa-row .p-col.w7p50, .phanterpwa-row .phanterpwa-col.w7p50 {
    width: 50%;
  }
  .row .col.w7p51, .row .p-col.w7p51, .row .phanterpwa-col.w7p51, .p-row .col.w7p51, .p-row .p-col.w7p51, .p-row .phanterpwa-col.w7p51, .phanterpwa-row .col.w7p51, .phanterpwa-row .p-col.w7p51, .phanterpwa-row .phanterpwa-col.w7p51 {
    width: 51%;
  }
  .row .col.w7p52, .row .p-col.w7p52, .row .phanterpwa-col.w7p52, .p-row .col.w7p52, .p-row .p-col.w7p52, .p-row .phanterpwa-col.w7p52, .phanterpwa-row .col.w7p52, .phanterpwa-row .p-col.w7p52, .phanterpwa-row .phanterpwa-col.w7p52 {
    width: 52%;
  }
  .row .col.w7p53, .row .p-col.w7p53, .row .phanterpwa-col.w7p53, .p-row .col.w7p53, .p-row .p-col.w7p53, .p-row .phanterpwa-col.w7p53, .phanterpwa-row .col.w7p53, .phanterpwa-row .p-col.w7p53, .phanterpwa-row .phanterpwa-col.w7p53 {
    width: 53%;
  }
  .row .col.w7p54, .row .p-col.w7p54, .row .phanterpwa-col.w7p54, .p-row .col.w7p54, .p-row .p-col.w7p54, .p-row .phanterpwa-col.w7p54, .phanterpwa-row .col.w7p54, .phanterpwa-row .p-col.w7p54, .phanterpwa-row .phanterpwa-col.w7p54 {
    width: 54%;
  }
  .row .col.w7p55, .row .p-col.w7p55, .row .phanterpwa-col.w7p55, .p-row .col.w7p55, .p-row .p-col.w7p55, .p-row .phanterpwa-col.w7p55, .phanterpwa-row .col.w7p55, .phanterpwa-row .p-col.w7p55, .phanterpwa-row .phanterpwa-col.w7p55 {
    width: 55%;
  }
  .row .col.w7p56, .row .p-col.w7p56, .row .phanterpwa-col.w7p56, .p-row .col.w7p56, .p-row .p-col.w7p56, .p-row .phanterpwa-col.w7p56, .phanterpwa-row .col.w7p56, .phanterpwa-row .p-col.w7p56, .phanterpwa-row .phanterpwa-col.w7p56 {
    width: 56%;
  }
  .row .col.w7p57, .row .p-col.w7p57, .row .phanterpwa-col.w7p57, .p-row .col.w7p57, .p-row .p-col.w7p57, .p-row .phanterpwa-col.w7p57, .phanterpwa-row .col.w7p57, .phanterpwa-row .p-col.w7p57, .phanterpwa-row .phanterpwa-col.w7p57 {
    width: 57%;
  }
  .row .col.w7p58, .row .p-col.w7p58, .row .phanterpwa-col.w7p58, .p-row .col.w7p58, .p-row .p-col.w7p58, .p-row .phanterpwa-col.w7p58, .phanterpwa-row .col.w7p58, .phanterpwa-row .p-col.w7p58, .phanterpwa-row .phanterpwa-col.w7p58 {
    width: 58%;
  }
  .row .col.w7p59, .row .p-col.w7p59, .row .phanterpwa-col.w7p59, .p-row .col.w7p59, .p-row .p-col.w7p59, .p-row .phanterpwa-col.w7p59, .phanterpwa-row .col.w7p59, .phanterpwa-row .p-col.w7p59, .phanterpwa-row .phanterpwa-col.w7p59 {
    width: 59%;
  }
  .row .col.w7p60, .row .p-col.w7p60, .row .phanterpwa-col.w7p60, .p-row .col.w7p60, .p-row .p-col.w7p60, .p-row .phanterpwa-col.w7p60, .phanterpwa-row .col.w7p60, .phanterpwa-row .p-col.w7p60, .phanterpwa-row .phanterpwa-col.w7p60 {
    width: 60%;
  }
  .row .col.w7p61, .row .p-col.w7p61, .row .phanterpwa-col.w7p61, .p-row .col.w7p61, .p-row .p-col.w7p61, .p-row .phanterpwa-col.w7p61, .phanterpwa-row .col.w7p61, .phanterpwa-row .p-col.w7p61, .phanterpwa-row .phanterpwa-col.w7p61 {
    width: 61%;
  }
  .row .col.w7p62, .row .p-col.w7p62, .row .phanterpwa-col.w7p62, .p-row .col.w7p62, .p-row .p-col.w7p62, .p-row .phanterpwa-col.w7p62, .phanterpwa-row .col.w7p62, .phanterpwa-row .p-col.w7p62, .phanterpwa-row .phanterpwa-col.w7p62 {
    width: 62%;
  }
  .row .col.w7p63, .row .p-col.w7p63, .row .phanterpwa-col.w7p63, .p-row .col.w7p63, .p-row .p-col.w7p63, .p-row .phanterpwa-col.w7p63, .phanterpwa-row .col.w7p63, .phanterpwa-row .p-col.w7p63, .phanterpwa-row .phanterpwa-col.w7p63 {
    width: 63%;
  }
  .row .col.w7p64, .row .p-col.w7p64, .row .phanterpwa-col.w7p64, .p-row .col.w7p64, .p-row .p-col.w7p64, .p-row .phanterpwa-col.w7p64, .phanterpwa-row .col.w7p64, .phanterpwa-row .p-col.w7p64, .phanterpwa-row .phanterpwa-col.w7p64 {
    width: 64%;
  }
  .row .col.w7p65, .row .p-col.w7p65, .row .phanterpwa-col.w7p65, .p-row .col.w7p65, .p-row .p-col.w7p65, .p-row .phanterpwa-col.w7p65, .phanterpwa-row .col.w7p65, .phanterpwa-row .p-col.w7p65, .phanterpwa-row .phanterpwa-col.w7p65 {
    width: 65%;
  }
  .row .col.w7p66, .row .p-col.w7p66, .row .phanterpwa-col.w7p66, .p-row .col.w7p66, .p-row .p-col.w7p66, .p-row .phanterpwa-col.w7p66, .phanterpwa-row .col.w7p66, .phanterpwa-row .p-col.w7p66, .phanterpwa-row .phanterpwa-col.w7p66 {
    width: 66%;
  }
  .row .col.w7p67, .row .p-col.w7p67, .row .phanterpwa-col.w7p67, .p-row .col.w7p67, .p-row .p-col.w7p67, .p-row .phanterpwa-col.w7p67, .phanterpwa-row .col.w7p67, .phanterpwa-row .p-col.w7p67, .phanterpwa-row .phanterpwa-col.w7p67 {
    width: 67%;
  }
  .row .col.w7p68, .row .p-col.w7p68, .row .phanterpwa-col.w7p68, .p-row .col.w7p68, .p-row .p-col.w7p68, .p-row .phanterpwa-col.w7p68, .phanterpwa-row .col.w7p68, .phanterpwa-row .p-col.w7p68, .phanterpwa-row .phanterpwa-col.w7p68 {
    width: 68%;
  }
  .row .col.w7p69, .row .p-col.w7p69, .row .phanterpwa-col.w7p69, .p-row .col.w7p69, .p-row .p-col.w7p69, .p-row .phanterpwa-col.w7p69, .phanterpwa-row .col.w7p69, .phanterpwa-row .p-col.w7p69, .phanterpwa-row .phanterpwa-col.w7p69 {
    width: 69%;
  }
  .row .col.w7p70, .row .p-col.w7p70, .row .phanterpwa-col.w7p70, .p-row .col.w7p70, .p-row .p-col.w7p70, .p-row .phanterpwa-col.w7p70, .phanterpwa-row .col.w7p70, .phanterpwa-row .p-col.w7p70, .phanterpwa-row .phanterpwa-col.w7p70 {
    width: 70%;
  }
  .row .col.w7p71, .row .p-col.w7p71, .row .phanterpwa-col.w7p71, .p-row .col.w7p71, .p-row .p-col.w7p71, .p-row .phanterpwa-col.w7p71, .phanterpwa-row .col.w7p71, .phanterpwa-row .p-col.w7p71, .phanterpwa-row .phanterpwa-col.w7p71 {
    width: 71%;
  }
  .row .col.w7p72, .row .p-col.w7p72, .row .phanterpwa-col.w7p72, .p-row .col.w7p72, .p-row .p-col.w7p72, .p-row .phanterpwa-col.w7p72, .phanterpwa-row .col.w7p72, .phanterpwa-row .p-col.w7p72, .phanterpwa-row .phanterpwa-col.w7p72 {
    width: 72%;
  }
  .row .col.w7p73, .row .p-col.w7p73, .row .phanterpwa-col.w7p73, .p-row .col.w7p73, .p-row .p-col.w7p73, .p-row .phanterpwa-col.w7p73, .phanterpwa-row .col.w7p73, .phanterpwa-row .p-col.w7p73, .phanterpwa-row .phanterpwa-col.w7p73 {
    width: 73%;
  }
  .row .col.w7p74, .row .p-col.w7p74, .row .phanterpwa-col.w7p74, .p-row .col.w7p74, .p-row .p-col.w7p74, .p-row .phanterpwa-col.w7p74, .phanterpwa-row .col.w7p74, .phanterpwa-row .p-col.w7p74, .phanterpwa-row .phanterpwa-col.w7p74 {
    width: 74%;
  }
  .row .col.w7p75, .row .p-col.w7p75, .row .phanterpwa-col.w7p75, .p-row .col.w7p75, .p-row .p-col.w7p75, .p-row .phanterpwa-col.w7p75, .phanterpwa-row .col.w7p75, .phanterpwa-row .p-col.w7p75, .phanterpwa-row .phanterpwa-col.w7p75 {
    width: 75%;
  }
  .row .col.w7p76, .row .p-col.w7p76, .row .phanterpwa-col.w7p76, .p-row .col.w7p76, .p-row .p-col.w7p76, .p-row .phanterpwa-col.w7p76, .phanterpwa-row .col.w7p76, .phanterpwa-row .p-col.w7p76, .phanterpwa-row .phanterpwa-col.w7p76 {
    width: 76%;
  }
  .row .col.w7p77, .row .p-col.w7p77, .row .phanterpwa-col.w7p77, .p-row .col.w7p77, .p-row .p-col.w7p77, .p-row .phanterpwa-col.w7p77, .phanterpwa-row .col.w7p77, .phanterpwa-row .p-col.w7p77, .phanterpwa-row .phanterpwa-col.w7p77 {
    width: 77%;
  }
  .row .col.w7p78, .row .p-col.w7p78, .row .phanterpwa-col.w7p78, .p-row .col.w7p78, .p-row .p-col.w7p78, .p-row .phanterpwa-col.w7p78, .phanterpwa-row .col.w7p78, .phanterpwa-row .p-col.w7p78, .phanterpwa-row .phanterpwa-col.w7p78 {
    width: 78%;
  }
  .row .col.w7p79, .row .p-col.w7p79, .row .phanterpwa-col.w7p79, .p-row .col.w7p79, .p-row .p-col.w7p79, .p-row .phanterpwa-col.w7p79, .phanterpwa-row .col.w7p79, .phanterpwa-row .p-col.w7p79, .phanterpwa-row .phanterpwa-col.w7p79 {
    width: 79%;
  }
  .row .col.w7p80, .row .p-col.w7p80, .row .phanterpwa-col.w7p80, .p-row .col.w7p80, .p-row .p-col.w7p80, .p-row .phanterpwa-col.w7p80, .phanterpwa-row .col.w7p80, .phanterpwa-row .p-col.w7p80, .phanterpwa-row .phanterpwa-col.w7p80 {
    width: 80%;
  }
  .row .col.w7p81, .row .p-col.w7p81, .row .phanterpwa-col.w7p81, .p-row .col.w7p81, .p-row .p-col.w7p81, .p-row .phanterpwa-col.w7p81, .phanterpwa-row .col.w7p81, .phanterpwa-row .p-col.w7p81, .phanterpwa-row .phanterpwa-col.w7p81 {
    width: 81%;
  }
  .row .col.w7p82, .row .p-col.w7p82, .row .phanterpwa-col.w7p82, .p-row .col.w7p82, .p-row .p-col.w7p82, .p-row .phanterpwa-col.w7p82, .phanterpwa-row .col.w7p82, .phanterpwa-row .p-col.w7p82, .phanterpwa-row .phanterpwa-col.w7p82 {
    width: 82%;
  }
  .row .col.w7p83, .row .p-col.w7p83, .row .phanterpwa-col.w7p83, .p-row .col.w7p83, .p-row .p-col.w7p83, .p-row .phanterpwa-col.w7p83, .phanterpwa-row .col.w7p83, .phanterpwa-row .p-col.w7p83, .phanterpwa-row .phanterpwa-col.w7p83 {
    width: 83%;
  }
  .row .col.w7p84, .row .p-col.w7p84, .row .phanterpwa-col.w7p84, .p-row .col.w7p84, .p-row .p-col.w7p84, .p-row .phanterpwa-col.w7p84, .phanterpwa-row .col.w7p84, .phanterpwa-row .p-col.w7p84, .phanterpwa-row .phanterpwa-col.w7p84 {
    width: 84%;
  }
  .row .col.w7p85, .row .p-col.w7p85, .row .phanterpwa-col.w7p85, .p-row .col.w7p85, .p-row .p-col.w7p85, .p-row .phanterpwa-col.w7p85, .phanterpwa-row .col.w7p85, .phanterpwa-row .p-col.w7p85, .phanterpwa-row .phanterpwa-col.w7p85 {
    width: 85%;
  }
  .row .col.w7p86, .row .p-col.w7p86, .row .phanterpwa-col.w7p86, .p-row .col.w7p86, .p-row .p-col.w7p86, .p-row .phanterpwa-col.w7p86, .phanterpwa-row .col.w7p86, .phanterpwa-row .p-col.w7p86, .phanterpwa-row .phanterpwa-col.w7p86 {
    width: 86%;
  }
  .row .col.w7p87, .row .p-col.w7p87, .row .phanterpwa-col.w7p87, .p-row .col.w7p87, .p-row .p-col.w7p87, .p-row .phanterpwa-col.w7p87, .phanterpwa-row .col.w7p87, .phanterpwa-row .p-col.w7p87, .phanterpwa-row .phanterpwa-col.w7p87 {
    width: 87%;
  }
  .row .col.w7p88, .row .p-col.w7p88, .row .phanterpwa-col.w7p88, .p-row .col.w7p88, .p-row .p-col.w7p88, .p-row .phanterpwa-col.w7p88, .phanterpwa-row .col.w7p88, .phanterpwa-row .p-col.w7p88, .phanterpwa-row .phanterpwa-col.w7p88 {
    width: 88%;
  }
  .row .col.w7p89, .row .p-col.w7p89, .row .phanterpwa-col.w7p89, .p-row .col.w7p89, .p-row .p-col.w7p89, .p-row .phanterpwa-col.w7p89, .phanterpwa-row .col.w7p89, .phanterpwa-row .p-col.w7p89, .phanterpwa-row .phanterpwa-col.w7p89 {
    width: 89%;
  }
  .row .col.w7p90, .row .p-col.w7p90, .row .phanterpwa-col.w7p90, .p-row .col.w7p90, .p-row .p-col.w7p90, .p-row .phanterpwa-col.w7p90, .phanterpwa-row .col.w7p90, .phanterpwa-row .p-col.w7p90, .phanterpwa-row .phanterpwa-col.w7p90 {
    width: 90%;
  }
  .row .col.w7p91, .row .p-col.w7p91, .row .phanterpwa-col.w7p91, .p-row .col.w7p91, .p-row .p-col.w7p91, .p-row .phanterpwa-col.w7p91, .phanterpwa-row .col.w7p91, .phanterpwa-row .p-col.w7p91, .phanterpwa-row .phanterpwa-col.w7p91 {
    width: 91%;
  }
  .row .col.w7p92, .row .p-col.w7p92, .row .phanterpwa-col.w7p92, .p-row .col.w7p92, .p-row .p-col.w7p92, .p-row .phanterpwa-col.w7p92, .phanterpwa-row .col.w7p92, .phanterpwa-row .p-col.w7p92, .phanterpwa-row .phanterpwa-col.w7p92 {
    width: 92%;
  }
  .row .col.w7p93, .row .p-col.w7p93, .row .phanterpwa-col.w7p93, .p-row .col.w7p93, .p-row .p-col.w7p93, .p-row .phanterpwa-col.w7p93, .phanterpwa-row .col.w7p93, .phanterpwa-row .p-col.w7p93, .phanterpwa-row .phanterpwa-col.w7p93 {
    width: 93%;
  }
  .row .col.w7p94, .row .p-col.w7p94, .row .phanterpwa-col.w7p94, .p-row .col.w7p94, .p-row .p-col.w7p94, .p-row .phanterpwa-col.w7p94, .phanterpwa-row .col.w7p94, .phanterpwa-row .p-col.w7p94, .phanterpwa-row .phanterpwa-col.w7p94 {
    width: 94%;
  }
  .row .col.w7p95, .row .p-col.w7p95, .row .phanterpwa-col.w7p95, .p-row .col.w7p95, .p-row .p-col.w7p95, .p-row .phanterpwa-col.w7p95, .phanterpwa-row .col.w7p95, .phanterpwa-row .p-col.w7p95, .phanterpwa-row .phanterpwa-col.w7p95 {
    width: 95%;
  }
  .row .col.w7p96, .row .p-col.w7p96, .row .phanterpwa-col.w7p96, .p-row .col.w7p96, .p-row .p-col.w7p96, .p-row .phanterpwa-col.w7p96, .phanterpwa-row .col.w7p96, .phanterpwa-row .p-col.w7p96, .phanterpwa-row .phanterpwa-col.w7p96 {
    width: 96%;
  }
  .row .col.w7p97, .row .p-col.w7p97, .row .phanterpwa-col.w7p97, .p-row .col.w7p97, .p-row .p-col.w7p97, .p-row .phanterpwa-col.w7p97, .phanterpwa-row .col.w7p97, .phanterpwa-row .p-col.w7p97, .phanterpwa-row .phanterpwa-col.w7p97 {
    width: 97%;
  }
  .row .col.w7p98, .row .p-col.w7p98, .row .phanterpwa-col.w7p98, .p-row .col.w7p98, .p-row .p-col.w7p98, .p-row .phanterpwa-col.w7p98, .phanterpwa-row .col.w7p98, .phanterpwa-row .p-col.w7p98, .phanterpwa-row .phanterpwa-col.w7p98 {
    width: 98%;
  }
  .row .col.w7p99, .row .p-col.w7p99, .row .phanterpwa-col.w7p99, .p-row .col.w7p99, .p-row .p-col.w7p99, .p-row .phanterpwa-col.w7p99, .phanterpwa-row .col.w7p99, .phanterpwa-row .p-col.w7p99, .phanterpwa-row .phanterpwa-col.w7p99 {
    width: 99%;
  }
  .row .col.w7p100, .row .p-col.w7p100, .row .phanterpwa-col.w7p100, .p-row .col.w7p100, .p-row .p-col.w7p100, .p-row .phanterpwa-col.w7p100, .phanterpwa-row .col.w7p100, .phanterpwa-row .p-col.w7p100, .phanterpwa-row .phanterpwa-col.w7p100 {
    width: 100%;
  }
}

@media screen and (min-width: 2100px) {
  .row .col.w8p1, .row .p-col.w8p1, .row .phanterpwa-col.w8p1, .p-row .col.w8p1, .p-row .p-col.w8p1, .p-row .phanterpwa-col.w8p1, .phanterpwa-row .col.w8p1, .phanterpwa-row .p-col.w8p1, .phanterpwa-row .phanterpwa-col.w8p1 {
    width: 1%;
  }
  .row .col.w8p2, .row .p-col.w8p2, .row .phanterpwa-col.w8p2, .p-row .col.w8p2, .p-row .p-col.w8p2, .p-row .phanterpwa-col.w8p2, .phanterpwa-row .col.w8p2, .phanterpwa-row .p-col.w8p2, .phanterpwa-row .phanterpwa-col.w8p2 {
    width: 2%;
  }
  .row .col.w8p3, .row .p-col.w8p3, .row .phanterpwa-col.w8p3, .p-row .col.w8p3, .p-row .p-col.w8p3, .p-row .phanterpwa-col.w8p3, .phanterpwa-row .col.w8p3, .phanterpwa-row .p-col.w8p3, .phanterpwa-row .phanterpwa-col.w8p3 {
    width: 3%;
  }
  .row .col.w8p4, .row .p-col.w8p4, .row .phanterpwa-col.w8p4, .p-row .col.w8p4, .p-row .p-col.w8p4, .p-row .phanterpwa-col.w8p4, .phanterpwa-row .col.w8p4, .phanterpwa-row .p-col.w8p4, .phanterpwa-row .phanterpwa-col.w8p4 {
    width: 4%;
  }
  .row .col.w8p5, .row .p-col.w8p5, .row .phanterpwa-col.w8p5, .p-row .col.w8p5, .p-row .p-col.w8p5, .p-row .phanterpwa-col.w8p5, .phanterpwa-row .col.w8p5, .phanterpwa-row .p-col.w8p5, .phanterpwa-row .phanterpwa-col.w8p5 {
    width: 5%;
  }
  .row .col.w8p6, .row .p-col.w8p6, .row .phanterpwa-col.w8p6, .p-row .col.w8p6, .p-row .p-col.w8p6, .p-row .phanterpwa-col.w8p6, .phanterpwa-row .col.w8p6, .phanterpwa-row .p-col.w8p6, .phanterpwa-row .phanterpwa-col.w8p6 {
    width: 6%;
  }
  .row .col.w8p7, .row .p-col.w8p7, .row .phanterpwa-col.w8p7, .p-row .col.w8p7, .p-row .p-col.w8p7, .p-row .phanterpwa-col.w8p7, .phanterpwa-row .col.w8p7, .phanterpwa-row .p-col.w8p7, .phanterpwa-row .phanterpwa-col.w8p7 {
    width: 7%;
  }
  .row .col.w8p8, .row .p-col.w8p8, .row .phanterpwa-col.w8p8, .p-row .col.w8p8, .p-row .p-col.w8p8, .p-row .phanterpwa-col.w8p8, .phanterpwa-row .col.w8p8, .phanterpwa-row .p-col.w8p8, .phanterpwa-row .phanterpwa-col.w8p8 {
    width: 8%;
  }
  .row .col.w8p9, .row .p-col.w8p9, .row .phanterpwa-col.w8p9, .p-row .col.w8p9, .p-row .p-col.w8p9, .p-row .phanterpwa-col.w8p9, .phanterpwa-row .col.w8p9, .phanterpwa-row .p-col.w8p9, .phanterpwa-row .phanterpwa-col.w8p9 {
    width: 9%;
  }
  .row .col.w8p10, .row .p-col.w8p10, .row .phanterpwa-col.w8p10, .p-row .col.w8p10, .p-row .p-col.w8p10, .p-row .phanterpwa-col.w8p10, .phanterpwa-row .col.w8p10, .phanterpwa-row .p-col.w8p10, .phanterpwa-row .phanterpwa-col.w8p10 {
    width: 10%;
  }
  .row .col.w8p11, .row .p-col.w8p11, .row .phanterpwa-col.w8p11, .p-row .col.w8p11, .p-row .p-col.w8p11, .p-row .phanterpwa-col.w8p11, .phanterpwa-row .col.w8p11, .phanterpwa-row .p-col.w8p11, .phanterpwa-row .phanterpwa-col.w8p11 {
    width: 11%;
  }
  .row .col.w8p12, .row .p-col.w8p12, .row .phanterpwa-col.w8p12, .p-row .col.w8p12, .p-row .p-col.w8p12, .p-row .phanterpwa-col.w8p12, .phanterpwa-row .col.w8p12, .phanterpwa-row .p-col.w8p12, .phanterpwa-row .phanterpwa-col.w8p12 {
    width: 12%;
  }
  .row .col.w8p13, .row .p-col.w8p13, .row .phanterpwa-col.w8p13, .p-row .col.w8p13, .p-row .p-col.w8p13, .p-row .phanterpwa-col.w8p13, .phanterpwa-row .col.w8p13, .phanterpwa-row .p-col.w8p13, .phanterpwa-row .phanterpwa-col.w8p13 {
    width: 13%;
  }
  .row .col.w8p14, .row .p-col.w8p14, .row .phanterpwa-col.w8p14, .p-row .col.w8p14, .p-row .p-col.w8p14, .p-row .phanterpwa-col.w8p14, .phanterpwa-row .col.w8p14, .phanterpwa-row .p-col.w8p14, .phanterpwa-row .phanterpwa-col.w8p14 {
    width: 14%;
  }
  .row .col.w8p15, .row .p-col.w8p15, .row .phanterpwa-col.w8p15, .p-row .col.w8p15, .p-row .p-col.w8p15, .p-row .phanterpwa-col.w8p15, .phanterpwa-row .col.w8p15, .phanterpwa-row .p-col.w8p15, .phanterpwa-row .phanterpwa-col.w8p15 {
    width: 15%;
  }
  .row .col.w8p16, .row .p-col.w8p16, .row .phanterpwa-col.w8p16, .p-row .col.w8p16, .p-row .p-col.w8p16, .p-row .phanterpwa-col.w8p16, .phanterpwa-row .col.w8p16, .phanterpwa-row .p-col.w8p16, .phanterpwa-row .phanterpwa-col.w8p16 {
    width: 16%;
  }
  .row .col.w8p17, .row .p-col.w8p17, .row .phanterpwa-col.w8p17, .p-row .col.w8p17, .p-row .p-col.w8p17, .p-row .phanterpwa-col.w8p17, .phanterpwa-row .col.w8p17, .phanterpwa-row .p-col.w8p17, .phanterpwa-row .phanterpwa-col.w8p17 {
    width: 17%;
  }
  .row .col.w8p18, .row .p-col.w8p18, .row .phanterpwa-col.w8p18, .p-row .col.w8p18, .p-row .p-col.w8p18, .p-row .phanterpwa-col.w8p18, .phanterpwa-row .col.w8p18, .phanterpwa-row .p-col.w8p18, .phanterpwa-row .phanterpwa-col.w8p18 {
    width: 18%;
  }
  .row .col.w8p19, .row .p-col.w8p19, .row .phanterpwa-col.w8p19, .p-row .col.w8p19, .p-row .p-col.w8p19, .p-row .phanterpwa-col.w8p19, .phanterpwa-row .col.w8p19, .phanterpwa-row .p-col.w8p19, .phanterpwa-row .phanterpwa-col.w8p19 {
    width: 19%;
  }
  .row .col.w8p20, .row .p-col.w8p20, .row .phanterpwa-col.w8p20, .p-row .col.w8p20, .p-row .p-col.w8p20, .p-row .phanterpwa-col.w8p20, .phanterpwa-row .col.w8p20, .phanterpwa-row .p-col.w8p20, .phanterpwa-row .phanterpwa-col.w8p20 {
    width: 20%;
  }
  .row .col.w8p21, .row .p-col.w8p21, .row .phanterpwa-col.w8p21, .p-row .col.w8p21, .p-row .p-col.w8p21, .p-row .phanterpwa-col.w8p21, .phanterpwa-row .col.w8p21, .phanterpwa-row .p-col.w8p21, .phanterpwa-row .phanterpwa-col.w8p21 {
    width: 21%;
  }
  .row .col.w8p22, .row .p-col.w8p22, .row .phanterpwa-col.w8p22, .p-row .col.w8p22, .p-row .p-col.w8p22, .p-row .phanterpwa-col.w8p22, .phanterpwa-row .col.w8p22, .phanterpwa-row .p-col.w8p22, .phanterpwa-row .phanterpwa-col.w8p22 {
    width: 22%;
  }
  .row .col.w8p23, .row .p-col.w8p23, .row .phanterpwa-col.w8p23, .p-row .col.w8p23, .p-row .p-col.w8p23, .p-row .phanterpwa-col.w8p23, .phanterpwa-row .col.w8p23, .phanterpwa-row .p-col.w8p23, .phanterpwa-row .phanterpwa-col.w8p23 {
    width: 23%;
  }
  .row .col.w8p24, .row .p-col.w8p24, .row .phanterpwa-col.w8p24, .p-row .col.w8p24, .p-row .p-col.w8p24, .p-row .phanterpwa-col.w8p24, .phanterpwa-row .col.w8p24, .phanterpwa-row .p-col.w8p24, .phanterpwa-row .phanterpwa-col.w8p24 {
    width: 24%;
  }
  .row .col.w8p25, .row .p-col.w8p25, .row .phanterpwa-col.w8p25, .p-row .col.w8p25, .p-row .p-col.w8p25, .p-row .phanterpwa-col.w8p25, .phanterpwa-row .col.w8p25, .phanterpwa-row .p-col.w8p25, .phanterpwa-row .phanterpwa-col.w8p25 {
    width: 25%;
  }
  .row .col.w8p26, .row .p-col.w8p26, .row .phanterpwa-col.w8p26, .p-row .col.w8p26, .p-row .p-col.w8p26, .p-row .phanterpwa-col.w8p26, .phanterpwa-row .col.w8p26, .phanterpwa-row .p-col.w8p26, .phanterpwa-row .phanterpwa-col.w8p26 {
    width: 26%;
  }
  .row .col.w8p27, .row .p-col.w8p27, .row .phanterpwa-col.w8p27, .p-row .col.w8p27, .p-row .p-col.w8p27, .p-row .phanterpwa-col.w8p27, .phanterpwa-row .col.w8p27, .phanterpwa-row .p-col.w8p27, .phanterpwa-row .phanterpwa-col.w8p27 {
    width: 27%;
  }
  .row .col.w8p28, .row .p-col.w8p28, .row .phanterpwa-col.w8p28, .p-row .col.w8p28, .p-row .p-col.w8p28, .p-row .phanterpwa-col.w8p28, .phanterpwa-row .col.w8p28, .phanterpwa-row .p-col.w8p28, .phanterpwa-row .phanterpwa-col.w8p28 {
    width: 28%;
  }
  .row .col.w8p29, .row .p-col.w8p29, .row .phanterpwa-col.w8p29, .p-row .col.w8p29, .p-row .p-col.w8p29, .p-row .phanterpwa-col.w8p29, .phanterpwa-row .col.w8p29, .phanterpwa-row .p-col.w8p29, .phanterpwa-row .phanterpwa-col.w8p29 {
    width: 29%;
  }
  .row .col.w8p30, .row .p-col.w8p30, .row .phanterpwa-col.w8p30, .p-row .col.w8p30, .p-row .p-col.w8p30, .p-row .phanterpwa-col.w8p30, .phanterpwa-row .col.w8p30, .phanterpwa-row .p-col.w8p30, .phanterpwa-row .phanterpwa-col.w8p30 {
    width: 30%;
  }
  .row .col.w8p31, .row .p-col.w8p31, .row .phanterpwa-col.w8p31, .p-row .col.w8p31, .p-row .p-col.w8p31, .p-row .phanterpwa-col.w8p31, .phanterpwa-row .col.w8p31, .phanterpwa-row .p-col.w8p31, .phanterpwa-row .phanterpwa-col.w8p31 {
    width: 31%;
  }
  .row .col.w8p32, .row .p-col.w8p32, .row .phanterpwa-col.w8p32, .p-row .col.w8p32, .p-row .p-col.w8p32, .p-row .phanterpwa-col.w8p32, .phanterpwa-row .col.w8p32, .phanterpwa-row .p-col.w8p32, .phanterpwa-row .phanterpwa-col.w8p32 {
    width: 32%;
  }
  .row .col.w8p33, .row .p-col.w8p33, .row .phanterpwa-col.w8p33, .p-row .col.w8p33, .p-row .p-col.w8p33, .p-row .phanterpwa-col.w8p33, .phanterpwa-row .col.w8p33, .phanterpwa-row .p-col.w8p33, .phanterpwa-row .phanterpwa-col.w8p33 {
    width: 33%;
  }
  .row .col.w8p34, .row .p-col.w8p34, .row .phanterpwa-col.w8p34, .p-row .col.w8p34, .p-row .p-col.w8p34, .p-row .phanterpwa-col.w8p34, .phanterpwa-row .col.w8p34, .phanterpwa-row .p-col.w8p34, .phanterpwa-row .phanterpwa-col.w8p34 {
    width: 34%;
  }
  .row .col.w8p35, .row .p-col.w8p35, .row .phanterpwa-col.w8p35, .p-row .col.w8p35, .p-row .p-col.w8p35, .p-row .phanterpwa-col.w8p35, .phanterpwa-row .col.w8p35, .phanterpwa-row .p-col.w8p35, .phanterpwa-row .phanterpwa-col.w8p35 {
    width: 35%;
  }
  .row .col.w8p36, .row .p-col.w8p36, .row .phanterpwa-col.w8p36, .p-row .col.w8p36, .p-row .p-col.w8p36, .p-row .phanterpwa-col.w8p36, .phanterpwa-row .col.w8p36, .phanterpwa-row .p-col.w8p36, .phanterpwa-row .phanterpwa-col.w8p36 {
    width: 36%;
  }
  .row .col.w8p37, .row .p-col.w8p37, .row .phanterpwa-col.w8p37, .p-row .col.w8p37, .p-row .p-col.w8p37, .p-row .phanterpwa-col.w8p37, .phanterpwa-row .col.w8p37, .phanterpwa-row .p-col.w8p37, .phanterpwa-row .phanterpwa-col.w8p37 {
    width: 37%;
  }
  .row .col.w8p38, .row .p-col.w8p38, .row .phanterpwa-col.w8p38, .p-row .col.w8p38, .p-row .p-col.w8p38, .p-row .phanterpwa-col.w8p38, .phanterpwa-row .col.w8p38, .phanterpwa-row .p-col.w8p38, .phanterpwa-row .phanterpwa-col.w8p38 {
    width: 38%;
  }
  .row .col.w8p39, .row .p-col.w8p39, .row .phanterpwa-col.w8p39, .p-row .col.w8p39, .p-row .p-col.w8p39, .p-row .phanterpwa-col.w8p39, .phanterpwa-row .col.w8p39, .phanterpwa-row .p-col.w8p39, .phanterpwa-row .phanterpwa-col.w8p39 {
    width: 39%;
  }
  .row .col.w8p40, .row .p-col.w8p40, .row .phanterpwa-col.w8p40, .p-row .col.w8p40, .p-row .p-col.w8p40, .p-row .phanterpwa-col.w8p40, .phanterpwa-row .col.w8p40, .phanterpwa-row .p-col.w8p40, .phanterpwa-row .phanterpwa-col.w8p40 {
    width: 40%;
  }
  .row .col.w8p41, .row .p-col.w8p41, .row .phanterpwa-col.w8p41, .p-row .col.w8p41, .p-row .p-col.w8p41, .p-row .phanterpwa-col.w8p41, .phanterpwa-row .col.w8p41, .phanterpwa-row .p-col.w8p41, .phanterpwa-row .phanterpwa-col.w8p41 {
    width: 41%;
  }
  .row .col.w8p42, .row .p-col.w8p42, .row .phanterpwa-col.w8p42, .p-row .col.w8p42, .p-row .p-col.w8p42, .p-row .phanterpwa-col.w8p42, .phanterpwa-row .col.w8p42, .phanterpwa-row .p-col.w8p42, .phanterpwa-row .phanterpwa-col.w8p42 {
    width: 42%;
  }
  .row .col.w8p43, .row .p-col.w8p43, .row .phanterpwa-col.w8p43, .p-row .col.w8p43, .p-row .p-col.w8p43, .p-row .phanterpwa-col.w8p43, .phanterpwa-row .col.w8p43, .phanterpwa-row .p-col.w8p43, .phanterpwa-row .phanterpwa-col.w8p43 {
    width: 43%;
  }
  .row .col.w8p44, .row .p-col.w8p44, .row .phanterpwa-col.w8p44, .p-row .col.w8p44, .p-row .p-col.w8p44, .p-row .phanterpwa-col.w8p44, .phanterpwa-row .col.w8p44, .phanterpwa-row .p-col.w8p44, .phanterpwa-row .phanterpwa-col.w8p44 {
    width: 44%;
  }
  .row .col.w8p45, .row .p-col.w8p45, .row .phanterpwa-col.w8p45, .p-row .col.w8p45, .p-row .p-col.w8p45, .p-row .phanterpwa-col.w8p45, .phanterpwa-row .col.w8p45, .phanterpwa-row .p-col.w8p45, .phanterpwa-row .phanterpwa-col.w8p45 {
    width: 45%;
  }
  .row .col.w8p46, .row .p-col.w8p46, .row .phanterpwa-col.w8p46, .p-row .col.w8p46, .p-row .p-col.w8p46, .p-row .phanterpwa-col.w8p46, .phanterpwa-row .col.w8p46, .phanterpwa-row .p-col.w8p46, .phanterpwa-row .phanterpwa-col.w8p46 {
    width: 46%;
  }
  .row .col.w8p47, .row .p-col.w8p47, .row .phanterpwa-col.w8p47, .p-row .col.w8p47, .p-row .p-col.w8p47, .p-row .phanterpwa-col.w8p47, .phanterpwa-row .col.w8p47, .phanterpwa-row .p-col.w8p47, .phanterpwa-row .phanterpwa-col.w8p47 {
    width: 47%;
  }
  .row .col.w8p48, .row .p-col.w8p48, .row .phanterpwa-col.w8p48, .p-row .col.w8p48, .p-row .p-col.w8p48, .p-row .phanterpwa-col.w8p48, .phanterpwa-row .col.w8p48, .phanterpwa-row .p-col.w8p48, .phanterpwa-row .phanterpwa-col.w8p48 {
    width: 48%;
  }
  .row .col.w8p49, .row .p-col.w8p49, .row .phanterpwa-col.w8p49, .p-row .col.w8p49, .p-row .p-col.w8p49, .p-row .phanterpwa-col.w8p49, .phanterpwa-row .col.w8p49, .phanterpwa-row .p-col.w8p49, .phanterpwa-row .phanterpwa-col.w8p49 {
    width: 49%;
  }
  .row .col.w8p50, .row .p-col.w8p50, .row .phanterpwa-col.w8p50, .p-row .col.w8p50, .p-row .p-col.w8p50, .p-row .phanterpwa-col.w8p50, .phanterpwa-row .col.w8p50, .phanterpwa-row .p-col.w8p50, .phanterpwa-row .phanterpwa-col.w8p50 {
    width: 50%;
  }
  .row .col.w8p51, .row .p-col.w8p51, .row .phanterpwa-col.w8p51, .p-row .col.w8p51, .p-row .p-col.w8p51, .p-row .phanterpwa-col.w8p51, .phanterpwa-row .col.w8p51, .phanterpwa-row .p-col.w8p51, .phanterpwa-row .phanterpwa-col.w8p51 {
    width: 51%;
  }
  .row .col.w8p52, .row .p-col.w8p52, .row .phanterpwa-col.w8p52, .p-row .col.w8p52, .p-row .p-col.w8p52, .p-row .phanterpwa-col.w8p52, .phanterpwa-row .col.w8p52, .phanterpwa-row .p-col.w8p52, .phanterpwa-row .phanterpwa-col.w8p52 {
    width: 52%;
  }
  .row .col.w8p53, .row .p-col.w8p53, .row .phanterpwa-col.w8p53, .p-row .col.w8p53, .p-row .p-col.w8p53, .p-row .phanterpwa-col.w8p53, .phanterpwa-row .col.w8p53, .phanterpwa-row .p-col.w8p53, .phanterpwa-row .phanterpwa-col.w8p53 {
    width: 53%;
  }
  .row .col.w8p54, .row .p-col.w8p54, .row .phanterpwa-col.w8p54, .p-row .col.w8p54, .p-row .p-col.w8p54, .p-row .phanterpwa-col.w8p54, .phanterpwa-row .col.w8p54, .phanterpwa-row .p-col.w8p54, .phanterpwa-row .phanterpwa-col.w8p54 {
    width: 54%;
  }
  .row .col.w8p55, .row .p-col.w8p55, .row .phanterpwa-col.w8p55, .p-row .col.w8p55, .p-row .p-col.w8p55, .p-row .phanterpwa-col.w8p55, .phanterpwa-row .col.w8p55, .phanterpwa-row .p-col.w8p55, .phanterpwa-row .phanterpwa-col.w8p55 {
    width: 55%;
  }
  .row .col.w8p56, .row .p-col.w8p56, .row .phanterpwa-col.w8p56, .p-row .col.w8p56, .p-row .p-col.w8p56, .p-row .phanterpwa-col.w8p56, .phanterpwa-row .col.w8p56, .phanterpwa-row .p-col.w8p56, .phanterpwa-row .phanterpwa-col.w8p56 {
    width: 56%;
  }
  .row .col.w8p57, .row .p-col.w8p57, .row .phanterpwa-col.w8p57, .p-row .col.w8p57, .p-row .p-col.w8p57, .p-row .phanterpwa-col.w8p57, .phanterpwa-row .col.w8p57, .phanterpwa-row .p-col.w8p57, .phanterpwa-row .phanterpwa-col.w8p57 {
    width: 57%;
  }
  .row .col.w8p58, .row .p-col.w8p58, .row .phanterpwa-col.w8p58, .p-row .col.w8p58, .p-row .p-col.w8p58, .p-row .phanterpwa-col.w8p58, .phanterpwa-row .col.w8p58, .phanterpwa-row .p-col.w8p58, .phanterpwa-row .phanterpwa-col.w8p58 {
    width: 58%;
  }
  .row .col.w8p59, .row .p-col.w8p59, .row .phanterpwa-col.w8p59, .p-row .col.w8p59, .p-row .p-col.w8p59, .p-row .phanterpwa-col.w8p59, .phanterpwa-row .col.w8p59, .phanterpwa-row .p-col.w8p59, .phanterpwa-row .phanterpwa-col.w8p59 {
    width: 59%;
  }
  .row .col.w8p60, .row .p-col.w8p60, .row .phanterpwa-col.w8p60, .p-row .col.w8p60, .p-row .p-col.w8p60, .p-row .phanterpwa-col.w8p60, .phanterpwa-row .col.w8p60, .phanterpwa-row .p-col.w8p60, .phanterpwa-row .phanterpwa-col.w8p60 {
    width: 60%;
  }
  .row .col.w8p61, .row .p-col.w8p61, .row .phanterpwa-col.w8p61, .p-row .col.w8p61, .p-row .p-col.w8p61, .p-row .phanterpwa-col.w8p61, .phanterpwa-row .col.w8p61, .phanterpwa-row .p-col.w8p61, .phanterpwa-row .phanterpwa-col.w8p61 {
    width: 61%;
  }
  .row .col.w8p62, .row .p-col.w8p62, .row .phanterpwa-col.w8p62, .p-row .col.w8p62, .p-row .p-col.w8p62, .p-row .phanterpwa-col.w8p62, .phanterpwa-row .col.w8p62, .phanterpwa-row .p-col.w8p62, .phanterpwa-row .phanterpwa-col.w8p62 {
    width: 62%;
  }
  .row .col.w8p63, .row .p-col.w8p63, .row .phanterpwa-col.w8p63, .p-row .col.w8p63, .p-row .p-col.w8p63, .p-row .phanterpwa-col.w8p63, .phanterpwa-row .col.w8p63, .phanterpwa-row .p-col.w8p63, .phanterpwa-row .phanterpwa-col.w8p63 {
    width: 63%;
  }
  .row .col.w8p64, .row .p-col.w8p64, .row .phanterpwa-col.w8p64, .p-row .col.w8p64, .p-row .p-col.w8p64, .p-row .phanterpwa-col.w8p64, .phanterpwa-row .col.w8p64, .phanterpwa-row .p-col.w8p64, .phanterpwa-row .phanterpwa-col.w8p64 {
    width: 64%;
  }
  .row .col.w8p65, .row .p-col.w8p65, .row .phanterpwa-col.w8p65, .p-row .col.w8p65, .p-row .p-col.w8p65, .p-row .phanterpwa-col.w8p65, .phanterpwa-row .col.w8p65, .phanterpwa-row .p-col.w8p65, .phanterpwa-row .phanterpwa-col.w8p65 {
    width: 65%;
  }
  .row .col.w8p66, .row .p-col.w8p66, .row .phanterpwa-col.w8p66, .p-row .col.w8p66, .p-row .p-col.w8p66, .p-row .phanterpwa-col.w8p66, .phanterpwa-row .col.w8p66, .phanterpwa-row .p-col.w8p66, .phanterpwa-row .phanterpwa-col.w8p66 {
    width: 66%;
  }
  .row .col.w8p67, .row .p-col.w8p67, .row .phanterpwa-col.w8p67, .p-row .col.w8p67, .p-row .p-col.w8p67, .p-row .phanterpwa-col.w8p67, .phanterpwa-row .col.w8p67, .phanterpwa-row .p-col.w8p67, .phanterpwa-row .phanterpwa-col.w8p67 {
    width: 67%;
  }
  .row .col.w8p68, .row .p-col.w8p68, .row .phanterpwa-col.w8p68, .p-row .col.w8p68, .p-row .p-col.w8p68, .p-row .phanterpwa-col.w8p68, .phanterpwa-row .col.w8p68, .phanterpwa-row .p-col.w8p68, .phanterpwa-row .phanterpwa-col.w8p68 {
    width: 68%;
  }
  .row .col.w8p69, .row .p-col.w8p69, .row .phanterpwa-col.w8p69, .p-row .col.w8p69, .p-row .p-col.w8p69, .p-row .phanterpwa-col.w8p69, .phanterpwa-row .col.w8p69, .phanterpwa-row .p-col.w8p69, .phanterpwa-row .phanterpwa-col.w8p69 {
    width: 69%;
  }
  .row .col.w8p70, .row .p-col.w8p70, .row .phanterpwa-col.w8p70, .p-row .col.w8p70, .p-row .p-col.w8p70, .p-row .phanterpwa-col.w8p70, .phanterpwa-row .col.w8p70, .phanterpwa-row .p-col.w8p70, .phanterpwa-row .phanterpwa-col.w8p70 {
    width: 70%;
  }
  .row .col.w8p71, .row .p-col.w8p71, .row .phanterpwa-col.w8p71, .p-row .col.w8p71, .p-row .p-col.w8p71, .p-row .phanterpwa-col.w8p71, .phanterpwa-row .col.w8p71, .phanterpwa-row .p-col.w8p71, .phanterpwa-row .phanterpwa-col.w8p71 {
    width: 71%;
  }
  .row .col.w8p72, .row .p-col.w8p72, .row .phanterpwa-col.w8p72, .p-row .col.w8p72, .p-row .p-col.w8p72, .p-row .phanterpwa-col.w8p72, .phanterpwa-row .col.w8p72, .phanterpwa-row .p-col.w8p72, .phanterpwa-row .phanterpwa-col.w8p72 {
    width: 72%;
  }
  .row .col.w8p73, .row .p-col.w8p73, .row .phanterpwa-col.w8p73, .p-row .col.w8p73, .p-row .p-col.w8p73, .p-row .phanterpwa-col.w8p73, .phanterpwa-row .col.w8p73, .phanterpwa-row .p-col.w8p73, .phanterpwa-row .phanterpwa-col.w8p73 {
    width: 73%;
  }
  .row .col.w8p74, .row .p-col.w8p74, .row .phanterpwa-col.w8p74, .p-row .col.w8p74, .p-row .p-col.w8p74, .p-row .phanterpwa-col.w8p74, .phanterpwa-row .col.w8p74, .phanterpwa-row .p-col.w8p74, .phanterpwa-row .phanterpwa-col.w8p74 {
    width: 74%;
  }
  .row .col.w8p75, .row .p-col.w8p75, .row .phanterpwa-col.w8p75, .p-row .col.w8p75, .p-row .p-col.w8p75, .p-row .phanterpwa-col.w8p75, .phanterpwa-row .col.w8p75, .phanterpwa-row .p-col.w8p75, .phanterpwa-row .phanterpwa-col.w8p75 {
    width: 75%;
  }
  .row .col.w8p76, .row .p-col.w8p76, .row .phanterpwa-col.w8p76, .p-row .col.w8p76, .p-row .p-col.w8p76, .p-row .phanterpwa-col.w8p76, .phanterpwa-row .col.w8p76, .phanterpwa-row .p-col.w8p76, .phanterpwa-row .phanterpwa-col.w8p76 {
    width: 76%;
  }
  .row .col.w8p77, .row .p-col.w8p77, .row .phanterpwa-col.w8p77, .p-row .col.w8p77, .p-row .p-col.w8p77, .p-row .phanterpwa-col.w8p77, .phanterpwa-row .col.w8p77, .phanterpwa-row .p-col.w8p77, .phanterpwa-row .phanterpwa-col.w8p77 {
    width: 77%;
  }
  .row .col.w8p78, .row .p-col.w8p78, .row .phanterpwa-col.w8p78, .p-row .col.w8p78, .p-row .p-col.w8p78, .p-row .phanterpwa-col.w8p78, .phanterpwa-row .col.w8p78, .phanterpwa-row .p-col.w8p78, .phanterpwa-row .phanterpwa-col.w8p78 {
    width: 78%;
  }
  .row .col.w8p79, .row .p-col.w8p79, .row .phanterpwa-col.w8p79, .p-row .col.w8p79, .p-row .p-col.w8p79, .p-row .phanterpwa-col.w8p79, .phanterpwa-row .col.w8p79, .phanterpwa-row .p-col.w8p79, .phanterpwa-row .phanterpwa-col.w8p79 {
    width: 79%;
  }
  .row .col.w8p80, .row .p-col.w8p80, .row .phanterpwa-col.w8p80, .p-row .col.w8p80, .p-row .p-col.w8p80, .p-row .phanterpwa-col.w8p80, .phanterpwa-row .col.w8p80, .phanterpwa-row .p-col.w8p80, .phanterpwa-row .phanterpwa-col.w8p80 {
    width: 80%;
  }
  .row .col.w8p81, .row .p-col.w8p81, .row .phanterpwa-col.w8p81, .p-row .col.w8p81, .p-row .p-col.w8p81, .p-row .phanterpwa-col.w8p81, .phanterpwa-row .col.w8p81, .phanterpwa-row .p-col.w8p81, .phanterpwa-row .phanterpwa-col.w8p81 {
    width: 81%;
  }
  .row .col.w8p82, .row .p-col.w8p82, .row .phanterpwa-col.w8p82, .p-row .col.w8p82, .p-row .p-col.w8p82, .p-row .phanterpwa-col.w8p82, .phanterpwa-row .col.w8p82, .phanterpwa-row .p-col.w8p82, .phanterpwa-row .phanterpwa-col.w8p82 {
    width: 82%;
  }
  .row .col.w8p83, .row .p-col.w8p83, .row .phanterpwa-col.w8p83, .p-row .col.w8p83, .p-row .p-col.w8p83, .p-row .phanterpwa-col.w8p83, .phanterpwa-row .col.w8p83, .phanterpwa-row .p-col.w8p83, .phanterpwa-row .phanterpwa-col.w8p83 {
    width: 83%;
  }
  .row .col.w8p84, .row .p-col.w8p84, .row .phanterpwa-col.w8p84, .p-row .col.w8p84, .p-row .p-col.w8p84, .p-row .phanterpwa-col.w8p84, .phanterpwa-row .col.w8p84, .phanterpwa-row .p-col.w8p84, .phanterpwa-row .phanterpwa-col.w8p84 {
    width: 84%;
  }
  .row .col.w8p85, .row .p-col.w8p85, .row .phanterpwa-col.w8p85, .p-row .col.w8p85, .p-row .p-col.w8p85, .p-row .phanterpwa-col.w8p85, .phanterpwa-row .col.w8p85, .phanterpwa-row .p-col.w8p85, .phanterpwa-row .phanterpwa-col.w8p85 {
    width: 85%;
  }
  .row .col.w8p86, .row .p-col.w8p86, .row .phanterpwa-col.w8p86, .p-row .col.w8p86, .p-row .p-col.w8p86, .p-row .phanterpwa-col.w8p86, .phanterpwa-row .col.w8p86, .phanterpwa-row .p-col.w8p86, .phanterpwa-row .phanterpwa-col.w8p86 {
    width: 86%;
  }
  .row .col.w8p87, .row .p-col.w8p87, .row .phanterpwa-col.w8p87, .p-row .col.w8p87, .p-row .p-col.w8p87, .p-row .phanterpwa-col.w8p87, .phanterpwa-row .col.w8p87, .phanterpwa-row .p-col.w8p87, .phanterpwa-row .phanterpwa-col.w8p87 {
    width: 87%;
  }
  .row .col.w8p88, .row .p-col.w8p88, .row .phanterpwa-col.w8p88, .p-row .col.w8p88, .p-row .p-col.w8p88, .p-row .phanterpwa-col.w8p88, .phanterpwa-row .col.w8p88, .phanterpwa-row .p-col.w8p88, .phanterpwa-row .phanterpwa-col.w8p88 {
    width: 88%;
  }
  .row .col.w8p89, .row .p-col.w8p89, .row .phanterpwa-col.w8p89, .p-row .col.w8p89, .p-row .p-col.w8p89, .p-row .phanterpwa-col.w8p89, .phanterpwa-row .col.w8p89, .phanterpwa-row .p-col.w8p89, .phanterpwa-row .phanterpwa-col.w8p89 {
    width: 89%;
  }
  .row .col.w8p90, .row .p-col.w8p90, .row .phanterpwa-col.w8p90, .p-row .col.w8p90, .p-row .p-col.w8p90, .p-row .phanterpwa-col.w8p90, .phanterpwa-row .col.w8p90, .phanterpwa-row .p-col.w8p90, .phanterpwa-row .phanterpwa-col.w8p90 {
    width: 90%;
  }
  .row .col.w8p91, .row .p-col.w8p91, .row .phanterpwa-col.w8p91, .p-row .col.w8p91, .p-row .p-col.w8p91, .p-row .phanterpwa-col.w8p91, .phanterpwa-row .col.w8p91, .phanterpwa-row .p-col.w8p91, .phanterpwa-row .phanterpwa-col.w8p91 {
    width: 91%;
  }
  .row .col.w8p92, .row .p-col.w8p92, .row .phanterpwa-col.w8p92, .p-row .col.w8p92, .p-row .p-col.w8p92, .p-row .phanterpwa-col.w8p92, .phanterpwa-row .col.w8p92, .phanterpwa-row .p-col.w8p92, .phanterpwa-row .phanterpwa-col.w8p92 {
    width: 92%;
  }
  .row .col.w8p93, .row .p-col.w8p93, .row .phanterpwa-col.w8p93, .p-row .col.w8p93, .p-row .p-col.w8p93, .p-row .phanterpwa-col.w8p93, .phanterpwa-row .col.w8p93, .phanterpwa-row .p-col.w8p93, .phanterpwa-row .phanterpwa-col.w8p93 {
    width: 93%;
  }
  .row .col.w8p94, .row .p-col.w8p94, .row .phanterpwa-col.w8p94, .p-row .col.w8p94, .p-row .p-col.w8p94, .p-row .phanterpwa-col.w8p94, .phanterpwa-row .col.w8p94, .phanterpwa-row .p-col.w8p94, .phanterpwa-row .phanterpwa-col.w8p94 {
    width: 94%;
  }
  .row .col.w8p95, .row .p-col.w8p95, .row .phanterpwa-col.w8p95, .p-row .col.w8p95, .p-row .p-col.w8p95, .p-row .phanterpwa-col.w8p95, .phanterpwa-row .col.w8p95, .phanterpwa-row .p-col.w8p95, .phanterpwa-row .phanterpwa-col.w8p95 {
    width: 95%;
  }
  .row .col.w8p96, .row .p-col.w8p96, .row .phanterpwa-col.w8p96, .p-row .col.w8p96, .p-row .p-col.w8p96, .p-row .phanterpwa-col.w8p96, .phanterpwa-row .col.w8p96, .phanterpwa-row .p-col.w8p96, .phanterpwa-row .phanterpwa-col.w8p96 {
    width: 96%;
  }
  .row .col.w8p97, .row .p-col.w8p97, .row .phanterpwa-col.w8p97, .p-row .col.w8p97, .p-row .p-col.w8p97, .p-row .phanterpwa-col.w8p97, .phanterpwa-row .col.w8p97, .phanterpwa-row .p-col.w8p97, .phanterpwa-row .phanterpwa-col.w8p97 {
    width: 97%;
  }
  .row .col.w8p98, .row .p-col.w8p98, .row .phanterpwa-col.w8p98, .p-row .col.w8p98, .p-row .p-col.w8p98, .p-row .phanterpwa-col.w8p98, .phanterpwa-row .col.w8p98, .phanterpwa-row .p-col.w8p98, .phanterpwa-row .phanterpwa-col.w8p98 {
    width: 98%;
  }
  .row .col.w8p99, .row .p-col.w8p99, .row .phanterpwa-col.w8p99, .p-row .col.w8p99, .p-row .p-col.w8p99, .p-row .phanterpwa-col.w8p99, .phanterpwa-row .col.w8p99, .phanterpwa-row .p-col.w8p99, .phanterpwa-row .phanterpwa-col.w8p99 {
    width: 99%;
  }
  .row .col.w8p100, .row .p-col.w8p100, .row .phanterpwa-col.w8p100, .p-row .col.w8p100, .p-row .p-col.w8p100, .p-row .phanterpwa-col.w8p100, .phanterpwa-row .col.w8p100, .phanterpwa-row .p-col.w8p100, .phanterpwa-row .phanterpwa-col.w8p100 {
    width: 100%;
  }
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\events\waves.sass */
.wave_on_click {
  overflow: hidden;
  position: relative;
  min-height: 20px;
}

.wave_on_click .wave {
  opacity: 0.8;
  width: 2px;
  height: 2px;
  background-color: rgba(200, 200, 200, 0.5);
  position: absolute;
  border-radius: 100%;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\extends\capcha_preload.sass */
.phanterpwa-components-preloaders-android * {
  box-sizing: border-box;
}

.phanterpwa-components-preloaders-android .left {
  float: left !important;
}

.phanterpwa-components-preloaders-android .right {
  float: right !important;
}

.phanterpwa-components-preloaders-android .preloader-wrapper {
  display: inline-block;
  position: relative;
  width: 200px;
  height: 200px;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_android {
  width: 100%;
  height: 100%;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-layer {
  position: absolute;
  width: 100%;
  height: 100%;
  opacity: 0;
  border-color: blue;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-one,
.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-one-only {
  border-color: blue;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-two,
.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-two-only {
  border-color: red;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-three,
.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-three-only {
  border-color: #f4b400;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-four,
.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-four-only {
  border-color: green;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_gap-patch {
  position: absolute;
  top: 0;
  left: 45%;
  width: 10%;
  height: 100%;
  overflow: hidden;
  border-color: inherit;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_gap-patch .phanterpwa_circle {
  width: 1000%;
  left: -450%;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_circle_clipper {
  display: inline-block;
  position: relative;
  width: 50%;
  height: 100%;
  overflow: hidden;
  border-color: inherit;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_circle_clipper .phanterpwa_circle {
  width: 200%;
  height: 100%;
  border-width: 10px;
  border-style: solid;
  border-color: inherit;
  border-bottom-color: transparent !important;
  border-radius: 50%;
  -webkit-animation: none;
  animation: none;
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_circle_clipper.left .phanterpwa_circle {
  left: 0;
  border-right-color: transparent !important;
  -webkit-transform: rotate(129deg);
  transform: rotate(129deg);
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_circle_clipper.right .phanterpwa_circle {
  left: -100%;
  border-left-color: transparent !important;
  -webkit-transform: rotate(-129deg);
  transform: rotate(-129deg);
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .phanterpwa_android {
  opacity: 1;
  -webkit-animation: container-rotate 1568ms linear infinite;
  animation: container-rotate 1568ms linear infinite;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-one {
  -webkit-animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_one_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_one_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-two {
  -webkit-animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_two_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_two_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-three {
  -webkit-animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_three_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_three_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-four {
  -webkit-animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_four_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_four_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer,
.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-one-only,
.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-two-only,
.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-three-only,
.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-four-only {
  opacity: 1;
  -webkit-animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .phanterpwa_circle_clipper.left .phanterpwa_circle {
  -webkit-animation: left-spin 1333ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: left-spin 1333ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .phanterpwa_circle_clipper.right .phanterpwa_circle {
  -webkit-animation: right-spin 1333ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: right-spin 1333ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

@-webkit-keyframes container-rotate {
  to {
    -webkit-transform: rotate(360deg);
  }
}

@keyframes container-rotate {
  to {
    transform: rotate(360deg);
  }
}

@-webkit-keyframes phanterpwa_android_rotate_animate {
  12.5% {
    webkit-transform: rotate(135deg);
  }
  25% {
    webkit-transform: rotate(270deg);
  }
  37.5% {
    webkit-transform: rotate(405deg);
  }
  50% {
    webkit-transform: rotate(540deg);
  }
  62.5% {
    webkit-transform: rotate(675deg);
  }
  75% {
    webkit-transform: rotate(810deg);
  }
  87.5% {
    webkit-transform: rotate(945deg);
  }
  to {
    webkit-transform: rotate(1080deg);
  }
}

@keyframes phanterpwa_android_rotate_animate {
  12.5% {
    transform: rotate(135deg);
  }
  25% {
    transform: rotate(270deg);
  }
  37.5% {
    transform: rotate(405deg);
  }
  50% {
    transform: rotate(540deg);
  }
  62.5% {
    transform: rotate(675deg);
  }
  75% {
    transform: rotate(810deg);
  }
  87.5% {
    transform: rotate(945deg);
  }
  to {
    transform: rotate(1080deg);
  }
}

@-webkit-keyframes phanterpwa_android_one_fade {
  from {
    opacity: 1;
  }
  25% {
    opacity: 1;
  }
  26% {
    opacity: 0;
  }
  89% {
    opacity: 0;
  }
  90% {
    opacity: 1;
  }
  100% {
    opacity: 1;
  }
}

@keyframes phanterpwa_android_one_fade {
  from {
    opacity: 1;
  }
  25% {
    opacity: 1;
  }
  26% {
    opacity: 0;
  }
  89% {
    opacity: 0;
  }
  90% {
    opacity: 1;
  }
  100% {
    opacity: 1;
  }
}

@-webkit-keyframes phanterpwa_android_two_fade {
  from {
    opacity: 0;
  }
  15% {
    opacity: 0;
  }
  25% {
    opacity: 1;
  }
  50% {
    opacity: 1;
  }
  51% {
    opacity: 0;
  }
}

@keyframes phanterpwa_android_two_fade {
  from {
    opacity: 0;
  }
  15% {
    opacity: 0;
  }
  25% {
    opacity: 1;
  }
  50% {
    opacity: 1;
  }
  51% {
    opacity: 0;
  }
}

@-webkit-keyframes phanterpwa_android_three_fade {
  from {
    opacity: 0;
  }
  40% {
    opacity: 0;
  }
  50% {
    opacity: 1;
  }
  75% {
    opacity: 1;
  }
  76% {
    opacity: 0;
  }
}

@keyframes phanterpwa_android_three_fade {
  from {
    opacity: 0;
  }
  40% {
    opacity: 0;
  }
  50% {
    opacity: 1;
  }
  75% {
    opacity: 1;
  }
  76% {
    opacity: 0;
  }
}

@-webkit-keyframes phanterpwa_android_four_fade {
  from {
    opacity: 0;
  }
  65% {
    opacity: 0;
  }
  75% {
    opacity: 1;
  }
  90% {
    opacity: 1;
  }
  100% {
    opacity: 0;
  }
}

@keyframes phanterpwa_android_four_fade {
  from {
    opacity: 0;
  }
  65% {
    opacity: 0;
  }
  75% {
    opacity: 1;
  }
  90% {
    opacity: 1;
  }
  100% {
    opacity: 0;
  }
}

@-webkit-keyframes left-spin {
  from {
    -webkit-transform: rotate(130deg);
  }
  50% {
    -webkit-transform: rotate(-5deg);
  }
  to {
    -webkit-transform: rotate(130deg);
  }
}

@keyframes left-spin {
  from {
    transform: rotate(130deg);
  }
  50% {
    transform: rotate(-5deg);
  }
  to {
    transform: rotate(130deg);
  }
}

@-webkit-keyframes right-spin {
  from {
    -webkit-transform: rotate(-130deg);
  }
  50% {
    -webkit-transform: rotate(5deg);
  }
  to {
    -webkit-transform: rotate(-130deg);
  }
}

@keyframes right-spin {
  from {
    transform: rotate(-130deg);
  }
  50% {
    transform: rotate(5deg);
  }
  to {
    transform: rotate(-130deg);
  }
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\extends\logo.sass */
.logo_fil0 {
  fill: #757575;
  fill-rule: nonzero;
}

.logo_fil1 {
  fill: #757575;
  fill-rule: nonzero;
}

.logo_fil3 {
  fill: #fff59d;
  fill-rule: nonzero;
}

.logo_fil2 {
  fill: url("#logo_id0");
}

.logo-svg {
  height: 58px;
  width: auto;
  font-size: 1rem;
  line-height: normal;
  padding: 10px;
}

.logo-svg svg {
  width: auto;
  height: 100%;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\extends\run_points.sass */
.phanterpwa-components-preloaders-run_points * {
  box-sizing: border-box;
}

.phanterpwa-components-preloaders-run_points .left {
  float: left !important;
}

.phanterpwa-components-preloaders-run_points .right {
  float: right !important;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper {
  display: inline-block;
  position: relative;
  width: 100px;
  height: 40px;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper .phanterpwa_run_points_container {
  width: 100%;
  height: 100%;
  position: relative;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper .phanterpwa_run_points {
  width: 20%;
  height: 50%;
  -moz-border-radius: 100%;
  -webkit-border-radius: 100%;
  border-radius: 100%;
  position: absolute;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_one_content {
  -webkit-animation: phanterpwa_run_points 2s linear infinite;
  animation: phanterpwa_run_points 2s linear infinite;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_one_content .phanterpwa_run_points_one {
  background-color: red;
  left: 10%;
  top: 10%;
  border-radius: 100%;
  width: 80%;
  height: 80%;
  position: absolute;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_two_content {
  -webkit-animation: phanterpwa_run_points 2s linear infinite -.4s;
  animation: phanterpwa_run_points 2s linear infinite -.4s;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_two_content .phanterpwa_run_points_two {
  background-color: blue;
  left: 10%;
  top: 10%;
  border-radius: 100%;
  width: 80%;
  height: 80%;
  position: absolute;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_three_content {
  -webkit-animation: phanterpwa_run_points 2s linear infinite -.8s;
  animation: phanterpwa_run_points 2s linear infinite -.8s;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_three_content .phanterpwa_run_points_three {
  background-color: yellow;
  left: 10%;
  top: 10%;
  border-radius: 100%;
  width: 80%;
  height: 80%;
  position: absolute;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_four_content {
  -webkit-animation: phanterpwa_run_points 2s linear infinite -1.2s;
  animation: phanterpwa_run_points 2s linear infinite -1.2s;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_four_content .phanterpwa_run_points_four {
  background-color: pink;
  left: 10%;
  top: 10%;
  border-radius: 100%;
  width: 80%;
  height: 80%;
  position: absolute;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_five_content {
  -webkit-animation: phanterpwa_run_points 2s linear infinite -1.6s;
  animation: phanterpwa_run_points 2s linear infinite -1.6s;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_five_content .phanterpwa_run_points_five {
  background-color: green;
  left: 10%;
  top: 10%;
  border-radius: 100%;
  width: 80%;
  height: 80%;
  position: absolute;
}

@keyframes phanterpwa_run_points {
  0% {
    width: 20%;
    left: 80%;
    top: 50%;
  }
  80% {
    width: 20%;
    left: 0;
    top: 50%;
  }
  85% {
    left: 0;
    top: 10%;
    width: 20%;
    height: 50%;
  }
  88% {
    top: 20%;
    width: 40%;
    height: 30%;
  }
  92.5% {
    top: 20%;
    width: 40%;
    height: 30%;
  }
  97% {
    top: 20%;
    left: 80%;
    width: 20%;
    height: 30%;
  }
  100% {
    width: 20%;
    left: 80%;
    top: 50%;
  }
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\gates\componentes.sass */
.icon_awesome-wrapper {
  font-size: 3rem;
  padding: 6px;
  text-align: center;
}

.icon_awesome-wrapper .icon_awesome-button:hover {
  color: orange;
  cursor: pointer;
}

.icon_awesome-wrapper .icon_awesome-panel-container {
  position: relative;
}

.icon_awesome-wrapper .icon_awesome_panel_wrapper {
  position: absolute;
  padding: 10px;
  font-size: 1rem;
  display: block;
  z-index: 900;
}

.icon_awesome-wrapper .icon_awesome_panel_wrapper .icon_awesome_panel {
  background-color: white;
  padding: 10px;
  font-size: 1rem;
  display: block;
  padding: 9px;
  box-shadow: 1px 1px 4px gray;
  padding-left: 33px;
}

.icon_awesome-wrapper .icon_awesome_panel_wrapper .icon_awesome_panel .copy_buttom {
  position: absolute;
  left: 22px;
  top: 21px;
}

.icon_awesome-wrapper .icon_awesome_panel_wrapper .icon_awesome_panel .copy_buttom:hover {
  color: orange;
  cursor: pointer;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\gates\main.sass */
.promo-container {
  text-align: center;
  padding: 20px;
}

.promo-container.link:hover .promo-icon {
  color: orange;
}

.promo-container .promo-icon {
  font-size: 6rem;
}

.title_page_container {
  display: table;
  margin: 0 0 30px 0;
  padding: 8px;
  width: 100%;
  text-transform: uppercase;
  font-size: 1.6rem;
  text-align: center;
}

.title_page_container.card {
  border-radius: 0px;
  min-height: 58px;
}

@media screen and (min-width: 0) and (max-width: 301px) {
  .title_page_container.card {
    min-height: auto;
  }
}

@media screen and (min-width: 300px) and (max-width: 601px) {
  .title_page_container.card {
    min-height: auto;
  }
}

.title_page_container h1, .title_page_container h2, .title_page_container h3, .title_page_container h4, .title_page_container h5 {
  margin: 0;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\plugins\codemirror.sass */
.source_code-codemirror-wrapper {
  margin: 10px;
  padding: 5px;
  border-radius: 5px;
  box-shadow: 0 0 6px inset gray;
}

.CodeMirror {
  height: auto;
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\preloaders\android.sass */
.phanterpwa-components-preloaders-android * {
  box-sizing: border-box;
}

.phanterpwa-components-preloaders-android .left {
  float: left !important;
}

.phanterpwa-components-preloaders-android .right {
  float: right !important;
}

.phanterpwa-components-preloaders-android .preloader-wrapper {
  display: inline-block;
  position: relative;
  width: 60px;
  height: 60px;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_android {
  width: 100%;
  height: 100%;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-layer {
  position: absolute;
  width: 100%;
  height: 100%;
  opacity: 0;
  border-color: red;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-one,
.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-one-only {
  border-color: red;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-two,
.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-two-only {
  border-color: red;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-three,
.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-three-only {
  border-color: red;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-four,
.phanterpwa-components-preloaders-android .preloader-wrapper .spinner-four-only {
  border-color: red;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_gap-patch {
  position: absolute;
  top: 0;
  left: 45%;
  width: 10%;
  height: 100%;
  overflow: hidden;
  border-color: inherit;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_gap-patch .phanterpwa_circle {
  width: 1000%;
  left: -450%;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_circle_clipper {
  display: inline-block;
  position: relative;
  width: 50%;
  height: 100%;
  overflow: hidden;
  border-color: inherit;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_circle_clipper .phanterpwa_circle {
  width: 200%;
  height: 100%;
  border-width: 8px;
  border-style: solid;
  border-color: inherit;
  border-bottom-color: transparent !important;
  border-radius: 50%;
  -webkit-animation: none;
  animation: none;
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_circle_clipper.left .phanterpwa_circle {
  left: 0;
  border-right-color: transparent !important;
  -webkit-transform: rotate(129deg);
  transform: rotate(129deg);
}

.phanterpwa-components-preloaders-android .preloader-wrapper .phanterpwa_circle_clipper.right .phanterpwa_circle {
  left: -100%;
  border-left-color: transparent !important;
  -webkit-transform: rotate(-129deg);
  transform: rotate(-129deg);
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .phanterpwa_android {
  opacity: 1;
  -webkit-animation: container-rotate 1568ms linear infinite;
  animation: container-rotate 1568ms linear infinite;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-one {
  -webkit-animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_one_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_one_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-two {
  -webkit-animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_two_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_two_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-three {
  -webkit-animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_three_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_three_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-four {
  -webkit-animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_four_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both, phanterpwa_android_four_fade 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer,
.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-one-only,
.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-two-only,
.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-three-only,
.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .spinner-layer.spinner-four-only {
  opacity: 1;
  -webkit-animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: phanterpwa_android_rotate_animate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .phanterpwa_circle_clipper.left .phanterpwa_circle {
  -webkit-animation: left-spin 1333ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: left-spin 1333ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

.phanterpwa-components-preloaders-android .preloader-wrapper.enabled .phanterpwa_circle_clipper.right .phanterpwa_circle {
  -webkit-animation: right-spin 1333ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
  animation: right-spin 1333ms cubic-bezier(0.4, 0, 0.2, 1) infinite both;
}

@-webkit-keyframes container-rotate {
  to {
    -webkit-transform: rotate(360deg);
  }
}

@keyframes container-rotate {
  to {
    transform: rotate(360deg);
  }
}

@-webkit-keyframes phanterpwa_android_rotate_animate {
  12.5% {
    webkit-transform: rotate(135deg);
  }
  25% {
    webkit-transform: rotate(270deg);
  }
  37.5% {
    webkit-transform: rotate(405deg);
  }
  50% {
    webkit-transform: rotate(540deg);
  }
  62.5% {
    webkit-transform: rotate(675deg);
  }
  75% {
    webkit-transform: rotate(810deg);
  }
  87.5% {
    webkit-transform: rotate(945deg);
  }
  to {
    webkit-transform: rotate(1080deg);
  }
}

@keyframes phanterpwa_android_rotate_animate {
  12.5% {
    transform: rotate(135deg);
  }
  25% {
    transform: rotate(270deg);
  }
  37.5% {
    transform: rotate(405deg);
  }
  50% {
    transform: rotate(540deg);
  }
  62.5% {
    transform: rotate(675deg);
  }
  75% {
    transform: rotate(810deg);
  }
  87.5% {
    transform: rotate(945deg);
  }
  to {
    transform: rotate(1080deg);
  }
}

@-webkit-keyframes phanterpwa_android_one_fade {
  from {
    opacity: 1;
  }
  25% {
    opacity: 1;
  }
  26% {
    opacity: 0;
  }
  89% {
    opacity: 0;
  }
  90% {
    opacity: 1;
  }
  100% {
    opacity: 1;
  }
}

@keyframes phanterpwa_android_one_fade {
  from {
    opacity: 1;
  }
  25% {
    opacity: 1;
  }
  26% {
    opacity: 0;
  }
  89% {
    opacity: 0;
  }
  90% {
    opacity: 1;
  }
  100% {
    opacity: 1;
  }
}

@-webkit-keyframes phanterpwa_android_two_fade {
  from {
    opacity: 0;
  }
  15% {
    opacity: 0;
  }
  25% {
    opacity: 1;
  }
  50% {
    opacity: 1;
  }
  51% {
    opacity: 0;
  }
}

@keyframes phanterpwa_android_two_fade {
  from {
    opacity: 0;
  }
  15% {
    opacity: 0;
  }
  25% {
    opacity: 1;
  }
  50% {
    opacity: 1;
  }
  51% {
    opacity: 0;
  }
}

@-webkit-keyframes phanterpwa_android_three_fade {
  from {
    opacity: 0;
  }
  40% {
    opacity: 0;
  }
  50% {
    opacity: 1;
  }
  75% {
    opacity: 1;
  }
  76% {
    opacity: 0;
  }
}

@keyframes phanterpwa_android_three_fade {
  from {
    opacity: 0;
  }
  40% {
    opacity: 0;
  }
  50% {
    opacity: 1;
  }
  75% {
    opacity: 1;
  }
  76% {
    opacity: 0;
  }
}

@-webkit-keyframes phanterpwa_android_four_fade {
  from {
    opacity: 0;
  }
  65% {
    opacity: 0;
  }
  75% {
    opacity: 1;
  }
  90% {
    opacity: 1;
  }
  100% {
    opacity: 0;
  }
}

@keyframes phanterpwa_android_four_fade {
  from {
    opacity: 0;
  }
  65% {
    opacity: 0;
  }
  75% {
    opacity: 1;
  }
  90% {
    opacity: 1;
  }
  100% {
    opacity: 0;
  }
}

@-webkit-keyframes left-spin {
  from {
    -webkit-transform: rotate(130deg);
  }
  50% {
    -webkit-transform: rotate(-5deg);
  }
  to {
    -webkit-transform: rotate(130deg);
  }
}

@keyframes left-spin {
  from {
    transform: rotate(130deg);
  }
  50% {
    transform: rotate(-5deg);
  }
  to {
    transform: rotate(130deg);
  }
}

@-webkit-keyframes right-spin {
  from {
    -webkit-transform: rotate(-130deg);
  }
  50% {
    -webkit-transform: rotate(5deg);
  }
  to {
    -webkit-transform: rotate(-130deg);
  }
}

@keyframes right-spin {
  from {
    transform: rotate(-130deg);
  }
  50% {
    transform: rotate(5deg);
  }
  to {
    transform: rotate(-130deg);
  }
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\preloaders\discs.sass */
.phanterpwa-components-preloaders-discs * {
  box-sizing: border-box;
}

.phanterpwa-components-preloaders-discs .preloader-wrapper {
  display: inline-block;
  position: relative;
  width: 60px;
  height: 60px;
}

.phanterpwa-components-preloaders-discs .preloader-wrapper .phanterpwa_discs {
  -moz-border-radius: 100%;
  -webkit-border-radius: 100%;
  border-radius: 100%;
  position: absolute;
}

.phanterpwa-components-preloaders-discs .preloader-wrapper .phanterpwa_discs_one {
  border-top: 5px solid red;
  border-left: 5px solid red;
  border-bottom: 5px solid transparent;
  border-right: 5px solid transparent;
  left: 40%;
  top: 40%;
  width: 20%;
  height: 20%;
}

.phanterpwa-components-preloaders-discs .preloader-wrapper .phanterpwa_discs_two {
  border-top: 5px solid blue;
  border-left: 5px solid blue;
  border-bottom: 5px solid transparent;
  border-right: 5px solid transparent;
  left: 30%;
  top: 30%;
  width: 40%;
  height: 40%;
}

.phanterpwa-components-preloaders-discs .preloader-wrapper .phanterpwa_discs_three {
  border-top: 5px solid orange;
  border-left: 5px solid orange;
  border-bottom: 5px solid transparent;
  border-right: 5px solid transparent;
  left: 20%;
  top: 20%;
  width: 60%;
  height: 60%;
}

.phanterpwa-components-preloaders-discs .preloader-wrapper .phanterpwa_discs_four {
  border-top: 5px solid green;
  border-left: 5px solid green;
  border-bottom: 5px solid transparent;
  border-right: 5px solid transparent;
  left: 10%;
  top: 10%;
  width: 80%;
  height: 80%;
}

.phanterpwa-components-preloaders-discs .preloader-wrapper .phanterpwa_discs_five {
  border-top: 5px solid black;
  border-left: 5px solid black;
  border-bottom: 5px solid transparent;
  border-right: 5px solid transparent;
  left: 0;
  top: 0;
  width: 100%;
  height: 100%;
}

.phanterpwa-components-preloaders-discs .preloader-wrapper.enabled .phanterpwa_discs {
  -webkit-animation: phanterpwa_discs_animate 2s infinite;
  animation: phanterpwa_discs_animate 2s infinite;
}

.phanterpwa-components-preloaders-discs .preloader-wrapper.enabled .phanterpwa_discs_two {
  -webkit-animation-delay: 0.2s;
  animation-delay: 0.2s;
}

.phanterpwa-components-preloaders-discs .preloader-wrapper.enabled .phanterpwa_discs_three {
  -webkit-animation-delay: 0.4s;
  animation-delay: 0.4s;
}

.phanterpwa-components-preloaders-discs .preloader-wrapper.enabled .phanterpwa_discs_four {
  -webkit-animation-delay: 0.6s;
  animation-delay: 0.6s;
}

.phanterpwa-components-preloaders-discs .preloader-wrapper.enabled .phanterpwa_discs_five {
  -webkit-animation-delay: 0.8s;
  animation-delay: 0.8s;
}

@-webkit-keyframes phanterpwa_discs_animate {
  50% {
    -ms-transform: rotate(360deg);
    -webkit-transform: rotate(360deg);
    transform: rotate(360deg);
  }
}

@keyframes phanterpwa_discs_animate {
  50% {
    -ms-transform: rotate(360deg);
    -webkit-transform: rotate(360deg);
    transform: rotate(360deg);
  }
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\preloaders\explosion.sass */
.phanterpwa-components-preloaders-explosion * {
  box-sizing: border-box;
}

.phanterpwa-components-preloaders-explosion .left {
  float: left !important;
}

.phanterpwa-components-preloaders-explosion .right {
  float: right !important;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper {
  display: inline-block;
  position: relative;
  width: 60px;
  height: 60px;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper .phanterpwa_explosion {
  width: 16%;
  height: 16%;
  background-color: red;
  position: absolute;
  left: 40%;
  top: 40%;
  border-radius: 100%;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper .phanterpwa_explosion:nth-child(2n+0) {
  margin-right: 0px;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper .phanterpwa_explosion_container {
  width: 100%;
  height: 100%;
  position: relative;
  transform: rotate(45deg);
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper.enabled .phanterpwa_explosion_one {
  background-color: red;
  -webkit-animation: phanterpwa_explosion_one_animate 2s infinite;
  animation: phanterpwa_explosion_one_animate 2s infinite;
  -webkit-animation-delay: 0.2s;
  animation-delay: 0.2s;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper.enabled .phanterpwa_explosion_two {
  background-color: red;
  -webkit-animation: phanterpwa_explosion_two_animate 2s infinite;
  animation: phanterpwa_explosion_two_animate 2s infinite;
  -webkit-animation-delay: 0.3s;
  animation-delay: 0.3s;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper.enabled .phanterpwa_explosion_three {
  background-color: red;
  -webkit-animation: phanterpwa_explosion_three_animate 2s infinite;
  animation: phanterpwa_explosion_three_animate 2s infinite;
  -webkit-animation-delay: 0.4s;
  animation-delay: 0.4s;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper.enabled .phanterpwa_explosion_four {
  background-color: red;
  -webkit-animation: phanterpwa_explosion_four_animate 2s infinite;
  animation: phanterpwa_explosion_four_animate 2s infinite;
  -webkit-animation-delay: 0.5s;
  animation-delay: 0.5s;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper.enabled .phanterpwa_explosion_five {
  background-color: red;
  -webkit-animation: phanterpwa_explosion_five_animate 2s infinite;
  animation: phanterpwa_explosion_five_animate 2s infinite;
  -webkit-animation-delay: 0.6s;
  animation-delay: 0.6s;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper.enabled .phanterpwa_explosion_six {
  background-color: red;
  -webkit-animation: phanterpwa_explosion_six_animate 2s infinite;
  animation: phanterpwa_explosion_six_animate 2s infinite;
  -webkit-animation-delay: 0.7s;
  animation-delay: 0.7s;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper.enabled .phanterpwa_explosion_seven {
  background-color: red;
  -webkit-animation: phanterpwa_explosion_seven_animate 2s infinite;
  animation: phanterpwa_explosion_seven_animate 2s infinite;
  -webkit-animation-delay: 0.8s;
  animation-delay: 0.8s;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper.enabled .phanterpwa_explosion_eight {
  background-color: red;
  -webkit-animation: phanterpwa_explosion_eight_animate 2s infinite;
  animation: phanterpwa_explosion_eight_animate 2s infinite;
  animation--webkit-animation-delay: 0.9s;
  animation-delay: 0.9s;
}

.phanterpwa-components-preloaders-explosion .preloader-wrapper.enabled .phanterpwa_explosion_big {
  background-color: red;
  position: absolute;
  width: 16%;
  height: 16%;
  left: 40%;
  top: 40%;
  -webkit-animation: phanterpwa_explosion_big_animate 2s infinite;
  animation: phanterpwa_explosion_big_animate 2s infinite;
  -webkit-animation-delay: 0.5s;
  animation-delay: 1.5s;
}

@-webkit-keyframes phanterpwa_explosion_big_animate {
  40% {
    width: 50%;
    height: 50%;
    top: 27%;
    left: 27%;
  }
}

@-webkit-keyframes phanterpwa_explosion_one_animate {
  60% {
    top: 42%;
    left: 0%;
  }
}

@-webkit-keyframes phanterpwa_explosion_two_animate {
  60% {
    top: 0;
    left: 0;
  }
}

@-webkit-keyframes phanterpwa_explosion_three_animate {
  60% {
    top: 0%;
    left: 42%;
  }
}

@-webkit-keyframes phanterpwa_explosion_four_animate {
  60% {
    top: 0%;
    left: 84%;
  }
}

@-webkit-keyframes phanterpwa_explosion_five_animate {
  60% {
    top: 42%;
    left: 84%;
  }
}

@-webkit-keyframes phanterpwa_explosion_six_animate {
  60% {
    top: 84%;
    left: 84%;
  }
}

@-webkit-keyframes phanterpwa_explosion_seven_animate {
  60% {
    top: 84%;
    left: 42%;
  }
}

@-webkit-keyframes phanterpwa_explosion_eight_animate {
  60% {
    top: 84%;
    left: 0%;
  }
}

@keyframes phanterpwa_explosion_big_animate {
  40% {
    width: 50%;
    height: 50%;
    top: 27%;
    left: 27%;
  }
}

@keyframes phanterpwa_explosion_one_animate {
  60% {
    top: 42%;
    left: 0%;
  }
}

@keyframes phanterpwa_explosion_two_animate {
  60% {
    top: 0;
    left: 0;
  }
}

@keyframes phanterpwa_explosion_three_animate {
  60% {
    top: 0%;
    left: 42%;
  }
}

@keyframes phanterpwa_explosion_four_animate {
  60% {
    top: 0%;
    left: 84%;
  }
}

@keyframes phanterpwa_explosion_five_animate {
  60% {
    top: 42%;
    left: 84%;
  }
}

@keyframes phanterpwa_explosion_six_animate {
  60% {
    top: 84%;
    left: 84%;
  }
}

@keyframes phanterpwa_explosion_seven_animate {
  60% {
    top: 84%;
    left: 42%;
  }
}

@keyframes phanterpwa_explosion_eight_animate {
  60% {
    top: 84%;
    left: 0%;
  }
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\preloaders\run_points.sass */
.phanterpwa-components-preloaders-run_points * {
  box-sizing: border-box;
}

.phanterpwa-components-preloaders-run_points .left {
  float: left !important;
}

.phanterpwa-components-preloaders-run_points .right {
  float: right !important;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper {
  display: inline-block;
  position: relative;
  width: 60px;
  height: 24px;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper .phanterpwa_run_points_container {
  width: 100%;
  height: 100%;
  position: relative;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper .phanterpwa_run_points {
  width: 20%;
  height: 50%;
  -moz-border-radius: 100%;
  -webkit-border-radius: 100%;
  border-radius: 100%;
  position: absolute;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_one_content {
  -webkit-animation: phanterpwa_run_points 2s linear infinite;
  animation: phanterpwa_run_points 2s linear infinite;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_one_content .phanterpwa_run_points_one {
  background-color: red;
  left: 10%;
  top: 10%;
  border-radius: 100%;
  width: 80%;
  height: 80%;
  position: absolute;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_two_content {
  -webkit-animation: phanterpwa_run_points 2s linear infinite -.4s;
  animation: phanterpwa_run_points 2s linear infinite -.4s;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_two_content .phanterpwa_run_points_two {
  background-color: red;
  left: 10%;
  top: 10%;
  border-radius: 100%;
  width: 80%;
  height: 80%;
  position: absolute;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_three_content {
  -webkit-animation: phanterpwa_run_points 2s linear infinite -.8s;
  animation: phanterpwa_run_points 2s linear infinite -.8s;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_three_content .phanterpwa_run_points_three {
  background-color: red;
  left: 10%;
  top: 10%;
  border-radius: 100%;
  width: 80%;
  height: 80%;
  position: absolute;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_four_content {
  -webkit-animation: phanterpwa_run_points 2s linear infinite -1.2s;
  animation: phanterpwa_run_points 2s linear infinite -1.2s;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_four_content .phanterpwa_run_points_four {
  background-color: red;
  left: 10%;
  top: 10%;
  border-radius: 100%;
  width: 80%;
  height: 80%;
  position: absolute;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_five_content {
  -webkit-animation: phanterpwa_run_points 2s linear infinite -1.6s;
  animation: phanterpwa_run_points 2s linear infinite -1.6s;
}

.phanterpwa-components-preloaders-run_points .preloader-wrapper.enabled .phanterpwa_run_points_five_content .phanterpwa_run_points_five {
  background-color: red;
  left: 10%;
  top: 10%;
  border-radius: 100%;
  width: 80%;
  height: 80%;
  position: absolute;
}

@keyframes phanterpwa_run_points {
  0% {
    width: 20%;
    left: 80%;
    top: 50%;
  }
  80% {
    width: 20%;
    left: 0;
    top: 50%;
  }
  85% {
    left: 0;
    top: 10%;
    width: 20%;
    height: 50%;
  }
  88% {
    top: 20%;
    width: 40%;
    height: 30%;
  }
  92.5% {
    top: 20%;
    width: 40%;
    height: 30%;
  }
  97% {
    top: 20%;
    left: 80%;
    width: 20%;
    height: 30%;
  }
  100% {
    width: 20%;
    left: 80%;
    top: 50%;
  }
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\preloaders\square.sass */
.phanterpwa-components-preloaders-square * {
  box-sizing: border-box;
}

.phanterpwa-components-preloaders-square .preloader-wrapper {
  display: inline-block;
  position: relative;
  width: 60px;
  height: 60px;
}

.phanterpwa-components-preloaders-square .preloader-wrapper .phanterpwa_square_container {
  perspective: 90px;
  width: 100%;
  height: 100%;
}

.phanterpwa-components-preloaders-square .preloader-wrapper.enabled .phanterpwa_square {
  width: 100%;
  height: 100%;
  background-color: red;
  -webkit-animation: phanterpwa_square_animate 1s infinite ease-in-out;
  animation: phanterpwa_square_animate 1s infinite ease-in-out;
  margin-right: auto;
  margin-left: auto;
  -webkit-border-radius: 0;
  border-radius: 0;
}

@-webkit-keyframes phanterpwa_square_animate {
  50% {
    -webkit-transform: rotateY(-180deg);
  }
  100% {
    -webkit-transform: rotateY(-180deg) rotateX(-180deg);
  }
}

@keyframes phanterpwa_square_animate {
  0% {
    -webkit-transform: rotateX(0deg) rotateY(0deg);
    transform: rotateX(0deg) rotateY(0deg);
  }
  50% {
    -webkit-transform: rotateX(-180deg) rotateY(0deg);
    transform: rotateX(-180deg) rotateY(0deg);
  }
  100% {
    -webkit-transform: rotateX(-180deg) rotateY(-180deg);
    transform: rotateX(-180deg) rotateY(-180deg);
  }
}

/* SASS Source Code: C:\virtualenv\py37\phanterpwaenv\lib\site-packages\phanterpwa\usual_sass\preloaders\squares.sass */
.phanterpwa-components-preloaders-squares * {
  box-sizing: border-box;
}

.phanterpwa-components-preloaders-squares .left {
  float: left !important;
}

.phanterpwa-components-preloaders-squares .right {
  float: right !important;
}

.phanterpwa-components-preloaders-squares .preloader-wrapper {
  display: inline-block;
  position: relative;
  width: 60px;
  height: 60px;
}

.phanterpwa-components-preloaders-squares .preloader-wrapper .phanterpwa_squares_container {
  perspective: 90px;
  height: 100%;
  width: 100%;
}

.phanterpwa-components-preloaders-squares .preloader-wrapper .phanterpwa_squares {
  width: 33.33%;
  height: 33.33%;
  background-color: rgba(255, 255, 255, 0);
  margin-right: auto;
  margin-left: auto;
  border: 4px solid red;
  left: 33.33%;
  top: 33.33%;
  position: absolute;
  -moz-border-radius: 0;
  -webkit-border-radius: 0;
  border-radius: 0;
}

.phanterpwa-components-preloaders-squares .preloader-wrapper.enabled .phanterpwa_squares_one {
  border: 4px solid red;
  -webkit-animation: phanterpwa_squares_one_animate 1s infinite ease-in-out;
  animation: phanterpwa_squares_one_animate 1s infinite ease-in-out;
}

.phanterpwa-components-preloaders-squares .preloader-wrapper.enabled .phanterpwa_squares_two {
  border: 4px solid red;
  -webkit-animation: phanterpwa_squares_two 1s forwards, phanterpwa_squares_two_animate 1s infinite ease-in-out;
  animation: phanterpwa_squares_two 1s forwards, phanterpwa_squares_two_animate 1s infinite ease-in-out;
}

.phanterpwa-components-preloaders-squares .preloader-wrapper.enabled .phanterpwa_squares_three {
  border: 4px solid red;
  -webkit-animation: phanterpwa_squares_three 1s forwards, phanterpwa_squares_three_animate 1s infinite ease-in-out;
  animation: phanterpwa_squares_three 1s forwards, phanterpwa_squares_three_animate 1s infinite ease-in-out;
}

@-webkit-keyframes phanterpwa_squares_two {
  100% {
    width: 66.66%;
    height: 66.66%;
    left: 48px;
    top: 48px;
  }
}

@keyframes phanterpwa_squares_two {
  100% {
    width: 66.66%;
    height: 66.66%;
    left: 16.66%;
    top: 16.66%;
  }
}

@-webkit-keyframes phanterpwa_squares_three {
  100% {
    width: 100%;
    height: 100%;
    left: 0;
    top: 0;
  }
}

@keyframes phanterpwa_squares_three {
  100% {
    width: 100%;
    height: 100%;
    left: 0;
    top: 0;
  }
}

@-webkit-keyframes phanterpwa_squares_one_animate {
  50% {
    -webkit-transform: rotateY(-180deg);
  }
  100% {
    -webkit-transform: rotateY(-180deg) rotateX(-180deg);
  }
}

@keyframes phanterpwa_squares_one_animate {
  0% {
    transform: rotateX(0deg) rotateY(0deg);
    -webkit-transform: rotateX(0deg) rotateY(0deg);
  }
  50% {
    transform: rotateX(-180deg) rotateY(0deg);
    -webkit-transform: rotateX(-180deg) rotateY(0deg);
  }
  100% {
    transform: rotateX(-180deg) rotateY(-180deg);
    -webkit-transform: rotateX(-180deg) rotateY(-180deg);
  }
}

@-webkit-keyframes phanterpwa_squares_two_animate {
  50% {
    -webkit-transform: rotateY(180deg);
  }
  100% {
    -webkit-transform: rotateY(180deg) rotateX(180deg);
  }
}

@keyframes phanterpwa_squares_two_animate {
  0% {
    transform: rotateX(0deg) rotateY(0deg);
    -webkit-transform: rotateX(0deg) rotateY(0deg);
  }
  50% {
    transform: rotateX(180deg) rotateY(0deg);
    -webkit-transform: rotateX(180deg) rotateY(0deg);
  }
  100% {
    transform: rotateX(180deg) rotateY(180deg);
    -webkit-transform: rotateX(180deg) rotateY(180deg);
  }
}

@-webkit-keyframes phanterpwa_squares_three_animate {
  50% {
    -webkit-transform: rotateY(-180deg);
  }
  100% {
    -webkit-transform: rotateY(-180deg) rotateX(-180deg);
  }
}

@keyframes phanterpwa_squares_three_animate {
  0% {
    transform: rotateX(0deg) rotateY(0deg);
    -webkit-transform: rotateX(0deg) rotateY(0deg);
  }
  50% {
    transform: rotateX(-180deg) rotateY(0deg);
    -webkit-transform: rotateX(-180deg) rotateY(0deg);
  }
  100% {
    transform: rotateX(-180deg) rotateY(-180deg);
    -webkit-transform: rotateX(-180deg) rotateY(-180deg);
  }
}
